;created as an attempt to replicate results found by Cremers as expressed in Key Exchange in IPsec Revisited.
;(Or rather from input of ikev1-quick.cpp)
(herald "IKEv1 Quick Mode" (algebra diffie-hellman) (bound 15))


;Macros to reflect what certian parties do not know
(defmacro (zi) (exp Gr i))
(defmacro (zr) (exp Gi r))
;(defmacro (kir) (hash ni nr (zi))) ; May come back to this or some form of bidirectionality
;(defmacro (kri) (hash  nr ni (zr)))



;Macros for the different messages sent back and forth
;	I have made duplicates for the sake of bidirectionality
;	and advise this file be replaced/revised once there is a function
;	available for bidirectional keys.
(defmacro (hash_1i) (hash "prf" (ltk I R) mid lst ni (exp (gen) i) I R))
(defmacro (hash_1ii) (hash "prf" (ltk R I) mid lst ni (exp (gen) i) I R))

(defmacro (hash_1r) (hash "prf" (ltk I R) mid lst ni Gi I R))
(defmacro (hash_1rr) (hash "prf" (ltk R I) mid lst ni Gi I R))

(defmacro (hash_2i) (hash "prf" (ltk I R) mid ni algo nr Gr I R))
(defmacro (hash_2ii) (hash "prf" (ltk R I) mid ni algo nr Gr I R))

(defmacro (hash_2r) (hash "prf" (ltk I R) mid ni algo nr (exp (gen) r) I R))
(defmacro (hash_2rr) (hash "prf" (ltk R I) mid ni algo nr (exp (gen) r) I R))

(defmacro (hash_3) (hash "prf" (ltk I R) mid ni nr))
(defmacro (hash_33) (hash "prf" (ltk R I) mid ni nr))

(defmacro (ilength) 2)
(defmacro (rlength) 3)

(defmacro (keymati) (hash "prf" (ltk I R) (exp Gr i) lst algo ni nr))
(defmacro (keymatii) (hash "prf" (ltk R I) (exp Gr i) lst algo ni nr))

(defmacro (keymatr) (hash "prf" (ltk I R) (exp Gi r) lst algo ni nr))
(defmacro (keymatrr) (hash "prf" (ltk R I) (exp Gi r) lst algo ni nr))

(defmacro (keymatf) (hash "prf" (ltk I R) (exp (gen) (mul i r)) lst algo ni nr))
(defmacro (keymatff) (hash "prf" (ltk R I) (exp (gen) (mul i r)) lst algo ni nr))

(defprotocol quick diffie-hellman
	     (defrole init
		       (vars (ni nr mid lst algo text) (i expn) (Gr base) (I R name))
		       (trace
			 (send (cat I R mid (enc (hash_1i) lst ni (exp (gen) i) I R (ltk I R))
				    (enc (hash_1ii) lst ni (exp (gen) i) I R (ltk R I))))
			 (recv (cat R I mid (enc (hash_2i) algo nr Gr I R (ltk I R))
				    (enc (hash_2ii) algo nr Gr I R (ltk R I))))
			 (send (cat I R mid (enc (hash_3) (ltk I R))
			       (enc (hash_33) (ltk R I)))))
		       (uniq-orig ni mid)
		       (uniq-gen i)
		       (neq ((gen) Gr)))
	     (defrole resp
		      (vars (ni nr mid lst algo text) (r expn) (Gi base) (I R name))
		      (trace
			(recv (cat I R mid (enc (hash_1r) lst ni Gi I R (ltk I R))
				   (enc (hash_1rr) lst ni Gi I R (ltk R I))))
			(send (cat R I mid (enc (hash_2r) algo nr (exp (gen) r) I R (ltk I R))
				   (enc (hash_2rr) algo nr (exp (gen) r) I R (ltk R I))))
			(recv (cat I R mid (enc (hash_3) (ltk I R))
				   (enc (hash_33) (ltk R I))))
                        (send "blab"))
			(uniq-orig nr)
			(uniq-gen r)
			(neq ((gen) Gi)))
             (defrole blabber-skey
               (vars (key skey))
               (trace
                (recv "blab")
                (send key))
               (uniq-orig key)))

;;;
;;; Authentication skeletons
;;;

;--------------------------------

(defgoal quick
  ; Aliveness - my peer has taken an action
  (forall ((iend node) (I R name) (ni nr mid lst algo text) (i expn) (Gr base))
	  (implies
	   (and (p "init" (ilength) iend) 
		(p "init" "R" iend R) (p "init" "I" iend I)
		(p "init" "ni" iend ni)
		(p "init" "nr" iend nr)
		(p "init" "mid" iend mid)
		(p "init" "lst" iend lst)
		(p "init" "i" iend i)
		(p "init" "Gr" iend Gr)
		(p "init" "algo" iend algo)
		(non (ltk R I)) (non (ltk I R)))
	   (or (exists ((r1 node))
		       (and (p "resp" 0 r1) (p "resp" "R" r1 R)))
	       (exists ((i1 node))
		       (and (p "init" 0 i1) (p "init" "I" i1 R))))))
  ; Weak agreement - my peer and I agree on our names
  (forall ((iend node) (I R name) (ni nr mid lst algo text) (i expn) (Gr base))
	  (implies
	   (and (p "init" (ilength) iend) 
		(p "init" "R" iend R) (p "init" "I" iend I)
		(p "init" "ni" iend ni)
		(p "init" "nr" iend nr)
		(p "init" "mid" iend mid)
		(p "init" "lst" iend lst)
		(p "init" "i" iend i)
		(p "init" "Gr" iend Gr)
		(p "init" "algo" iend algo)
		(non (ltk R I)) (non (ltk I R)))
	   (or (exists ((r1 node))
		       (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)))
	       (exists ((i1 node))
		       (and (p "init" 0 i1) (p "init" "I" i1 R) (p "init" "R" i1 I))))))
  ; Minimal agreement - my peer and I agree on our names and roles, and on the DH key
  (forall ((iend node) (I R name) (ni nr mid lst algo text) (i expn) (Gr base))
	  (implies
	   (and (p "init" (ilength) iend) 
		(p "init" "R" iend R) (p "init" "I" iend I)
		(p "init" "ni" iend ni)
		(p "init" "nr" iend nr)
		(p "init" "mid" iend mid)
		(p "init" "lst" iend lst)
		(p "init" "i" iend i)
		(p "init" "Gr" iend Gr)
		(p "init" "algo" iend algo)
		(non (ltk R I)) (non (ltk I R)))
	   (exists ((r1 node) (Gi base) (r expn))
		   (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)
			(p "resp" "Gi" r1 Gi) (p "resp" "r" r1 r)
			(= (exp Gi r) (exp Gr i))))))
  ; Full agreement - my peer and I agree on the full transcript
  (forall ((iend node) (I R name) (ni nr mid lst algo text) (i expn) (Gr base))
	  (implies
	   (and (p "init" (ilength) iend) 
		(p "init" "R" iend R) (p "init" "I" iend I)
		(p "init" "ni" iend ni)
		(p "init" "nr" iend nr)
		(p "init" "mid" iend mid)
		(p "init" "lst" iend lst)
		(p "init" "i" iend i)
		(p "init" "Gr" iend Gr)
		(p "init" "algo" iend algo)
		(non (ltk R I)) (non (ltk I R)))
	   (exists ((r1 node) (r expn))
		   (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)
			(p "resp" "ni" r1 ni) (p "resp" "nr" r1 nr)
			(p "resp" "r" r1 r) (p "resp" "mid" r1 mid)
			(p "resp" "lst" r1 lst)
			(p "resp" "Gi" r1 (exp (gen) i))
			(p "resp" "algo" r1 algo)))))
)

(defgoal quick
  ; Aliveness - my peer has taken an action
  (forall ((rend node) (I R name) (ni nr mid lst algo text) (r expn) (Gi base))
	  (implies
	   (and (p "resp" (rlength) rend) 
		(p "resp" "R" rend R) (p "resp" "I" rend I) (p "resp" "ni" rend ni)
		(p "resp" "nr" rend nr) (p "resp" "mid" rend mid)
		(p "resp" "lst" rend lst) (p "resp" "algo" rend algo) (p "resp" "r" rend r)
		(p "resp" "Gi" rend Gi)
		(non (ltk R I)) (non (ltk I R)))
	   (or (exists ((r1 node))
		       (and (p "resp" 0 r1) (p "resp" "R" r1 I)))
	       (exists ((i1 node))
		       (and (p "init" 0 i1) (p "init" "I" i1 I))))))
  ; Weak agreement - my peer and I agree on our names
  (forall ((rend node) (I R name) (ni nr mid lst algo text) (r expn) (Gi base))
	  (implies
	   (and (p "resp" (rlength) rend) 
		(p "resp" "R" rend R) (p "resp" "I" rend I) (p "resp" "ni" rend ni)
		(p "resp" "nr" rend nr) (p "resp" "mid" rend mid)
		(p "resp" "lst" rend lst) (p "resp" "algo" rend algo) (p "resp" "r" rend r)
		(p "resp" "Gi" rend Gi)
		(non (ltk R I)) (non (ltk I R)))
	   (or (exists ((r1 node))
		       (and (p "resp" 0 r1) (p "resp" "R" r1 I) (p "resp" "I" r1 R)))
	       (exists ((i1 node))
		       (and (p "init" 0 i1) (p "init" "I" i1 I) (p "init" "R" i1 R))))))
  ; Minimal agreement - my peer and I agree on our names and roles, and on the DH key
  (forall ((rend node) (I R name) (ni nr mid lst algo text) (r expn) (Gi base))
	  (implies
	   (and (p "resp" (rlength) rend) 
		(p "resp" "R" rend R) (p "resp" "I" rend I) (p "resp" "ni" rend ni)
		(p "resp" "nr" rend nr) (p "resp" "mid" rend mid)
		(p "resp" "lst" rend lst) (p "resp" "algo" rend algo) (p "resp" "r" rend r)
		(p "resp" "Gi" rend Gi)
		(non (ltk R I)) (non (ltk I R)))
	   (exists ((i1 node) (Gr base) (i expn))
		   (and (p "init" 0 i1) (p "init" "R" i1 R) (p "init" "I" i1 I)
		        (p "init" "Gr" i1 Gr) (p "init" "i" i1 i)
			(= (exp Gi r) (exp Gr i))))))
  ; Full agreement - my peer and I agree on the full transcript
  (forall ((rend node) (I R name) (ni nr mid lst algo text) (r expn) (Gi base))
	  (implies
	   (and (p "resp" (rlength) rend) 
		(p "resp" "R" rend R) (p "resp" "I" rend I) (p "resp" "ni" rend ni)
		(p "resp" "nr" rend nr) (p "resp" "mid" rend mid)
		(p "resp" "lst" rend lst) (p "resp" "algo" rend algo) (p "resp" "r" rend r)
		(p "resp" "Gi" rend Gi)
		(non (ltk R I)) (non (ltk I R)))
	   (exists ((i1 node) (i expn))
		   (and (p "init" 0 i1) (p "init" "R" i1 R) (p "init" "I" i1 I)
			(p "init" "ni" i1 ni) (p "init" "nr" i1 nr)
			(p "init" "mid" i1 mid)
			(p "init" "algo" i1 algo) (p "init" "lst" i1 lst)
		        (p "init" "Gr" i1 (exp (gen) r)) (p "init" "i" i1 i)
			))))
  (comment "Responder point of view; agreement / weak agreement / aliveness goals")
  )

;;;
;;; Key secrecy skeletons (no privk leaks)
;;;

(defskeleton quick
	    (vars (ni nr ci cr lst algo text) (i expn) (Gr base) (I R name))
	    (defstrand init 3 (I I) (R R) (Gr Gr) (i i) (ni ni) (nr nr))
            (deflistener (keymati))
	    (non-orig (ltk I R) (ltk R I)))

(defskeleton quick
	    (vars (ni nr ci cr lst algo text) (i expn) (Gr base) (I R name))
	    (defstrand init 3 (I I) (R R) (Gr Gr) (i i) (ni ni) (nr nr))
            (deflistener (keymatii))
	    (non-orig (ltk I R) (ltk R I)))

(defskeleton quick
	    (vars (ni nr ci cr lst algo text) (r expn) (Gi base) (I R name))
	    (defstrand resp 3 (I I) (R R) (Gi Gi) (r r) (ni ni) (nr nr))
            (deflistener (keymatr))
	    (non-orig (ltk I R) (ltk R I)))

(defskeleton quick
	    (vars (ni nr ci cr lst algo text) (r expn) (Gi base) (I R name))
	    (defstrand resp 3 (I I) (R R) (Gi Gi) (r r) (ni ni) (nr nr))
            (deflistener (keymatrr))
	    (non-orig (ltk I R) (ltk R I)))



;;;
;;; Forward Secrecy skeletons (privk leaks)
;;;

;;; Weak Forward Secrecy
(defskeleton quick
  (vars (nr algo ni mid lst text) (I R name) (i r expn))
  (defstrand resp 4 (ni ni) (nr nr) (mid mid) (lst lst) (algo algo)
    (I I) (R R) (Gi (exp (gen) i)) (r r))
  (defstrand init 3 (ni ni) (nr nr) (mid mid) (lst lst) (algo algo)
    (I I) (R R) (Gr (exp (gen) r)) (i i))
  (defstrand blabber-skey 2 (key (ltk I R)))
  (defstrand blabber-skey 2 (key (ltk R I)))
  (deflistener (keymatf))
  (precedes ((0 1) (1 1)) ((1 0) (0 0)) ((1 2) (0 2)) ((0 3) (2 0)) ((0 3) (3 0)))
  (ind-zero-in (r (exp (gen) i)))
  (neq ((gen) (exp (gen) i)) ((gen) (exp (gen) r)))
  (pen-non-orig (ltk I R) (ltk R I))
  (uniq-gen i r)
  (uniq-orig nr ni mid))

(defskeleton quick
  (vars (nr algo ni mid lst text) (I R name) (i r expn))
  (defstrand resp 4 (ni ni) (nr nr) (mid mid) (lst lst) (algo algo)
    (I I) (R R) (Gi (exp (gen) i)) (r r))
  (defstrand init 3 (ni ni) (nr nr) (mid mid) (lst lst) (algo algo)
    (I I) (R R) (Gr (exp (gen) r)) (i i))
  (defstrand blabber-skey 2 (key (ltk I R)))
  (defstrand blabber-skey 2 (key (ltk R I)))
  (deflistener (keymatff))
  (precedes ((0 1) (1 1)) ((1 0) (0 0)) ((1 2) (0 2)) ((0 3) (2 0)) ((0 3) (3 0)))
  (ind-zero-in (r (exp (gen) i)))
  (neq ((gen) (exp (gen) i)) ((gen) (exp (gen) r)))
  (pen-non-orig (ltk I R) (ltk R I))
  (uniq-gen i r)
  (uniq-orig nr ni mid))

;;; Perfect Forward Secrecy
(defskeleton quick
	    (vars (ni nr ci cr lst algo text) (i expn) (Gr base) (I R name))
	    (defstrand init 3 (I I) (R R) (Gr Gr) (i i) (ni ni) (nr nr))
            (defstrand blabber-skey 2 (key (ltk I R)))
            (defstrand blabber-skey 2 (key (ltk R I)))
            (deflistener (keymati))
            (precedes ((0 2) (1 0)) ((0 2) (2 0)))
	    (pen-non-orig (ltk I R) (ltk R I)))

(defskeleton quick
	    (vars (ni nr ci cr lst algo text) (i expn) (Gr base) (I R name))
	    (defstrand init 3 (I I) (R R) (Gr Gr) (i i) (ni ni) (nr nr))
            (defstrand blabber-skey 2 (key (ltk I R)))
            (defstrand blabber-skey 2 (key (ltk R I)))
            (deflistener (keymatii))
            (precedes ((0 2) (1 0)) ((0 2) (2 0)))
	    (pen-non-orig (ltk I R) (ltk R I)))

(defskeleton quick
	    (vars (ni nr ci cr lst algo text) (r expn) (Gi base) (I R name))
	    (defstrand resp 4 (I I) (R R) (Gi Gi) (r r) (ni ni) (nr nr))
            (defstrand blabber-skey 2 (key (ltk I R)))
            (defstrand blabber-skey 2 (key (ltk R I)))
            (deflistener (keymatr))
            (precedes ((0 3) (1 0)) ((0 3) (2 0)))
	    (pen-non-orig (ltk I R) (ltk R I)))

(defskeleton quick
	    (vars (ni nr ci cr lst algo text) (r expn) (Gi base) (I R name))
	    (defstrand resp 4 (I I) (R R) (Gi Gi) (r r) (ni ni) (nr nr))
            (defstrand blabber-skey 2 (key (ltk I R)))
            (defstrand blabber-skey 2 (key (ltk R I)))
            (deflistener (keymatrr))
            (precedes ((0 3) (1 0)) ((0 3) (2 0)))
	    (pen-non-orig (ltk I R) (ltk R I)))







