;created as an attempt to replicate results found by Cremers as expressed in Key Exchange in IPsec Revisited.
;(Or rather from input of ikev2-sig-child.cpp)
;This refers to sections 1.3, 2.17, 2.18 and 3.14 of rfc 4306
;(This does not seem to be the first Child_SA 
(herald "IKEv2 Sig" (algebra diffie-hellman) (bound 15))


;Macros to reflect what certian parties do not know
(defmacro (zi) (exp Gr i))
(defmacro (zr) (exp Gi r))
(defmacro (zj) (exp Gt j))
(defmacro (zt) (exp Gj t))
;Note: Instead of using the terms SPIi, SPIr 
;I have decided to stick with ci, cr as described in IKEv1
;to avoid confusion of terms

(defmacro (ilength) 6)
(defmacro (rlength) 5)

;Macros based on those for IKEV2 in common.h
(defmacro (SKi) (hash "KDF" ni nr (zi) ci cr))
(defmacro (SKr) (hash "KDF" ni nr (zr) ci cr))
;Macro for the weak ("perfect") forward secrecy
(defmacro (SKf) (hash "KDF" ni nr (exp (gen) (mul i r)) ci cr))

(defmacro (AUTHii) (enc ci "0" lst1 (exp (gen) i) ni nr (hash (SKi) I) (privk I)))
(defmacro (AUTHir) (enc ci "0" lst1 Gi ni nr (hash (SKr) I) (privk I)))
(defmacro (AUTHri) (enc ci cr algo1 Gr nr ni (hash (SKi) R) (privk R)))
(defmacro (AUTHrr) (enc ci cr algo1 (exp (gen) r) nr ni (hash (SKr) R) (privk R)))



(defprotocol sigChild diffie-hellman
	     (defrole init
		      (vars (ci cr ni nr mi mr lst1 algo1 lst2 algo2 lst3 algo3 TSi TSr text) 
			    (Gr Gt base) (i j expn) (R I name))
		      (trace
			(send (cat I R ci "0" lst1 (exp (gen) i) ni))
			(recv (cat R I ci cr algo1 Gr nr))
			(send (cat I R ci cr (enc I (AUTHii) lst2 TSi TSr (SKi))))
			(recv (cat R I ci cr (enc R (AUTHri) algo2 TSi TSr (SKi))))
			(send (cat I R ci cr (enc I lst3 mi (exp (gen) j) TSi TSr (SKi))))
			(recv (cat R I ci cr (enc R algo3 mr Gt TSi TSr (SKi))))
			(send "blab"))
		      (uniq-orig mi ni ci)
		      (uniq-gen i j))
	     (defrole resp
		      (vars (ci cr ni nr mi mr lst1 algo1 lst2 algo2 lst3 algo3 TSi TSr text)
			    (Gi Gj base) (r t expn) (R I name))
		      (trace
			(recv (cat I R ci "0" lst1 Gi ni))
			(send (cat R I ci cr algo1 (exp (gen) r) nr))
			(recv (cat I R ci cr (enc I (AUTHir) lst2 TSi TSr (SKr))))
			(send (cat R I ci cr (enc R (AUTHrr) algo2 TSi TSr (SKr))))
			(recv (cat I R ci cr (enc I lst3 mi Gj TSi TSr (SKr))))
			(send (cat R I ci cr (enc R algo3 mr (exp (gen) t) TSi TSr (SKr)))))
		      (uniq-orig mr nr cr)
		      (uniq-gen r t))
	     (defrole blab-akey
               (vars (key akey))
               (trace
                (recv "blab")
                (send key))
               (uniq-orig key)))

;Standard skeletons:

(defgoal sigChild
  ; Aliveness - my peer has taken an action
  (forall ((iend brend brstart node) (I R name) (ni nr ci cr lst1 lst2 algo1 algo2 lst3
						    algo3 mi mr TSi TSr text) (i j expn) (Gr Gt base))
	  (implies
	   (and (p "init" (ilength) iend) 
	        (p "init" "R" iend R) (p "init" "I" iend I)
		(p "init" "ni" iend ni)	(p "init" "nr" iend nr)
		(p "init" "ci" iend ci)	(p "init" "cr" iend cr)
		(p "init" "mi" iend mi)	(p "init" "mr" iend mr)
		(p "init" "lst1" iend lst1) (p "init" "lst2" iend lst2)
		(p "init" "algo3" iend algo3) (p "init" "lst3" iend lst3)
		(p "init" "i" iend i) (p "init" "Gr" iend Gr)
		(p "init" "j" iend j) (p "init" "Gt" iend Gt)
		(p "init" "algo1" iend algo1) (p "init" "algo2" iend algo2)
		(p "init" "TSi" iend TSi) (p "init" "TSr" iend TSr)
		(p "blab-akey" 1 brend) (str-prec brstart brend)
		(p "blab-akey" 0 brstart) (p "blab-akey" "key" brend (privk R))
		(prec iend brstart)
		(pnon (privk R)))
	   (or (exists ((r1 node))
		       (and (p "resp" 0 r1) (p "resp" "R" r1 R)))
	       (exists ((i1 node))
		       (and (p "init" 0 i1) (p "init" "I" i1 R))))))
  ; Weak agreement - my peer and I agree on our names
  (forall ((iend brend brstart node) (I R name) (ni nr ci cr lst1 lst2 algo1 algo2 lst3
						    algo3 mi mr TSi TSr text) (i j expn) (Gr Gt base))
	  (implies
	   (and (p "init" (ilength) iend) 
	        (p "init" "R" iend R) (p "init" "I" iend I)
		(p "init" "ni" iend ni)	(p "init" "nr" iend nr)
		(p "init" "ci" iend ci)	(p "init" "cr" iend cr)
		(p "init" "mi" iend mi)	(p "init" "mr" iend mr)
		(p "init" "lst1" iend lst1) (p "init" "lst2" iend lst2)
		(p "init" "algo3" iend algo3) (p "init" "lst3" iend lst3)
		(p "init" "i" iend i) (p "init" "Gr" iend Gr)
		(p "init" "j" iend j) (p "init" "Gt" iend Gt)
		(p "init" "algo1" iend algo1) (p "init" "algo2" iend algo2)
		(p "init" "TSi" iend TSi) (p "init" "TSr" iend TSr)
		(p "blab-akey" 1 brend) (str-prec brstart brend)
		(p "blab-akey" 0 brstart) (p "blab-akey" "key" brend (privk R))
		(prec iend brstart)
		(pnon (privk R)))
	   (or (exists ((r1 node))
		       (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)))
	       (exists ((i1 node))
		       (and (p "init" 0 i1) (p "init" "I" i1 R) (p "init" "R" i1 I))))))
  ; Minimal agreement - my peer and I agree on our names and roles, and on the DH key
  (forall ((iend brend brstart node) (I R name) (ni nr ci cr lst1 lst2 algo1 algo2 lst3
						    algo3 mi mr TSi TSr text) (i j expn) (Gr Gt base))
	  (implies
	   (and (p "init" (ilength) iend) 
	        (p "init" "R" iend R) (p "init" "I" iend I)
		(p "init" "ni" iend ni)	(p "init" "nr" iend nr)
		(p "init" "ci" iend ci)	(p "init" "cr" iend cr)
		(p "init" "mi" iend mi)	(p "init" "mr" iend mr)
		(p "init" "lst1" iend lst1) (p "init" "lst2" iend lst2)
		(p "init" "algo3" iend algo3) (p "init" "lst3" iend lst3)
		(p "init" "i" iend i) (p "init" "Gr" iend Gr)
		(p "init" "j" iend j) (p "init" "Gt" iend Gt)
		(p "init" "algo1" iend algo1) (p "init" "algo2" iend algo2)
		(p "init" "TSi" iend TSi) (p "init" "TSr" iend TSr)
		(p "blab-akey" 1 brend) (str-prec brstart brend)
		(p "blab-akey" 0 brstart) (p "blab-akey" "key" brend (privk R))
		(prec iend brstart)
		(pnon (privk R)))
	   (exists ((r1 node) (Gi Gj base) (r t expn))
		   (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)
			(p "resp" "Gi" r1 Gi) (p "resp" "r" r1 r)
			(p "resp" "Gj" r1 Gj) (p "resp" "t" r1 t)
			(= (exp Gj t) (exp Gt j))
			(= (exp Gi r) (exp Gr i))))))
  ; Full agreement - my peer and I agree on the full transcript
  (forall ((iend brend brstart node) (I R name) (ni nr ci cr lst1 lst2 algo1 algo2 lst3
						    algo3 mi mr TSi TSr text) (i j expn) (Gr Gt base))
	  (implies
	   (and (p "init" (ilength) iend) 
	        (p "init" "R" iend R) (p "init" "I" iend I)
		(p "init" "ni" iend ni)	(p "init" "nr" iend nr)
		(p "init" "ci" iend ci)	(p "init" "cr" iend cr)
		(p "init" "mi" iend mi)	(p "init" "mr" iend mr)
		(p "init" "lst1" iend lst1) (p "init" "lst2" iend lst2)
		(p "init" "algo3" iend algo3) (p "init" "lst3" iend lst3)
		(p "init" "i" iend i) (p "init" "Gr" iend Gr)
		(p "init" "j" iend j) (p "init" "Gt" iend Gt)
		(p "init" "algo1" iend algo1) (p "init" "algo2" iend algo2)
		(p "init" "TSi" iend TSi) (p "init" "TSr" iend TSr)
		(p "blab-akey" 1 brend) (str-prec brstart brend)
		(p "blab-akey" 0 brstart) (p "blab-akey" "key" brend (privk R))
		(prec iend brstart)
		(pnon (privk R)))
	   (exists ((r1 node) (r expn))
		   (and (p "resp" 0 r1) (p "resp" "R" r1 R) (p "resp" "I" r1 I)
			(p "resp" "ni" r1 ni) (p "resp" "nr" r1 nr)
			(p "resp" "r" r1 r) (p "resp" "ci" r1 ci)
			(p "resp" "cr" r1 cr) (p "resp" "lst1" r1 lst1)
			(p "resp" "lst2" r1 lst2) (p "resp" "algo1" r1 algo1)
			(p "resp" "TSi" r1 TSi) (p "resp" "TSr" r1 TSr)
			(p "resp" "Gi" r1 (exp (gen) i))
			(p "resp" "algo2" r1 algo2)))))
  )

(defgoal sigChild
  ; Aliveness - my peer has taken an action
  (forall ((rend biend bistart node) (I R name) (ni nr ci cr lst1 lst2 algo1 algo2 TSi TSr text)
	   (r expn) (Gi base))
	  (implies
	   (and (p "resp" (rlength) rend) 
		(p "resp" "R" rend R) (p "resp" "I" rend I) (p "resp" "ni" rend ni)
		(p "resp" "nr" rend nr) (p "resp" "ci" rend ci) (p "resp" "cr" rend cr)
		(p "resp" "lst1" rend lst1) (p "resp" "algo1" rend algo1) (p "resp" "r" rend r)
		(p "resp" "lst2" rend lst2) (p "resp" "algo2" rend algo2)
		(p "resp" "Gi" rend Gi) (p "resp" "TSi" rend TSi) (p "resp" "TSr" rend TSr)
		(p "blab-akey" 1 biend) (str-prec bistart biend)
		(p "blab-akey" 0 bistart) (p "blab-akey" "key" biend (privk I))
		(prec rend bistart)
		(pnon (privk I)))
	   (or (exists ((r1 node))
		       (and (p "resp" 0 r1) (p "resp" "R" r1 I)))
	       (exists ((i1 node))
		       (and (p "init" 0 i1) (p "init" "I" i1 I))))))
  ; Weak agreement - my peer and I agree on our names
  (forall ((rend biend bistart node) (I R name) (ni nr ci cr lst1 lst2 algo1 algo2 TSi TSr text)
	   (r expn) (Gi base))
	  (implies
	   (and (p "resp" (rlength) rend) 
		(p "resp" "R" rend R) (p "resp" "I" rend I) (p "resp" "ni" rend ni)
		(p "resp" "nr" rend nr) (p "resp" "ci" rend ci) (p "resp" "cr" rend cr)
		(p "resp" "lst1" rend lst1) (p "resp" "algo1" rend algo1) (p "resp" "r" rend r)
		(p "resp" "lst2" rend lst2) (p "resp" "algo2" rend algo2)
		(p "resp" "Gi" rend Gi) (p "resp" "TSi" rend TSi) (p "resp" "TSr" rend TSr)
		(p "blab-akey" 1 biend) (str-prec bistart biend)
		(p "blab-akey" 0 bistart) (p "blab-akey" "key" biend (privk I))
		(prec rend bistart)
		(pnon (privk I)))
	   (or (exists ((r1 node))
		       (and (p "resp" 0 r1) (p "resp" "R" r1 I) (p "resp" "I" r1 R)))
	       (exists ((i1 node))
		       (and (p "init" 0 i1) (p "init" "I" i1 I) (p "init" "R" i1 R))))))
  ; Minimal agreement - my peer and I agree on our names and roles, and on the DH key
  (forall ((rend biend bistart node) (I R name) (ni nr ci cr lst1 lst2 algo1 algo2 TSi TSr text)
	   (r expn) (Gi base))
	  (implies
	   (and (p "resp" (rlength) rend) 
		(p "resp" "R" rend R) (p "resp" "I" rend I) (p "resp" "ni" rend ni)
		(p "resp" "nr" rend nr) (p "resp" "ci" rend ci) (p "resp" "cr" rend cr)
		(p "resp" "lst1" rend lst1) (p "resp" "algo1" rend algo1) (p "resp" "r" rend r)
		(p "resp" "lst2" rend lst2) (p "resp" "algo2" rend algo2)
		(p "resp" "Gi" rend Gi) (p "resp" "TSi" rend TSi) (p "resp" "TSr" rend TSr)
		(p "blab-akey" 1 biend) (str-prec bistart biend)
		(p "blab-akey" 0 bistart) (p "blab-akey" "key" biend (privk I))
		(prec rend bistart)
		(pnon (privk I)))
	   (exists ((i1 node) (Gr base) (i expn))
		   (and (p "init" 0 i1) (p "init" "R" i1 R) (p "init" "I" i1 I)
		        (p "init" "Gr" i1 Gr) (p "init" "i" i1 i)
			(= (exp Gi r) (exp Gr i))
			))))
  ; Full agreement - my peer and I agree on the full transcript
  (forall ((rend biend bistart node) (I R name) (ni nr ci cr lst1 lst2 algo1 algo2 TSi TSr text)
	   (r expn) (Gi base))
	  (implies
	   (and (p "resp" (rlength) rend) 
		(p "resp" "R" rend R) (p "resp" "I" rend I) (p "resp" "ni" rend ni)
		(p "resp" "nr" rend nr) (p "resp" "ci" rend ci) (p "resp" "cr" rend cr)
		(p "resp" "lst1" rend lst1) (p "resp" "algo1" rend algo1) (p "resp" "r" rend r)
		(p "resp" "lst2" rend lst2) (p "resp" "algo2" rend algo2)
		(p "resp" "Gi" rend Gi) (p "resp" "TSi" rend TSi) (p "resp" "TSr" rend TSr)
		(p "blab-akey" 1 biend) (str-prec bistart biend)
		(p "blab-akey" 0 bistart) (p "blab-akey" "key" biend (privk I))
		(prec rend bistart)
		(pnon (privk I)))
	   (exists ((i1 node) (i expn))
		   (and (p "init" 0 i1) (p "init" "R" i1 R) (p "init" "I" i1 I)
			(p "init" "ni" i1 ni) (p "init" "nr" i1 nr)
			(p "init" "ci" i1 ci) (p "init" "cr" i1 cr)
			(p "init" "algo1" i1 algo1) (p "init" "lst1" i1 lst1)
			(p "init" "algo2" i1 algo2) (p "init" "lst2" i1 lst2)
			(p "init" "TSr" i1 TSr) (p "init" "TSi" i1 TSi)
		        (p "init" "Gr" i1 (exp (gen) r)) (p "init" "i" i1 i)
			))))
  (comment "Responder point of view; agreement / weak agreement / aliveness goals")
  )

;Weakest: Keys are never compromised:
(defskeleton sigChild
	     (vars (ci cr ni nr mi mr lst1 algo1 lst2 algo2 lst3 algo3 TSi TSr text) 
			    (Gr Gt base) (i j expn) (R I name))
	     (defstrand init 6 (I I) (R R) (i i) (ci ci) (cr cr) (nr nr) (ni ni) (mi mi)
			(TSi TSi) (TSr TSr) (Gr Gr) (Gt Gt))
	     (deflistener (SKi))
	     (non-orig (privk I) (privk R)))
(defskeleton sigChild
	     (vars (ci cr mi mr ni nr lst1 algo1 lst2 algo2 lst3 algo3 TSi TSr text)
		   (Gi Gj base) (r t expn) (R I name))
	     (defstrand resp 6 (I I) (R R) (r r) (mi mi) (mr mr) (ci ci) (cr cr) 
			(ni ni) (nr nr) (Gi Gi) (Gj Gj) (TSi TSi) (TSr TSr))
	     (deflistener (SKr))
	     (non-orig (privk I) (privk R)))


;;; Weak Forward Secrecy
(defskeleton sigChild
  (vars (ci cr ni nr mi mr lst1 algo1 lst2 algo2 lst3 algo3 TSi TSr text) 
			    (Gr Gt base) (i j r t expn) (R I name))
  (defstrand init 7 (ci ci) (cr cr) (ni ni) (nr nr) (mi mi) (mr mr) (lst1 lst1)
    (algo1 algo1) (lst2 lst2) (algo2 algo2) (lst3 lst3) (algo3 algo3) (TSi TSi) (TSr TSr) (R R)
    (I I) (Gr (exp (gen) r)) (Gt (exp (gen) t)) (i i) (j j))
  (defstrand resp 6 (ci ci) (cr cr) (ni ni) (nr nr) (lst1 lst1)
    (algo1 algo1) (lst2 lst2) (algo2 algo2) (lst3 lst3) (algo3 algo3) (TSi TSi) (TSr TSr) (R R)
    (I I) (Gi (exp (gen) i)) (Gj (exp (gen) j)) (r r) (t t) (mr mr))
  (defstrand blab-akey 2 (key (privk I)))
  (defstrand blab-akey 2 (key (privk R)))
  (deflistener (SKf))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 5) (0 5)) ((0 6) (2 0)) ((0 6) (3 0)))
  (ind-zero-in (r (exp (gen) i)))
  (pen-non-orig (privk R) (privk I))
  (uniq-gen r i j t)
  (uniq-orig ci ni mi cr nr mr))


;;; Perfect Forward Secrecy
(defskeleton sigChild
	     (vars (ci cr ni nr lst1 algo1 lst2 algo2 TSi TSr text) (Gr base) (i r expn) (R I name))
	     (defstrand init 7 (I I) (R R) (i i) (ci ci) (cr cr) (ni ni) (nr nr) (TSi TSi) (TSr TSr) 
			(lst1 lst1) (algo1 algo1) (lst2 lst2) (algo2 algo2) (Gr Gr))
	     (defstrand blab-akey 2 (key (privk I)))
             (defstrand blab-akey 2 (key (privk R)))
	     (deflistener (SKi))
	     (precedes ((0 6) (1 0)) ((0 6) (2 0)))
	     (pen-non-orig (privk I) (privk R)))

(defskeleton sigChild
	     (vars (ci cr ni nr lst1 algo1 lst2 algo2 TSi TSr text) (Gi base) (i r expn) (R I name))
	     (defstrand resp 6 (I I) (R R) (r r) (ci ci) (cr cr) (ni ni) (nr nr) (TSi TSi) (TSr TSr) 
			(lst1 lst1) (algo1 algo1) (lst2 lst2) (algo2 algo2) (Gi Gi))
	     (defstrand blab-akey 2 (key (privk I)))
             (defstrand blab-akey 2 (key (privk R)))
	     (deflistener (SKr))
	     (precedes ((0 5) (1 0)) ((0 5) (2 0)))
	     (pen-non-orig (privk I) (privk R)))
