(herald "Diffie-Hellman protocol, man-in-the-middle attack"
  (algebra diffie-hellman))

(comment "CPSA 3.4.0")
(comment "All input read from dh_mim.scm")

(defprotocol dh_mim diffie-hellman
  (defrole init
    (vars (x expn) (h base) (n text))
    (trace (send (exp (gen) x)) (recv h) (send (enc n (exp h x))))
    (uniq-gen x))
  (defrole resp
    (vars (y expn) (h base) (n text))
    (trace (recv h) (send (exp (gen) y)) (recv (enc n (exp h y))))
    (uniq-gen y)
    (absent (y h)))
  (comment "Diffie-hellman key exchange followed by an encryption"))

(defskeleton dh_mim
  (vars (n text) (hx hy base) (x y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h hx) (y y))
  (precedes ((0 2) (1 2)))
  (absent (y hx))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (comment "Agreement on the encrypted text only")
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv hx) (send (exp (gen) y)) (recv (enc n (exp hx y)))))
  (label 0)
  (unrealized (1 2))
  (origs (n (0 2)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x y expn))
  (defstrand init 3 (n n) (h (exp hy y)) (x x))
  (defstrand resp 3 (n n) (h (exp hy x)) (y y))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)))
  (absent (y (exp hy x)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (operation encryption-test (displaced 2 0 init 3) (enc n (exp hx y-0))
    (1 2))
  (traces
    ((send (exp (gen) x)) (recv (exp hy y))
      (send (enc n (exp hy (mul x y)))))
    ((recv (exp hy x)) (send (exp (gen) y))
      (recv (enc n (exp hy (mul x y))))))
  (label 1)
  (parent 0)
  (unrealized (0 1) (1 0))
  (origs (n (0 2)))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hx hy base) (x y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h hx) (y y))
  (deflistener (exp hx y))
  (precedes ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2)))
  (absent (y hx))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (operation encryption-test (added-listener (exp hx y))
    (enc n (exp hx y)) (1 2))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv hx) (send (exp (gen) y)) (recv (enc n (exp hx y))))
    ((recv (exp hx y)) (send (exp hx y))))
  (label 2)
  (parent 0)
  (unrealized (1 2) (2 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (hy (exp (gen) (rec x)))) (gen)
    (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))))
  (label 3)
  (parent 1)
  (unrealized (0 1))
  (origs (n (0 2)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x)) (y y))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)))
  (absent (y (exp (gen) x)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 2 0 init 1) (exp (gen) x-0) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul y x)))))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x))))))
  (label 4)
  (parent 1)
  (unrealized)
  (shape)
  (maps
    ((0 1) ((n n) (hx (exp (gen) x)) (hy (exp (gen) y)) (x x) (y y))))
  (origs (n (0 2))))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 0) (1 0)))
  (absent (y (exp (gen) x-0)))
  (pen-non-orig x y)
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0))))) ((send (exp (gen) x-0))))
  (label 5)
  (parent 1)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (pen-non-orig x y)
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 6)
  (parent 1)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x y expn) (w expr))
  (defstrand init 3 (n n) (h (exp hy y)) (x x))
  (defstrand resp 3 (n n) (h (exp hy x)) (y y))
  (deflistener (cat (exp hy (mul x (rec w))) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0)))
  (absent (y (exp hy x)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test
    (added-listener (cat (exp hy (mul x (rec w))) w)) (exp hy x) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp hy y))
      (send (enc n (exp hy (mul x y)))))
    ((recv (exp hy x)) (send (exp (gen) y))
      (recv (enc n (exp hy (mul x y)))))
    ((recv (cat (exp hy (mul x (rec w))) w))
      (send (cat (exp hy (mul x (rec w))) w))))
  (label 7)
  (parent 1)
  (unrealized (0 1) (2 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (precedes ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 3 1 resp 2) (exp (gen) y-0) (2 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y))))
  (label 8)
  (parent 2)
  (unrealized (1 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hx hy base) (x y expn) (w expr))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h hx) (y y))
  (deflistener (exp hx y))
  (deflistener (cat (exp hx (mul y (rec w))) w))
  (precedes ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2)) ((3 1) (2 0)))
  (absent (y hx))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp hx (mul y (rec w))) w)) (exp hx y) (2 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv hx) (send (exp (gen) y)) (recv (enc n (exp hx y))))
    ((recv (exp hx y)) (send (exp hx y)))
    ((recv (cat (exp hx (mul y (rec w))) w))
      (send (cat (exp hx (mul y (rec w))) w))))
  (label 9)
  (parent 2)
  (unrealized (1 2) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) (mul (rec x) y (rec w))) w))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y (rec w))) w))
    (exp (gen) (mul (rec x) y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (cat (exp (gen) (mul (rec x) y (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y (rec w))) w))))
  (label 10)
  (parent 3)
  (unrealized (2 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y x-0 (rec w))) w))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (pen-non-orig x y)
  (uniq-gen x y x-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y x-0 (rec w))) w))
    (exp (gen) (mul (rec x) y x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0))))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) y x-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y x-0 (rec w))) w))))
  (label 11)
  (parent 5)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y y-0 (rec w))) w))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (pen-non-orig x y)
  (uniq-gen x y y-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y y-0 (rec w))) w))
    (exp (gen) (mul (rec x) y y-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) (mul (rec x) y y-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y y-0 (rec w))) w))))
  (label 12)
  (parent 6)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y w))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (gen) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test (contracted (hy (exp (gen) (mul (rec x) w))))
    (gen) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w)))
      (send (enc n (exp (gen) (mul y w)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w)))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 13)
  (parent 7)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y w))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x))) (y y))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0)))
  (absent (y (exp (gen) (mul w x))))
  (pen-non-orig y x)
  (precur (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 3 0 init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y w)))
      (send (enc n (exp (gen) (mul y w x)))))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w x)))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 14)
  (parent 7)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y w x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0))
    ((3 0) (2 0)))
  (absent (y (exp (gen) (mul w x-0))))
  (pen-non-orig x y)
  (precur (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w x-0)))
      (send (enc n (exp (gen) (mul y w x-0)))))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w x-0)))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0))))
  (label 15)
  (parent 7)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y w y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w y-0))) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0))
    ((3 1) (2 0)))
  (absent (y-0 h) (y (exp (gen) (mul w y-0))))
  (pen-non-orig x y)
  (precur (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w y-0)))
      (send (enc n (exp (gen) (mul y w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w y-0)))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0))))
  (label 16)
  (parent 7)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (precedes ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (2 0)) ((2 1) (1 2)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (hy (exp (gen) (mul (rec x) y)))) n
    (1 2) (enc n (exp (gen) y)))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y))))
  (label 17)
  (parent 8)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp hy x))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener (exp hy x)) n (1 2)
    (enc n (exp hy x)))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp hy x)) (send (exp hy x))))
  (label 18)
  (parent 8)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (gen) (mul w y)))
  (precedes ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2)) ((3 1) (2 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (contracted (hx (exp (gen) w)) (y-0 y) (w (mul w y))) (gen) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y)))))
  (label 19)
  (parent 9)
  (unrealized (1 2) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (w expr) (x y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x))) (y y))
  (deflistener (exp (gen) (mul w x y)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)))
  (absent (y (exp (gen) (mul w x))))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x y)))))
    ((recv (exp (gen) (mul w x y))) (send (exp (gen) (mul w x y))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))))
  (label 20)
  (parent 9)
  (unrealized (1 2) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x-0))) (y y))
  (deflistener (exp (gen) (mul w x-0 y)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 0) (1 0)))
  (absent (y (exp (gen) (mul w x-0))))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x-0 y)))))
    ((recv (exp (gen) (mul w x-0 y))) (send (exp (gen) (mul w x-0 y))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0))))
  (label 21)
  (parent 9)
  (unrealized (1 2) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2)) ((3 1) (2 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 22)
  (parent 9)
  (unrealized (1 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w y))) (y y-0))
  (deflistener (exp (gen) (mul w y y-0)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 0)))
  (absent (y h) (y-0 (exp (gen) (mul w y))))
  (pen-non-orig x y-0)
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y y-0)))))
    ((recv (exp (gen) (mul w y y-0))) (send (exp (gen) (mul w y y-0))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 23)
  (parent 9)
  (unrealized (1 2) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test (contracted (x-0 x) (y-0 y) (w (mul (rec x) y)))
    (gen) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 24)
  (parent 10)
  (unrealized (0 1) (2 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 3 0 init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 25)
  (parent 10)
  (unrealized (0 1) (2 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 0) (2 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 26)
  (parent 10)
  (unrealized (0 1) (2 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 3 1 resp 2) (exp (gen) y-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 27)
  (parent 10)
  (unrealized (0 1) (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (0 1))
    ((3 1) (2 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig x y)
  (precur (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 28)
  (parent 10)
  (unrealized (0 1) (2 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul (rec x) y x-0)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (pen-non-orig x y)
  (uniq-gen x y x-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (contracted (x-1 x) (y-0 y) (x-2 x-0) (w (mul (rec x) y x-0))) (gen)
    (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0))))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul (rec x) y x-0)))
      (send (cat (gen) (mul (rec x) y x-0)))))
  (label 29)
  (parent 11)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) x)) (y y))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul y x (rec x-0) (rec x-0))))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) x)))
  (pen-non-orig y x-0)
  (precur (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-1) (3 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul y x (rec x-0))))
      (send (enc n (exp (gen) (mul y x)))))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x))))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul y x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul y x (rec x-0) (rec x-0))))))
  (label 30)
  (parent 11)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 init 1) (exp (gen) x-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0))))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y)))
      (send (cat (exp (gen) x-0) (mul (rec x) y)))))
  (label 31)
  (parent 11)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul (rec x) y x-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)) ((4 0) (3 0)))
  (absent (y (exp (gen) x-0)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0))))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul (rec x) y x-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul (rec x) y x-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 32)
  (parent 11)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0 y)))
      (send (enc n (exp (gen) (mul x-0 y)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x-0 y))))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 33)
  (parent 11)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y x-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y-0 h) (y (exp (gen) x-0)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0))))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y x-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y x-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 34)
  (parent 11)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul (rec x) y y-0)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (pen-non-orig x y)
  (uniq-gen x y y-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (contracted (x-0 x) (y-1 y) (y-2 y-0) (w (mul (rec x) y y-0))) (gen)
    (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul (rec x) y y-0)))
      (send (cat (gen) (mul (rec x) y y-0)))))
  (label 35)
  (parent 12)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul y y-0 (rec x) (rec x))))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (pen-non-orig y x)
  (precur (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y y-0 (rec x))))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul y y-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y y-0 (rec x) (rec x))))))
  (label 36)
  (parent 12)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y y-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 0) (3 0)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y y-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 37)
  (parent 12)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y)) (y y-0))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y-0 (exp (gen) y)) (y h))
  (pen-non-orig x y-0)
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y)))
      (send (cat (exp (gen) y-0) (mul (rec x) y)))))
  (label 38)
  (parent 12)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 resp 2) (exp (gen) y-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y)))
      (send (cat (exp (gen) y-0) (mul (rec x) y)))))
  (label 39)
  (parent 12)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (x y y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y y-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y-1 h-0) (y-0 h) (y (exp (gen) y-0)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y y-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 40)
  (parent 12)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x)) (y y))
  (deflistener (cat (gen) x))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0)))
  (absent (y (exp (gen) x)))
  (pen-non-orig x y)
  (precur (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 3 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x y)))))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 41)
  (parent 13)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y w))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w)))
      (send (enc n (exp (gen) (mul y w)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w)))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))))
  (label 42)
  (parent 13)
  (unrealized (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y w))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x))) (y y))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)))
  (absent (y (exp (gen) (mul w x))))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y w)))
      (send (enc n (exp (gen) (mul y w x)))))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w x))))))
  (label 43)
  (parent 14)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((n n) (hx (exp (gen) (mul w x))) (hy (exp (gen) (mul y w))) (x x)
        (y y))))
  (origs (n (0 2))))

(defskeleton dh_mim
  (vars (n text) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x)) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0))
    ((3 0) (2 0)))
  (absent (y (exp (gen) x)))
  (pen-non-orig x y)
  (precur (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 44)
  (parent 15)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr) (x-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y w x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul w x-0))))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w x-0)))
      (send (enc n (exp (gen) (mul y w x-0)))))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w x-0)))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 45)
  (parent 15)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x)) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0))
    ((3 1) (2 0)))
  (absent (y-0 (exp (gen) x)) (y h))
  (pen-non-orig x y-0)
  (precur (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (exp (gen) x)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((recv h) (send (exp (gen) y))))
  (label 46)
  (parent 16)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y w y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w y-0))) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) (mul w y-0))))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w y-0)))
      (send (enc n (exp (gen) (mul y w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w y-0)))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 47)
  (parent 16)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) (mul (rec x) y (rec w))) w))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y (rec w))) w))
    (exp (gen) (mul (rec x) y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) (mul (rec x) y (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y (rec w))) w))))
  (label 48)
  (parent 17)
  (unrealized (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (hy (exp (gen) (rec x)))) (gen)
    (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen))))
  (label 49)
  (parent 18)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x)) (send (exp (gen) x))))
  (label 50)
  (parent 18)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 0) (0 1)) ((4 0) (3 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0))))
  (label 51)
  (parent 18)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (2 0))
    ((1 1) (3 0)) ((2 1) (1 2)) ((3 1) (1 2)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y))))
  (label 52)
  (parent 18)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (0 1)) ((4 1) (3 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0))))
  (label 53)
  (parent 18)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x y expn) (w expr))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp hy x))
  (deflistener (cat (exp hy (mul x (rec w))) w))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp hy (mul x (rec w))) w)) (exp hy x) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp hy x)) (send (exp hy x)))
    ((recv (cat (exp hy (mul x (rec w))) w))
      (send (cat (exp hy (mul x (rec w))) w))))
  (label 54)
  (parent 18)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (gen) (mul w y)))
  (deflistener y)
  (precedes ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv y) (send y)))
  (label 55)
  (parent 19)
  (unrealized (1 2) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (hy base) (w expr) (x y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x))) (y y))
  (deflistener (exp (gen) (mul w x y)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (absent (y (exp (gen) (mul w x))))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x y)))))
    ((recv (exp (gen) (mul w x y))) (send (exp (gen) (mul w x y))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))) ((recv y) (send y)))
  (label 56)
  (parent 20)
  (unrealized (1 2) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x-0))) (y y))
  (deflistener (exp (gen) (mul w x-0 y)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (deflistener y)
  (precedes ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 0) (1 0)) ((5 1) (3 0)))
  (absent (y (exp (gen) (mul w x-0))))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x-0 y)))))
    ((recv (exp (gen) (mul w x-0 y))) (send (exp (gen) (mul w x-0 y))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0)))
    ((recv y) (send y)))
  (label 57)
  (parent 21)
  (unrealized (1 2) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (hy (exp (gen) (mul (rec x) w y))))
    n (1 2) (enc n (exp (gen) (mul w y))))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w y)))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 58)
  (parent 22)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp hy x))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener (exp hy x)) n (1 2)
    (enc n (exp hy x)))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp hy x)) (send (exp hy x))))
  (label 59)
  (parent 22)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w y))) (y y-0))
  (deflistener (exp (gen) (mul w y y-0)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 0)) ((5 1) (3 0)))
  (absent (y h) (y-0 (exp (gen) (mul w y))))
  (pen-non-orig x y-0)
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y y-0)))))
    ((recv (exp (gen) (mul w y y-0))) (send (exp (gen) (mul w y y-0))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 60)
  (parent 23)
  (unrealized (1 2) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (0 1)) ((3 1) (2 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 61)
  (parent 27)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w-0 (mul x w)) (w-1 (mul w y)) (y-1 y)
      (w (mul w y))) (gen) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y)))))
  (label 62)
  (parent 42)
  (unrealized (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x x))) (y y))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) (mul w x x))))
  (pen-non-orig x y)
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x x y)))))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))))
  (label 63)
  (parent 42)
  (unrealized (0 1) (1 0) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 0) (2 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (pen-non-orig x y)
  (precur (3 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0))))
  (label 64)
  (parent 42)
  (unrealized (0 1) (1 0) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (pen-non-orig x y)
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 65)
  (parent 42)
  (unrealized (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (pen-non-orig x y-0)
  (precur (3 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 66)
  (parent 42)
  (unrealized (0 1) (1 0) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (y-0 y) (w-0 (mul x (rec x-0) w)) (x-2 x-0)
      (w-1 (mul w y)) (y-1 y) (w (mul w y))) (gen) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y)))))
  (label 67)
  (parent 45)
  (unrealized (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x-0 x-0))) (y y))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul w x-0 x-0))))
  (pen-non-orig x-0 y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-1) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul w x-0 x-0 y)))))
    ((recv (exp (gen) (mul w x-0 x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x-0 x-0 y)))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))))
  (label 68)
  (parent 45)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w x-0))))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 init 1) (exp (gen) x-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))))
  (label 69)
  (parent 45)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (x-1 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-1 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-1))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w y)))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 0) (2 0)))
  (absent (y (exp (gen) (mul x w x-1))))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1 y)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-1 y)))
      (send (enc n (exp (gen) (mul x w x-1 y)))))
    ((recv (exp (gen) (mul x w x-1))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-1 y)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w y)))
      (send (cat (exp (gen) x-1) (mul w y)))) ((send (exp (gen) x-1))))
  (label 70)
  (parent 45)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 71)
  (parent 45)
  (unrealized (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x x-0 expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (2 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 72)
  (parent 45)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) (mul w y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y h) (y-0 (exp (gen) (mul x w))))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w-0 (mul x (rec y) w)) (y-2 y)
      (w-1 (mul w y-0)) (y-3 y-0) (w (mul w y-0))) (gen) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) (mul w y-0))) (send (cat (gen) (mul w y-0)))))
  (label 73)
  (parent 47)
  (unrealized (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x x))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x x)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul w y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y h) (y-0 (exp (gen) (mul w x x))))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y-0)))
      (send (enc n (exp (gen) (mul w x x y-0)))))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w x x y-0)))))
    ((recv (cat (exp (gen) y) (mul (rec y) w x x)))
      (send (cat (exp (gen) y) (mul (rec y) w x x))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul w y-0)))
      (send (cat (exp (gen) x) (mul w y-0)))))
  (label 74)
  (parent 47)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) (mul w y-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 0) (2 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w x-0))))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y-0)))
      (send (enc n (exp (gen) (mul x w x-0 y-0)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w x-0 y-0)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul w y-0)))
      (send (cat (exp (gen) x-0) (mul w y-0))))
    ((send (exp (gen) x-0))))
  (label 75)
  (parent 47)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y-0 (exp (gen) (mul x w))) (y h))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 76)
  (parent 47)
  (unrealized (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 2) (exp (gen) y-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))))
  (label 77)
  (parent 47)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (x y expn) (w expr) (y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 y-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y-0))) (y y-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w y-1)))
  (defstrand resp 2 (h h-0) (y y-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (2 0)))
  (absent (y-0 h-0) (y h) (y-1 (exp (gen) (mul x w y-0))))
  (pen-non-orig x y-1)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 y-1)))
      (send (enc n (exp (gen) (mul x w y-0 y-1)))))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) y-1))
      (recv (enc n (exp (gen) (mul x w y-0 y-1)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w y-1)))
      (send (cat (exp (gen) y-0) (mul w y-1))))
    ((recv h-0) (send (exp (gen) y-0))))
  (label 78)
  (parent 47)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test (contracted (x-0 x) (y-0 y) (w (mul (rec x) y)))
    (gen) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 79)
  (parent 48)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 80)
  (parent 48)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (0 1)) ((4 0) (3 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 81)
  (parent 48)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 82)
  (parent 48)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 83)
  (parent 48)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (exp (gen) (mul (rec x) (rec w))) w))
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) (rec w))) w))
    (exp (gen) (rec x)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) (mul (rec x) (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) (rec w))) w))))
  (label 84)
  (parent 49)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) x))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((2 1) (1 2)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) x)) (send (exp (gen) x))))
  (label 85)
  (parent 50)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0 (rec w))) w))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 0) (3 0)) ((4 0) (5 0))
    ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y x-0)
  (uniq-orig n)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0 (rec w))) w))
    (exp (gen) (mul (rec x) x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) x-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) x-0 (rec w))) w))))
  (label 86)
  (parent 51)
  (unrealized (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) (mul (rec x) y (rec w))) w))
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y (rec w))) w))
    (exp (gen) (mul (rec x) y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) (mul (rec x) y (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y (rec w))) w))))
  (label 87)
  (parent 52)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0 (rec w))) w))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y y-0)
  (uniq-orig n)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0 (rec w))) w))
    (exp (gen) (mul (rec x) y-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) (mul (rec x) y-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y-0 (rec w))) w))))
  (label 88)
  (parent 53)
  (unrealized (5 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (gen) w))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test (contracted (hy (exp (gen) (mul (rec x) w))))
    (gen) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w)))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 89)
  (parent 54)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul w x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 90)
  (parent 54)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (0 1)) ((5 0) (4 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w x-0)))
      (send (enc n (exp (gen) (mul w x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) (mul w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0))))
  (label 91)
  (parent 54)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w y)))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 92)
  (parent 54)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w y-0)))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0))))
  (label 93)
  (parent 54)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x)) (y y))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)))
  (absent (y (exp (gen) x)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 94)
  (parent 58)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w y)))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))))
  (label 95)
  (parent 58)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (hy (exp (gen) (rec x)))) (gen)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen))))
  (label 96)
  (parent 59)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x))))
  (label 97)
  (parent 59)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (0 1)) ((5 0) (4 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0))))
  (label 98)
  (parent 59)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y))))
  (label 99)
  (parent 59)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0))))
  (label 100)
  (parent 59)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (y expn) (w-0 expr))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp hy x))
  (deflistener (cat (exp hy (mul x (rec w-0))) w-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp hy (mul x (rec w-0))) w-0)) (exp hy x)
    (4 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp hy x)) (send (exp hy x)))
    ((recv (cat (exp hy (mul x (rec w-0))) w-0))
      (send (cat (exp hy (mul x (rec w-0))) w-0))))
  (label 101)
  (parent 59)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y (exp (gen) (mul x w))))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv y) (send y)))
  (label 102)
  (parent 62)
  (unrealized (2 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x x))) (y y))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y (exp (gen) (mul w x x))))
  (pen-non-orig x y)
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x x y)))))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))) ((recv y) (send y)))
  (label 103)
  (parent 63)
  (unrealized (1 0) (2 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 0) (2 0)) ((5 1) (3 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (pen-non-orig x y)
  (precur (3 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0)))
    ((recv y) (send y)))
  (label 104)
  (parent 64)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (3 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 105)
  (parent 65)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (x w) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (3 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w) w (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))))
  (label 106)
  (parent 65)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)))
  (absent (y (exp (gen) (mul x w))))
  (pen-non-orig x y)
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 107)
  (parent 65)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)) ((5 1) (3 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (pen-non-orig x y-0)
  (precur (3 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 108)
  (parent 66)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul x w))))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv y) (send y)))
  (label 109)
  (parent 67)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x-0 x-0))) (y y))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul w x-0 x-0))))
  (pen-non-orig x-0 y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul w x-0 x-0 y)))))
    ((recv (exp (gen) (mul w x-0 x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x-0 x-0 y)))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((recv y) (send y)))
  (label 110)
  (parent 68)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((recv y) (send y)))
  (label 111)
  (parent 69)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (x-1 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-1 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-1))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w y)))
  (defstrand init 1 (x x-1))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (6 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 0) (2 0)) ((6 1) (4 0)))
  (absent (y (exp (gen) (mul x w x-1))))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-1 y)))
      (send (enc n (exp (gen) (mul x w x-1 y)))))
    ((recv (exp (gen) (mul x w x-1))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-1 y)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w y)))
      (send (cat (exp (gen) x-1) (mul w y)))) ((send (exp (gen) x-1)))
    ((recv y) (send y)))
  (label 112)
  (parent 70)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (pen-non-orig y x)
  (precur (4 0) (2 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x) x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0)))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 113)
  (parent 71)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w (rec x-0))))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (x-0 (mul (rec x) w)) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w))
    (mul (rec x) w) (2 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w y (rec x-0))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w (rec x-0))))
      (send (cat (exp (gen) y) (mul w (rec x-0))))))
  (label 114)
  (parent 71)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w x-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (x-0 (mul x w)) (y (exp (gen) (mul x w x-0))))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w)) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y x-0)))
      (send (enc n (exp (gen) (mul x w y x-0)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y x-0)))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w x-0)))
      (send (cat (exp (gen) y) (mul w x-0)))))
  (label 115)
  (parent 71)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x x-0 expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (6 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (2 0)) ((6 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 116)
  (parent 72)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w))))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) (mul w y-0))) (send (cat (gen) (mul w y-0))))
    ((recv y-0) (send y-0)))
  (label 117)
  (parent 73)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x x))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x x)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul w x x))))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y-0)))
      (send (enc n (exp (gen) (mul w x x y-0)))))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w x x y-0)))))
    ((recv (cat (exp (gen) y) (mul (rec y) w x x)))
      (send (cat (exp (gen) y) (mul (rec y) w x x))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul w y-0)))
      (send (cat (exp (gen) x) (mul w y-0)))) ((recv y-0) (send y-0)))
  (label 118)
  (parent 74)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) (mul w y-0)))
  (defstrand init 1 (x x-0))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (6 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 0) (2 0)) ((6 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w x-0))))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y-0)))
      (send (enc n (exp (gen) (mul x w x-0 y-0)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w x-0 y-0)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul w y-0)))
      (send (cat (exp (gen) x-0) (mul w y-0)))) ((send (exp (gen) x-0)))
    ((recv y-0) (send y-0)))
  (label 119)
  (parent 75)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) y-0)) (y-0 h))
  (pen-non-orig y x)
  (precur (4 0) (2 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x) y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 120)
  (parent 76)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (x (mul (rec y) w)) (y-0 (exp (gen) w)) (y h))
  (pen-non-orig y-0 x)
  (precur (4 0) (2 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w))
    (mul (rec y) w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x))))))
  (label 121)
  (parent 76)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y-0))) (y y))
  (deflistener (cat (exp (gen) y-0) (mul x w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y-0 (mul x w)) (y (exp (gen) (mul x w y-0))) (y-0 h))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w)) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (exp (gen) y-0) (mul x w)))
      (send (cat (exp (gen) y-0) (mul x w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))))
  (label 122)
  (parent 76)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))) ((recv y-0) (send y-0)))
  (label 123)
  (parent 77)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (x y expn) (w expr) (y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 y-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y-0))) (y y-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w y-1)))
  (defstrand resp 2 (h h-0) (y y-0))
  (deflistener y-1)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (6 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (2 0)) ((6 1) (4 0)))
  (absent (y-0 h-0) (y h) (y-1 (exp (gen) (mul x w y-0))))
  (pen-non-orig x y-1)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-listener y-1) (mul w y-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 y-1)))
      (send (enc n (exp (gen) (mul x w y-0 y-1)))))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) y-1))
      (recv (enc n (exp (gen) (mul x w y-0 y-1)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w y-1)))
      (send (cat (exp (gen) y-0) (mul w y-1))))
    ((recv h-0) (send (exp (gen) y-0))) ((recv y-1) (send y-1)))
  (label 124)
  (parent 78)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 125)
  (parent 82)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (gen) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 126)
  (parent 84)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 127)
  (parent 84)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (0 1)) ((5 0) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 128)
  (parent 84)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y))))))
  (label 129)
  (parent 84)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 130)
  (parent 84)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (precedes ((0 2) (1 2)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))))
  (label 131)
  (parent 85)
  (unrealized)
  (shape)
  (maps ((0 1) ((n n) (hx (gen)) (hy (gen)) (x x) (y y))))
  (origs (n (0 2))))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0)))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 0) (3 0)) ((4 0) (5 0))
    ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x x-0)
  (uniq-orig n)
  (precur (5 0))
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (gen) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul (rec x) x-0)))
      (send (cat (gen) (mul (rec x) x-0)))))
  (label 132)
  (parent 86)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 0) (3 0)) ((4 0) (5 0))
    ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x-0)
  (precur (5 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-1) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x)) (send (exp (gen) x))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))))
  (label 133)
  (parent 86)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (rec x)))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 0) (3 0)) ((4 0) (5 0))
    ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 4 init 1) (exp (gen) x-1) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (rec x)))
      (send (cat (exp (gen) x-0) (rec x)))))
  (label 134)
  (parent 86)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 0) (3 0)) ((4 0) (5 0))
    ((5 1) (0 1)) ((6 0) (5 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 135)
  (parent 86)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (5 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))))
  (label 136)
  (parent 86)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 0) (3 0)) ((4 0) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 137)
  (parent 86)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test (contracted (x-0 x) (y-0 y) (w (mul (rec x) y)))
    (gen) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 138)
  (parent 87)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 139)
  (parent 87)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (0 1)) ((5 0) (4 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 140)
  (parent 87)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 141)
  (parent 87)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 142)
  (parent 87)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul (rec x) y-0)))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x y-0)
  (uniq-orig n)
  (precur (5 0))
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (gen) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul (rec x) y-0)))
      (send (cat (gen) (mul (rec x) y-0)))))
  (label 143)
  (parent 88)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y-0 (rec x))))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))))
  (label 144)
  (parent 88)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 0) (5 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 145)
  (parent 88)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y-0))
  (deflistener (exp (gen) y-0))
  (deflistener (exp (gen) y))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0))
    ((4 1) (5 0)) ((5 1) (0 1)))
  (absent (y-0 (gen)) (y h))
  (pen-non-orig x y-0)
  (precur (5 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-1) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y-0)) (recv (enc n (exp (gen) y-0))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))))
  (label 146)
  (parent 88)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (rec x)))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 4 resp 2) (exp (gen) y-1) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (rec x)))
      (send (cat (exp (gen) y-0) (rec x)))))
  (label 147)
  (parent 88)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (y x y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (y-1 h-0) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-1) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 148)
  (parent 88)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x))
  (deflistener (cat (gen) x))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test (contracted (x-0 x) (w x)) (gen) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 149)
  (parent 89)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) x)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (gen) (mul x x)))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (gen) (mul x x))) (send (cat (gen) (mul x x)))))
  (label 150)
  (parent 89)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (gen) (mul x x-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (0 1)) ((5 0) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (gen) (mul x x-0))) (send (cat (gen) (mul x x-0))))
    ((send (exp (gen) x-0))))
  (label 151)
  (parent 89)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (gen) (mul x y)))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (gen) (mul x y))) (send (cat (gen) (mul x y)))))
  (label 152)
  (parent 89)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (gen) (mul x y-0)))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (gen) (mul x y-0))) (send (cat (gen) (mul x y-0))))
    ((recv h) (send (exp (gen) y-0))))
  (label 153)
  (parent 89)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w)))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))))
  (label 154)
  (parent 89)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((2 1) (1 2)) ((3 1) (2 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul w x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 155)
  (parent 90)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (0 1)) ((5 0) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w (mul x (rec x-0))) (x-2 x-0)) (gen) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 156)
  (parent 91)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x-0 x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
  (defstrand init 1 (x x))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (0 1)) ((5 0) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x-0)
  (precur (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-1) (0 1))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x-0 x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x-0 x-0))) (send (exp (gen) (mul x-0 x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) x-0 x-0))))
    ((send (exp (gen) x))))
  (label 157)
  (parent 91)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (exp (gen) x-0) x))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (0 1)) ((5 0) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 5 init 1) (exp (gen) x-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x)))
    ((send (exp (gen) x-0))))
  (label 158)
  (parent 91)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-1)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x x-1)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 1 (x x-0))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (0 1)) ((5 0) (4 0))
    ((6 0) (0 1)) ((6 0) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-1))
      (send (enc n (exp (gen) (mul x x-1)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x x-1))) (send (exp (gen) (mul x x-1))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((send (exp (gen) x-0))) ((send (exp (gen) x-1))))
  (label 159)
  (parent 91)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((send (exp (gen) x-0))))
  (label 160)
  (parent 91)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y-0)))
  (defstrand init 1 (x x-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (0 1)) ((5 0) (4 0))
    ((6 1) (0 1)) ((6 1) (4 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y-0))))
    ((send (exp (gen) x-0))) ((recv h) (send (exp (gen) y-0))))
  (label 161)
  (parent 91)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr) (x-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (6 0) (4 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w x-0)))
      (send (enc n (exp (gen) (mul w x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) (mul w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))))
  (label 162)
  (parent 91)
  (unrealized (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w (mul x (rec y))) (y-0 y))
    (gen) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y))))))
  (label 163)
  (parent 92)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) x)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) x x)))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (exp (gen) y) (mul (rec y) x x)))
      (send (cat (exp (gen) y) (mul (rec y) x x)))))
  (label 164)
  (parent 92)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((send (exp (gen) x-0))))
  (label 165)
  (parent 92)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 166)
  (parent 92)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0))
    ((5 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((recv h) (send (exp (gen) y-0))))
  (label 167)
  (parent 92)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w y)))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))))
  (label 168)
  (parent 92)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w (mul x (rec y-0))) (y-1 y-0)) (gen) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0))))
      (send (cat (exp (gen) y-0) (mul x (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 169)
  (parent 93)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) x)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) x x)))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) x x)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) x x))))
    ((recv h) (send (exp (gen) y-0))))
  (label 170)
  (parent 93)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)) ((5 1) (4 0))
    ((6 0) (0 1)) ((6 0) (4 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) x-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) x-0))))
    ((recv h) (send (exp (gen) y-0))) ((send (exp (gen) x-0))))
  (label 171)
  (parent 93)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y-0))
  (deflistener (exp (gen) y-0))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (2 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0))
    ((5 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 (gen)) (y h))
  (pen-non-orig x y-0)
  (precur (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (gen)) (send (exp (gen) y-0)) (recv (enc n (exp (gen) y-0))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 172)
  (parent 93)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) y-0) x))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 6 5 resp 2) (exp (gen) y-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) y-0) x)) (send (cat (exp (gen) y-0) x)))
    ((recv h) (send (exp (gen) y-0))))
  (label 173)
  (parent 93)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (y x y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-1)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x y-1)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) y-1)))
  (defstrand resp 2 (h h) (y y-0))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)) ((5 1) (4 0))
    ((6 1) (0 1)) ((6 1) (4 0)))
  (absent (y-1 h-0) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-1))
      (send (enc n (exp (gen) (mul x y-1)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x y-1))) (send (exp (gen) (mul x y-1))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) y-1)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) y-1))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 174)
  (parent 93)
  (unrealized (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) w y-0 (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig x y)
  (precur (6 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w y-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w y-0)))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) (mul (rec x) w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w y-0 (rec w-0))) w-0))))
  (label 175)
  (parent 93)
  (unrealized (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (deflistener (cat (gen) (mul w y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x w)) (y-0 y) (w-1 (mul w y)) (y-1 y)
      (w (mul w y))) (gen) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y)))))
  (label 176)
  (parent 95)
  (unrealized (2 0) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x x))) (y y))
  (deflistener (exp (gen) (mul w x x y)))
  (deflistener (cat (exp (gen) y) (mul w x x)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul w x x))))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x x y)))))
    ((recv (exp (gen) (mul w x x y))) (send (exp (gen) (mul w x x y))))
    ((recv (cat (exp (gen) y) (mul w x x)))
      (send (cat (exp (gen) y) (mul w x x))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))))
  (label 177)
  (parent 95)
  (unrealized (0 1) (1 0) (2 0) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (exp (gen) (mul x w x-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 0) (1 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0 y)))
      (send (exp (gen) (mul x w x-0 y))))
    ((recv (cat (exp (gen) y) (mul x w x-0)))
      (send (cat (exp (gen) y) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0))))
  (label 178)
  (parent 95)
  (unrealized (0 1) (1 0) (2 0) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 179)
  (parent 95)
  (unrealized (2 0) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (exp (gen) (mul x w y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (1 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (pen-non-orig x y-0)
  (precur (4 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y y-0)))
      (send (exp (gen) (mul x w y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w y)))
      (send (cat (exp (gen) y-0) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 180)
  (parent 95)
  (unrealized (0 1) (1 0) (2 0) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (exp (gen) (mul (rec x) (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) (rec w-0))) w-0))
    (exp (gen) (rec x)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) (mul (rec x) (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) (rec w-0))) w-0))))
  (label 181)
  (parent 96)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x))))
  (label 182)
  (parent 97)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0 (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) x-0 (rec w-0))) w-0))))
  (label 183)
  (parent 98)
  (unrealized (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) (mul (rec x) y (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) (mul (rec x) y (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y (rec w-0))) w-0))))
  (label 184)
  (parent 99)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0 (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) (mul (rec x) y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y-0 (rec w-0))) w-0))))
  (label 185)
  (parent 100)
  (unrealized (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (gen) w-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (hy (exp (gen) (mul (rec x) w-0))))
    (gen) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0)))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0))))
  (label 186)
  (parent 101)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 187)
  (parent 101)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) w-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0 x-0)))
      (send (enc n (exp (gen) (mul w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x-0))) (send (exp (gen) (mul w-0 x-0))))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0)))
    ((send (exp (gen) x-0))))
  (label 188)
  (parent 101)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) y) w-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0 y)))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0))))
  (label 189)
  (parent 101)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y expn) (w-0 expr)
    (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0 y-0)))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0))))
  (label 190)
  (parent 101)
  (unrealized (0 1))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (3 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 191)
  (parent 105)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (x w) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (3 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))) ((recv x) (send x)))
  (label 192)
  (parent 106)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (4 0))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (x-0 (mul (rec x) w)) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w y (rec x-0))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w (rec x-0))))
      (send (cat (exp (gen) y) (mul w (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 193)
  (parent 114)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w x-0)))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (x-0 (mul x w)) (y (exp (gen) (mul x w x-0))))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y x-0)))
      (send (enc n (exp (gen) (mul x w y x-0)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y x-0)))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w x-0)))
      (send (cat (exp (gen) y) (mul w x-0)))) ((recv x-0) (send x-0)))
  (label 194)
  (parent 115)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (4 0))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (x (mul (rec y) w)) (y-0 (exp (gen) w)) (y h))
  (pen-non-orig y-0 x)
  (precur (4 0) (2 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x))))) ((recv x) (send x)))
  (label 195)
  (parent 121)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y-0))) (y y))
  (deflistener (cat (exp (gen) y-0) (mul x w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 (mul x w)) (y (exp (gen) (mul x w y-0))) (y-0 h))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (exp (gen) y-0) (mul x w)))
      (send (cat (exp (gen) y-0) (mul x w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))) ((recv y-0) (send y-0)))
  (label 196)
  (parent 122)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (precur (4 0))
  (operation nonce-test (added-listener x) (rec x) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 197)
  (parent 126)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 198)
  (parent 127)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (rec x)))
  (deflistener x)
  (precedes ((0 0) (3 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 0) (3 0)) ((4 0) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (rec x)))
      (send (cat (exp (gen) x-0) (rec x)))) ((recv x) (send x)))
  (label 199)
  (parent 134)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (3 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (3 0)) ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 200)
  (parent 141)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (rec x)))
  (deflistener x)
  (precedes ((0 0) (3 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (rec x)))
      (send (cat (exp (gen) y-0) (rec x)))) ((recv x) (send x)))
  (label 201)
  (parent 147)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) x)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (gen) (mul x x)))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x x) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (gen) (mul x x))) (send (cat (gen) (mul x x))))
    ((recv x) (send x)))
  (label 202)
  (parent 150)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) w))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (contracted (x-0 x) (w-0 (mul x w)) (w-1 w))
    (gen) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 203)
  (parent 154)
  (unrealized (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x x)))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x)))
      (send (enc n (exp (gen) (mul w x x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) (mul w x x))))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 204)
  (parent 154)
  (unrealized (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0))))
  (label 205)
  (parent 154)
  (unrealized (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 206)
  (parent 154)
  (unrealized (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y-0)))
  (deflistener (cat (gen) (mul x w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (4 0)) ((6 1) (5 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) (mul x w y-0))))
    ((recv (cat (gen) (mul x w y-0))) (send (cat (gen) (mul x w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0))))
  (label 207)
  (parent 154)
  (unrealized (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((2 1) (1 2)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul w x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 208)
  (parent 155)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) w))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul x (rec x-0) w)) (x-2 x-0) (w-1 w))
    (gen) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 209)
  (parent 162)
  (unrealized (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x-0 x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-1) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul w x-0 x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x-0 x-0)))
      (send (exp (gen) (mul w x-0 x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 210)
  (parent 162)
  (unrealized (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 init 1) (exp (gen) x-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 211)
  (parent 162)
  (unrealized (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn) (w expr) (x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-1))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-1)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-1)))
      (send (enc n (exp (gen) (mul x w x-1)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-1))) (send (exp (gen) (mul x w x-1))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((send (exp (gen) x-1))))
  (label 212)
  (parent 162)
  (unrealized (3 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (6 0) (4 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 213)
  (parent 162)
  (unrealized (3 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x x-0 expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) (mul x w y-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0))))
  (label 214)
  (parent 162)
  (unrealized (3 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec y) w)) (y-0 y) (w-1 w)) (gen)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 215)
  (parent 168)
  (unrealized (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x x)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x)))
      (send (enc n (exp (gen) (mul w x x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) (mul w x x))))
    ((recv (cat (exp (gen) y) (mul (rec y) w x x)))
      (send (cat (exp (gen) y) (mul (rec y) w x x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 216)
  (parent 168)
  (unrealized (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0))))
  (label 217)
  (parent 168)
  (unrealized (3 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 218)
  (parent 168)
  (unrealized (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (4 0)) ((6 1) (5 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) (mul x w y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0))))
  (label 219)
  (parent 168)
  (unrealized (3 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) w))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec y-0) w)) (y-1 y-0) (w-1 w))
    (gen) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 220)
  (parent 175)
  (unrealized (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x x)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w x x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x)))
      (send (enc n (exp (gen) (mul w x x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) (mul w x x))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w x x)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w x x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 221)
  (parent 175)
  (unrealized (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w x-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w x-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0))))
  (label 222)
  (parent 175)
  (unrealized (3 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y-0))
  (deflistener (exp (gen) y-0))
  (deflistener (exp (gen) (mul x w y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 (gen)) (y h))
  (pen-non-orig x y-0)
  (precur (6 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (gen)) (send (exp (gen) y-0)) (recv (enc n (exp (gen) y-0))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) (mul x w y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 223)
  (parent 175)
  (unrealized (3 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 resp 2) (exp (gen) y-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) (mul x w y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w)))
      (send (cat (exp (gen) y-0) (mul x w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 224)
  (parent 175)
  (unrealized (3 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (y x y-0 expn) (w expr) (y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-1))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y-1)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w y-1)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) w))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0)))
  (absent (y-1 h-0) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-1)))
      (send (enc n (exp (gen) (mul x w y-1)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y-1))) (send (exp (gen) (mul x w y-1))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w y-1)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w y-1))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) w)) (send (cat (exp (gen) y-1) w)))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 225)
  (parent 175)
  (unrealized (3 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (deflistener (cat (gen) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (5 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul x w))))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv y) (send y)))
  (label 226)
  (parent 176)
  (unrealized (2 0) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x x))) (y y))
  (deflistener (exp (gen) (mul w x x y)))
  (deflistener (cat (exp (gen) y) (mul w x x)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (5 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul w x x))))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x x y)))))
    ((recv (exp (gen) (mul w x x y))) (send (exp (gen) (mul w x x y))))
    ((recv (cat (exp (gen) y) (mul w x x)))
      (send (cat (exp (gen) y) (mul w x x))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))) ((recv y) (send y)))
  (label 227)
  (parent 177)
  (unrealized (1 0) (2 0) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (exp (gen) (mul x w x-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (deflistener y)
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (6 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 0) (1 0))
    ((6 1) (4 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0 y)))
      (send (exp (gen) (mul x w x-0 y))))
    ((recv (cat (exp (gen) y) (mul x w x-0)))
      (send (cat (exp (gen) y) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0)))
    ((recv y) (send y)))
  (label 228)
  (parent 178)
  (unrealized (1 0) (2 0) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 229)
  (parent 179)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (x w) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (4 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w) w (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))))
  (label 230)
  (parent 179)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (3 0)))
  (absent (y (exp (gen) (mul x w))))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 231)
  (parent 179)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (exp (gen) (mul x w y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (6 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (1 0))
    ((6 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (pen-non-orig x y-0)
  (precur (4 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y y-0)))
      (send (exp (gen) (mul x w y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w y)))
      (send (cat (exp (gen) y-0) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 232)
  (parent 180)
  (unrealized (1 0) (2 0) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w-0 (rec x))) (gen) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 233)
  (parent 181)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 234)
  (parent 181)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1))
    ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 235)
  (parent 181)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y))))))
  (label 236)
  (parent 181)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 237)
  (parent 181)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) x))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((2 1) (1 2)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) x)) (send (exp (gen) x))))
  (label 238)
  (parent 182)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w-0 (mul (rec x) x-0))) (gen) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul (rec x) x-0)))
      (send (cat (gen) (mul (rec x) x-0)))))
  (label 239)
  (parent 183)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (6 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-1) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))))
  (label 240)
  (parent 183)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (rec x)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 init 1) (exp (gen) x-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (rec x)))
      (send (cat (exp (gen) x-0) (rec x)))))
  (label 241)
  (parent 183)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 0) (6 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 242)
  (parent 183)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))))
  (label 243)
  (parent 183)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 244)
  (parent 183)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w-0 (mul (rec x) y))) (gen) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 245)
  (parent 184)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 246)
  (parent 184)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)) ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 247)
  (parent 184)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 248)
  (parent 184)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 249)
  (parent 184)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul (rec x) y-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w-0 (mul (rec x) y-0))) (gen) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul (rec x) y-0)))
      (send (cat (gen) (mul (rec x) y-0)))))
  (label 250)
  (parent 185)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y-0 (rec x))))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))))
  (label 251)
  (parent 185)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 0) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 252)
  (parent 185)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) y))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 (exp (gen) w)) (y h))
  (pen-non-orig x y-0)
  (precur (6 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))))
  (label 253)
  (parent 185)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (rec x)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 resp 2) (exp (gen) y-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (rec x)))
      (send (cat (exp (gen) y-0) (rec x)))))
  (label 254)
  (parent 185)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y-1 h-0) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 255)
  (parent 185)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (deflistener (cat (gen) x))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w-0 x)) (gen) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 256)
  (parent 186)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) x)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (gen) (mul x x)))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (gen) (mul x x))) (send (cat (gen) (mul x x)))))
  (label 257)
  (parent 186)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (gen) (mul x x-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (gen) (mul x x-0))) (send (cat (gen) (mul x x-0))))
    ((send (exp (gen) x-0))))
  (label 258)
  (parent 186)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (gen) (mul x y)))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (gen) (mul x y))) (send (cat (gen) (mul x y)))))
  (label 259)
  (parent 186)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (gen) (mul x y-0)))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (gen) (mul x y-0))) (send (cat (gen) (mul x y-0))))
    ((recv h) (send (exp (gen) y-0))))
  (label 260)
  (parent 186)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 w-1 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (gen) w-0))
  (deflistener (cat (exp (gen) (mul (rec x) w-0 (rec w-1))) w-1))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w-0 (rec w-1))) w-1))
    (exp (gen) (mul (rec x) w-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0)))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0)))
    ((recv (cat (exp (gen) (mul (rec x) w-0 (rec w-1))) w-1))
      (send (cat (exp (gen) (mul (rec x) w-0 (rec w-1))) w-1))))
  (label 261)
  (parent 186)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (4 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 262)
  (parent 187)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul x (rec x-0))) (x-2 x-0)) (gen) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 263)
  (parent 188)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x-0 x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
  (defstrand init 1 (x x))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-1) (0 1))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x-0 x-0))) (send (exp (gen) (mul x-0 x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) x-0 x-0))))
    ((send (exp (gen) x))))
  (label 264)
  (parent 188)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (exp (gen) x-0) x))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 6 init 1) (exp (gen) x-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x)))
    ((send (exp (gen) x-0))))
  (label 265)
  (parent 188)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-1)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x-1)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 1 (x x-0))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)) ((7 0) (0 1)) ((7 0) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-1))
      (send (enc n (exp (gen) (mul x x-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x-1))) (send (exp (gen) (mul x x-1))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((send (exp (gen) x-0))) ((send (exp (gen) x-1))))
  (label 266)
  (parent 188)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((send (exp (gen) x-0))))
  (label 267)
  (parent 188)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y-0)))
  (defstrand init 1 (x x-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)) ((7 1) (0 1)) ((7 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y-0))))
    ((send (exp (gen) x-0))) ((recv h) (send (exp (gen) y-0))))
  (label 268)
  (parent 188)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr) (x-0 expn)
    (w-1 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) w-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) w-0 x-0 (rec w-1))) w-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) w-0 x-0 (rec w-1))) w-1))
    (exp (gen) (mul (rec x) w-0 x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0 x-0)))
      (send (enc n (exp (gen) (mul w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x-0))) (send (exp (gen) (mul w-0 x-0))))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) w-0 x-0 (rec w-1))) w-1))
      (send (cat (exp (gen) (mul (rec x) w-0 x-0 (rec w-1))) w-1))))
  (label 269)
  (parent 188)
  (unrealized (7 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec y))) (y-0 y)) (gen) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y))))))
  (label 270)
  (parent 189)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) x)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) x x)))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (exp (gen) y) (mul (rec y) x x)))
      (send (cat (exp (gen) y) (mul (rec y) x x)))))
  (label 271)
  (parent 189)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((send (exp (gen) x-0))))
  (label 272)
  (parent 189)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 273)
  (parent 189)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)) ((6 1) (0 1)) ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((recv h) (send (exp (gen) y-0))))
  (label 274)
  (parent 189)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w w-0 expr) (y expn) (w-1 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) y) w-0))
  (deflistener (cat (exp (gen) (mul (rec x) w-0 y (rec w-1))) w-1))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w-0 y (rec w-1))) w-1))
    (exp (gen) (mul (rec x) w-0 y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0 y)))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0)))
    ((recv (cat (exp (gen) (mul (rec x) w-0 y (rec w-1))) w-1))
      (send (cat (exp (gen) (mul (rec x) w-0 y (rec w-1))) w-1))))
  (label 275)
  (parent 189)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec y-0))) (y-1 y-0)) (gen) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0))))
      (send (cat (exp (gen) y-0) (mul x (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 276)
  (parent 190)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) x)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) x x)))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) x x)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) x x))))
    ((recv h) (send (exp (gen) y-0))))
  (label 277)
  (parent 190)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)) ((7 0) (0 1)) ((7 0) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) x-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) x-0))))
    ((recv h) (send (exp (gen) y-0))) ((send (exp (gen) x-0))))
  (label 278)
  (parent 190)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)) ((6 1) (0 1)) ((6 1) (5 0)))
  (absent (y-0 (exp (gen) w)) (y h))
  (pen-non-orig x y-0)
  (precur (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 279)
  (parent 190)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) y-0) x))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 6 resp 2) (exp (gen) y-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) y-0) x)) (send (cat (exp (gen) y-0) x)))
    ((recv h) (send (exp (gen) y-0))))
  (label 280)
  (parent 190)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-1)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y-1)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) y-1)))
  (defstrand resp 2 (h h) (y y-0))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)) ((7 1) (0 1)) ((7 1) (5 0)))
  (absent (y-1 h-0) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-1))
      (send (enc n (exp (gen) (mul x y-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y-1))) (send (exp (gen) (mul x y-1))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) y-1)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) y-1))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 281)
  (parent 190)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y expn) (w-0 expr)
    (y-0 expn) (w-1 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) w-0 y-0 (rec w-1))) w-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) w-0 y-0 (rec w-1))) w-1))
    (exp (gen) (mul (rec x) w-0 y-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0 y-0)))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) (mul (rec x) w-0 y-0 (rec w-1))) w-1))
      (send (cat (exp (gen) (mul (rec x) w-0 y-0 (rec w-1))) w-1))))
  (label 282)
  (parent 190)
  (unrealized (7 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 283)
  (parent 203)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (gen) w))
  (deflistener (cat (gen) (mul w (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (x w) (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence x w) w (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (gen) (mul w (rec x))))
      (send (cat (gen) (mul w (rec x))))))
  (label 284)
  (parent 203)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) w))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener x) (mul x w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))) ((recv x) (send x)))
  (label 285)
  (parent 203)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (w (mul (rec x) (rec x))) (x-0 x))
    (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 286)
  (parent 204)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) x) (mul w (rec x) (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (x w) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w) w (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) x) (mul w (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w (rec x) (rec x))))))
  (label 287)
  (parent 204)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x x)))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) w))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w x x) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x)))
      (send (enc n (exp (gen) (mul w x x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) (mul w x x))))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w)))
    ((recv x) (send x)))
  (label 288)
  (parent 204)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w (mul (rec x) (rec x-0))) (x-2 x-0)) (one)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 289)
  (parent 205)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (gen) (mul w x)))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (defstrand init 1 (x x))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (x-0 (mul w x)) (y (gen)))
  (pen-non-orig y x-0)
  (precur (5 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul w x)) (mul w x) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (gen) (mul w x))) (send (cat (gen) (mul w x))))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0)))))
    ((send (exp (gen) x))))
  (label 290)
  (parent 205)
  (unrealized (0 1) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (x-0 (mul x w)) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w)) (mul x w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 291)
  (parent 205)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec y))) (y-0 y)) (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y))))))
  (label 292)
  (parent 206)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (x (mul w y)) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul w y)) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))))
  (label 293)
  (parent 206)
  (unrealized (0 1) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul w (rec y))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (mul x w)) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x w)) (mul x w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul w (rec y))))
      (send (cat (exp (gen) y) (mul w (rec y))))))
  (label 294)
  (parent 206)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (4 0)) ((6 1) (5 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec y-0))) (y-1 y-0)) (one)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 295)
  (parent 207)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (4 0)) ((6 1) (5 0)))
  (absent (x (mul w y-0)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul w y-0)) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (gen) (mul w y-0))) (send (cat (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 296)
  (parent 207)
  (unrealized (0 1) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y-0) (mul w (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (4 0)) ((6 1) (5 0)))
  (absent (y-0 (mul x w)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w)) (mul x w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y-0) (mul w (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 297)
  (parent 207)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (precedes ((0 2) (1 2)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul w x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))))
  (label 298)
  (parent 208)
  (unrealized)
  (shape)
  (maps ((0 1) ((n n) (hx (gen)) (hy (exp (gen) w)) (x x) (y y))))
  (origs (n (0 2))))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul (rec x) x-0)))
      (send (cat (gen) (mul (rec x) x-0)))))
  (label 299)
  (parent 209)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (gen) (mul w (rec x-0))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (x-0 (mul (rec x) w)) (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w))
    (mul (rec x) w) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w (rec x-0))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (gen) (mul w (rec x-0))))
      (send (cat (gen) (mul w (rec x-0))))))
  (label 300)
  (parent 209)
  (unrealized (0 1) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w x-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (x-0 (mul x w)) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w)) (mul x w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w x-0))) (send (cat (gen) (mul w x-0)))))
  (label 301)
  (parent 209)
  (unrealized (3 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w (mul x (rec x-0) (rec x-0))) (x-2 x-0)) (one)
    (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))))
  (label 302)
  (parent 210)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x x x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w x x)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x) (mul w x-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (x-0 (mul w x x)) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul w x x)) (mul w x x)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x x-0)))
      (send (enc n (exp (gen) (mul w x x x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x x x-0)))
      (send (exp (gen) (mul w x x x-0))))
    ((recv (cat (exp (gen) x-0) (mul w x x)))
      (send (cat (exp (gen) x-0) (mul w x x)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x) (mul w x-0)))
      (send (cat (exp (gen) x) (mul w x-0)))))
  (label 303)
  (parent 210)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0) (rec x-0))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (x-0 (mul (rec x) w)) (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w))
    (mul (rec x) w) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w (rec x-0))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0) (rec x-0))))))
  (label 304)
  (parent 210)
  (unrealized (0 1) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (rec x-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (contracted (x-1 x-0) (w (rec x-0))) (one)
    (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (rec x-0)))
      (send (cat (exp (gen) x) (rec x-0)))))
  (label 305)
  (parent 211)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) x) w))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (x-0 w) (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 w) w (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w)))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0))))))
  (label 306)
  (parent 211)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) w))
  (deflistener x)
  (precedes ((0 0) (6 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((recv x) (send x)))
  (label 307)
  (parent 211)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-2 x) (x-3 x-0) (w (mul (rec x) x-0 (rec x-1)))
      (x-4 x-1)) (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 308)
  (parent 212)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 (rec x-1)))) (x x-1))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-1))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (x-1 (mul (rec x) w x-0)) (y (gen)))
  (pen-non-orig y x-1)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-absence x-1 (mul (rec x) w x-0))
    (mul (rec x) w x-0) (4 0))
  (traces
    ((send (exp (gen) x-1)) (recv (exp (gen) (mul w x-0 (rec x-1))))
      (send (enc n (exp (gen) (mul w x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) (mul w x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-1))))
      (send (cat (exp (gen) x-0) (mul w (rec x-1)))))
    ((send (exp (gen) x-0))))
  (label 309)
  (parent 212)
  (unrealized (0 1) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 x-1))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0 x-1)))
  (deflistener (cat (exp (gen) x-1) (mul x w x-0)))
  (defstrand init 1 (x x-1))
  (deflistener (cat (exp (gen) x-0) (mul w x-1)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (x-1 (mul x w x-0)) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-absence x-1 (mul x w x-0)) (mul x w x-0)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 x-1)))
      (send (enc n (exp (gen) (mul x w x-0 x-1)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0 x-1)))
      (send (exp (gen) (mul x w x-0 x-1))))
    ((recv (cat (exp (gen) x-1) (mul x w x-0)))
      (send (cat (exp (gen) x-1) (mul x w x-0))))
    ((send (exp (gen) x-1)))
    ((recv (cat (exp (gen) x-0) (mul w x-1)))
      (send (cat (exp (gen) x-0) (mul w x-1))))
    ((send (exp (gen) x-0))))
  (label 310)
  (parent 212)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn) (w expr) (x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (x-1 (mul x (rec x-0) w)) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-absence x-1 (mul x (rec x-0) w))
    (mul x (rec x-0) w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w (rec x-1))))
      (send (cat (exp (gen) x-1) (mul w (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 311)
  (parent 212)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (6 0) (4 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec y))) (y-0 y))
    (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))))
  (label 312)
  (parent 213)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w y)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w (rec x-0))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (x-0 (mul (rec x) w y)) (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w y))
    (mul (rec x) w y) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w y (rec x-0))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) x) (mul (rec x) w y)))
      (send (cat (exp (gen) x) (mul (rec x) w y))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w (rec x-0))))
      (send (cat (exp (gen) y) (mul w (rec x-0))))))
  (label 313)
  (parent 213)
  (unrealized (0 1) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w x-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (x-0 (mul x w y)) (y (gen)))
  (pen-non-orig x y)
  (precur (6 0) (4 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w y)) (mul x w y)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y x-0)))
      (send (enc n (exp (gen) (mul x w y x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y x-0)))
      (send (exp (gen) (mul x w y x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w y)))
      (send (cat (exp (gen) x-0) (mul x w y)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w x-0)))
      (send (cat (exp (gen) y) (mul w x-0)))))
  (label 314)
  (parent 213)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w (rec y))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (mul x (rec x-0) w)) (y (gen)))
  (pen-non-orig x y)
  (precur (6 0) (4 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x (rec x-0) w))
    (mul x (rec x-0) w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w (rec y))))
      (send (cat (exp (gen) y) (mul w (rec y))))))
  (label 315)
  (parent 213)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec y-0)))
      (y-1 y-0)) (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 316)
  (parent 214)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w y-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0)))
  (absent (x-0 (mul (rec x) w y-0)) (y-0 h) (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w y-0))
    (mul (rec x) w y-0) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w y-0 (rec x-0))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w y-0)))
      (send (cat (exp (gen) x) (mul (rec x) w y-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x-0))))
      (send (cat (exp (gen) y-0) (mul w (rec x-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 317)
  (parent 214)
  (unrealized (0 1) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w y-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul w x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0)))
  (absent (x-0 (mul x w y-0)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w y-0)) (mul x w y-0)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 x-0)))
      (send (enc n (exp (gen) (mul x w y-0 x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y-0 x-0)))
      (send (exp (gen) (mul x w y-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w y-0)))
      (send (cat (exp (gen) x-0) (mul x w y-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul w x-0)))
      (send (cat (exp (gen) y-0) (mul w x-0))))
    ((recv h) (send (exp (gen) y-0))))
  (label 318)
  (parent 214)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x x-0 expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul w (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0)))
  (absent (y-0 (mul x (rec x-0) w)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x (rec x-0) w))
    (mul x (rec x-0) w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul w (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 319)
  (parent 214)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (y-0 y) (w (mul (rec x) y)))
    (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 320)
  (parent 215)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (deflistener (cat (gen) (mul w (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (x (mul (rec y) w)) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w))
    (mul (rec y) w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv (cat (gen) (mul w (rec x))))
      (send (cat (gen) (mul w (rec x))))))
  (label 321)
  (parent 215)
  (unrealized (0 1) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (deflistener (cat (gen) (mul w y)))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (mul x w)) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x w)) (mul x w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y)))))
  (label 322)
  (parent 215)
  (unrealized (3 0) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test
    (contracted (y-0 y) (w (mul y (rec x) (rec x))) (x-0 x)) (one)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 323)
  (parent 216)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x x y)))
  (deflistener (cat (exp (gen) y) (mul w x x)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (mul w x x)) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul w x x)) (mul w x x) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x x y))) (send (exp (gen) (mul w x x y))))
    ((recv (cat (exp (gen) y) (mul w x x)))
      (send (cat (exp (gen) y) (mul w x x))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))))
  (label 324)
  (parent 216)
  (unrealized (0 1) (3 0) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (deflistener (cat (exp (gen) x) (mul w (rec x) (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (x (mul (rec y) w)) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w))
    (mul (rec y) w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv (cat (exp (gen) x) (mul w (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w (rec x) (rec x))))))
  (label 325)
  (parent 216)
  (unrealized (0 1) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (y-0 y) (w (mul (rec x) y (rec x-0))) (x-2 x-0))
    (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 326)
  (parent 217)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x)))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (defstrand init 1 (x x))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (x-0 (mul (rec y) w x)) (y (gen)))
  (pen-non-orig y x-0)
  (precur (5 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec y) w x))
    (mul (rec y) w x) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) y) (mul (rec y) w x)))
      (send (cat (exp (gen) y) (mul (rec y) w x))))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0)))))
    ((send (exp (gen) x))))
  (label 327)
  (parent 217)
  (unrealized (0 1) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (y (mul x w x-0)) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x w x-0)) (mul x w x-0)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0 y)))
      (send (exp (gen) (mul x w x-0 y))))
    ((recv (cat (exp (gen) y) (mul x w x-0)))
      (send (cat (exp (gen) y) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0))))
  (label 328)
  (parent 217)
  (unrealized (0 1) (3 0) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (x-0 (mul x (rec y) w)) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x (rec y) w))
    (mul x (rec y) w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 329)
  (parent 217)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 330)
  (parent 218)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)))
  (absent (x w) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w) w (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))))
  (label 331)
  (parent 218)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (4 0)))
  (absent (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 332)
  (parent 218)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (4 0)) ((6 1) (5 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y) (w (mul (rec x) y (rec y-0))) (y-2 y-0))
    (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 333)
  (parent 219)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (4 0)) ((6 1) (5 0)))
  (absent (x (mul (rec y) w y-0)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w y-0))
    (mul (rec y) w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w y-0))))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 334)
  (parent 219)
  (unrealized (0 1) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y-0))
  (deflistener (exp (gen) y-0))
  (deflistener (exp (gen) (mul x w y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (4 0)) ((6 1) (5 0)))
  (absent (y-0 (mul x w y)) (y h) (y-0 (gen)))
  (pen-non-orig x y-0)
  (precur (5 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w y)) (mul x w y)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (gen)) (send (exp (gen) y-0)) (recv (enc n (exp (gen) y-0))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (exp (gen) (mul x w y y-0)))
      (send (exp (gen) (mul x w y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w y)))
      (send (cat (exp (gen) y-0) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 335)
  (parent 219)
  (unrealized (0 1) (3 0) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (deflistener (cat (exp (gen) y-0) (mul w (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (4 0)) ((6 1) (5 0)))
  (absent (y-0 (mul x (rec y) w)) (y-0 h) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x (rec y) w))
    (mul x (rec y) w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv (cat (exp (gen) y-0) (mul w (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 336)
  (parent 219)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul (rec x) y-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul (rec x) y-0)))
      (send (cat (gen) (mul (rec x) y-0)))))
  (label 337)
  (parent 220)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul w (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (x (mul (rec y-0) w)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y-0) w))
    (mul (rec y-0) w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul w (rec x))))
      (send (cat (gen) (mul w (rec x))))))
  (label 338)
  (parent 220)
  (unrealized (0 1) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul w y-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 (mul x w)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w)) (mul x w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) (mul x w y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w)))
      (send (cat (exp (gen) y-0) (mul x w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul w y-0))) (send (cat (gen) (mul w y-0)))))
  (label 339)
  (parent 220)
  (unrealized (3 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test
    (contracted (y-1 y-0) (w (mul y-0 (rec x) (rec x))) (x-0 x)) (one)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y-0 (rec x))))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))))
  (label 340)
  (parent 221)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x x y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w x x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w y-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 (mul w x x)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul w x x)) (mul w x x)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y-0)))
      (send (enc n (exp (gen) (mul w x x y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x x y-0)))
      (send (exp (gen) (mul w x x y-0))))
    ((recv (cat (exp (gen) y-0) (mul w x x)))
      (send (cat (exp (gen) y-0) (mul w x x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w y-0)))
      (send (cat (exp (gen) x) (mul w y-0)))))
  (label 341)
  (parent 221)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w (rec x) (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (x (mul (rec y-0) w)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y-0) w))
    (mul (rec y-0) w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w (rec x) (rec x))))))
  (label 342)
  (parent 221)
  (unrealized (0 1) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (y-1 y-0) (w (mul (rec x) y-0 (rec x-0)))
      (x-2 x-0)) (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 343)
  (parent 222)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 expn) (w expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (defstrand init 1 (x x))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (x-0 (mul (rec y-0) w x)) (y-0 h) (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec y-0) w x))
    (mul (rec y-0) w x) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w x)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0)))))
    ((send (exp (gen) x))))
  (label 344)
  (parent 222)
  (unrealized (0 1) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul w y-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (y-0 (mul x w x-0)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w x-0)) (mul x w x-0)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y-0)))
      (send (enc n (exp (gen) (mul x w x-0 y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0 y-0)))
      (send (exp (gen) (mul x w x-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w x-0)))
      (send (cat (exp (gen) y-0) (mul x w x-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul w y-0)))
      (send (cat (exp (gen) x-0) (mul w y-0))))
    ((send (exp (gen) x-0))))
  (label 345)
  (parent 222)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (x-0 (mul x (rec y-0) w)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x (rec y-0) w))
    (mul x (rec y-0) w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 346)
  (parent 222)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y-0))
  (deflistener (exp (gen) y-0))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 (gen)) (y h))
  (pen-non-orig x y-0)
  (precur (6 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y) (w (mul (rec x) y (rec y-0))) (y-2 y-0))
    (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y-0)) (recv (enc n (exp (gen) y-0))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))))
  (label 347)
  (parent 223)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y-0))
  (deflistener (exp (gen) y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (x (mul (rec y) w y-0)) (y-0 (gen)) (y h))
  (pen-non-orig y-0 x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w y-0))
    (mul (rec y) w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (gen)) (send (exp (gen) y-0)) (recv (enc n (exp (gen) y-0))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x))))))
  (label 348)
  (parent 223)
  (unrealized (0 1) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w y)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 (mul x w y)) (y (gen)) (y-0 h))
  (pen-non-orig x y)
  (precur (6 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w y)) (mul x w y)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y y-0)))
      (send (exp (gen) (mul x w y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w y)))
      (send (cat (exp (gen) y-0) (mul x w y))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))))
  (label 349)
  (parent 223)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y-0))
  (deflistener (exp (gen) y-0))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec y-0))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 (mul x (rec y) w)) (y-0 (gen)) (y h))
  (pen-non-orig x y-0)
  (precur (6 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x (rec y) w))
    (mul x (rec y) w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y-0)) (recv (enc n (exp (gen) y-0))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w (rec y-0))))))
  (label 350)
  (parent 223)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (rec x)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y-0 (rec x))))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (rec x)))
      (send (cat (exp (gen) y-0) (rec x)))))
  (label 351)
  (parent 224)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (x w) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w) w (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x))))))
  (label 352)
  (parent 224)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener x)
  (precedes ((0 0) (6 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) (mul x w y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w)))
      (send (cat (exp (gen) y-0) (mul x w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv x) (send x)))
  (label 353)
  (parent 224)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (y x y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0)))
  (absent (y-1 h-0) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-2 y-0) (w (mul (rec x) y-0 (rec y-1)))
      (y-3 y-1)) (one) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 354)
  (parent 225)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (y y-0 expn) (w expr) (y-1 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-1 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y-1)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w y-1)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul w (rec x))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0)))
  (absent (x (mul (rec y-0) w y-1)) (y-1 h-0) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 y-1 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y-0) w y-1))
    (mul (rec y-0) w y-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-1 (rec x))))
      (send (enc n (exp (gen) (mul w y-1)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y-1))) (send (exp (gen) (mul w y-1))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w y-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w y-1))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul w (rec x))))
      (send (cat (exp (gen) y-1) (mul w (rec x)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 355)
  (parent 225)
  (unrealized (0 1) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (y x expn) (w expr) (y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 y-1))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y-0 y-1)))
  (deflistener (cat (exp (gen) y-1) (mul x w y-0)))
  (defstrand resp 2 (h h) (y y-1))
  (deflistener (cat (exp (gen) y-0) (mul w y-1)))
  (defstrand resp 2 (h h-0) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0)))
  (absent (y-1 (mul x w y-0)) (y-0 h-0) (y-1 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-absence y-1 (mul x w y-0)) (mul x w y-0)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 y-1)))
      (send (enc n (exp (gen) (mul x w y-0 y-1)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y-0 y-1)))
      (send (exp (gen) (mul x w y-0 y-1))))
    ((recv (cat (exp (gen) y-1) (mul x w y-0)))
      (send (cat (exp (gen) y-1) (mul x w y-0))))
    ((recv h) (send (exp (gen) y-1)))
    ((recv (cat (exp (gen) y-0) (mul w y-1)))
      (send (cat (exp (gen) y-0) (mul w y-1))))
    ((recv h-0) (send (exp (gen) y-0))))
  (label 356)
  (parent 225)
  (unrealized (0 1) (3 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (y x y-0 expn) (w expr) (y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul w (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0)))
  (absent (y-1 (mul x (rec y-0) w)) (y-1 h-0) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-absence y-1 (mul x (rec y-0) w))
    (mul x (rec y-0) w) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul w (rec y-1))))
      (send (cat (exp (gen) y-1) (mul w (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 357)
  (parent 225)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (4 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 358)
  (parent 229)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (absent (x w) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (4 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))) ((recv x) (send x)))
  (label 359)
  (parent 230)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 360)
  (parent 233)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 361)
  (parent 234)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (gen)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (precedes ((0 2) (1 2)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y))))))
  (label 362)
  (parent 238)
  (unrealized)
  (shape)
  (maps ((0 1) ((n n) (hx (exp (gen) w)) (hy (gen)) (x x) (y y))))
  (origs (n (0 2))))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (rec x)))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (rec x)))
      (send (cat (exp (gen) x-0) (rec x)))) ((recv x) (send x)))
  (label 363)
  (parent 241)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 364)
  (parent 248)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (rec x)))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (rec x)))
      (send (cat (exp (gen) y-0) (rec x)))) ((recv x) (send x)))
  (label 365)
  (parent 254)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) x)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (gen) (mul x x)))
  (deflistener x)
  (precedes ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x x) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (gen) (mul x x))) (send (cat (gen) (mul x x))))
    ((recv x) (send x)))
  (label 366)
  (parent 257)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (gen) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w-1 (mul x w-0)) (w-2 w-0))
    (gen) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0))))
  (label 367)
  (parent 261)
  (unrealized (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x)))
  (deflistener (cat (gen) (mul w-0 x x)))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x)))
      (send (enc n (exp (gen) (mul w-0 x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x))) (send (exp (gen) (mul w-0 x x))))
    ((recv (cat (gen) (mul w-0 x x))) (send (cat (gen) (mul w-0 x x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 368)
  (parent 261)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) w-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (gen) (mul x w-0 x-0)))
      (send (cat (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0)))
    ((send (exp (gen) x-0))))
  (label 369)
  (parent 261)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y)))
  (deflistener (cat (gen) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y)))
      (send (enc n (exp (gen) (mul x w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y))) (send (exp (gen) (mul x w-0 y))))
    ((recv (cat (gen) (mul x w-0 y))) (send (cat (gen) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0))))
  (label 370)
  (parent 261)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)) ((7 1) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (gen) (mul x w-0 y-0)))
      (send (cat (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0))))
  (label 371)
  (parent 261)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((2 1) (1 2)) ((3 1) (2 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 372)
  (parent 262)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w-1 (mul x (rec x-0) w-0)) (x-2 x-0) (w-2 w-0))
    (gen) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0))))
  (label 373)
  (parent 269)
  (unrealized (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x-0 x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 x-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 0 init 1) (exp (gen) x-1) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul w-0 x-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x-0 x-0)))
      (send (exp (gen) (mul w-0 x-0 x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 x-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0))))
  (label 374)
  (parent 269)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 6 init 1) (exp (gen) x-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0)))
      (send (cat (exp (gen) x-0) (mul x w-0)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0))))
  (label 375)
  (parent 269)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn) (w-0 expr) (x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-1)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0 x-1)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) w-0))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-1)))
      (send (enc n (exp (gen) (mul x w-0 x-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-1)))
      (send (exp (gen) (mul x w-0 x-1))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0 x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0 x-1))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) w-0)) (send (cat (exp (gen) x-1) w-0)))
    ((send (exp (gen) x-1))))
  (label 376)
  (parent 269)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x x-0 expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0 y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 8 1 resp 2) (exp (gen) y-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y)))
      (send (enc n (exp (gen) (mul x w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y))) (send (exp (gen) (mul x w-0 y))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0 y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0 y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0))))
  (label 377)
  (parent 269)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x x-0 expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0 y-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0 y-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0 y-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0))))
  (label 378)
  (parent 269)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (deflistener (cat (gen) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-1 (mul x (rec y) w-0)) (y-0 y) (w-2 w-0))
    (gen) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0))))
  (label 379)
  (parent 275)
  (unrealized (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 x x)))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x)))
      (send (enc n (exp (gen) (mul w-0 x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x))) (send (exp (gen) (mul w-0 x x))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 x x)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 x x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 380)
  (parent 275)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) w-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0 x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0 x-0))))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0)))
    ((send (exp (gen) x-0))))
  (label 381)
  (parent 275)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w-0)))
  (deflistener (cat (exp (gen) y) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y)))
      (send (enc n (exp (gen) (mul x w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y))) (send (exp (gen) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) (mul x w-0)))
      (send (cat (exp (gen) y) (mul x w-0))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0))))
  (label 382)
  (parent 275)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0 y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0 y-0))))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0))))
  (label 383)
  (parent 275)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-1 (mul x (rec y-0) w-0)) (y-1 y-0) (w-2 w-0))
    (gen) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0))))
  (label 384)
  (parent 282)
  (unrealized (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0 x x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (displaced 8 0 init 1) (exp (gen) x-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x)))
      (send (enc n (exp (gen) (mul w-0 x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x))) (send (exp (gen) (mul w-0 x x))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0 x x)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0 x x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 385)
  (parent 282)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0 x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) w-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0 x-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0 x-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0)))
    ((send (exp (gen) x-0))))
  (label 386)
  (parent 282)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0 y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)))
  (absent (y-0 (exp (gen) w)) (y h))
  (pen-non-orig x y-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 1 resp 2) (exp (gen) y-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0 y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0 y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0))))
  (label 387)
  (parent 282)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 6 resp 2) (exp (gen) y-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0)))
      (send (cat (exp (gen) y-0) (mul x w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0))))
  (label 388)
  (parent 282)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x y-0 expn) (w-0 expr)
    (y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-1)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0 y-1)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) w-0))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-1 h-0) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-1)))
      (send (enc n (exp (gen) (mul x w-0 y-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-1)))
      (send (exp (gen) (mul x w-0 y-1))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0 y-1)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0 y-1))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) w-0)) (send (cat (exp (gen) y-1) w-0)))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 389)
  (parent 282)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener x) (rec x) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 390)
  (parent 283)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (gen) w))
  (deflistener (cat (gen) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (absent (x w) (y (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener x) (mul w (rec x)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (gen) (mul w (rec x))))
      (send (cat (gen) (mul w (rec x))))) ((recv x) (send x)))
  (label 391)
  (parent 284)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 392)
  (parent 286)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) x) (mul w (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (absent (x w) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x) (rec x))
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) x) (mul w (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 393)
  (parent 287)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (gen) (mul w x)))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (defstrand init 1 (x x))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 0) (4 0)) ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (x-0 (mul w x)) (y (gen)))
  (pen-non-orig y x-0)
  (precur (5 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (gen) (mul w x))) (send (cat (gen) (mul w x))))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0)))))
    ((send (exp (gen) x))) ((recv x-0) (send x-0)))
  (label 394)
  (parent 290)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0))))
  (defstrand init 1 (x x-0))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 0) (4 0)) ((6 0) (7 0)) ((7 1) (5 0)))
  (absent (x-0 (mul x w)) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0)))))
    ((send (exp (gen) x-0))) ((recv x-0) (send x-0)))
  (label 395)
  (parent 291)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (x (mul w y)) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))) ((recv x) (send x)))
  (label 396)
  (parent 293)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul w (rec y))))
  (deflistener y)
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (y (mul x w)) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w (rec y)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul w (rec y))))
      (send (cat (exp (gen) y) (mul w (rec y))))) ((recv y) (send y)))
  (label 397)
  (parent 294)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (4 0)) ((6 1) (5 0)) ((7 1) (5 0)))
  (absent (x (mul w y-0)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (gen) (mul w y-0))) (send (cat (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x)))))
    ((recv h) (send (exp (gen) y-0))) ((recv x) (send x)))
  (label 398)
  (parent 296)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y-0) (mul w (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener y-0)
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (0 1))
    ((6 1) (4 0)) ((6 1) (7 0)) ((7 1) (5 0)))
  (absent (y-0 (mul x w)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w (rec y-0)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y-0) (mul w (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))) ((recv y-0) (send y-0)))
  (label 399)
  (parent 297)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (gen) (mul w (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x-0 (mul (rec x) w)) (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w (rec x-0))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (gen) (mul w (rec x-0))))
      (send (cat (gen) (mul w (rec x-0))))) ((recv x-0) (send x-0)))
  (label 400)
  (parent 300)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w x-0)))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x-0 (mul x w)) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w x-0))) (send (cat (gen) (mul w x-0))))
    ((recv x-0) (send x-0)))
  (label 401)
  (parent 301)
  (unrealized (3 0) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x x x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w x x)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x) (mul w x-0)))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (7 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x-0 (mul w x x)) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x x-0)))
      (send (enc n (exp (gen) (mul w x x x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x x x-0)))
      (send (exp (gen) (mul w x x x-0))))
    ((recv (cat (exp (gen) x-0) (mul w x x)))
      (send (cat (exp (gen) x-0) (mul w x x)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x) (mul w x-0)))
      (send (cat (exp (gen) x) (mul w x-0)))) ((recv x-0) (send x-0)))
  (label 402)
  (parent 303)
  (unrealized (3 0) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0) (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x-0 (mul (rec x) w)) (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0) (rec x-0))
    (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w (rec x-0))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0) (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 403)
  (parent 304)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (rec x-0)))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (rec x-0) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (rec x-0)))
      (send (cat (exp (gen) x) (rec x-0)))) ((recv x-0) (send x-0)))
  (label 404)
  (parent 305)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) x) w))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x-0 w) (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w)))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 405)
  (parent 306)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 (rec x-1)))) (x x-1))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-1))))
  (defstrand init 1 (x x-0))
  (deflistener x-1)
  (precedes ((0 0) (4 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (x-1 (mul (rec x) w x-0)) (y (gen)))
  (pen-non-orig y x-1)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-listener x-1) (mul w (rec x-1)) (6 0))
  (traces
    ((send (exp (gen) x-1)) (recv (exp (gen) (mul w x-0 (rec x-1))))
      (send (enc n (exp (gen) (mul w x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) (mul w x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-1))))
      (send (cat (exp (gen) x-0) (mul w (rec x-1)))))
    ((send (exp (gen) x-0))) ((recv x-1) (send x-1)))
  (label 406)
  (parent 309)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn) (w expr) (x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 x-1))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0 x-1)))
  (deflistener (cat (exp (gen) x-1) (mul x w x-0)))
  (defstrand init 1 (x x-1))
  (deflistener (cat (exp (gen) x-0) (mul w x-1)))
  (defstrand init 1 (x x-0))
  (deflistener x-1)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (8 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (x-1 (mul x w x-0)) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-listener x-1) (mul w x-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 x-1)))
      (send (enc n (exp (gen) (mul x w x-0 x-1)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0 x-1)))
      (send (exp (gen) (mul x w x-0 x-1))))
    ((recv (cat (exp (gen) x-1) (mul x w x-0)))
      (send (cat (exp (gen) x-1) (mul x w x-0))))
    ((send (exp (gen) x-1)))
    ((recv (cat (exp (gen) x-0) (mul w x-1)))
      (send (cat (exp (gen) x-0) (mul w x-1)))) ((send (exp (gen) x-0)))
    ((recv x-1) (send x-1)))
  (label 407)
  (parent 310)
  (unrealized (3 0) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn) (w expr) (x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w (rec x-1))))
  (defstrand init 1 (x x-1))
  (deflistener x-1)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (8 0))
    ((8 1) (6 0)))
  (absent (x-1 (mul x (rec x-0) w)) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-listener x-1) (mul w (rec x-1)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w (rec x-1))))
      (send (cat (exp (gen) x-1) (mul w (rec x-1)))))
    ((send (exp (gen) x-1))) ((recv x-1) (send x-1)))
  (label 408)
  (parent 311)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w y)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 1))
    ((7 1) (6 0)))
  (absent (x-0 (mul (rec x) w y)) (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w y (rec x-0))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) x) (mul (rec x) w y)))
      (send (cat (exp (gen) x) (mul (rec x) w y))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w (rec x-0))))
      (send (cat (exp (gen) y) (mul w (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 409)
  (parent 313)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w x-0)))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 1))
    ((7 1) (6 0)))
  (absent (x-0 (mul x w y)) (y (gen)))
  (pen-non-orig x y)
  (precur (6 0) (4 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y x-0)))
      (send (enc n (exp (gen) (mul x w y x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y x-0)))
      (send (exp (gen) (mul x w y x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w y)))
      (send (cat (exp (gen) x-0) (mul x w y)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w x-0)))
      (send (cat (exp (gen) y) (mul w x-0)))) ((recv x-0) (send x-0)))
  (label 410)
  (parent 314)
  (unrealized (3 0) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w (rec y))))
  (deflistener y)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 1))
    ((7 1) (6 0)))
  (absent (y (mul x (rec x-0) w)) (y (gen)))
  (pen-non-orig x y)
  (precur (6 0) (4 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w (rec y)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w (rec y))))
      (send (cat (exp (gen) y) (mul w (rec y))))) ((recv y) (send y)))
  (label 411)
  (parent 315)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w y-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x-0))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0))
    ((8 1) (6 0)))
  (absent (x-0 (mul (rec x) w y-0)) (y-0 h) (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w y-0 (rec x-0))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w y-0)))
      (send (cat (exp (gen) x) (mul (rec x) w y-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x-0))))
      (send (cat (exp (gen) y-0) (mul w (rec x-0)))))
    ((recv h) (send (exp (gen) y-0))) ((recv x-0) (send x-0)))
  (label 412)
  (parent 317)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w y-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul w x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (8 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0))
    ((8 1) (6 0)))
  (absent (x-0 (mul x w y-0)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 x-0)))
      (send (enc n (exp (gen) (mul x w y-0 x-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y-0 x-0)))
      (send (exp (gen) (mul x w y-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w y-0)))
      (send (cat (exp (gen) x-0) (mul x w y-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul w x-0)))
      (send (cat (exp (gen) y-0) (mul w x-0))))
    ((recv h) (send (exp (gen) y-0))) ((recv x-0) (send x-0)))
  (label 413)
  (parent 318)
  (unrealized (3 0) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x x-0 expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul w (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener y-0)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (absent (y-0 (mul x (rec x-0) w)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w (rec y-0)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul w (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))) ((recv y-0) (send y-0)))
  (label 414)
  (parent 319)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (deflistener (cat (gen) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (x (mul (rec y) w)) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv (cat (gen) (mul w (rec x))))
      (send (cat (gen) (mul w (rec x))))) ((recv x) (send x)))
  (label 415)
  (parent 321)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (deflistener (cat (gen) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (y (mul x w)) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv y) (send y)))
  (label 416)
  (parent 322)
  (unrealized (3 0) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x x y)))
  (deflistener (cat (exp (gen) y) (mul w x x)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (y (mul w x x)) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x x y))) (send (exp (gen) (mul w x x y))))
    ((recv (cat (exp (gen) y) (mul w x x)))
      (send (cat (exp (gen) y) (mul w x x))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))) ((recv y) (send y)))
  (label 417)
  (parent 324)
  (unrealized (3 0) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (deflistener (cat (exp (gen) x) (mul w (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (x (mul (rec y) w)) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x) (rec x))
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv (cat (exp (gen) x) (mul w (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 418)
  (parent 325)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x)))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (defstrand init 1 (x x))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (x-0 (mul (rec y) w x)) (y (gen)))
  (pen-non-orig y x-0)
  (precur (5 0) (4 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) y) (mul (rec y) w x)))
      (send (cat (exp (gen) y) (mul (rec y) w x))))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0)))))
    ((send (exp (gen) x))) ((recv x-0) (send x-0)))
  (label 419)
  (parent 327)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (deflistener y)
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (y (mul x w x-0)) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0 y)))
      (send (exp (gen) (mul x w x-0 y))))
    ((recv (cat (exp (gen) y) (mul x w x-0)))
      (send (cat (exp (gen) y) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0)))
    ((recv y) (send y)))
  (label 420)
  (parent 328)
  (unrealized (3 0) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0))))
  (defstrand init 1 (x x-0))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (x-0 (mul x (rec y) w)) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0)))))
    ((send (exp (gen) x-0))) ((recv x-0) (send x-0)))
  (label 421)
  (parent 329)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 422)
  (parent 330)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (x w) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))) ((recv x) (send x)))
  (label 423)
  (parent 331)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (4 0)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (absent (x (mul (rec y) w y-0)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (5 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w y-0))))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x)))))
    ((recv h) (send (exp (gen) y-0))) ((recv x) (send x)))
  (label 424)
  (parent 334)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y-0))
  (deflistener (exp (gen) y-0))
  (deflistener (exp (gen) (mul x w y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (4 0)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (absent (y-0 (mul x w y)) (y h) (y-0 (gen)))
  (pen-non-orig x y-0)
  (precur (5 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (gen)) (send (exp (gen) y-0)) (recv (enc n (exp (gen) y-0))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (exp (gen) (mul x w y y-0)))
      (send (exp (gen) (mul x w y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w y)))
      (send (cat (exp (gen) y-0) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 425)
  (parent 335)
  (unrealized (3 0) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (deflistener (cat (exp (gen) y-0) (mul w (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener y-0)
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (0 1)) ((6 1) (4 0)) ((6 1) (7 0))
    ((7 1) (5 0)))
  (absent (y-0 (mul x (rec y) w)) (y-0 h) (y (gen)))
  (pen-non-orig x y)
  (precur (5 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w (rec y-0)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv (cat (exp (gen) y-0) (mul w (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))) ((recv y-0) (send y-0)))
  (label 426)
  (parent 336)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x (mul (rec y-0) w)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul w (rec x))))
      (send (cat (gen) (mul w (rec x))))) ((recv x) (send x)))
  (label 427)
  (parent 338)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y-0 (mul x w)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) (mul x w y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w)))
      (send (cat (exp (gen) y-0) (mul x w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul w y-0))) (send (cat (gen) (mul w y-0))))
    ((recv y-0) (send y-0)))
  (label 428)
  (parent 339)
  (unrealized (3 0) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x x y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w x x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (7 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y-0 (mul w x x)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y-0)))
      (send (enc n (exp (gen) (mul w x x y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x x y-0)))
      (send (exp (gen) (mul w x x y-0))))
    ((recv (cat (exp (gen) y-0) (mul w x x)))
      (send (cat (exp (gen) y-0) (mul w x x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w y-0)))
      (send (cat (exp (gen) x) (mul w y-0)))) ((recv y-0) (send y-0)))
  (label 429)
  (parent 341)
  (unrealized (3 0) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x (mul (rec y-0) w)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x) (rec x))
    (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 430)
  (parent 342)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 expn) (w expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (defstrand init 1 (x x))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (x-0 (mul (rec y-0) w x)) (y-0 h) (y (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w x)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0)))))
    ((send (exp (gen) x))) ((recv x-0) (send x-0)))
  (label 431)
  (parent 344)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x expn) (w expr) (x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w x-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul w y-0)))
  (defstrand init 1 (x x-0))
  (deflistener y-0)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (8 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (6 0))
    ((8 1) (6 0)))
  (absent (y-0 (mul x w x-0)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y-0)))
      (send (enc n (exp (gen) (mul x w x-0 y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w x-0 y-0)))
      (send (exp (gen) (mul x w x-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w x-0)))
      (send (cat (exp (gen) y-0) (mul x w x-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul w y-0)))
      (send (cat (exp (gen) x-0) (mul w y-0)))) ((send (exp (gen) x-0)))
    ((recv y-0) (send y-0)))
  (label 432)
  (parent 345)
  (unrealized (3 0) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0))))
  (defstrand init 1 (x x-0))
  (deflistener x-0)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 0) (4 0)) ((7 0) (8 0))
    ((8 1) (6 0)))
  (absent (x-0 (mul x (rec y-0) w)) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0)))))
    ((send (exp (gen) x-0))) ((recv x-0) (send x-0)))
  (label 433)
  (parent 346)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y-0))
  (deflistener (exp (gen) y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (6 0)))
  (absent (x (mul (rec y) w y-0)) (y-0 (gen)) (y h))
  (pen-non-orig y-0 x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (gen)) (send (exp (gen) y-0)) (recv (enc n (exp (gen) y-0))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x))))) ((recv x) (send x)))
  (label 434)
  (parent 348)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w y)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (4 0)) ((5 1) (7 0)) ((6 1) (0 1))
    ((7 1) (6 0)))
  (absent (y-0 (mul x w y)) (y (gen)) (y-0 h))
  (pen-non-orig x y)
  (precur (6 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y y-0)))
      (send (exp (gen) (mul x w y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w y)))
      (send (cat (exp (gen) y-0) (mul x w y))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))) ((recv y-0) (send y-0)))
  (label 435)
  (parent 349)
  (unrealized (3 0) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y-0))
  (deflistener (exp (gen) y-0))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec y-0))))
  (deflistener y-0)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 1) (4 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (1 2))
    ((4 1) (3 0)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1))
    ((7 1) (6 0)))
  (absent (y-0 (mul x (rec y) w)) (y-0 (gen)) (y h))
  (pen-non-orig x y-0)
  (precur (6 0) (4 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w (rec y-0)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y-0)) (recv (enc n (exp (gen) y-0))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w (rec y-0)))))
    ((recv y-0) (send y-0)))
  (label 436)
  (parent 350)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (rec x)))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y-0 (rec x))))
      (send (enc n (exp (gen) y-0))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (rec x)))
      (send (cat (exp (gen) y-0) (rec x)))) ((recv x) (send x)))
  (label 437)
  (parent 351)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x w) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x))))) ((recv x) (send x)))
  (label 438)
  (parent 352)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (y y-0 expn) (w expr) (y-1 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-1 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul w y-1)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w y-1)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul w (rec x))))
  (defstrand resp 2 (h h-0) (y y-1))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0))
    ((8 1) (6 0)))
  (absent (x (mul (rec y-0) w y-1)) (y-1 h-0) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y y-0 y-1 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-1 (rec x))))
      (send (enc n (exp (gen) (mul w y-1)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul w y-1))) (send (exp (gen) (mul w y-1))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w y-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w y-1))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul w (rec x))))
      (send (cat (exp (gen) y-1) (mul w (rec x)))))
    ((recv h-0) (send (exp (gen) y-1))) ((recv x) (send x)))
  (label 439)
  (parent 355)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (y x expn) (w expr) (y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 y-1))) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w y-0 y-1)))
  (deflistener (cat (exp (gen) y-1) (mul x w y-0)))
  (defstrand resp 2 (h h) (y y-1))
  (deflistener (cat (exp (gen) y-0) (mul w y-1)))
  (defstrand resp 2 (h h-0) (y y-0))
  (deflistener y-1)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (8 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (6 0))
    ((8 1) (6 0)))
  (absent (y-1 (mul x w y-0)) (y-0 h-0) (y-1 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-listener y-1) (mul w y-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 y-1)))
      (send (enc n (exp (gen) (mul x w y-0 y-1)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w y-0 y-1)))
      (send (exp (gen) (mul x w y-0 y-1))))
    ((recv (cat (exp (gen) y-1) (mul x w y-0)))
      (send (cat (exp (gen) y-1) (mul x w y-0))))
    ((recv h) (send (exp (gen) y-1)))
    ((recv (cat (exp (gen) y-0) (mul w y-1)))
      (send (cat (exp (gen) y-0) (mul w y-1))))
    ((recv h-0) (send (exp (gen) y-0))) ((recv y-1) (send y-1)))
  (label 440)
  (parent 356)
  (unrealized (3 0) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (y x y-0 expn) (w expr) (y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) w)) (x x))
  (defstrand resp 3 (n n) (h (gen)) (y y))
  (deflistener (exp (gen) y))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul w (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (deflistener y-1)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((2 1) (1 2)) ((3 1) (1 2)) ((4 1) (3 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (4 0)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (absent (y-1 (mul x (rec y-0) w)) (y-1 h-0) (y-0 h) (y (gen)))
  (pen-non-orig y x)
  (precur (6 0) (4 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-listener y-1) (mul w (rec y-1)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y))))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul w (rec y-1))))
      (send (cat (exp (gen) y-1) (mul w (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))) ((recv y-1) (send y-1)))
  (label 441)
  (parent 357)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w-0 (rec x))) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 442)
  (parent 367)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (gen) w-0))
  (deflistener (cat (gen) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (x w-0) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w-0) w-0 (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0)))
    ((recv (cat (gen) (mul w-0 (rec x))))
      (send (cat (gen) (mul w-0 (rec x))))))
  (label 443)
  (parent 367)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (gen) w-0))
  (deflistener x)
  (precedes ((0 0) (6 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0))) ((recv x) (send x)))
  (label 444)
  (parent 367)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (w-0 (mul (rec x) (rec x))) (x-0 x))
    (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 445)
  (parent 368)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (gen) w-0))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (x w-0) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w-0) w-0 (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))))
  (label 446)
  (parent 368)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x)))
  (deflistener (cat (gen) (mul w-0 x x)))
  (deflistener (cat (exp (gen) x) w-0))
  (deflistener x)
  (precedes ((0 0) (6 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 x x) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x)))
      (send (enc n (exp (gen) (mul w-0 x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x))) (send (exp (gen) (mul w-0 x x))))
    ((recv (cat (gen) (mul w-0 x x))) (send (cat (gen) (mul w-0 x x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0)))
    ((recv x) (send x)))
  (label 447)
  (parent 368)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul (rec x) (rec x-0))) (x-2 x-0)) (one)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 448)
  (parent 369)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (x-0 (mul w-0 x)) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul w-0 x)) (mul w-0 x)
    (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (gen) (mul w-0 x))) (send (cat (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x))))
  (label 449)
  (parent 369)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (x-0 (mul x w-0)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w-0)) (mul x w-0)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 450)
  (parent 369)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul (rec x) (rec y))) (y-0 y)) (one)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y))))))
  (label 451)
  (parent 370)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w w-0 expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (x (mul w-0 y)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul w-0 y)) (mul w-0 y) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y (rec x))))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (gen) (mul w-0 y))) (send (cat (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) y) (mul w-0 (rec x))))
      (send (cat (exp (gen) y) (mul w-0 (rec x))))))
  (label 452)
  (parent 370)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec y))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (mul x w-0)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x w-0)) (mul x w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul w-0 (rec y))))
      (send (cat (exp (gen) y) (mul w-0 (rec y))))))
  (label 453)
  (parent 370)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)) ((7 1) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul (rec x) (rec y-0))) (y-1 y-0)) (one)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 454)
  (parent 371)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)) ((7 1) (6 0)))
  (absent (x (mul w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul w-0 y-0)) (mul w-0 y-0)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (gen) (mul w-0 y-0))) (send (cat (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 455)
  (parent 371)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)) ((7 1) (6 0)))
  (absent (y-0 (mul x w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w-0)) (mul x w-0)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 456)
  (parent 371)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((2 1) (1 2)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 457)
  (parent 372)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w-0 (mul (rec x) x-0))) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul (rec x) x-0)))
      (send (cat (gen) (mul (rec x) x-0)))))
  (label 458)
  (parent 373)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (gen) (mul w-0 (rec x-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (x-0 (mul (rec x) w-0)) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w-0))
    (mul (rec x) w-0) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 (rec x-0))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0))))
    ((send (exp (gen) x)))
    ((recv (cat (gen) (mul w-0 (rec x-0))))
      (send (cat (gen) (mul w-0 (rec x-0))))))
  (label 459)
  (parent 373)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w-0 x-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (x-0 (mul x w-0)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w-0)) (mul x w-0)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0)))
      (send (cat (exp (gen) x-0) (mul x w-0)))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w-0 x-0))) (send (cat (gen) (mul w-0 x-0)))))
  (label 460)
  (parent 373)
  (unrealized (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul x (rec x-0) (rec x-0))) (x-2 x-0))
    (one) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))))
  (label 461)
  (parent 374)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 x x)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x) (mul w-0 x-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (x-0 (mul w-0 x x)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul w-0 x x)) (mul w-0 x x)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x x-0)))
      (send (enc n (exp (gen) (mul w-0 x x x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x x-0)))
      (send (exp (gen) (mul w-0 x x x-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 x x)))
      (send (cat (exp (gen) x-0) (mul w-0 x x))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x) (mul w-0 x-0)))
      (send (cat (exp (gen) x) (mul w-0 x-0)))))
  (label 462)
  (parent 374)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0) (rec x-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (x-0 (mul (rec x) w-0)) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w-0))
    (mul (rec x) w-0) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 (rec x-0))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0) (rec x-0))))))
  (label 463)
  (parent 374)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (rec x-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (contracted (x-1 x-0) (w-0 (rec x-0))) (one)
    (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (rec x-0)))
      (send (cat (exp (gen) x) (rec x-0)))))
  (label 464)
  (parent 375)
  (unrealized (0 1) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) w-0))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (x-0 w-0) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 w-0) w-0 (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0)))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0))))))
  (label 465)
  (parent 375)
  (unrealized (0 1) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) w-0))
  (deflistener x)
  (precedes ((0 0) (7 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0)))
      (send (cat (exp (gen) x-0) (mul x w-0)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0)))
    ((recv x) (send x)))
  (label 466)
  (parent 375)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-2 x) (x-3 x-0) (w-0 (mul (rec x) x-0 (rec x-1)))
      (x-4 x-1)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 467)
  (parent 376)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 (rec x-1))))
    (x x-1))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-1))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (x-1 (mul (rec x) w-0 x-0)) (y (exp (gen) w)))
  (pen-non-orig y x-1)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-absence x-1 (mul (rec x) w-0 x-0))
    (mul (rec x) w-0 x-0) (5 0))
  (traces
    ((send (exp (gen) x-1)) (recv (exp (gen) (mul w-0 x-0 (rec x-1))))
      (send (enc n (exp (gen) (mul w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x-0))) (send (exp (gen) (mul w-0 x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-1))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-1)))))
    ((send (exp (gen) x-0))))
  (label 468)
  (parent 376)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 x-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0 x-1)))
  (deflistener (cat (exp (gen) x-1) (mul x w-0 x-0)))
  (defstrand init 1 (x x-1))
  (deflistener (cat (exp (gen) x-0) (mul w-0 x-1)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (x-1 (mul x w-0 x-0)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-absence x-1 (mul x w-0 x-0))
    (mul x w-0 x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0 x-1)))
      (send (enc n (exp (gen) (mul x w-0 x-0 x-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0 x-1)))
      (send (exp (gen) (mul x w-0 x-0 x-1))))
    ((recv (cat (exp (gen) x-1) (mul x w-0 x-0)))
      (send (cat (exp (gen) x-1) (mul x w-0 x-0))))
    ((send (exp (gen) x-1)))
    ((recv (cat (exp (gen) x-0) (mul w-0 x-1)))
      (send (cat (exp (gen) x-0) (mul w-0 x-1))))
    ((send (exp (gen) x-0))))
  (label 469)
  (parent 376)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn) (w-0 expr) (x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (x-1 (mul x (rec x-0) w-0)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-absence x-1 (mul x (rec x-0) w-0))
    (mul x (rec x-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul w-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 470)
  (parent 376)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w-0 (mul (rec x) x-0 (rec y)))
      (y-0 y)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))))
  (label 471)
  (parent 377)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 y)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec x-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)))
  (absent (x-0 (mul (rec x) w-0 y)) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w-0 y))
    (mul (rec x) w-0 y) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 y (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 y)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 y))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) y) (mul w-0 (rec x-0))))))
  (label 472)
  (parent 377)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0 y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w-0 x-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)))
  (absent (x-0 (mul x w-0 y)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w-0 y)) (mul x w-0 y)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y x-0)))
      (send (enc n (exp (gen) (mul x w-0 y x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y x-0)))
      (send (exp (gen) (mul x w-0 y x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0 y)))
      (send (cat (exp (gen) x-0) (mul x w-0 y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w-0 x-0)))
      (send (cat (exp (gen) y) (mul w-0 x-0)))))
  (label 473)
  (parent 377)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x x-0 expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec y))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)))
  (absent (y (mul x (rec x-0) w-0)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x (rec x-0) w-0))
    (mul x (rec x-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w-0 (rec y))))
      (send (cat (exp (gen) y) (mul w-0 (rec y))))))
  (label 474)
  (parent 377)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w-0 (mul (rec x) x-0 (rec y-0)))
      (y-1 y-0)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 475)
  (parent 378)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x-0))))
    (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 y-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (x-0 (mul (rec x) w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w-0 y-0))
    (mul (rec x) w-0 y-0) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 y-0 (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 y-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 y-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 476)
  (parent 378)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0 y-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul w-0 x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (x-0 (mul x w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w-0 y-0))
    (mul x w-0 y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0 x-0)))
      (send (exp (gen) (mul x w-0 y-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0 y-0)))
      (send (cat (exp (gen) x-0) (mul x w-0 y-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul w-0 x-0)))
      (send (cat (exp (gen) y-0) (mul w-0 x-0))))
    ((recv h) (send (exp (gen) y-0))))
  (label 477)
  (parent 378)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x x-0 expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-0 (mul x (rec x-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x (rec x-0) w-0))
    (mul x (rec x-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 478)
  (parent 378)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w-0 (mul (rec x) y))) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 479)
  (parent 379)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0)))
  (deflistener (cat (gen) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (x (mul (rec y) w-0)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w-0))
    (mul (rec y) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0))))
    ((recv (cat (gen) (mul w-0 (rec x))))
      (send (cat (gen) (mul w-0 (rec x))))))
  (label 480)
  (parent 379)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w-0)))
  (deflistener (cat (gen) (mul w-0 y)))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (mul x w-0)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x w-0)) (mul x w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y)))
      (send (enc n (exp (gen) (mul x w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y))) (send (exp (gen) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) (mul x w-0)))
      (send (cat (exp (gen) y) (mul x w-0))))
    ((recv (cat (gen) (mul w-0 y))) (send (cat (gen) (mul w-0 y)))))
  (label 481)
  (parent 379)
  (unrealized (4 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test
    (contracted (y-0 y) (w-0 (mul y (rec x) (rec x))) (x-0 x)) (one)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 482)
  (parent 380)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w w-0 expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x y)))
  (deflistener (cat (exp (gen) y) (mul w-0 x x)))
  (deflistener (cat (exp (gen) x) (mul w-0 y)))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (mul w-0 x x)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul w-0 x x)) (mul w-0 x x)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x y)))
      (send (enc n (exp (gen) (mul w-0 x x y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x y)))
      (send (exp (gen) (mul w-0 x x y))))
    ((recv (cat (exp (gen) y) (mul w-0 x x)))
      (send (cat (exp (gen) y) (mul w-0 x x))))
    ((recv (cat (exp (gen) x) (mul w-0 y)))
      (send (cat (exp (gen) x) (mul w-0 y)))))
  (label 483)
  (parent 380)
  (unrealized (0 1) (4 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0)))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (x (mul (rec y) w-0)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w-0))
    (mul (rec y) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0))))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))))
  (label 484)
  (parent 380)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (y-0 y) (w-0 (mul (rec x) y (rec x-0)))
      (x-2 x-0)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 485)
  (parent 381)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 x)))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)))
  (absent (x-0 (mul (rec y) w-0 x)) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec y) w-0 x))
    (mul (rec y) w-0 x) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 x)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 x))))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x))))
  (label 486)
  (parent 381)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)))
  (absent (y (mul x w-0 x-0)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x w-0 x-0))
    (mul x w-0 x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0 y)))
      (send (enc n (exp (gen) (mul x w-0 x-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0 y)))
      (send (exp (gen) (mul x w-0 x-0 y))))
    ((recv (cat (exp (gen) y) (mul x w-0 x-0)))
      (send (cat (exp (gen) y) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 y)))
      (send (cat (exp (gen) x-0) (mul w-0 y))))
    ((send (exp (gen) x-0))))
  (label 487)
  (parent 381)
  (unrealized (0 1) (4 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)))
  (absent (x-0 (mul x (rec y) w-0)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x (rec y) w-0))
    (mul x (rec y) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 488)
  (parent 381)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w-0 (rec x))) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 489)
  (parent 382)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w w-0 expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) y) w-0))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (x w-0) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w-0) w-0 (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y (rec x))))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0)))
    ((recv (cat (exp (gen) y) (mul w-0 (rec x))))
      (send (cat (exp (gen) y) (mul w-0 (rec x))))))
  (label 490)
  (parent 382)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w-0)))
  (deflistener (cat (exp (gen) y) w-0))
  (deflistener x)
  (precedes ((0 0) (6 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y)))
      (send (enc n (exp (gen) (mul x w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y))) (send (exp (gen) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) (mul x w-0)))
      (send (cat (exp (gen) y) (mul x w-0))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0)))
    ((recv x) (send x)))
  (label 491)
  (parent 382)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y) (w-0 (mul (rec x) y (rec y-0)))
      (y-2 y-0)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 492)
  (parent 383)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)))
  (absent (x (mul (rec y) w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w-0 y-0))
    (mul (rec y) w-0 y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 493)
  (parent 383)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x expn) (w-0 expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul x w-0 y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) (mul w-0 y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)))
  (absent (y-0 (mul x w-0 y)) (y h) (y-0 (exp (gen) w)))
  (pen-non-orig x y-0)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w-0 y)) (mul x w-0 y)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y y-0)))
      (send (enc n (exp (gen) (mul x w-0 y y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul x w-0 y y-0)))
      (send (exp (gen) (mul x w-0 y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0 y)))
      (send (cat (exp (gen) y-0) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) (mul w-0 y-0)))
      (send (cat (exp (gen) y) (mul w-0 y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 494)
  (parent 383)
  (unrealized (0 1) (4 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)))
  (absent (y-0 (mul x (rec y) w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x (rec y) w-0))
    (mul x (rec y) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 495)
  (parent 383)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul (rec x) y-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w-0 (mul (rec x) y-0))) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul (rec x) y-0)))
      (send (cat (gen) (mul (rec x) y-0)))))
  (label 496)
  (parent 384)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (x (mul (rec y-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y-0) w-0))
    (mul (rec y-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul w-0 (rec x))))
      (send (cat (gen) (mul w-0 (rec x))))))
  (label 497)
  (parent 384)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul w-0 y-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 (mul x w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w-0)) (mul x w-0)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0)))
      (send (cat (exp (gen) y-0) (mul x w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul w-0 y-0))) (send (cat (gen) (mul w-0 y-0)))))
  (label 498)
  (parent 384)
  (unrealized (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test
    (contracted (y-1 y-0) (w-0 (mul y-0 (rec x) (rec x))) (x-0 x)) (one)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y-0 (rec x))))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))))
  (label 499)
  (parent 385)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 x x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w-0 y-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 (mul w-0 x x)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul w-0 x x)) (mul w-0 x x)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x y-0)))
      (send (enc n (exp (gen) (mul w-0 x x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x y-0)))
      (send (exp (gen) (mul w-0 x x y-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 x x)))
      (send (cat (exp (gen) y-0) (mul w-0 x x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w-0 y-0)))
      (send (cat (exp (gen) x) (mul w-0 y-0)))))
  (label 500)
  (parent 385)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (x (mul (rec y-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y-0) w-0))
    (mul (rec y-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))))
  (label 501)
  (parent 385)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (y-1 y-0) (w-0 (mul (rec x) y-0 (rec x-0)))
      (x-2 x-0)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 502)
  (parent 386)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0 x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (x-0 (mul (rec y-0) w-0 x)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec y-0) w-0 x))
    (mul (rec y-0) w-0 x) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0 x)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0 x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x))))
  (label 503)
  (parent 386)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0 x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul w-0 y-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (y-0 (mul x w-0 x-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w-0 x-0))
    (mul x w-0 x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0 y-0)))
      (send (exp (gen) (mul x w-0 x-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0 x-0)))
      (send (cat (exp (gen) y-0) (mul x w-0 x-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul w-0 y-0)))
      (send (cat (exp (gen) x-0) (mul w-0 y-0))))
    ((send (exp (gen) x-0))))
  (label 504)
  (parent 386)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (x-0 (mul x (rec y-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x (rec y-0) w-0))
    (mul x (rec y-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 505)
  (parent 386)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)))
  (absent (y-0 (exp (gen) w)) (y h))
  (pen-non-orig x y-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y) (w-0 (mul (rec x) y (rec y-0)))
      (y-2 y-0)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))))
  (label 506)
  (parent 387)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)))
  (absent (x (mul (rec y) w-0 y-0)) (y-0 (exp (gen) w)) (y h))
  (pen-non-orig y-0 x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w-0 y-0))
    (mul (rec y) w-0 y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x))))))
  (label 507)
  (parent 387)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x expn) (w-0 expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0 y)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w-0 y-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)))
  (absent (y-0 (mul x w-0 y)) (y (exp (gen) w)) (y-0 h))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w-0 y)) (mul x w-0 y)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y y-0)))
      (send (enc n (exp (gen) (mul x w-0 y y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y y-0)))
      (send (exp (gen) (mul x w-0 y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0 y)))
      (send (cat (exp (gen) y-0) (mul x w-0 y))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w-0 y-0)))
      (send (cat (exp (gen) y) (mul w-0 y-0)))))
  (label 508)
  (parent 387)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)))
  (absent (y-0 (mul x (rec y) w-0)) (y-0 (exp (gen) w)) (y h))
  (pen-non-orig x y-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x (rec y) w-0))
    (mul x (rec y) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0))))))
  (label 509)
  (parent 387)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (rec x)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w-0 (rec x))) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y-0 (rec x))))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (rec x)))
      (send (cat (exp (gen) y-0) (rec x)))))
  (label 510)
  (parent 388)
  (unrealized (0 1) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (x w-0) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w-0) w-0 (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x))))))
  (label 511)
  (parent 388)
  (unrealized (0 1) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) w-0))
  (deflistener x)
  (precedes ((0 0) (7 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0)))
      (send (cat (exp (gen) y-0) (mul x w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv x) (send x)))
  (label 512)
  (parent 388)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-1 h-0) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-2 y-0) (w-0 (mul (rec x) y-0 (rec y-1)))
      (y-3 y-1)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 513)
  (parent 389)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y y-0 expn) (w-0 expr)
    (y-1 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-1 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-1)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0 y-1)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul w-0 (rec x))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (x (mul (rec y-0) w-0 y-1)) (y-1 h-0) (y-0 h)
    (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 y-1 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y-0) w-0 y-1))
    (mul (rec y-0) w-0 y-1) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-1 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-1))) (send (exp (gen) (mul w-0 y-1))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0 y-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0 y-1))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-1) (mul w-0 (rec x)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 514)
  (parent 389)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x expn) (w-0 expr)
    (y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 y-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0 y-1)))
  (deflistener (cat (exp (gen) y-1) (mul x w-0 y-0)))
  (defstrand resp 2 (h h) (y y-1))
  (deflistener (cat (exp (gen) y-0) (mul w-0 y-1)))
  (defstrand resp 2 (h h-0) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-1 (mul x w-0 y-0)) (y-0 h-0) (y-1 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-absence y-1 (mul x w-0 y-0))
    (mul x w-0 y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 y-1)))
      (send (enc n (exp (gen) (mul x w-0 y-0 y-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0 y-1)))
      (send (exp (gen) (mul x w-0 y-0 y-1))))
    ((recv (cat (exp (gen) y-1) (mul x w-0 y-0)))
      (send (cat (exp (gen) y-1) (mul x w-0 y-0))))
    ((recv h) (send (exp (gen) y-1)))
    ((recv (cat (exp (gen) y-0) (mul w-0 y-1)))
      (send (cat (exp (gen) y-0) (mul w-0 y-1))))
    ((recv h-0) (send (exp (gen) y-0))))
  (label 515)
  (parent 389)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x y-0 expn) (w-0 expr)
    (y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul w-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-1 (mul x (rec y-0) w-0)) (y-1 h-0) (y-0 h)
    (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-absence y-1 (mul x (rec y-0) w-0))
    (mul x (rec y-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul w-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul w-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 516)
  (parent 389)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 517)
  (parent 442)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (gen) w-0))
  (deflistener (cat (gen) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x w-0) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0)))
    ((recv (cat (gen) (mul w-0 (rec x))))
      (send (cat (gen) (mul w-0 (rec x))))) ((recv x) (send x)))
  (label 518)
  (parent 443)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 519)
  (parent 445)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (gen) w-0))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x w-0) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x) (rec x))
    (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w-0 (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 520)
  (parent 446)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (x-0 (mul w-0 x)) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (gen) (mul w-0 x))) (send (cat (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x))) ((recv x-0) (send x-0)))
  (label 521)
  (parent 449)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 0) (5 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (x-0 (mul x w-0)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x-0))) ((recv x-0) (send x-0)))
  (label 522)
  (parent 450)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w w-0 expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x (mul w-0 y)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y (rec x))))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (gen) (mul w-0 y))) (send (cat (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) y) (mul w-0 (rec x))))
      (send (cat (exp (gen) y) (mul w-0 (rec x))))) ((recv x) (send x)))
  (label 523)
  (parent 452)
  (unrealized (5 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec y))))
  (deflistener y)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (mul x w-0)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w-0 (rec y)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul w-0 (rec y))))
      (send (cat (exp (gen) y) (mul w-0 (rec y))))) ((recv y) (send y)))
  (label 524)
  (parent 453)
  (unrealized (5 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)) ((7 1) (6 0)) ((8 1) (6 0)))
  (absent (x (mul w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (gen) (mul w-0 y-0))) (send (cat (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x)))))
    ((recv h) (send (exp (gen) y-0))) ((recv x) (send x)))
  (label 525)
  (parent 455)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)) ((7 1) (8 0)) ((8 1) (6 0)))
  (absent (y-0 (mul x w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 (rec y-0)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))) ((recv y-0) (send y-0)))
  (label 526)
  (parent 456)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (precedes ((0 2) (1 2)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y))))))
  (label 527)
  (parent 457)
  (unrealized)
  (shape)
  (maps
    ((0 1) ((n n) (hx (exp (gen) w)) (hy (exp (gen) w-0)) (x x) (y y))))
  (origs (n (0 2))))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (gen) (mul w-0 (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 (mul (rec x) w-0)) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 (rec x-0))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0))))
    ((send (exp (gen) x)))
    ((recv (cat (gen) (mul w-0 (rec x-0))))
      (send (cat (gen) (mul w-0 (rec x-0))))) ((recv x-0) (send x-0)))
  (label 528)
  (parent 459)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w-0 x-0)))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (8 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 (mul x w-0)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 x-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0)))
      (send (cat (exp (gen) x-0) (mul x w-0)))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w-0 x-0))) (send (cat (gen) (mul w-0 x-0))))
    ((recv x-0) (send x-0)))
  (label 529)
  (parent 460)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 x x)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x) (mul w-0 x-0)))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (8 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 (mul w-0 x x)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 x-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x x-0)))
      (send (enc n (exp (gen) (mul w-0 x x x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x x-0)))
      (send (exp (gen) (mul w-0 x x x-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 x x)))
      (send (cat (exp (gen) x-0) (mul w-0 x x))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x) (mul w-0 x-0)))
      (send (cat (exp (gen) x) (mul w-0 x-0)))) ((recv x-0) (send x-0)))
  (label 530)
  (parent 462)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0) (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 (mul (rec x) w-0)) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0)
    (mul w-0 (rec x-0) (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 (rec x-0))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0) (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 531)
  (parent 463)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (rec x-0)))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (rec x-0) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (rec x-0)))
      (send (cat (exp (gen) x) (rec x-0)))) ((recv x-0) (send x-0)))
  (label 532)
  (parent 464)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) w-0))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 w-0) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0)))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 533)
  (parent 465)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 (rec x-1))))
    (x x-1))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-1))))
  (defstrand init 1 (x x-0))
  (deflistener x-1)
  (precedes ((0 0) (5 0)) ((0 0) (9 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (x-1 (mul (rec x) w-0 x-0)) (y (exp (gen) w)))
  (pen-non-orig y x-1)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-listener x-1) (mul w-0 (rec x-1)) (7 0))
  (traces
    ((send (exp (gen) x-1)) (recv (exp (gen) (mul w-0 x-0 (rec x-1))))
      (send (enc n (exp (gen) (mul w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x-0))) (send (exp (gen) (mul w-0 x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-1))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-1)))))
    ((send (exp (gen) x-0))) ((recv x-1) (send x-1)))
  (label 534)
  (parent 468)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 x-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0 x-1)))
  (deflistener (cat (exp (gen) x-1) (mul x w-0 x-0)))
  (defstrand init 1 (x x-1))
  (deflistener (cat (exp (gen) x-0) (mul w-0 x-1)))
  (defstrand init 1 (x x-0))
  (deflistener x-1)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (9 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (x-1 (mul x w-0 x-0)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-listener x-1) (mul w-0 x-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0 x-1)))
      (send (enc n (exp (gen) (mul x w-0 x-0 x-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0 x-1)))
      (send (exp (gen) (mul x w-0 x-0 x-1))))
    ((recv (cat (exp (gen) x-1) (mul x w-0 x-0)))
      (send (cat (exp (gen) x-1) (mul x w-0 x-0))))
    ((send (exp (gen) x-1)))
    ((recv (cat (exp (gen) x-0) (mul w-0 x-1)))
      (send (cat (exp (gen) x-0) (mul w-0 x-1))))
    ((send (exp (gen) x-0))) ((recv x-1) (send x-1)))
  (label 535)
  (parent 469)
  (unrealized (4 0) (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn) (w-0 expr) (x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (deflistener x-1)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (9 0)) ((9 1) (7 0)))
  (absent (x-1 (mul x (rec x-0) w-0)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-listener x-1) (mul w-0 (rec x-1)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul w-0 (rec x-1)))))
    ((send (exp (gen) x-1))) ((recv x-1) (send x-1)))
  (label 536)
  (parent 470)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 y)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 (mul (rec x) w-0 y)) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 y (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 y)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 y))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) y) (mul w-0 (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 537)
  (parent 472)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0 y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w-0 x-0)))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (8 0))
    ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 (mul x w-0 y)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 x-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y x-0)))
      (send (enc n (exp (gen) (mul x w-0 y x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y x-0)))
      (send (exp (gen) (mul x w-0 y x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0 y)))
      (send (cat (exp (gen) x-0) (mul x w-0 y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w-0 x-0)))
      (send (cat (exp (gen) y) (mul w-0 x-0)))) ((recv x-0) (send x-0)))
  (label 538)
  (parent 473)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x x-0 expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec y))))
  (deflistener y)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (8 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y (mul x (rec x-0) w-0)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w-0 (rec y)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w-0 (rec y))))
      (send (cat (exp (gen) y) (mul w-0 (rec y))))) ((recv y) (send y)))
  (label 539)
  (parent 474)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x-0))))
    (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 y-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x-0))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (9 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (absent (x-0 (mul (rec x) w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 y-0 (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 y-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 y-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x-0)))))
    ((recv h) (send (exp (gen) y-0))) ((recv x-0) (send x-0)))
  (label 540)
  (parent 476)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0 y-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul w-0 x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (9 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (absent (x-0 (mul x w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 x-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0 x-0)))
      (send (exp (gen) (mul x w-0 y-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0 y-0)))
      (send (cat (exp (gen) x-0) (mul x w-0 y-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul w-0 x-0)))
      (send (cat (exp (gen) y-0) (mul w-0 x-0))))
    ((recv h) (send (exp (gen) y-0))) ((recv x-0) (send x-0)))
  (label 541)
  (parent 477)
  (unrealized (4 0) (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x x-0 expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (9 0)) ((9 1) (7 0)))
  (absent (y-0 (mul x (rec x-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 (rec y-0)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))) ((recv y-0) (send y-0)))
  (label 542)
  (parent 478)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0)))
  (deflistener (cat (gen) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x (mul (rec y) w-0)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0))))
    ((recv (cat (gen) (mul w-0 (rec x))))
      (send (cat (gen) (mul w-0 (rec x))))) ((recv x) (send x)))
  (label 543)
  (parent 480)
  (unrealized (5 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w-0)))
  (deflistener (cat (gen) (mul w-0 y)))
  (deflistener y)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (mul x w-0)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w-0 y) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y)))
      (send (enc n (exp (gen) (mul x w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y))) (send (exp (gen) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) (mul x w-0)))
      (send (cat (exp (gen) y) (mul x w-0))))
    ((recv (cat (gen) (mul w-0 y))) (send (cat (gen) (mul w-0 y))))
    ((recv y) (send y)))
  (label 544)
  (parent 481)
  (unrealized (4 0) (5 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w w-0 expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x y)))
  (deflistener (cat (exp (gen) y) (mul w-0 x x)))
  (deflistener (cat (exp (gen) x) (mul w-0 y)))
  (deflistener y)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (mul w-0 x x)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w-0 y) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x y)))
      (send (enc n (exp (gen) (mul w-0 x x y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x y)))
      (send (exp (gen) (mul w-0 x x y))))
    ((recv (cat (exp (gen) y) (mul w-0 x x)))
      (send (cat (exp (gen) y) (mul w-0 x x))))
    ((recv (cat (exp (gen) x) (mul w-0 y)))
      (send (cat (exp (gen) x) (mul w-0 y)))) ((recv y) (send y)))
  (label 545)
  (parent 483)
  (unrealized (4 0) (5 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0)))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x (mul (rec y) w-0)) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x) (rec x))
    (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0))))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w-0 (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 546)
  (parent 484)
  (unrealized (5 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 x)))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (x-0 (mul (rec y) w-0 x)) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 x)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 x))))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x))) ((recv x-0) (send x-0)))
  (label 547)
  (parent 486)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 y)))
  (defstrand init 1 (x x-0))
  (deflistener y)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (8 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (y (mul x w-0 x-0)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w-0 y) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0 y)))
      (send (enc n (exp (gen) (mul x w-0 x-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0 y)))
      (send (exp (gen) (mul x w-0 x-0 y))))
    ((recv (cat (exp (gen) y) (mul x w-0 x-0)))
      (send (cat (exp (gen) y) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 y)))
      (send (cat (exp (gen) x-0) (mul w-0 y)))) ((send (exp (gen) x-0)))
    ((recv y) (send y)))
  (label 548)
  (parent 487)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (x-0 (mul x (rec y) w-0)) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x-0))) ((recv x-0) (send x-0)))
  (label 549)
  (parent 488)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 550)
  (parent 489)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w w-0 expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) y) w-0))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x w-0) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y (rec x))))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0)))
    ((recv (cat (exp (gen) y) (mul w-0 (rec x))))
      (send (cat (exp (gen) y) (mul w-0 (rec x))))) ((recv x) (send x)))
  (label 551)
  (parent 490)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (absent (x (mul (rec y) w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x)))))
    ((recv h) (send (exp (gen) y-0))) ((recv x) (send x)))
  (label 552)
  (parent 493)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x expn) (w-0 expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul x w-0 y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) (mul w-0 y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (8 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (absent (y-0 (mul x w-0 y)) (y h) (y-0 (exp (gen) w)))
  (pen-non-orig x y-0)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y y-0)))
      (send (enc n (exp (gen) (mul x w-0 y y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul x w-0 y y-0)))
      (send (exp (gen) (mul x w-0 y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0 y)))
      (send (cat (exp (gen) y-0) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) (mul w-0 y-0)))
      (send (cat (exp (gen) y) (mul w-0 y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 553)
  (parent 494)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (absent (y-0 (mul x (rec y) w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 (rec y-0)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))) ((recv y-0) (send y-0)))
  (label 554)
  (parent 495)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x (mul (rec y-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul w-0 (rec x))))
      (send (cat (gen) (mul w-0 (rec x))))) ((recv x) (send x)))
  (label 555)
  (parent 497)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul w-0 y-0)))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y-0 (mul x w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 y-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0)))
      (send (cat (exp (gen) y-0) (mul x w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul w-0 y-0))) (send (cat (gen) (mul w-0 y-0))))
    ((recv y-0) (send y-0)))
  (label 556)
  (parent 498)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 x x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w-0 y-0)))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y-0 (mul w-0 x x)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 y-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x y-0)))
      (send (enc n (exp (gen) (mul w-0 x x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x y-0)))
      (send (exp (gen) (mul w-0 x x y-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 x x)))
      (send (cat (exp (gen) y-0) (mul w-0 x x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w-0 y-0)))
      (send (cat (exp (gen) x) (mul w-0 y-0)))) ((recv y-0) (send y-0)))
  (label 557)
  (parent 500)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x (mul (rec y-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x) (rec x))
    (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w-0 (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 558)
  (parent 501)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0 x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (9 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (x-0 (mul (rec y-0) w-0 x)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0 x)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0 x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x))) ((recv x-0) (send x-0)))
  (label 559)
  (parent 503)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0 x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul w-0 y-0)))
  (defstrand init 1 (x x-0))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (y-0 (mul x w-0 x-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 y-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0 y-0)))
      (send (exp (gen) (mul x w-0 x-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0 x-0)))
      (send (cat (exp (gen) y-0) (mul x w-0 x-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul w-0 y-0)))
      (send (cat (exp (gen) x-0) (mul w-0 y-0))))
    ((send (exp (gen) x-0))) ((recv y-0) (send y-0)))
  (label 560)
  (parent 504)
  (unrealized (4 0) (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (9 0)) ((9 1) (7 0)))
  (absent (x-0 (mul x (rec y-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x-0))) ((recv x-0) (send x-0)))
  (label 561)
  (parent 505)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x (mul (rec y) w-0 y-0)) (y-0 (exp (gen) w)) (y h))
  (pen-non-orig y-0 x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x)))))
    ((recv x) (send x)))
  (label 562)
  (parent 507)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x expn) (w-0 expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0 y)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w-0 y-0)))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (8 0))
    ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y-0 (mul x w-0 y)) (y (exp (gen) w)) (y-0 h))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 y-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y y-0)))
      (send (enc n (exp (gen) (mul x w-0 y y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y y-0)))
      (send (exp (gen) (mul x w-0 y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0 y)))
      (send (cat (exp (gen) y-0) (mul x w-0 y))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w-0 y-0)))
      (send (cat (exp (gen) y) (mul w-0 y-0)))) ((recv y-0) (send y-0)))
  (label 563)
  (parent 508)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (8 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y-0 (mul x (rec y) w-0)) (y-0 (exp (gen) w)) (y h))
  (pen-non-orig x y-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 (rec y-0)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv y-0) (send y-0)))
  (label 564)
  (parent 509)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (rec x)))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y-0 (rec x))))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (rec x)))
      (send (cat (exp (gen) y-0) (rec x)))) ((recv x) (send x)))
  (label 565)
  (parent 510)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x w-0) (y-0 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x)))))
    ((recv x) (send x)))
  (label 566)
  (parent 511)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y y-0 expn) (w-0 expr)
    (y-1 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-1 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-1)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0 y-1)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul w-0 (rec x))))
  (defstrand resp 2 (h h-0) (y y-1))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (9 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (absent (x (mul (rec y-0) w-0 y-1)) (y-1 h-0) (y-0 h)
    (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 y-1 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-1 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-1))) (send (exp (gen) (mul w-0 y-1))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0 y-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0 y-1))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-1) (mul w-0 (rec x)))))
    ((recv h-0) (send (exp (gen) y-1))) ((recv x) (send x)))
  (label 567)
  (parent 514)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x expn) (w-0 expr)
    (y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 y-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0 y-1)))
  (deflistener (cat (exp (gen) y-1) (mul x w-0 y-0)))
  (defstrand resp 2 (h h) (y y-1))
  (deflistener (cat (exp (gen) y-0) (mul w-0 y-1)))
  (defstrand resp 2 (h h-0) (y y-0))
  (deflistener y-1)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (absent (y-1 (mul x w-0 y-0)) (y-0 h-0) (y-1 h) (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-listener y-1) (mul w-0 y-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 y-1)))
      (send (enc n (exp (gen) (mul x w-0 y-0 y-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0 y-1)))
      (send (exp (gen) (mul x w-0 y-0 y-1))))
    ((recv (cat (exp (gen) y-1) (mul x w-0 y-0)))
      (send (cat (exp (gen) y-1) (mul x w-0 y-0))))
    ((recv h) (send (exp (gen) y-1)))
    ((recv (cat (exp (gen) y-0) (mul w-0 y-1)))
      (send (cat (exp (gen) y-0) (mul w-0 y-1))))
    ((recv h-0) (send (exp (gen) y-0))) ((recv y-1) (send y-1)))
  (label 568)
  (parent 515)
  (unrealized (4 0) (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x y-0 expn) (w-0 expr)
    (y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul w-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (deflistener y-1)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (9 0)) ((9 1) (7 0)))
  (absent (y-1 (mul x (rec y-0) w-0)) (y-1 h-0) (y-0 h)
    (y (exp (gen) w)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-listener y-1) (mul w-0 (rec y-1)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul w-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul w-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))) ((recv y-1) (send y-1)))
  (label 569)
  (parent 516)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(comment "Nothing left to do")
