(herald "Diffie-Hellman protocol, man-in-the-middle attack"
  (algebra diffie-hellman))

(comment "CPSA 3.4.0")
(comment "All input read from dh_mim2.scm")

(defprotocol dh_mim diffie-hellman
  (defrole init
    (vars (x expn) (h base) (n text))
    (trace (send (exp (gen) x)) (recv h) (send (enc n (exp h x))))
    (uniq-orig n)
    (uniq-gen x)
    (neq (h (gen))))
  (defrole resp
    (vars (y expn) (h base) (n text))
    (trace (recv h) (send (exp (gen) y)) (recv (enc n (exp h y))))
    (uniq-gen y)
    (neq (h (gen)))
    (absent (y h)))
  (comment "Diffie-hellman key exchange followed by an encryption"))

(defskeleton dh_mim
  (vars (n text) (hx hy base) (x y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h hx) (y y))
  (precedes ((0 2) (1 2)))
  (absent (y hx))
  (neq (hx (gen)) (hy (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (comment "Agreement on the encrypted text only")
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv hx) (send (exp (gen) y)) (recv (enc n (exp hx y)))))
  (label 0)
  (unrealized (1 2))
  (origs (n (0 2)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x y expn))
  (defstrand init 3 (n n) (h (exp hy y)) (x x))
  (defstrand resp 3 (n n) (h (exp hy x)) (y y))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)))
  (absent (y (exp hy x)))
  (neq ((exp hy y) (gen)) ((exp hy x) (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (operation encryption-test (displaced 2 0 init 3) (enc n (exp hx y-0))
    (1 2))
  (traces
    ((send (exp (gen) x)) (recv (exp hy y))
      (send (enc n (exp hy (mul x y)))))
    ((recv (exp hy x)) (send (exp (gen) y))
      (recv (enc n (exp hy (mul x y))))))
  (label 1)
  (parent 0)
  (unrealized (0 1) (1 0))
  (origs (n (0 2)))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hx hy base) (x y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h hx) (y y))
  (deflistener (exp hx y))
  (precedes ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2)))
  (absent (y hx))
  (neq (hx (gen)) (hy (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (operation encryption-test (added-listener (exp hx y))
    (enc n (exp hx y)) (1 2))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv hx) (send (exp (gen) y)) (recv (enc n (exp hx y))))
    ((recv (exp hx y)) (send (exp hx y))))
  (label 2)
  (parent 0)
  (unrealized (1 2) (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x)) (y y))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)))
  (absent (y (exp (gen) x)))
  (neq ((exp (gen) y) (gen)) ((exp (gen) x) (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 2 0 init 1) (exp (gen) x-0) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul y x)))))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x))))))
  (label 3)
  (parent 1)
  (unrealized)
  (shape)
  (maps
    ((0 1) ((n n) (hx (exp (gen) x)) (hy (exp (gen) y)) (x x) (y y))))
  (origs (n (0 2))))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 0) (1 0)))
  (absent (y (exp (gen) x-0)))
  (neq ((exp (gen) (mul (rec x) y x-0)) (gen)) ((exp (gen) x-0) (gen)))
  (pen-non-orig x y)
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0))))) ((send (exp (gen) x-0))))
  (label 4)
  (parent 1)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (neq (h (gen)) ((exp (gen) (mul (rec x) y y-0)) (gen))
    ((exp (gen) y-0) (gen)))
  (pen-non-orig x y)
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 5)
  (parent 1)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x y expn) (w expr))
  (defstrand init 3 (n n) (h (exp hy y)) (x x))
  (defstrand resp 3 (n n) (h (exp hy x)) (y y))
  (deflistener (cat (exp hy (mul x (rec w))) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0)))
  (absent (y (exp hy x)))
  (neq ((exp hy y) (gen)) ((exp hy x) (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test
    (added-listener (cat (exp hy (mul x (rec w))) w)) (exp hy x) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp hy y))
      (send (enc n (exp hy (mul x y)))))
    ((recv (exp hy x)) (send (exp (gen) y))
      (recv (enc n (exp hy (mul x y)))))
    ((recv (cat (exp hy (mul x (rec w))) w))
      (send (cat (exp hy (mul x (rec w))) w))))
  (label 6)
  (parent 1)
  (unrealized (0 1) (2 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hx hy base) (x y expn) (w expr))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h hx) (y y))
  (deflistener (exp hx y))
  (deflistener (cat (exp hx (mul y (rec w))) w))
  (precedes ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2)) ((3 1) (2 0)))
  (absent (y hx))
  (neq (hx (gen)) (hy (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp hx (mul y (rec w))) w)) (exp hx y) (2 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv hx) (send (exp (gen) y)) (recv (enc n (exp hx y))))
    ((recv (exp hx y)) (send (exp hx y)))
    ((recv (cat (exp hx (mul y (rec w))) w))
      (send (cat (exp hx (mul y (rec w))) w))))
  (label 7)
  (parent 2)
  (unrealized (1 2) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y x-0 (rec w))) w))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (neq ((exp (gen) (mul (rec x) y x-0)) (gen)) ((exp (gen) x-0) (gen)))
  (pen-non-orig x y)
  (uniq-gen x y x-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y x-0 (rec w))) w))
    (exp (gen) (mul (rec x) y x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0))))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) y x-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y x-0 (rec w))) w))))
  (label 8)
  (parent 4)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y y-0 (rec w))) w))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (neq (h (gen)) ((exp (gen) (mul (rec x) y y-0)) (gen))
    ((exp (gen) y-0) (gen)))
  (pen-non-orig x y)
  (uniq-gen x y y-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y y-0 (rec w))) w))
    (exp (gen) (mul (rec x) y y-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) (mul (rec x) y y-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y y-0 (rec w))) w))))
  (label 9)
  (parent 5)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y w))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (gen) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) (mul (rec x) y w)) (gen)) ((exp (gen) w) (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test (contracted (hy (exp (gen) (mul (rec x) w))))
    (gen) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w)))
      (send (enc n (exp (gen) (mul y w)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w)))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 10)
  (parent 6)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y w))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x))) (y y))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0)))
  (absent (y (exp (gen) (mul w x))))
  (neq ((exp (gen) (mul y w)) (gen)) ((exp (gen) (mul w x)) (gen)))
  (pen-non-orig y x)
  (precur (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 3 0 init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y w)))
      (send (enc n (exp (gen) (mul y w x)))))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w x)))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 11)
  (parent 6)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y w x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0))
    ((3 0) (2 0)))
  (absent (y (exp (gen) (mul w x-0))))
  (neq ((exp (gen) (mul (rec x) y w x-0)) (gen))
    ((exp (gen) (mul w x-0)) (gen)))
  (pen-non-orig x y)
  (precur (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w x-0)))
      (send (enc n (exp (gen) (mul y w x-0)))))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w x-0)))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0))))
  (label 12)
  (parent 6)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y w y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w y-0))) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0))
    ((3 1) (2 0)))
  (absent (y-0 h) (y (exp (gen) (mul w y-0))))
  (neq (h (gen)) ((exp (gen) (mul (rec x) y w y-0)) (gen))
    ((exp (gen) (mul w y-0)) (gen)))
  (pen-non-orig x y)
  (precur (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w y-0)))
      (send (enc n (exp (gen) (mul y w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w y-0)))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0))))
  (label 13)
  (parent 6)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (gen) (mul w y)))
  (precedes ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2)) ((3 1) (2 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) (hy (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (contracted (hx (exp (gen) w)) (y-0 y) (w (mul w y))) (gen) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y)))))
  (label 14)
  (parent 7)
  (unrealized (1 2) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (w expr) (x y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x))) (y y))
  (deflistener (exp (gen) (mul w x y)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)))
  (absent (y (exp (gen) (mul w x))))
  (neq ((exp (gen) (mul w x)) (gen)) (hy (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x y)))))
    ((recv (exp (gen) (mul w x y))) (send (exp (gen) (mul w x y))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))))
  (label 15)
  (parent 7)
  (unrealized (1 2) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x-0))) (y y))
  (deflistener (exp (gen) (mul w x-0 y)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 0) (1 0)))
  (absent (y (exp (gen) (mul w x-0))))
  (neq ((exp (gen) (mul w x-0)) (gen)) (hy (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x-0 y)))))
    ((recv (exp (gen) (mul w x-0 y))) (send (exp (gen) (mul w x-0 y))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0))))
  (label 16)
  (parent 7)
  (unrealized (1 2) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2)) ((3 1) (2 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) (hy (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 17)
  (parent 7)
  (unrealized (1 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w y))) (y y-0))
  (deflistener (exp (gen) (mul w y y-0)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 0)))
  (absent (y h) (y-0 (exp (gen) (mul w y))))
  (neq (h (gen)) ((exp (gen) (mul w y)) (gen)) (hy (gen)))
  (pen-non-orig x y-0)
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y y-0)))))
    ((recv (exp (gen) (mul w y y-0))) (send (exp (gen) (mul w y y-0))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 18)
  (parent 7)
  (unrealized (1 2) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul (rec x) y x-0)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (neq ((exp (gen) (mul (rec x) y x-0)) (gen)) ((exp (gen) x-0) (gen)))
  (pen-non-orig x y)
  (uniq-gen x y x-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (contracted (x-1 x) (y-0 y) (x-2 x-0) (w (mul (rec x) y x-0))) (gen)
    (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0))))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul (rec x) y x-0)))
      (send (cat (gen) (mul (rec x) y x-0)))))
  (label 19)
  (parent 8)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) x)) (y y))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul y x (rec x-0) (rec x-0))))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) x)))
  (neq ((exp (gen) (mul y x (rec x-0))) (gen)) ((exp (gen) x) (gen)))
  (pen-non-orig y x-0)
  (precur (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-1) (3 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul y x (rec x-0))))
      (send (enc n (exp (gen) (mul y x)))))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x))))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul y x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul y x (rec x-0) (rec x-0))))))
  (label 20)
  (parent 8)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (neq ((exp (gen) (mul (rec x) y x-0)) (gen)) ((exp (gen) x-0) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 init 1) (exp (gen) x-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0))))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y)))
      (send (cat (exp (gen) x-0) (mul (rec x) y)))))
  (label 21)
  (parent 8)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul (rec x) y x-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)) ((4 0) (3 0)))
  (absent (y (exp (gen) x-0)))
  (neq ((exp (gen) (mul (rec x) y x-0)) (gen)) ((exp (gen) x-0) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0))))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul (rec x) y x-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul (rec x) y x-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 22)
  (parent 8)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (neq ((exp (gen) x-0) (gen)) ((exp (gen) (mul (rec x) x-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0 y)))
      (send (enc n (exp (gen) (mul x-0 y)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x-0 y))))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 23)
  (parent 8)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y x-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 0) (1 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y-0 h) (y (exp (gen) x-0)))
  (neq (h (gen)) ((exp (gen) (mul (rec x) y x-0)) (gen))
    ((exp (gen) x-0) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0))))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y x-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y x-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 24)
  (parent 8)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul (rec x) y y-0)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (neq (h (gen)) ((exp (gen) (mul (rec x) y y-0)) (gen))
    ((exp (gen) y-0) (gen)))
  (pen-non-orig x y)
  (uniq-gen x y y-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (contracted (x-0 x) (y-1 y) (y-2 y-0) (w (mul (rec x) y y-0))) (gen)
    (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul (rec x) y y-0)))
      (send (cat (gen) (mul (rec x) y y-0)))))
  (label 25)
  (parent 9)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul y y-0 (rec x) (rec x))))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (neq (h (gen)) ((exp (gen) (mul y y-0 (rec x))) (gen))
    ((exp (gen) y-0) (gen)))
  (pen-non-orig y x)
  (precur (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y y-0 (rec x))))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul y y-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y y-0 (rec x) (rec x))))))
  (label 26)
  (parent 9)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y y-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 0) (3 0)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (neq (h (gen)) ((exp (gen) (mul (rec x) y y-0)) (gen))
    ((exp (gen) y-0) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y y-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 27)
  (parent 9)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y)) (y y-0))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y-0 (exp (gen) y)) (y h))
  (neq ((exp (gen) y) (gen)) (h (gen))
    ((exp (gen) (mul (rec x) y y-0)) (gen)))
  (pen-non-orig x y-0)
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y)))
      (send (cat (exp (gen) y-0) (mul (rec x) y)))))
  (label 28)
  (parent 9)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (neq (h (gen)) ((exp (gen) (mul (rec x) y y-0)) (gen))
    ((exp (gen) y-0) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 resp 2) (exp (gen) y-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y)))
      (send (cat (exp (gen) y-0) (mul (rec x) y)))))
  (label 29)
  (parent 9)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (x y y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y y-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y-1 h-0) (y-0 h) (y (exp (gen) y-0)))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) (mul (rec x) y y-0)) (gen))
    ((exp (gen) y-0) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y y-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 30)
  (parent 9)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x)) (y y))
  (deflistener (cat (gen) x))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0)))
  (absent (y (exp (gen) x)))
  (neq ((exp (gen) x) (gen)) ((exp (gen) y) (gen)))
  (pen-non-orig x y)
  (precur (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 3 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x y)))))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 31)
  (parent 10)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y w))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) (mul (rec x) y w)) (gen)) ((exp (gen) w) (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w)))
      (send (enc n (exp (gen) (mul y w)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w)))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))))
  (label 32)
  (parent 10)
  (unrealized (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y w))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x))) (y y))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)))
  (absent (y (exp (gen) (mul w x))))
  (neq ((exp (gen) (mul y w)) (gen)) ((exp (gen) (mul w x)) (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y w)))
      (send (enc n (exp (gen) (mul y w x)))))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w x))))))
  (label 33)
  (parent 11)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((n n) (hx (exp (gen) (mul w x))) (hy (exp (gen) (mul y w))) (x x)
        (y y))))
  (origs (n (0 2))))

(defskeleton dh_mim
  (vars (n text) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x)) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0))
    ((3 0) (2 0)))
  (absent (y (exp (gen) x)))
  (neq ((exp (gen) x) (gen)) ((exp (gen) y) (gen)))
  (pen-non-orig x y)
  (precur (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x y)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 34)
  (parent 12)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x y expn) (w expr) (x-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y w x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul w x-0))))
  (neq ((exp (gen) (mul (rec x) y w x-0)) (gen))
    ((exp (gen) (mul w x-0)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w x-0)))
      (send (enc n (exp (gen) (mul y w x-0)))))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w x-0)))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 35)
  (parent 12)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x)) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((2 1) (1 0))
    ((3 1) (2 0)))
  (absent (y-0 (exp (gen) x)) (y h))
  (neq ((exp (gen) x) (gen)) (h (gen)) ((exp (gen) y-0) (gen)))
  (pen-non-orig x y-0)
  (precur (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (exp (gen) x)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x y-0)))))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((recv h) (send (exp (gen) y))))
  (label 36)
  (parent 13)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y w y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w y-0))) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) (mul w y-0))))
  (neq (h (gen)) ((exp (gen) (mul (rec x) y w y-0)) (gen))
    ((exp (gen) (mul w y-0)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w y-0)))
      (send (enc n (exp (gen) (mul y w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w y-0)))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 37)
  (parent 13)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (gen) (mul w y)))
  (deflistener y)
  (precedes ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) (hy (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv y) (send y)))
  (label 38)
  (parent 14)
  (unrealized (1 2) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (hy base) (w expr) (x y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x))) (y y))
  (deflistener (exp (gen) (mul w x y)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (3 0)))
  (absent (y (exp (gen) (mul w x))))
  (neq ((exp (gen) (mul w x)) (gen)) (hy (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x y)))))
    ((recv (exp (gen) (mul w x y))) (send (exp (gen) (mul w x y))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))) ((recv y) (send y)))
  (label 39)
  (parent 15)
  (unrealized (1 2) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x-0))) (y y))
  (deflistener (exp (gen) (mul w x-0 y)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (deflistener y)
  (precedes ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 0) (1 0)) ((5 1) (3 0)))
  (absent (y (exp (gen) (mul w x-0))))
  (neq ((exp (gen) (mul w x-0)) (gen)) (hy (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x-0 y)))))
    ((recv (exp (gen) (mul w x-0 y))) (send (exp (gen) (mul w x-0 y))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0)))
    ((recv y) (send y)))
  (label 40)
  (parent 16)
  (unrealized (1 2) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) w y)) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (hy (exp (gen) (mul (rec x) w y))))
    n (1 2) (enc n (exp (gen) (mul w y))))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w y)))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 41)
  (parent 17)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp hy x))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) (hy (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener (exp hy x)) n (1 2)
    (enc n (exp hy x)))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp hy x)) (send (exp hy x))))
  (label 42)
  (parent 17)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w y))) (y y-0))
  (deflistener (exp (gen) (mul w y y-0)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 0)) ((5 1) (3 0)))
  (absent (y h) (y-0 (exp (gen) (mul w y))))
  (neq (h (gen)) ((exp (gen) (mul w y)) (gen)) (hy (gen)))
  (pen-non-orig x y-0)
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (3 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y y-0)))))
    ((recv (exp (gen) (mul w y y-0))) (send (exp (gen) (mul w y y-0))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 43)
  (parent 18)
  (unrealized (1 2) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (neq ((exp (gen) (mul w y)) (gen)) ((exp (gen) (mul x w)) (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w-0 (mul x w)) (w-1 (mul w y)) (y-1 y)
      (w (mul w y))) (gen) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y)))))
  (label 44)
  (parent 32)
  (unrealized (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x x))) (y y))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) (mul w x x))))
  (neq ((exp (gen) (mul w x y)) (gen)) ((exp (gen) (mul w x x)) (gen)))
  (pen-non-orig x y)
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x x y)))))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))))
  (label 45)
  (parent 32)
  (unrealized (0 1) (1 0) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 0) (2 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (neq ((exp (gen) (mul w x-0 y)) (gen))
    ((exp (gen) (mul x w x-0)) (gen)))
  (pen-non-orig x y)
  (precur (3 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0))))
  (label 46)
  (parent 32)
  (unrealized (0 1) (1 0) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (neq ((exp (gen) (mul x w)) (gen)) ((exp (gen) (mul w y)) (gen)))
  (pen-non-orig x y)
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 47)
  (parent 32)
  (unrealized (2 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (neq (h (gen)) ((exp (gen) (mul w y y-0)) (gen))
    ((exp (gen) (mul x w y)) (gen)))
  (pen-non-orig x y-0)
  (precur (3 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 48)
  (parent 32)
  (unrealized (0 1) (1 0) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (neq ((exp (gen) (mul w y)) (gen)) ((exp (gen) (mul x w)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (y-0 y) (w-0 (mul x (rec x-0) w)) (x-2 x-0)
      (w-1 (mul w y)) (y-1 y) (w (mul w y))) (gen) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y)))))
  (label 49)
  (parent 35)
  (unrealized (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x-0 x-0))) (y y))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul w x-0 x-0))))
  (neq ((exp (gen) (mul w x-0 y)) (gen))
    ((exp (gen) (mul w x-0 x-0)) (gen)))
  (pen-non-orig x-0 y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-1) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul w x-0 x-0 y)))))
    ((recv (exp (gen) (mul w x-0 x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x-0 x-0 y)))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))))
  (label 50)
  (parent 35)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w x-0))))
  (neq ((exp (gen) (mul w x-0 y)) (gen))
    ((exp (gen) (mul x w x-0)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 init 1) (exp (gen) x-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))))
  (label 51)
  (parent 35)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (x-1 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-1 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-1))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w y)))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 0) (2 0)))
  (absent (y (exp (gen) (mul x w x-1))))
  (neq ((exp (gen) (mul w x-1 y)) (gen))
    ((exp (gen) (mul x w x-1)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1 y)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-1 y)))
      (send (enc n (exp (gen) (mul x w x-1 y)))))
    ((recv (exp (gen) (mul x w x-1))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-1 y)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w y)))
      (send (cat (exp (gen) x-1) (mul w y)))) ((send (exp (gen) x-1))))
  (label 52)
  (parent 35)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (neq ((exp (gen) (mul x w)) (gen)) ((exp (gen) (mul w y)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 53)
  (parent 35)
  (unrealized (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x x-0 expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (2 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (neq (h (gen)) ((exp (gen) (mul w y y-0)) (gen))
    ((exp (gen) (mul x w y)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 54)
  (parent 35)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) (mul w y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y h) (y-0 (exp (gen) (mul x w))))
  (neq (h (gen)) ((exp (gen) (mul w y-0)) (gen))
    ((exp (gen) (mul x w)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w-0 (mul x (rec y) w)) (y-2 y)
      (w-1 (mul w y-0)) (y-3 y-0) (w (mul w y-0))) (gen) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) (mul w y-0))) (send (cat (gen) (mul w y-0)))))
  (label 55)
  (parent 37)
  (unrealized (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x x))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x x)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul w y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y h) (y-0 (exp (gen) (mul w x x))))
  (neq (h (gen)) ((exp (gen) (mul w x y-0)) (gen))
    ((exp (gen) (mul w x x)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y-0)))
      (send (enc n (exp (gen) (mul w x x y-0)))))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w x x y-0)))))
    ((recv (cat (exp (gen) y) (mul (rec y) w x x)))
      (send (cat (exp (gen) y) (mul (rec y) w x x))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul w y-0)))
      (send (cat (exp (gen) x) (mul w y-0)))))
  (label 56)
  (parent 37)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) (mul w y-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 0) (2 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w x-0))))
  (neq (h (gen)) ((exp (gen) (mul w x-0 y-0)) (gen))
    ((exp (gen) (mul x w x-0)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y-0)))
      (send (enc n (exp (gen) (mul x w x-0 y-0)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w x-0 y-0)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul w y-0)))
      (send (cat (exp (gen) x-0) (mul w y-0))))
    ((send (exp (gen) x-0))))
  (label 57)
  (parent 37)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y-0 (exp (gen) (mul x w))) (y h))
  (neq ((exp (gen) (mul x w)) (gen)) (h (gen))
    ((exp (gen) (mul w y-0)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 58)
  (parent 37)
  (unrealized (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (neq (h (gen)) ((exp (gen) (mul w y y-0)) (gen))
    ((exp (gen) (mul x w y)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 2) (exp (gen) y-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))))
  (label 59)
  (parent 37)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (x y expn) (w expr) (y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 y-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y-0))) (y y-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w y-1)))
  (defstrand resp 2 (h h-0) (y y-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (2 0)))
  (absent (y-0 h-0) (y h) (y-1 (exp (gen) (mul x w y-0))))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) (mul w y-0 y-1)) (gen))
    ((exp (gen) (mul x w y-0)) (gen)))
  (pen-non-orig x y-1)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 y-1)))
      (send (enc n (exp (gen) (mul x w y-0 y-1)))))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) y-1))
      (recv (enc n (exp (gen) (mul x w y-0 y-1)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w y-1)))
      (send (cat (exp (gen) y-0) (mul w y-1))))
    ((recv h-0) (send (exp (gen) y-0))))
  (label 60)
  (parent 37)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x)) (y y))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)))
  (absent (y (exp (gen) x)))
  (neq ((exp (gen) x) (gen)) ((exp (gen) y) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 61)
  (parent 41)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) w y)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w y)))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))))
  (label 62)
  (parent 41)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (hy (exp (gen) (rec x)))) (gen)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen))))
  (label 63)
  (parent 42)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (0 1)) ((5 0) (4 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) x-0)) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0))))
  (label 64)
  (parent 42)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) y)) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y))))
  (label 65)
  (parent 42)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) y-0)) (gen)))
  (pen-non-orig x y)
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0))))
  (label 66)
  (parent 42)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (hy base) (x expn) (w expr) (y expn) (w-0 expr))
  (defstrand init 3 (n n) (h hy) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp hy x))
  (deflistener (cat (exp hy (mul x (rec w-0))) w-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) (hy (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp hy (mul x (rec w-0))) w-0)) (exp hy x)
    (4 0))
  (traces ((send (exp (gen) x)) (recv hy) (send (enc n (exp hy x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp hy x)) (send (exp hy x)))
    ((recv (cat (exp hy (mul x (rec w-0))) w-0))
      (send (cat (exp hy (mul x (rec w-0))) w-0))))
  (label 67)
  (parent 42)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y (exp (gen) (mul x w))))
  (neq ((exp (gen) (mul w y)) (gen)) ((exp (gen) (mul x w)) (gen)))
  (pen-non-orig x y)
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv y) (send y)))
  (label 68)
  (parent 44)
  (unrealized (2 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x x))) (y y))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y (exp (gen) (mul w x x))))
  (neq ((exp (gen) (mul w x y)) (gen)) ((exp (gen) (mul w x x)) (gen)))
  (pen-non-orig x y)
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x x y)))))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))) ((recv y) (send y)))
  (label 69)
  (parent 45)
  (unrealized (1 0) (2 0) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 0) (2 0)) ((5 1) (3 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (neq ((exp (gen) (mul w x-0 y)) (gen))
    ((exp (gen) (mul x w x-0)) (gen)))
  (pen-non-orig x y)
  (precur (3 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0)))
    ((recv y) (send y)))
  (label 70)
  (parent 46)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (absent (x w) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w y (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (3 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w) w (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))))
  (label 71)
  (parent 47)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)))
  (absent (y (exp (gen) (mul x w))))
  (neq ((exp (gen) (mul x w)) (gen)) ((exp (gen) (mul w y)) (gen)))
  (pen-non-orig x y)
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 72)
  (parent 47)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)) ((5 1) (3 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (neq (h (gen)) ((exp (gen) (mul w y y-0)) (gen))
    ((exp (gen) (mul x w y)) (gen)))
  (pen-non-orig x y-0)
  (precur (3 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 73)
  (parent 48)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul x w))))
  (neq ((exp (gen) (mul w y)) (gen)) ((exp (gen) (mul x w)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv y) (send y)))
  (label 74)
  (parent 49)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x-0 x-0))) (y y))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul w x-0 x-0))))
  (neq ((exp (gen) (mul w x-0 y)) (gen))
    ((exp (gen) (mul w x-0 x-0)) (gen)))
  (pen-non-orig x-0 y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul w x-0 x-0 y)))))
    ((recv (exp (gen) (mul w x-0 x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x-0 x-0 y)))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((recv y) (send y)))
  (label 75)
  (parent 50)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (neq ((exp (gen) (mul w x-0 y)) (gen))
    ((exp (gen) (mul x w x-0)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((recv y) (send y)))
  (label 76)
  (parent 51)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x x-0 expn) (w expr) (x-1 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-1 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-1))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w y)))
  (defstrand init 1 (x x-1))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (6 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 0) (2 0)) ((6 1) (4 0)))
  (absent (y (exp (gen) (mul x w x-1))))
  (neq ((exp (gen) (mul w x-1 y)) (gen))
    ((exp (gen) (mul x w x-1)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-1 y)))
      (send (enc n (exp (gen) (mul x w x-1 y)))))
    ((recv (exp (gen) (mul x w x-1))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-1 y)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w y)))
      (send (cat (exp (gen) x-1) (mul w y)))) ((send (exp (gen) x-1)))
    ((recv y) (send y)))
  (label 77)
  (parent 52)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) x-0)) (y y))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (neq ((exp (gen) x-0) (gen)) ((exp (gen) (mul y (rec x) x-0)) (gen)))
  (pen-non-orig y x)
  (precur (4 0) (2 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x) x-0)))
      (send (enc n (exp (gen) (mul y x-0)))))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0)))))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 78)
  (parent 53)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w (rec x-0))))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (x-0 (mul (rec x) w)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w y (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w))
    (mul (rec x) w) (2 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w y (rec x-0))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w (rec x-0))))
      (send (cat (exp (gen) y) (mul w (rec x-0))))))
  (label 79)
  (parent 53)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w x-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (x-0 (mul x w)) (y (exp (gen) (mul x w x-0))))
  (neq ((exp (gen) (mul x w x-0)) (gen))
    ((exp (gen) (mul w y x-0)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w)) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y x-0)))
      (send (enc n (exp (gen) (mul x w y x-0)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y x-0)))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w x-0)))
      (send (cat (exp (gen) y) (mul w x-0)))))
  (label 80)
  (parent 53)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x x-0 expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (6 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (2 0)) ((6 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (neq (h (gen)) ((exp (gen) (mul w y y-0)) (gen))
    ((exp (gen) (mul x w y)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 81)
  (parent 54)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w))))
  (neq (h (gen)) ((exp (gen) (mul w y-0)) (gen))
    ((exp (gen) (mul x w)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y-0)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) (mul w y-0))) (send (cat (gen) (mul w y-0))))
    ((recv y-0) (send y-0)))
  (label 82)
  (parent 55)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x x))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x x)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul w x x))))
  (neq (h (gen)) ((exp (gen) (mul w x y-0)) (gen))
    ((exp (gen) (mul w x x)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y-0)))
      (send (enc n (exp (gen) (mul w x x y-0)))))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w x x y-0)))))
    ((recv (cat (exp (gen) y) (mul (rec y) w x x)))
      (send (cat (exp (gen) y) (mul (rec y) w x x))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul w y-0)))
      (send (cat (exp (gen) x) (mul w y-0)))) ((recv y-0) (send y-0)))
  (label 83)
  (parent 56)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x y expn) (w expr) (x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) (mul w y-0)))
  (defstrand init 1 (x x-0))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (6 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 0) (2 0)) ((6 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w x-0))))
  (neq (h (gen)) ((exp (gen) (mul w x-0 y-0)) (gen))
    ((exp (gen) (mul x w x-0)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y-0)))
      (send (enc n (exp (gen) (mul x w x-0 y-0)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w x-0 y-0)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul w y-0)))
      (send (cat (exp (gen) x-0) (mul w y-0)))) ((send (exp (gen) x-0)))
    ((recv y-0) (send y-0)))
  (label 84)
  (parent 57)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) y-0)) (y y))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) y-0)) (y-0 h))
  (neq ((exp (gen) y-0) (gen)) (h (gen))
    ((exp (gen) (mul y (rec x) y-0)) (gen)))
  (pen-non-orig y x)
  (precur (4 0) (2 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x) y-0)))
      (send (enc n (exp (gen) (mul y y-0)))))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 85)
  (parent 58)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (x (mul (rec y) w)) (y-0 (exp (gen) w)) (y h))
  (neq ((exp (gen) w) (gen)) (h (gen))
    ((exp (gen) (mul w y-0 (rec x))) (gen)))
  (pen-non-orig y-0 x)
  (precur (4 0) (2 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w))
    (mul (rec y) w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x))))))
  (label 86)
  (parent 58)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y-0))) (y y))
  (deflistener (cat (exp (gen) y-0) (mul x w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y-0 (mul x w)) (y (exp (gen) (mul x w y-0))) (y-0 h))
  (neq ((exp (gen) (mul x w y-0)) (gen)) (h (gen))
    ((exp (gen) (mul w y y-0)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w)) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (exp (gen) y-0) (mul x w)))
      (send (cat (exp (gen) y-0) (mul x w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))))
  (label 87)
  (parent 58)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (neq (h (gen)) ((exp (gen) (mul w y y-0)) (gen))
    ((exp (gen) (mul x w y)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))) ((recv y-0) (send y-0)))
  (label 88)
  (parent 59)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (x y expn) (w expr) (y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 y-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y-0))) (y y-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w y-1)))
  (defstrand resp 2 (h h-0) (y y-0))
  (deflistener y-1)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (6 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (2 0)) ((6 1) (4 0)))
  (absent (y-0 h-0) (y h) (y-1 (exp (gen) (mul x w y-0))))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) (mul w y-0 y-1)) (gen))
    ((exp (gen) (mul x w y-0)) (gen)))
  (pen-non-orig x y-1)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-listener y-1) (mul w y-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 y-1)))
      (send (enc n (exp (gen) (mul x w y-0 y-1)))))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) y-1))
      (recv (enc n (exp (gen) (mul x w y-0 y-1)))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w y-1)))
      (send (cat (exp (gen) y-0) (mul w y-1))))
    ((recv h-0) (send (exp (gen) y-0))) ((recv y-1) (send y-1)))
  (label 89)
  (parent 60)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (deflistener (cat (gen) (mul w y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (neq ((exp (gen) (mul x w)) (gen)) ((exp (gen) (mul w y)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x w)) (y-0 y) (w-1 (mul w y)) (y-1 y)
      (w (mul w y))) (gen) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y)))))
  (label 90)
  (parent 62)
  (unrealized (2 0) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x x))) (y y))
  (deflistener (exp (gen) (mul w x x y)))
  (deflistener (cat (exp (gen) y) (mul w x x)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul w x x))))
  (neq ((exp (gen) (mul w x x)) (gen)) ((exp (gen) (mul w x y)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x x y)))))
    ((recv (exp (gen) (mul w x x y))) (send (exp (gen) (mul w x x y))))
    ((recv (cat (exp (gen) y) (mul w x x)))
      (send (cat (exp (gen) y) (mul w x x))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))))
  (label 91)
  (parent 62)
  (unrealized (0 1) (1 0) (2 0) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (exp (gen) (mul x w x-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 0) (1 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (neq ((exp (gen) (mul x w x-0)) (gen))
    ((exp (gen) (mul w x-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0 y)))
      (send (exp (gen) (mul x w x-0 y))))
    ((recv (cat (exp (gen) y) (mul x w x-0)))
      (send (cat (exp (gen) y) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0))))
  (label 92)
  (parent 62)
  (unrealized (0 1) (1 0) (2 0) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (neq ((exp (gen) (mul x w)) (gen)) ((exp (gen) (mul w y)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 93)
  (parent 62)
  (unrealized (2 0) (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (exp (gen) (mul x w y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (1 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (neq (h (gen)) ((exp (gen) (mul x w y)) (gen))
    ((exp (gen) (mul w y y-0)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y y-0)))
      (send (exp (gen) (mul x w y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w y)))
      (send (cat (exp (gen) y-0) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 94)
  (parent 62)
  (unrealized (0 1) (1 0) (2 0) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (exp (gen) (mul (rec x) (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) (rec w-0))) w-0))
    (exp (gen) (rec x)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) (mul (rec x) (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) (rec w-0))) w-0))))
  (label 95)
  (parent 63)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0 (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) x-0)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) x-0 (rec w-0))) w-0))))
  (label 96)
  (parent 64)
  (unrealized (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) (mul (rec x) y (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) y)) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) (mul (rec x) y (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y (rec w-0))) w-0))))
  (label 97)
  (parent 65)
  (unrealized (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y-0 (rec w-0))) w-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) y-0)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) (mul (rec x) y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y-0 (rec w-0))) w-0))))
  (label 98)
  (parent 66)
  (unrealized (6 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (gen) w-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) w-0)) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (hy (exp (gen) (mul (rec x) w-0))))
    (gen) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0)))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0))))
  (label 99)
  (parent 67)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 100)
  (parent 67)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) w-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) w-0 x-0)) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0 x-0)))
      (send (enc n (exp (gen) (mul w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x-0))) (send (exp (gen) (mul w-0 x-0))))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0)))
    ((send (exp (gen) x-0))))
  (label 101)
  (parent 67)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) y) w-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) w-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0 y)))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0))))
  (label 102)
  (parent 67)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y expn) (w-0 expr)
    (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) w-0 y-0)) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0 y-0)))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0))))
  (label 103)
  (parent 67)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (x w) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w y (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (3 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))) ((recv x) (send x)))
  (label 104)
  (parent 71)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (4 0))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (x-0 (mul (rec x) w)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w y (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w y (rec x-0))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w (rec x-0))))
      (send (cat (exp (gen) y) (mul w (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 105)
  (parent 79)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w x-0)))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (x-0 (mul x w)) (y (exp (gen) (mul x w x-0))))
  (neq ((exp (gen) (mul x w x-0)) (gen))
    ((exp (gen) (mul w y x-0)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y x-0)))
      (send (enc n (exp (gen) (mul x w y x-0)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y x-0)))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w x-0)))
      (send (cat (exp (gen) y) (mul w x-0)))) ((recv x-0) (send x-0)))
  (label 106)
  (parent 80)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (4 0))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (x (mul (rec y) w)) (y-0 (exp (gen) w)) (y h))
  (neq ((exp (gen) w) (gen)) (h (gen))
    ((exp (gen) (mul w y-0 (rec x))) (gen)))
  (pen-non-orig y-0 x)
  (precur (4 0) (2 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x))))) ((recv x) (send x)))
  (label 107)
  (parent 86)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y-0))) (y y))
  (deflistener (cat (exp (gen) y-0) (mul x w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y-0 (mul x w)) (y (exp (gen) (mul x w y-0))) (y-0 h))
  (neq ((exp (gen) (mul x w y-0)) (gen)) (h (gen))
    ((exp (gen) (mul w y y-0)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (cat (exp (gen) y-0) (mul x w)))
      (send (cat (exp (gen) y-0) (mul x w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))) ((recv y-0) (send y-0)))
  (label 108)
  (parent 87)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (deflistener (cat (gen) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (5 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul x w))))
  (neq ((exp (gen) (mul x w)) (gen)) ((exp (gen) (mul w y)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv y) (send y)))
  (label 109)
  (parent 90)
  (unrealized (2 0) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul w x x))) (y y))
  (deflistener (exp (gen) (mul w x x y)))
  (deflistener (cat (exp (gen) y) (mul w x x)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (5 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul w x x))))
  (neq ((exp (gen) (mul w x x)) (gen)) ((exp (gen) (mul w x y)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x x y)))))
    ((recv (exp (gen) (mul w x x y))) (send (exp (gen) (mul w x x y))))
    ((recv (cat (exp (gen) y) (mul w x x)))
      (send (cat (exp (gen) y) (mul w x x))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))) ((recv y) (send y)))
  (label 110)
  (parent 91)
  (unrealized (1 0) (2 0) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (exp (gen) (mul x w x-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (deflistener y)
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (6 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 0) (1 0))
    ((6 1) (4 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (neq ((exp (gen) (mul x w x-0)) (gen))
    ((exp (gen) (mul w x-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))))
    ((recv (exp (gen) (mul x w x-0 y)))
      (send (exp (gen) (mul x w x-0 y))))
    ((recv (cat (exp (gen) y) (mul x w x-0)))
      (send (cat (exp (gen) y) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0)))
    ((recv y) (send y)))
  (label 111)
  (parent 92)
  (unrealized (1 0) (2 0) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (4 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (x w) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w y (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (4 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w) w (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))))
  (label 112)
  (parent 93)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (3 0)))
  (absent (y (exp (gen) (mul x w))))
  (neq ((exp (gen) (mul x w)) (gen)) ((exp (gen) (mul w y)) (gen)))
  (pen-non-orig x y)
  (precur (4 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 113)
  (parent 93)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (exp (gen) (mul x w y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 1) (6 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (1 0))
    ((6 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (neq (h (gen)) ((exp (gen) (mul x w y)) (gen))
    ((exp (gen) (mul w y y-0)) (gen)))
  (pen-non-orig x y-0)
  (precur (4 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))))
    ((recv (exp (gen) (mul x w y y-0)))
      (send (exp (gen) (mul x w y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w y)))
      (send (cat (exp (gen) y-0) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 114)
  (parent 94)
  (unrealized (1 0) (2 0) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w-0 (rec x))) (gen) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 115)
  (parent 95)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 116)
  (parent 95)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1))
    ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 117)
  (parent 95)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y))))))
  (label 118)
  (parent 95)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 119)
  (parent 95)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) x-0)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w-0 (mul (rec x) x-0))) (gen) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul (rec x) x-0)))
      (send (cat (gen) (mul (rec x) x-0)))))
  (label 120)
  (parent 96)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul x (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-1) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))))
  (label 121)
  (parent 96)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (rec x)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) x-0)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 init 1) (exp (gen) x-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (rec x)))
      (send (cat (exp (gen) x-0) (rec x)))))
  (label 122)
  (parent 96)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 0) (6 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) x-0)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 123)
  (parent 96)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) x-0)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))))
  (label 124)
  (parent 96)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) x-0)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 125)
  (parent 96)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) y)) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w-0 (mul (rec x) y))) (gen) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 126)
  (parent 97)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul y (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 127)
  (parent 97)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)) ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) y)) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 128)
  (parent 97)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) y)) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 129)
  (parent 97)
  (unrealized (0 1) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) y)) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 130)
  (parent 97)
  (unrealized (0 1) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul (rec x) y-0)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) y-0)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w-0 (mul (rec x) y-0))) (gen) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul (rec x) y-0)))
      (send (cat (gen) (mul (rec x) y-0)))))
  (label 131)
  (parent 98)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul y-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y-0 (rec x))))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))))
  (label 132)
  (parent 98)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 0) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) y-0)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 133)
  (parent 98)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) y))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 (exp (gen) w)) (y h))
  (neq ((exp (gen) w) (gen)) (h (gen))
    ((exp (gen) (mul (rec x) y)) (gen)))
  (pen-non-orig x y-0)
  (precur (6 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))))
  (label 134)
  (parent 98)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (rec x)))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) y-0)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 5 resp 2) (exp (gen) y-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (rec x)))
      (send (cat (exp (gen) y-0) (rec x)))))
  (label 135)
  (parent 98)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y-1 h-0) (y-0 h) (y (exp (gen) w)))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) y-0)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-1) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 136)
  (parent 98)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) x)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (gen) (mul x x)))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) x) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (gen) (mul x x))) (send (cat (gen) (mul x x)))))
  (label 137)
  (parent 99)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (gen) (mul x x-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) x-0) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (gen) (mul x x-0))) (send (cat (gen) (mul x x-0))))
    ((send (exp (gen) x-0))))
  (label 138)
  (parent 99)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (gen) (mul x y)))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) y) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (gen) (mul x y))) (send (cat (gen) (mul x y)))))
  (label 139)
  (parent 99)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (gen) (mul x y-0)))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) y-0) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (gen) (mul x y-0))) (send (cat (gen) (mul x y-0))))
    ((recv h) (send (exp (gen) y-0))))
  (label 140)
  (parent 99)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 w-1 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (gen) w-0))
  (deflistener (cat (exp (gen) (mul (rec x) w-0 (rec w-1))) w-1))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) w-0)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w-0 (rec w-1))) w-1))
    (exp (gen) (mul (rec x) w-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0)))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0)))
    ((recv (cat (exp (gen) (mul (rec x) w-0 (rec w-1))) w-1))
      (send (cat (exp (gen) (mul (rec x) w-0 (rec w-1))) w-1))))
  (label 141)
  (parent 99)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((2 1) (1 2))
    ((3 1) (1 2)) ((4 1) (3 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (4 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 142)
  (parent 100)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x-0 x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
  (defstrand init 1 (x x))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) x-0) (gen)))
  (pen-non-orig y x-0)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-1) (0 1))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x-0 x-0))) (send (exp (gen) (mul x-0 x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) x-0 x-0))))
    ((send (exp (gen) x))))
  (label 143)
  (parent 101)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (exp (gen) x-0) x))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) x-0) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 6 init 1) (exp (gen) x-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x)))
    ((send (exp (gen) x-0))))
  (label 144)
  (parent 101)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-1)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x-1)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 1 (x x-0))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)) ((7 0) (0 1)) ((7 0) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) x-1) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-1))
      (send (enc n (exp (gen) (mul x x-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x-1))) (send (exp (gen) (mul x x-1))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((send (exp (gen) x-0))) ((send (exp (gen) x-1))))
  (label 145)
  (parent 101)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) y) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((send (exp (gen) x-0))))
  (label 146)
  (parent 101)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y-0)))
  (defstrand init 1 (x x-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (0 1))
    ((6 0) (5 0)) ((7 1) (0 1)) ((7 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) y-0) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y-0))))
    ((send (exp (gen) x-0))) ((recv h) (send (exp (gen) y-0))))
  (label 147)
  (parent 101)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w expr) (y expn) (w-0 expr) (x-0 expn)
    (w-1 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) w-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) w-0 x-0 (rec w-1))) w-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) w-0 x-0)) (gen)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) w-0 x-0 (rec w-1))) w-1))
    (exp (gen) (mul (rec x) w-0 x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0 x-0)))
      (send (enc n (exp (gen) (mul w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x-0))) (send (exp (gen) (mul w-0 x-0))))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) w-0 x-0 (rec w-1))) w-1))
      (send (cat (exp (gen) (mul (rec x) w-0 x-0 (rec w-1))) w-1))))
  (label 148)
  (parent 101)
  (unrealized (7 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) x)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) x x)))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) x) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 6 0 init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (exp (gen) y) (mul (rec y) x x)))
      (send (cat (exp (gen) y) (mul (rec y) x x)))))
  (label 149)
  (parent 102)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) x-0) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((send (exp (gen) x-0))))
  (label 150)
  (parent 102)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (exp (gen) y) x))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) y) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 6 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x))))
  (label 151)
  (parent 102)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)) ((6 1) (0 1)) ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) y-0) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((recv h) (send (exp (gen) y-0))))
  (label 152)
  (parent 102)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (x expn) (w w-0 expr) (y expn) (w-1 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) y) w-0))
  (deflistener (cat (exp (gen) (mul (rec x) w-0 y (rec w-1))) w-1))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) w-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w-0 y (rec w-1))) w-1))
    (exp (gen) (mul (rec x) w-0 y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0 y)))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0)))
    ((recv (cat (exp (gen) (mul (rec x) w-0 y (rec w-1))) w-1))
      (send (cat (exp (gen) (mul (rec x) w-0 y (rec w-1))) w-1))))
  (label 153)
  (parent 102)
  (unrealized (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) x)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) x x)))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) x) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) x x)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) x x))))
    ((recv h) (send (exp (gen) y-0))))
  (label 154)
  (parent 103)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) x-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)) ((7 0) (0 1)) ((7 0) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) x-0) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) x-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) x-0))))
    ((recv h) (send (exp (gen) y-0))) ((send (exp (gen) x-0))))
  (label 155)
  (parent 103)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 1) (3 0))
    ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2))
    ((5 1) (4 0)) ((6 1) (0 1)) ((6 1) (5 0)))
  (absent (y-0 (exp (gen) w)) (y h))
  (neq ((exp (gen) w) (gen)) (h (gen)) ((exp (gen) y-0) (gen)))
  (pen-non-orig x y-0)
  (precur (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 156)
  (parent 103)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) y-0) x))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) y-0) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 7 6 resp 2) (exp (gen) y-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) y-0) x)) (send (cat (exp (gen) y-0) x)))
    ((recv h) (send (exp (gen) y-0))))
  (label 157)
  (parent 103)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) y-1)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x y-1)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) y-1)))
  (defstrand resp 2 (h h) (y y-0))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1))
    ((6 1) (5 0)) ((7 1) (0 1)) ((7 1) (5 0)))
  (absent (y-1 h-0) (y-0 h) (y (exp (gen) w)))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) y-1) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-1))
      (send (enc n (exp (gen) (mul x y-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x y-1))) (send (exp (gen) (mul x y-1))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) y-1)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) y-1))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 158)
  (parent 103)
  (unrealized (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (x expn) (w expr) (y expn) (w-0 expr)
    (y-0 expn) (w-1 expr))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) w-0 y-0 (rec w-1))) w-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) w-0 y-0)) (gen)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) w-0 y-0 (rec w-1))) w-1))
    (exp (gen) (mul (rec x) w-0 y-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w-0 y-0)))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) (mul (rec x) w-0 y-0 (rec w-1))) w-1))
      (send (cat (exp (gen) (mul (rec x) w-0 y-0 (rec w-1))) w-1))))
  (label 159)
  (parent 103)
  (unrealized (7 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (1 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (absent (x w) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w y (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (4 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))) ((recv x) (send x)))
  (label 160)
  (parent 112)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 161)
  (parent 115)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (0 1))
    ((6 1) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 162)
  (parent 116)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (rec x)))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 0) (4 0))
    ((5 0) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) x-0)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (rec x)))
      (send (cat (exp (gen) x-0) (rec x)))) ((recv x) (send x)))
  (label 163)
  (parent 122)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (4 0)) ((1 1) (5 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (0 1)) ((6 1) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) y)) (gen)))
  (pen-non-orig x y)
  (precur (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 164)
  (parent 129)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (rec x)))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) y-0)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (rec x)))
      (send (cat (exp (gen) y-0) (rec x)))) ((recv x) (send x)))
  (label 165)
  (parent 135)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) x)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (gen) (mul x x)))
  (deflistener x)
  (precedes ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((2 1) (1 2))
    ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) x) (gen)))
  (pen-non-orig y x)
  (precur (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x x) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (gen) (mul x x))) (send (cat (gen) (mul x x))))
    ((recv x) (send x)))
  (label 166)
  (parent 137)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (gen) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w-1 (mul x w-0)) (w-2 w-0))
    (gen) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0))))
  (label 167)
  (parent 141)
  (unrealized (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x)))
  (deflistener (cat (gen) (mul w-0 x x)))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x)))
      (send (enc n (exp (gen) (mul w-0 x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x))) (send (exp (gen) (mul w-0 x x))))
    ((recv (cat (gen) (mul w-0 x x))) (send (cat (gen) (mul w-0 x x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 168)
  (parent 141)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) w-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x-0)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (gen) (mul x w-0 x-0)))
      (send (cat (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0)))
    ((send (exp (gen) x-0))))
  (label 169)
  (parent 141)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y)))
  (deflistener (cat (gen) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y)))
      (send (enc n (exp (gen) (mul x w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y))) (send (exp (gen) (mul x w-0 y))))
    ((recv (cat (gen) (mul x w-0 y))) (send (cat (gen) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0))))
  (label 170)
  (parent 141)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)) ((7 1) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (gen) (mul x w-0 y-0)))
      (send (cat (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0))))
  (label 171)
  (parent 141)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (3 0)) ((0 2) (1 2)) ((2 1) (1 2)) ((3 1) (2 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 172)
  (parent 142)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w-1 (mul x (rec x-0) w-0)) (x-2 x-0) (w-2 w-0))
    (gen) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0))))
  (label 173)
  (parent 148)
  (unrealized (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x-0 x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 x-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x-0)) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 0 init 1) (exp (gen) x-1) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul w-0 x-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x-0 x-0)))
      (send (exp (gen) (mul w-0 x-0 x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 x-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0))))
  (label 174)
  (parent 148)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 6 init 1) (exp (gen) x-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0)))
      (send (cat (exp (gen) x-0) (mul x w-0)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0))))
  (label 175)
  (parent 148)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn) (w-0 expr) (x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-1)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0 x-1)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) w-0))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x-1)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-1)))
      (send (enc n (exp (gen) (mul x w-0 x-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-1)))
      (send (exp (gen) (mul x w-0 x-1))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0 x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0 x-1))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) w-0)) (send (cat (exp (gen) x-1) w-0)))
    ((send (exp (gen) x-1))))
  (label 176)
  (parent 148)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x x-0 expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0 y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 8 1 resp 2) (exp (gen) y-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y)))
      (send (enc n (exp (gen) (mul x w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y))) (send (exp (gen) (mul x w-0 y))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0 y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0 y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0))))
  (label 177)
  (parent 148)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x x-0 expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0 y-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0 y-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0 y-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0))))
  (label 178)
  (parent 148)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (deflistener (cat (gen) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-1 (mul x (rec y) w-0)) (y-0 y) (w-2 w-0))
    (gen) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0))))
  (label 179)
  (parent 153)
  (unrealized (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 x x)))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 7 0 init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x)))
      (send (enc n (exp (gen) (mul w-0 x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x))) (send (exp (gen) (mul w-0 x x))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 x x)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 x x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 180)
  (parent 153)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) w-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x-0)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0 x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0 x-0))))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0)))
    ((send (exp (gen) x-0))))
  (label 181)
  (parent 153)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w-0)))
  (deflistener (cat (exp (gen) y) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 7 1 resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y)))
      (send (enc n (exp (gen) (mul x w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y))) (send (exp (gen) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) (mul x w-0)))
      (send (cat (exp (gen) y) (mul x w-0))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0))))
  (label 182)
  (parent 153)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0 y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0 y-0))))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0))))
  (label 183)
  (parent 153)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-1 (mul x (rec y-0) w-0)) (y-1 y-0) (w-2 w-0))
    (gen) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0))))
  (label 184)
  (parent 159)
  (unrealized (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0 x x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (displaced 8 0 init 1) (exp (gen) x-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x)))
      (send (enc n (exp (gen) (mul w-0 x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x))) (send (exp (gen) (mul w-0 x x))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0 x x)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0 x x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 185)
  (parent 159)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0 x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) w-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 x-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0 x-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0 x-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0)))
    ((send (exp (gen) x-0))))
  (label 186)
  (parent 159)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0 y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)))
  (absent (y-0 (exp (gen) w)) (y h))
  (neq ((exp (gen) w) (gen)) (h (gen))
    ((exp (gen) (mul w-0 y-0)) (gen)))
  (pen-non-orig x y-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 1 resp 2) (exp (gen) y-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0 y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0 y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0))))
  (label 187)
  (parent 159)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) w-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 8 6 resp 2) (exp (gen) y-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0)))
      (send (cat (exp (gen) y-0) (mul x w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0))))
  (label 188)
  (parent 159)
  (unrealized (4 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x y-0 expn) (w-0 expr)
    (y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-1)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0 y-1)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) w-0))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-1 h-0) (y-0 h) (y (exp (gen) w)))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-1)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-1)))
      (send (enc n (exp (gen) (mul x w-0 y-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-1)))
      (send (exp (gen) (mul x w-0 y-1))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0 y-1)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0 y-1))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) w-0)) (send (cat (exp (gen) y-1) w-0)))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 189)
  (parent 159)
  (unrealized (4 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w-0 (rec x))) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 190)
  (parent 167)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (gen) w-0))
  (deflistener (cat (gen) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (x w-0) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w-0) w-0 (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0)))
    ((recv (cat (gen) (mul w-0 (rec x))))
      (send (cat (gen) (mul w-0 (rec x))))))
  (label 191)
  (parent 167)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (gen) w-0))
  (deflistener x)
  (precedes ((0 0) (6 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0))) ((recv x) (send x)))
  (label 192)
  (parent 167)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (w-0 (mul (rec x) (rec x))) (x-0 x))
    (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 193)
  (parent 168)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (gen) w-0))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)))
  (absent (x w-0) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w-0) w-0 (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))))
  (label 194)
  (parent 168)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x)))
  (deflistener (cat (gen) (mul w-0 x x)))
  (deflistener (cat (exp (gen) x) w-0))
  (deflistener x)
  (precedes ((0 0) (6 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 x x) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x)))
      (send (enc n (exp (gen) (mul w-0 x x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x))) (send (exp (gen) (mul w-0 x x))))
    ((recv (cat (gen) (mul w-0 x x))) (send (cat (gen) (mul w-0 x x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0)))
    ((recv x) (send x)))
  (label 195)
  (parent 168)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul (rec x) (rec x-0))) (x-2 x-0)) (one)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 196)
  (parent 169)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (x-0 (mul w-0 x)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul w-0 x)) (mul w-0 x)
    (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (gen) (mul w-0 x))) (send (cat (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x))))
  (label 197)
  (parent 169)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 0) (5 0)) ((7 0) (6 0)))
  (absent (x-0 (mul x w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w-0)) (mul x w-0)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 198)
  (parent 169)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul (rec x) (rec y))) (y-0 y)) (one)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y))))))
  (label 199)
  (parent 170)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w w-0 expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (x (mul w-0 y)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul w-0 y)) (mul w-0 y) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y (rec x))))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (gen) (mul w-0 y))) (send (cat (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) y) (mul w-0 (rec x))))
      (send (cat (exp (gen) y) (mul w-0 (rec x))))))
  (label 200)
  (parent 170)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec y))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (mul x w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x w-0)) (mul x w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul w-0 (rec y))))
      (send (cat (exp (gen) y) (mul w-0 (rec y))))))
  (label 201)
  (parent 170)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)) ((7 1) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul (rec x) (rec y-0))) (y-1 y-0)) (one)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 202)
  (parent 171)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)) ((7 1) (6 0)))
  (absent (x (mul w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul w-0 y-0)) (mul w-0 y-0)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (gen) (mul w-0 y-0))) (send (cat (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 203)
  (parent 171)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)) ((7 1) (6 0)))
  (absent (y-0 (mul x w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w-0)) (mul x w-0)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 204)
  (parent 171)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (exp (gen) x) w-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((2 1) (1 2)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0))))
  (label 205)
  (parent 172)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) x-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w-0 (mul (rec x) x-0))) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul (rec x) x-0)))
      (send (cat (gen) (mul (rec x) x-0)))))
  (label 206)
  (parent 173)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (gen) (mul w-0 (rec x-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (x-0 (mul (rec x) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w-0))
    (mul (rec x) w-0) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 (rec x-0))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0))))
    ((send (exp (gen) x)))
    ((recv (cat (gen) (mul w-0 (rec x-0))))
      (send (cat (gen) (mul w-0 (rec x-0))))))
  (label 207)
  (parent 173)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w-0 x-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (x-0 (mul x w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w-0)) (mul x w-0)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0)))
      (send (cat (exp (gen) x-0) (mul x w-0)))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w-0 x-0))) (send (cat (gen) (mul w-0 x-0)))))
  (label 208)
  (parent 173)
  (unrealized (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul x (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul x (rec x-0) (rec x-0))) (x-2 x-0))
    (one) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))))
  (label 209)
  (parent 174)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 x x)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x) (mul w-0 x-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (x-0 (mul w-0 x x)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x x-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul w-0 x x)) (mul w-0 x x)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x x-0)))
      (send (enc n (exp (gen) (mul w-0 x x x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x x-0)))
      (send (exp (gen) (mul w-0 x x x-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 x x)))
      (send (cat (exp (gen) x-0) (mul w-0 x x))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x) (mul w-0 x-0)))
      (send (cat (exp (gen) x) (mul w-0 x-0)))))
  (label 210)
  (parent 174)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0) (rec x-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (x-0 (mul (rec x) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w-0))
    (mul (rec x) w-0) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 (rec x-0))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0) (rec x-0))))))
  (label 211)
  (parent 174)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (rec x-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul x (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (contracted (x-1 x-0) (w-0 (rec x-0))) (one)
    (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (rec x-0)))
      (send (cat (exp (gen) x) (rec x-0)))))
  (label 212)
  (parent 175)
  (unrealized (0 1) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) w-0))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)))
  (absent (x-0 w-0) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 w-0) w-0 (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0)))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0))))))
  (label 213)
  (parent 175)
  (unrealized (0 1) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) w-0))
  (deflistener x)
  (precedes ((0 0) (7 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0)))
      (send (cat (exp (gen) x-0) (mul x w-0)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) w-0)) (send (cat (exp (gen) x-0) w-0)))
    ((recv x) (send x)))
  (label 214)
  (parent 175)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) x-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-2 x) (x-3 x-0) (w-0 (mul (rec x) x-0 (rec x-1)))
      (x-4 x-1)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 215)
  (parent 176)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 (rec x-1))))
    (x x-1))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-1))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (x-1 (mul (rec x) w-0 x-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 x-0 (rec x-1))) (gen)))
  (pen-non-orig y x-1)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-absence x-1 (mul (rec x) w-0 x-0))
    (mul (rec x) w-0 x-0) (5 0))
  (traces
    ((send (exp (gen) x-1)) (recv (exp (gen) (mul w-0 x-0 (rec x-1))))
      (send (enc n (exp (gen) (mul w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x-0))) (send (exp (gen) (mul w-0 x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-1))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-1)))))
    ((send (exp (gen) x-0))))
  (label 216)
  (parent 176)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 x-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0 x-1)))
  (deflistener (cat (exp (gen) x-1) (mul x w-0 x-0)))
  (defstrand init 1 (x x-1))
  (deflistener (cat (exp (gen) x-0) (mul w-0 x-1)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (x-1 (mul x w-0 x-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x-0 x-1)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-absence x-1 (mul x w-0 x-0))
    (mul x w-0 x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0 x-1)))
      (send (enc n (exp (gen) (mul x w-0 x-0 x-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0 x-1)))
      (send (exp (gen) (mul x w-0 x-0 x-1))))
    ((recv (cat (exp (gen) x-1) (mul x w-0 x-0)))
      (send (cat (exp (gen) x-1) (mul x w-0 x-0))))
    ((send (exp (gen) x-1)))
    ((recv (cat (exp (gen) x-0) (mul w-0 x-1)))
      (send (cat (exp (gen) x-0) (mul w-0 x-1))))
    ((send (exp (gen) x-0))))
  (label 217)
  (parent 176)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn) (w-0 expr) (x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (x-1 (mul x (rec x-0) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-absence x-1 (mul x (rec x-0) w-0))
    (mul x (rec x-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul w-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 218)
  (parent 176)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) x-0)) (gen)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w-0 (mul (rec x) x-0 (rec y)))
      (y-0 y)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))))
  (label 219)
  (parent 177)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 y)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec x-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)))
  (absent (x-0 (mul (rec x) w-0 y)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w-0 y))
    (mul (rec x) w-0 y) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 y (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 y)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 y))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) y) (mul w-0 (rec x-0))))))
  (label 220)
  (parent 177)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0 y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w-0 x-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)))
  (absent (x-0 (mul x w-0 y)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y x-0)) (gen)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w-0 y)) (mul x w-0 y)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y x-0)))
      (send (enc n (exp (gen) (mul x w-0 y x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y x-0)))
      (send (exp (gen) (mul x w-0 y x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0 y)))
      (send (cat (exp (gen) x-0) (mul x w-0 y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w-0 x-0)))
      (send (cat (exp (gen) y) (mul w-0 x-0)))))
  (label 221)
  (parent 177)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x x-0 expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec y))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)))
  (absent (y (mul x (rec x-0) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x (rec x-0) w-0))
    (mul x (rec x-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w-0 (rec y))))
      (send (cat (exp (gen) y) (mul w-0 (rec y))))))
  (label 222)
  (parent 177)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) x-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w-0 (mul (rec x) x-0 (rec y-0)))
      (y-1 y-0)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) x-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 223)
  (parent 178)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x-0))))
    (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 y-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (x-0 (mul (rec x) w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0 (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w-0 y-0))
    (mul (rec x) w-0 y-0) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 y-0 (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 y-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 y-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 224)
  (parent 178)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0 y-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul w-0 x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (x-0 (mul x w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0 x-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w-0 y-0))
    (mul x w-0 y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0 x-0)))
      (send (exp (gen) (mul x w-0 y-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0 y-0)))
      (send (cat (exp (gen) x-0) (mul x w-0 y-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul w-0 x-0)))
      (send (cat (exp (gen) y-0) (mul w-0 x-0))))
    ((recv h) (send (exp (gen) y-0))))
  (label 225)
  (parent 178)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x x-0 expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-0 (mul x (rec x-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x (rec x-0) w-0))
    (mul x (rec x-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 226)
  (parent 178)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w-0 (mul (rec x) y))) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 227)
  (parent 179)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0)))
  (deflistener (cat (gen) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (x (mul (rec y) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w-0))
    (mul (rec y) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0))))
    ((recv (cat (gen) (mul w-0 (rec x))))
      (send (cat (gen) (mul w-0 (rec x))))))
  (label 228)
  (parent 179)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w-0)))
  (deflistener (cat (gen) (mul w-0 y)))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (mul x w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x w-0)) (mul x w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y)))
      (send (enc n (exp (gen) (mul x w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y))) (send (exp (gen) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) (mul x w-0)))
      (send (cat (exp (gen) y) (mul x w-0))))
    ((recv (cat (gen) (mul w-0 y))) (send (cat (gen) (mul w-0 y)))))
  (label 229)
  (parent 179)
  (unrealized (4 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul y (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test
    (contracted (y-0 y) (w-0 (mul y (rec x) (rec x))) (x-0 x)) (one)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 230)
  (parent 180)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w w-0 expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x y)))
  (deflistener (cat (exp (gen) y) (mul w-0 x x)))
  (deflistener (cat (exp (gen) x) (mul w-0 y)))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (mul w-0 x x)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul w-0 x x)) (mul w-0 x x)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x y)))
      (send (enc n (exp (gen) (mul w-0 x x y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x y)))
      (send (exp (gen) (mul w-0 x x y))))
    ((recv (cat (exp (gen) y) (mul w-0 x x)))
      (send (cat (exp (gen) y) (mul w-0 x x))))
    ((recv (cat (exp (gen) x) (mul w-0 y)))
      (send (cat (exp (gen) x) (mul w-0 y)))))
  (label 231)
  (parent 180)
  (unrealized (0 1) (4 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0)))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (x (mul (rec y) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w-0))
    (mul (rec y) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0))))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))))
  (label 232)
  (parent 180)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec x) y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (y-0 y) (w-0 (mul (rec x) y (rec x-0)))
      (x-2 x-0)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 233)
  (parent 181)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 x)))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)))
  (absent (x-0 (mul (rec y) w-0 x)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec y) w-0 x))
    (mul (rec y) w-0 x) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 x)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 x))))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x))))
  (label 234)
  (parent 181)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)))
  (absent (y (mul x w-0 x-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x w-0 x-0))
    (mul x w-0 x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0 y)))
      (send (enc n (exp (gen) (mul x w-0 x-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0 y)))
      (send (exp (gen) (mul x w-0 x-0 y))))
    ((recv (cat (exp (gen) y) (mul x w-0 x-0)))
      (send (cat (exp (gen) y) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 y)))
      (send (cat (exp (gen) x-0) (mul w-0 y))))
    ((send (exp (gen) x-0))))
  (label 235)
  (parent 181)
  (unrealized (0 1) (4 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)))
  (absent (x-0 (mul x (rec y) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x (rec y) w-0))
    (mul x (rec y) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 236)
  (parent 181)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul y (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w-0 (rec x))) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 237)
  (parent 182)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w w-0 expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) y) w-0))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)))
  (absent (x w-0) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w-0) w-0 (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y (rec x))))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0)))
    ((recv (cat (exp (gen) y) (mul w-0 (rec x))))
      (send (cat (exp (gen) y) (mul w-0 (rec x))))))
  (label 238)
  (parent 182)
  (unrealized (0 1) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w-0)))
  (deflistener (cat (exp (gen) y) w-0))
  (deflistener x)
  (precedes ((0 0) (6 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y)))
      (send (enc n (exp (gen) (mul x w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y))) (send (exp (gen) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) (mul x w-0)))
      (send (cat (exp (gen) y) (mul x w-0))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0)))
    ((recv x) (send x)))
  (label 239)
  (parent 182)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y) (w-0 (mul (rec x) y (rec y-0)))
      (y-2 y-0)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 240)
  (parent 183)
  (unrealized (0 1) (6 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)))
  (absent (x (mul (rec y) w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w-0 y-0))
    (mul (rec y) w-0 y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 241)
  (parent 183)
  (unrealized (0 1) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x expn) (w-0 expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul x w-0 y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) (mul w-0 y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)))
  (absent (y-0 (mul x w-0 y)) (y h) (y-0 (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y y-0)) (gen)))
  (pen-non-orig x y-0)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w-0 y)) (mul x w-0 y)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y y-0)))
      (send (enc n (exp (gen) (mul x w-0 y y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul x w-0 y y-0)))
      (send (exp (gen) (mul x w-0 y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0 y)))
      (send (cat (exp (gen) y-0) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) (mul w-0 y-0)))
      (send (cat (exp (gen) y) (mul w-0 y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 242)
  (parent 183)
  (unrealized (0 1) (4 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)))
  (absent (y-0 (mul x (rec y) w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x (rec y) w-0))
    (mul x (rec y) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 243)
  (parent 183)
  (unrealized (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul (rec x) y-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) y-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w-0 (mul (rec x) y-0))) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul (rec x) y-0)))
      (send (cat (gen) (mul (rec x) y-0)))))
  (label 244)
  (parent 184)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (x (mul (rec y-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y-0) w-0))
    (mul (rec y-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul w-0 (rec x))))
      (send (cat (gen) (mul w-0 (rec x))))))
  (label 245)
  (parent 184)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul w-0 y-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 (mul x w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w-0)) (mul x w-0)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0)))
      (send (cat (exp (gen) y-0) (mul x w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul w-0 y-0))) (send (cat (gen) (mul w-0 y-0)))))
  (label 246)
  (parent 184)
  (unrealized (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul y-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test
    (contracted (y-1 y-0) (w-0 (mul y-0 (rec x) (rec x))) (x-0 x)) (one)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y-0 (rec x))))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y-0 (rec x) (rec x))))))
  (label 247)
  (parent 185)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 x x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w-0 y-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 (mul w-0 x x)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 x y-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul w-0 x x)) (mul w-0 x x)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x y-0)))
      (send (enc n (exp (gen) (mul w-0 x x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x y-0)))
      (send (exp (gen) (mul w-0 x x y-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 x x)))
      (send (cat (exp (gen) y-0) (mul w-0 x x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w-0 y-0)))
      (send (cat (exp (gen) x) (mul w-0 y-0)))))
  (label 248)
  (parent 185)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (x (mul (rec y-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y-0) w-0))
    (mul (rec y-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))))
  (label 249)
  (parent 185)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) y-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (y-1 y-0) (w-0 (mul (rec x) y-0 (rec x-0)))
      (x-2 x-0)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 250)
  (parent 186)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0 x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (x-0 (mul (rec y-0) w-0 x)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 x (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec y-0) w-0 x))
    (mul (rec y-0) w-0 x) (5 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0 x)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0 x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x))))
  (label 251)
  (parent 186)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0 x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul w-0 y-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (y-0 (mul x w-0 x-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 x-0 y-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w-0 x-0))
    (mul x w-0 x-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0 y-0)))
      (send (exp (gen) (mul x w-0 x-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0 x-0)))
      (send (cat (exp (gen) y-0) (mul x w-0 x-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul w-0 y-0)))
      (send (cat (exp (gen) x-0) (mul w-0 y-0))))
    ((send (exp (gen) x-0))))
  (label 252)
  (parent 186)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)))
  (absent (x-0 (mul x (rec y-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x (rec y-0) w-0))
    (mul x (rec y-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 253)
  (parent 186)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)))
  (absent (y-0 (exp (gen) w)) (y h))
  (neq ((exp (gen) w) (gen)) (h (gen))
    ((exp (gen) (mul (rec x) y)) (gen)))
  (pen-non-orig x y-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y) (w-0 (mul (rec x) y (rec y-0)))
      (y-2 y-0)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))))
  (label 254)
  (parent 187)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)))
  (absent (x (mul (rec y) w-0 y-0)) (y-0 (exp (gen) w)) (y h))
  (neq ((exp (gen) w) (gen)) (h (gen))
    ((exp (gen) (mul w-0 y-0 (rec x))) (gen)))
  (pen-non-orig y-0 x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w-0 y-0))
    (mul (rec y) w-0 y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x))))))
  (label 255)
  (parent 187)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x expn) (w-0 expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0 y)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w-0 y-0)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)))
  (absent (y-0 (mul x w-0 y)) (y (exp (gen) w)) (y-0 h))
  (neq ((exp (gen) w) (gen)) (h (gen))
    ((exp (gen) (mul w-0 y y-0)) (gen)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w-0 y)) (mul x w-0 y)
    (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y y-0)))
      (send (enc n (exp (gen) (mul x w-0 y y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y y-0)))
      (send (exp (gen) (mul x w-0 y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0 y)))
      (send (cat (exp (gen) y-0) (mul x w-0 y))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w-0 y-0)))
      (send (cat (exp (gen) y) (mul w-0 y-0)))))
  (label 256)
  (parent 187)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)))
  (absent (y-0 (mul x (rec y) w-0)) (y-0 (exp (gen) w)) (y h))
  (neq ((exp (gen) w) (gen)) (h (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig x y-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x (rec y) w-0))
    (mul x (rec y) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0))))))
  (label 257)
  (parent 187)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (rec x)))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul y-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w-0 (rec x))) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y-0 (rec x))))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (rec x)))
      (send (cat (exp (gen) y-0) (rec x)))))
  (label 258)
  (parent 188)
  (unrealized (0 1) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)))
  (absent (x w-0) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w-0) w-0 (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x))))))
  (label 259)
  (parent 188)
  (unrealized (0 1) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) w-0))
  (deflistener x)
  (precedes ((0 0) (7 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0)))
      (send (cat (exp (gen) y-0) (mul x w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv x) (send x)))
  (label 260)
  (parent 188)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul (rec x) y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-1 h-0) (y-0 h) (y (exp (gen) w)))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul (rec x) y-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-2 y-0) (w-0 (mul (rec x) y-0 (rec y-1)))
      (y-3 y-1)) (one) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y-0)))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 261)
  (parent 189)
  (unrealized (0 1) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y y-0 expn) (w-0 expr)
    (y-1 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-1 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-1)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0 y-1)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul w-0 (rec x))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (x (mul (rec y-0) w-0 y-1)) (y-1 h-0) (y-0 h)
    (y (exp (gen) w)))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-1 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 y-1 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y-0) w-0 y-1))
    (mul (rec y-0) w-0 y-1) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-1 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-1))) (send (exp (gen) (mul w-0 y-1))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0 y-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0 y-1))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-1) (mul w-0 (rec x)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 262)
  (parent 189)
  (unrealized (0 1) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x expn) (w-0 expr)
    (y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 y-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0 y-1)))
  (deflistener (cat (exp (gen) y-1) (mul x w-0 y-0)))
  (defstrand resp 2 (h h) (y y-1))
  (deflistener (cat (exp (gen) y-0) (mul w-0 y-1)))
  (defstrand resp 2 (h h-0) (y y-0))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-1 (mul x w-0 y-0)) (y-0 h-0) (y-1 h) (y (exp (gen) w)))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0 y-1)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-absence y-1 (mul x w-0 y-0))
    (mul x w-0 y-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 y-1)))
      (send (enc n (exp (gen) (mul x w-0 y-0 y-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0 y-1)))
      (send (exp (gen) (mul x w-0 y-0 y-1))))
    ((recv (cat (exp (gen) y-1) (mul x w-0 y-0)))
      (send (cat (exp (gen) y-1) (mul x w-0 y-0))))
    ((recv h) (send (exp (gen) y-1)))
    ((recv (cat (exp (gen) y-0) (mul w-0 y-1)))
      (send (cat (exp (gen) y-0) (mul w-0 y-1))))
    ((recv h-0) (send (exp (gen) y-0))))
  (label 263)
  (parent 189)
  (unrealized (0 1) (4 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x y-0 expn) (w-0 expr)
    (y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul w-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)))
  (absent (y-1 (mul x (rec y-0) w-0)) (y-1 h-0) (y-0 h)
    (y (exp (gen) w)))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-absence y-1 (mul x (rec y-0) w-0))
    (mul x (rec y-0) w-0) (5 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul w-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul w-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 264)
  (parent 189)
  (unrealized (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 265)
  (parent 190)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (gen) w-0))
  (deflistener (cat (gen) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x w-0) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0)))
    ((recv (cat (gen) (mul w-0 (rec x))))
      (send (cat (gen) (mul w-0 (rec x))))) ((recv x) (send x)))
  (label 266)
  (parent 191)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (rec x))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (rec x)) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 267)
  (parent 193)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (gen) w-0))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x w-0) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x) (rec x))
    (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (gen) w-0)) (send (cat (gen) w-0)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w-0 (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 268)
  (parent 194)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 0) (5 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (x-0 (mul w-0 x)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (gen) (mul w-0 x))) (send (cat (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x))) ((recv x-0) (send x-0)))
  (label 269)
  (parent 197)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 0) (5 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (x-0 (mul x w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x-0))) ((recv x-0) (send x-0)))
  (label 270)
  (parent 198)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w w-0 expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x (mul w-0 y)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y (rec x))))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (gen) (mul w-0 y))) (send (cat (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) y) (mul w-0 (rec x))))
      (send (cat (exp (gen) y) (mul w-0 (rec x))))) ((recv x) (send x)))
  (label 271)
  (parent 200)
  (unrealized (5 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec y))))
  (deflistener y)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (mul x w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w-0 (rec y)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul w-0 (rec y))))
      (send (cat (exp (gen) y) (mul w-0 (rec y))))) ((recv y) (send y)))
  (label 272)
  (parent 201)
  (unrealized (5 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)) ((7 1) (6 0)) ((8 1) (6 0)))
  (absent (x (mul w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (gen) (mul w-0 y-0))) (send (cat (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x)))))
    ((recv h) (send (exp (gen) y-0))) ((recv x) (send x)))
  (label 273)
  (parent 203)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (0 1)) ((7 1) (5 0)) ((7 1) (8 0)) ((8 1) (6 0)))
  (absent (y-0 (mul x w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 (rec y-0)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (gen) (mul x w-0))) (send (cat (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))) ((recv y-0) (send y-0)))
  (label 274)
  (parent 204)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (precedes ((0 2) (1 2)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (uniq-gen y x)
  (uniq-orig n)
  (operation generalization deleted (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y))))))
  (label 275)
  (parent 205)
  (unrealized)
  (shape)
  (maps
    ((0 1) ((n n) (hx (exp (gen) w)) (hy (exp (gen) w-0)) (x x) (y y))))
  (origs (n (0 2))))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (gen) (mul w-0 (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 (mul (rec x) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 (rec x-0))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0))))
    ((send (exp (gen) x)))
    ((recv (cat (gen) (mul w-0 (rec x-0))))
      (send (cat (gen) (mul w-0 (rec x-0))))) ((recv x-0) (send x-0)))
  (label 276)
  (parent 207)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w-0 x-0)))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (8 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 (mul x w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 x-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0)))
      (send (exp (gen) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0)))
      (send (cat (exp (gen) x-0) (mul x w-0)))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w-0 x-0))) (send (cat (gen) (mul w-0 x-0))))
    ((recv x-0) (send x-0)))
  (label 277)
  (parent 208)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 x x)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x) (mul w-0 x-0)))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (8 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 (mul w-0 x x)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x x-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 x-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x x-0)))
      (send (enc n (exp (gen) (mul w-0 x x x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x x-0)))
      (send (exp (gen) (mul w-0 x x x-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 x x)))
      (send (cat (exp (gen) x-0) (mul w-0 x x))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x) (mul w-0 x-0)))
      (send (cat (exp (gen) x) (mul w-0 x-0)))) ((recv x-0) (send x-0)))
  (label 278)
  (parent 210)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0) (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 (mul (rec x) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0)
    (mul w-0 (rec x-0) (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 (rec x-0))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0) (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 279)
  (parent 211)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (rec x-0)))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul x (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (rec x-0) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (rec x-0)))
      (send (cat (exp (gen) x) (rec x-0)))) ((recv x-0) (send x-0)))
  (label 280)
  (parent 212)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) x) w-0))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 w-0) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) x) w-0)) (send (cat (exp (gen) x) w-0)))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 281)
  (parent 213)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 (rec x-1))))
    (x x-1))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-1))))
  (defstrand init 1 (x x-0))
  (deflistener x-1)
  (precedes ((0 0) (5 0)) ((0 0) (9 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (x-1 (mul (rec x) w-0 x-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 x-0 (rec x-1))) (gen)))
  (pen-non-orig y x-1)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-listener x-1) (mul w-0 (rec x-1)) (7 0))
  (traces
    ((send (exp (gen) x-1)) (recv (exp (gen) (mul w-0 x-0 (rec x-1))))
      (send (enc n (exp (gen) (mul w-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x-0))) (send (exp (gen) (mul w-0 x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-1))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-1)))))
    ((send (exp (gen) x-0))) ((recv x-1) (send x-1)))
  (label 282)
  (parent 216)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn) (w-0 expr) (x-0 x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 x-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0 x-1)))
  (deflistener (cat (exp (gen) x-1) (mul x w-0 x-0)))
  (defstrand init 1 (x x-1))
  (deflistener (cat (exp (gen) x-0) (mul w-0 x-1)))
  (defstrand init 1 (x x-0))
  (deflistener x-1)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (9 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (x-1 (mul x w-0 x-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x-0 x-1)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-listener x-1) (mul w-0 x-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0 x-1)))
      (send (enc n (exp (gen) (mul x w-0 x-0 x-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0 x-1)))
      (send (exp (gen) (mul x w-0 x-0 x-1))))
    ((recv (cat (exp (gen) x-1) (mul x w-0 x-0)))
      (send (cat (exp (gen) x-1) (mul x w-0 x-0))))
    ((send (exp (gen) x-1)))
    ((recv (cat (exp (gen) x-0) (mul w-0 x-1)))
      (send (cat (exp (gen) x-0) (mul w-0 x-1))))
    ((send (exp (gen) x-0))) ((recv x-1) (send x-1)))
  (label 283)
  (parent 217)
  (unrealized (4 0) (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x x-0 expn) (w-0 expr) (x-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (deflistener x-1)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (9 0)) ((9 1) (7 0)))
  (absent (x-1 (mul x (rec x-0) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-listener x-1) (mul w-0 (rec x-1)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul w-0 (rec x-1)))))
    ((send (exp (gen) x-1))) ((recv x-1) (send x-1)))
  (label 284)
  (parent 218)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 y)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 (mul (rec x) w-0 y)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 y (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 y)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 y))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) y) (mul w-0 (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 285)
  (parent 220)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0 y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w-0 x-0)))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (8 0))
    ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x-0 (mul x w-0 y)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y x-0)) (gen)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 x-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y x-0)))
      (send (enc n (exp (gen) (mul x w-0 y x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y x-0)))
      (send (exp (gen) (mul x w-0 y x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0 y)))
      (send (cat (exp (gen) x-0) (mul x w-0 y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w-0 x-0)))
      (send (cat (exp (gen) y) (mul w-0 x-0)))) ((recv x-0) (send x-0)))
  (label 286)
  (parent 221)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x x-0 expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec y))))
  (deflistener y)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (8 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 0) (5 0)) ((6 0) (7 0))
    ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y (mul x (rec x-0) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w-0 (rec y)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w-0 (rec y))))
      (send (cat (exp (gen) y) (mul w-0 (rec y))))) ((recv y) (send y)))
  (label 287)
  (parent 222)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x-0))))
    (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w-0 y-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x-0))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (9 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (absent (x-0 (mul (rec x) w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0 (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 y-0 (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w-0 y-0)))
      (send (cat (exp (gen) x) (mul (rec x) w-0 y-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x-0)))))
    ((recv h) (send (exp (gen) y-0))) ((recv x-0) (send x-0)))
  (label 288)
  (parent 224)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 x-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w-0 y-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul w-0 x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (9 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (absent (x-0 (mul x w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0 x-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 x-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 x-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0 x-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0 x-0)))
      (send (exp (gen) (mul x w-0 y-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w-0 y-0)))
      (send (cat (exp (gen) x-0) (mul x w-0 y-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul w-0 x-0)))
      (send (cat (exp (gen) y-0) (mul w-0 x-0))))
    ((recv h) (send (exp (gen) y-0))) ((recv x-0) (send x-0)))
  (label 289)
  (parent 225)
  (unrealized (4 0) (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x x-0 expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 0) (5 0)) ((6 0) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (9 0)) ((9 1) (7 0)))
  (absent (y-0 (mul x (rec x-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 (rec y-0)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w-0)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w-0))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))) ((recv y-0) (send y-0)))
  (label 290)
  (parent 226)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0)))
  (deflistener (cat (gen) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x (mul (rec y) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0))))
    ((recv (cat (gen) (mul w-0 (rec x))))
      (send (cat (gen) (mul w-0 (rec x))))) ((recv x) (send x)))
  (label 291)
  (parent 228)
  (unrealized (5 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w-0)))
  (deflistener (cat (gen) (mul w-0 y)))
  (deflistener y)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (mul x w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w-0 y) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y)))
      (send (enc n (exp (gen) (mul x w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y))) (send (exp (gen) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) (mul x w-0)))
      (send (cat (exp (gen) y) (mul x w-0))))
    ((recv (cat (gen) (mul w-0 y))) (send (cat (gen) (mul w-0 y))))
    ((recv y) (send y)))
  (label 292)
  (parent 229)
  (unrealized (4 0) (5 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w w-0 expr) (x y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x y)))
  (deflistener (cat (exp (gen) y) (mul w-0 x x)))
  (deflistener (cat (exp (gen) x) (mul w-0 y)))
  (deflistener y)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (mul w-0 x x)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w-0 y) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x y)))
      (send (enc n (exp (gen) (mul w-0 x x y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x y)))
      (send (exp (gen) (mul w-0 x x y))))
    ((recv (cat (exp (gen) y) (mul w-0 x x)))
      (send (cat (exp (gen) y) (mul w-0 x x))))
    ((recv (cat (exp (gen) x) (mul w-0 y)))
      (send (cat (exp (gen) x) (mul w-0 y)))) ((recv y) (send y)))
  (label 293)
  (parent 231)
  (unrealized (4 0) (5 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0)))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x (mul (rec y) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x) (rec x))
    (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0))))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w-0 (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 294)
  (parent 232)
  (unrealized (5 0) (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 x)))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (x-0 (mul (rec y) w-0 x)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 x)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 x))))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x))) ((recv x-0) (send x-0)))
  (label 295)
  (parent 234)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x expn) (w-0 expr) (x-0 y expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 y))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w-0 x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 y)))
  (defstrand init 1 (x x-0))
  (deflistener y)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (8 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (y (mul x w-0 x-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 x-0 y)) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w-0 y) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0 y)))
      (send (enc n (exp (gen) (mul x w-0 x-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0 y)))
      (send (exp (gen) (mul x w-0 x-0 y))))
    ((recv (cat (exp (gen) y) (mul x w-0 x-0)))
      (send (cat (exp (gen) y) (mul x w-0 x-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 y)))
      (send (cat (exp (gen) x-0) (mul w-0 y)))) ((send (exp (gen) x-0)))
    ((recv y) (send y)))
  (label 296)
  (parent 235)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (x y expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 0) (5 0))
    ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (x-0 (mul x (rec y) w-0)) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x-0))) ((recv x-0) (send x-0)))
  (label 297)
  (parent 236)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul y (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 298)
  (parent 237)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (w w-0 expr) (y x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y)))
  (deflistener (cat (exp (gen) y) w-0))
  (deflistener (cat (exp (gen) y) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (6 0)))
  (absent (x w-0) (y (exp (gen) w)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w-0 y (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y (rec x))))
      (send (enc n (exp (gen) (mul w-0 y)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y))) (send (exp (gen) (mul w-0 y))))
    ((recv (cat (exp (gen) y) w-0)) (send (cat (exp (gen) y) w-0)))
    ((recv (cat (exp (gen) y) (mul w-0 (rec x))))
      (send (cat (exp (gen) y) (mul w-0 (rec x))))) ((recv x) (send x)))
  (label 299)
  (parent 238)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (absent (x (mul (rec y) w-0 y-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x)))))
    ((recv h) (send (exp (gen) y-0))) ((recv x) (send x)))
  (label 300)
  (parent 241)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x expn) (w-0 expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul x w-0 y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0 y)))
  (deflistener (cat (exp (gen) y) (mul w-0 y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (8 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (absent (y-0 (mul x w-0 y)) (y h) (y-0 (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y y-0)) (gen)))
  (pen-non-orig x y-0)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 y-0) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y y-0)))
      (send (enc n (exp (gen) (mul x w-0 y y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul x w-0 y y-0)))
      (send (exp (gen) (mul x w-0 y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0 y)))
      (send (cat (exp (gen) y-0) (mul x w-0 y))))
    ((recv (cat (exp (gen) y) (mul w-0 y-0)))
      (send (cat (exp (gen) y) (mul w-0 y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 301)
  (parent 242)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (6 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (6 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (0 1)) ((7 1) (5 0))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (absent (y-0 (mul x (rec y) w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig x y)
  (precur (6 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 (rec y-0)) (6 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))) ((recv y-0) (send y-0)))
  (label 302)
  (parent 243)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x (mul (rec y-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul w-0 (rec x))))
      (send (cat (gen) (mul w-0 (rec x))))) ((recv x) (send x)))
  (label 303)
  (parent 245)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul w-0 y-0)))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y-0 (mul x w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 y-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0)))
      (send (exp (gen) (mul x w-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0)))
      (send (cat (exp (gen) y-0) (mul x w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul w-0 y-0))) (send (cat (gen) (mul w-0 y-0))))
    ((recv y-0) (send y-0)))
  (label 304)
  (parent 246)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (x y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x x y-0)))
  (deflistener (cat (exp (gen) y-0) (mul w-0 x x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w-0 y-0)))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (8 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y-0 (mul w-0 x x)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 x y-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 y-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x y-0)))
      (send (enc n (exp (gen) (mul w-0 x x y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x x y-0)))
      (send (exp (gen) (mul w-0 x x y-0))))
    ((recv (cat (exp (gen) y-0) (mul w-0 x x)))
      (send (cat (exp (gen) y-0) (mul w-0 x x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w-0 y-0)))
      (send (cat (exp (gen) x) (mul w-0 y-0)))) ((recv y-0) (send y-0)))
  (label 305)
  (parent 248)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) w-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x (mul (rec y-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x) (rec x))
    (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 (rec x))))
      (send (enc n (exp (gen) w-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) w-0)) (send (exp (gen) w-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w-0 (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 306)
  (parent 249)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 expn) (w-0 expr) (x x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x (rec x-0)))) (x x-0))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 x)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0 x)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (9 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (x-0 (mul (rec y-0) w-0 x)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 x (rec x-0))) (gen)))
  (pen-non-orig y x-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w-0 x (rec x-0))))
      (send (enc n (exp (gen) (mul w-0 x)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 x))) (send (exp (gen) (mul w-0 x))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0 x)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0 x))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x))) ((recv x-0) (send x-0)))
  (label 307)
  (parent 251)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x expn) (w-0 expr) (x-0 y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 x-0 y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 x-0 y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0 x-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul w-0 y-0)))
  (defstrand init 1 (x x-0))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (7 0)) ((9 1) (7 0)))
  (absent (y-0 (mul x w-0 x-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 x-0 y-0)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 y-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 x-0 y-0)))
      (send (enc n (exp (gen) (mul x w-0 x-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 x-0 y-0)))
      (send (exp (gen) (mul x w-0 x-0 y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0 x-0)))
      (send (cat (exp (gen) y-0) (mul x w-0 x-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul w-0 y-0)))
      (send (cat (exp (gen) x-0) (mul w-0 y-0))))
    ((send (exp (gen) x-0))) ((recv y-0) (send y-0)))
  (label 308)
  (parent 252)
  (unrealized (4 0) (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y x y-0 expn) (w-0 expr) (x-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (deflistener x-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 0) (5 0))
    ((8 0) (9 0)) ((9 1) (7 0)))
  (absent (x-0 (mul x (rec y-0) w-0)) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w-0 (rec x-0)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul w-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w-0 (rec x-0)))))
    ((send (exp (gen) x-0))) ((recv x-0) (send x-0)))
  (label 309)
  (parent 253)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x (mul (rec y) w-0 y-0)) (y-0 (exp (gen) w)) (y h))
  (neq ((exp (gen) w) (gen)) (h (gen))
    ((exp (gen) (mul w-0 y-0 (rec x))) (gen)))
  (pen-non-orig y-0 x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w-0 y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w-0 y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x)))))
    ((recv x) (send x)))
  (label 310)
  (parent 255)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x expn) (w-0 expr) (y y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y y-0))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w-0 y)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w-0 y-0)))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (7 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (8 0))
    ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y-0 (mul x w-0 y)) (y (exp (gen) w)) (y-0 h))
  (neq ((exp (gen) w) (gen)) (h (gen))
    ((exp (gen) (mul w-0 y y-0)) (gen)))
  (pen-non-orig x y)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 y-0) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y y-0)))
      (send (enc n (exp (gen) (mul x w-0 y y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y y-0)))
      (send (exp (gen) (mul x w-0 y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w-0 y)))
      (send (cat (exp (gen) y-0) (mul x w-0 y))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w-0 y-0)))
      (send (cat (exp (gen) y) (mul w-0 y-0)))) ((recv y-0) (send y-0)))
  (label 311)
  (parent 256)
  (unrealized (4 0) (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (x y expn) (w-0 expr) (y-0 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y-0) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
  (deflistener y-0)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 1) (5 0)) ((1 1) (8 0)) ((2 1) (1 2)) ((3 1) (2 0))
    ((4 1) (1 2)) ((5 1) (4 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y-0 (mul x (rec y) w-0)) (y-0 (exp (gen) w)) (y h))
  (neq ((exp (gen) w) (gen)) (h (gen)) ((exp (gen) w-0) (gen)))
  (pen-non-orig x y-0)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w-0 (rec y-0)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec y-0)))))
    ((recv y-0) (send y-0)))
  (label 312)
  (parent 257)
  (unrealized (5 0) (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) y-0))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (rec x)))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul y-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y-0 (rec x))))
      (send (enc n (exp (gen) y-0))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) y-0)) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (rec x)))
      (send (cat (exp (gen) y-0) (rec x)))) ((recv x) (send x)))
  (label 313)
  (parent 258)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h base) (w expr) (y expn) (w-0 expr) (y-0 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-0)))
  (deflistener (cat (exp (gen) y-0) w-0))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (mul w-0 (rec x))))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (8 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (7 0)))
  (absent (x w-0) (y-0 h) (y (exp (gen) w)))
  (neq (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-0))) (send (exp (gen) (mul w-0 y-0))))
    ((recv (cat (exp (gen) y-0) w-0)) (send (cat (exp (gen) y-0) w-0)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-0) (mul w-0 (rec x)))))
    ((recv x) (send x)))
  (label 314)
  (parent 259)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y y-0 expn) (w-0 expr)
    (y-1 x expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-1 (rec x)))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul w-0 y-1)))
  (deflistener (cat (exp (gen) y-0) (mul (rec y-0) w-0 y-1)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul w-0 (rec x))))
  (defstrand resp 2 (h h-0) (y y-1))
  (deflistener x)
  (precedes ((0 0) (5 0)) ((0 0) (9 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (absent (x (mul (rec y-0) w-0 y-1)) (y-1 h-0) (y-0 h)
    (y (exp (gen) w)))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-1 (rec x))) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y y-0 y-1 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w-0 (rec x)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-1 (rec x))))
      (send (enc n (exp (gen) (mul w-0 y-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul w-0 y-1))) (send (exp (gen) (mul w-0 y-1))))
    ((recv (cat (exp (gen) y-0) (mul (rec y-0) w-0 y-1)))
      (send (cat (exp (gen) y-0) (mul (rec y-0) w-0 y-1))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul w-0 (rec x))))
      (send (cat (exp (gen) y-1) (mul w-0 (rec x)))))
    ((recv h-0) (send (exp (gen) y-1))) ((recv x) (send x)))
  (label 315)
  (parent 262)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x expn) (w-0 expr)
    (y-0 y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) (mul w-0 y-0 y-1))) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0 y-0 y-1)))
  (deflistener (cat (exp (gen) y-1) (mul x w-0 y-0)))
  (defstrand resp 2 (h h) (y y-1))
  (deflistener (cat (exp (gen) y-0) (mul w-0 y-1)))
  (defstrand resp 2 (h h-0) (y y-0))
  (deflistener y-1)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (9 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (absent (y-1 (mul x w-0 y-0)) (y-0 h-0) (y-1 h) (y (exp (gen) w)))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) (mul w-0 y-0 y-1)) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-listener y-1) (mul w-0 y-1) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w-0 y-0 y-1)))
      (send (enc n (exp (gen) (mul x w-0 y-0 y-1)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0 y-0 y-1)))
      (send (exp (gen) (mul x w-0 y-0 y-1))))
    ((recv (cat (exp (gen) y-1) (mul x w-0 y-0)))
      (send (cat (exp (gen) y-1) (mul x w-0 y-0))))
    ((recv h) (send (exp (gen) y-1)))
    ((recv (cat (exp (gen) y-0) (mul w-0 y-1)))
      (send (cat (exp (gen) y-0) (mul w-0 y-1))))
    ((recv h-0) (send (exp (gen) y-0))) ((recv y-1) (send y-1)))
  (label 316)
  (parent 263)
  (unrealized (4 0) (5 0) (9 0))
  (comment "empty cohort"))

(defskeleton dh_mim
  (vars (n text) (h h-0 base) (w expr) (y x y-0 expn) (w-0 expr)
    (y-1 expn))
  (defstrand init 3 (n n) (h (exp (gen) w-0)) (x x))
  (defstrand resp 3 (n n) (h (exp (gen) w)) (y y))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener (exp (gen) (mul x w-0)))
  (deflistener (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul w-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (deflistener y-1)
  (precedes ((0 0) (5 0)) ((0 0) (7 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((2 1) (1 2)) ((3 1) (2 0)) ((4 1) (1 2)) ((5 1) (4 0))
    ((6 1) (5 0)) ((6 1) (7 0)) ((7 1) (0 1)) ((8 1) (5 0))
    ((8 1) (9 0)) ((9 1) (7 0)))
  (absent (y-1 (mul x (rec y-0) w-0)) (y-1 h-0) (y-0 h)
    (y (exp (gen) w)))
  (neq (h-0 (gen)) (h (gen)) ((exp (gen) w) (gen))
    ((exp (gen) w-0) (gen)))
  (pen-non-orig y x)
  (precur (7 0) (5 0) (3 0))
  (uniq-gen y x y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-listener y-1) (mul w-0 (rec y-1)) (7 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w-0))
      (send (enc n (exp (gen) (mul x w-0)))))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv (exp (gen) (mul x w-0))) (send (exp (gen) (mul x w-0))))
    ((recv (cat (exp (gen) y-0) (mul x (rec y-0) w-0)))
      (send (cat (exp (gen) y-0) (mul x (rec y-0) w-0))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul w-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul w-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))) ((recv y-1) (send y-1)))
  (label 317)
  (parent 264)
  (unrealized (5 0) (9 0))
  (comment "empty cohort"))

(comment "Nothing left to do")
