(herald "IADH: unified model (UM)" (bound 20) (limit 2000)
  (algebra diffie-hellman))

(comment "CPSA 3.4.0")
(comment "All input read from iadh_um.scm")
(comment "Strand count bounded at 20")

(defprotocol iadh-um diffie-hellman
  (defrole participant
    (vars (l e expn) (hl he base) (self peer name) (key data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))
      (recv he) (send key))
    (uniq-gen e)
    (fn-of (foo ((hash (exp hl l) (exp he e)) key)))
    (neq (he (gen)))
    (absent (e (exp (gen) l)) (e hl)))
  (defrole ltx-gen
    (vars (self name) (l expn))
    (trace (send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    (uniq-gen l)
    (fn-of ("principal-of" (l self)))))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (hl he hl-0 he-0 base)
    (eA lA eB lB expn))
  (defstrand participant 5 (key key) (self A) (peer C) (hl hl) (he he)
    (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D) (hl hl-0)
    (he he-0) (l lB) (e eB))
  (absent (eB (exp (gen) lB)) (eB hl-0) (eA (exp (gen) lA)) (eA hl))
  (fn-of
    (foo ((hash (exp hl-0 lB) (exp he-0 eB)) key)
      ((hash (exp hl lA) (exp he eA)) key)))
  (neq (he-0 (gen)) (eA eB) (he (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA lB)
  (uniq-gen eA eB)
  (comment "Implicit authentication")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" hl C (privk C))) (send (exp (gen) eA)) (recv he)
      (send key))
    ((recv (enc "cert" (exp (gen) lB) B (privk B)))
      (recv (enc "cert" hl-0 D (privk D))) (send (exp (gen) eB))
      (recv he-0) (send key)))
  (label 0)
  (unrealized (0 0) (0 1) (1 0) (1 1))
  (preskeleton)
  (comment "Not a skeleton"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (hl he base) (lA lB eA eB expn))
  (defstrand participant 5 (key key) (self A) (peer C) (hl hl) (he he)
    (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp hl (mul lA (rec lB)))) (he (exp he (mul eA (rec eB))))
    (l lB) (e eB))
  (precedes ((0 2) (1 3)))
  (absent (eB (exp (gen) lB)) (eB (exp hl (mul lA (rec lB))))
    (eA (exp (gen) lA)) (eA hl))
  (fn-of (foo ((hash (exp hl lA) (exp he eA)) key)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (eA eB) (he (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA lB)
  (uniq-gen eA eB)
  (comment "Implicit authentication")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" hl C (privk C))) (send (exp (gen) eA)) (recv he)
      (send key))
    ((recv (enc "cert" (exp (gen) lB) B (privk B)))
      (recv (enc "cert" (exp hl (mul lA (rec lB))) D (privk D)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB))))
      (send key)))
  (label 1)
  (parent 0)
  (unrealized (0 0) (0 1) (1 0) (1 1) (1 3))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (hl he base) (lA eA eB l expn))
  (defstrand participant 5 (key key) (self A) (peer C) (hl hl) (he he)
    (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp hl (mul lA (rec l)))) (he (exp he (mul eA (rec eB)))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eB (exp hl (mul lA (rec l))))
    (eA (exp (gen) lA)) (eA hl))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp hl lA) (exp he eA)) key)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (eA eB) (he (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l) B (privk B)) (1 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" hl C (privk C))) (send (exp (gen) eA)) (recv he)
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp hl (mul lA (rec l))) D (privk D)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 2)
  (parent 1)
  (unrealized (0 0) (0 1) (1 1) (1 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (he base) (lA eA eB l expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l))) (he he) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp he (mul eA (rec eB)))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp he eA)) key)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (eA eB) (he (gen)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) D (privk D)) (1 1))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 3)
  (parent 2)
  (unrealized (0 0) (0 1) (1 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (he base) (lA eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0))) (he he) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp he (mul eA (rec eB)))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 1))
    ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) key)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (eA eB) (he (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) D (privk D)) (1 1))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 4)
  (parent 2)
  (unrealized (0 0) (0 1) (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (eB l l-0 e expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0)))) (he (exp (gen) eB)) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l l (rec l-0))))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e eB) ((exp (gen) eB) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (uniq-gen eB l e)
  (operation nonce-test (displaced 3 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 5)
  (parent 3)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (lA eA l e expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) e e))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA l e)
  (operation nonce-test (displaced 3 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 6)
  (parent 3)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C self peer name) (hl base)
    (lA eA eB l l-0 e expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB e))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 2) (0 3)) ((3 2) (1 3)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) lA)) (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 7)
  (parent 3)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (lA eA eB l expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB l))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 3 2 ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 8)
  (parent 3)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C self name) (lA eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 3)) ((3 0) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 9)
  (parent 3)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (he base) (lA eA eB l expn) (w expr))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l))) (he he) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp he (mul eA (rec eB)))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp he (mul eA (rec eB) (rec w))) w))
  (precedes ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp he eA)) key)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (eA eB) (he (gen)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp he (mul eA (rec eB) (rec w))) w))
    (exp he (mul eA (rec eB))) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp he (mul eA (rec eB) (rec w))) w))
      (send (cat (exp he (mul eA (rec eB) (rec w))) w))))
  (label 10)
  (parent 3)
  (unrealized (0 0) (0 1) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eB l l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1)))) (he (exp (gen) eB)) (l l-1)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e eB) ((exp (gen) eB) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 11)
  (parent 4)
  (unrealized (0 0) (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA eA l l-0 e expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) e e))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l-0)
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 12)
  (parent 4)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D self peer name) (hl base)
    (lA eA eB l l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB e))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3)) ((4 2) (1 3)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 13)
  (parent 4)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l-0)
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 14)
  (parent 4)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 15)
  (parent 4)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D self name) (lA eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB l-1))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 16)
  (parent 4)
  (unrealized (0 0) (0 1) (0 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (he base) (lA eA eB l l-0 expn)
    (w expr))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0))) (he he) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp he (mul eA (rec eB)))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp he (mul eA (rec eB) (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) key)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (eA eB) (he (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp he (mul eA (rec eB) (rec w))) w))
    (exp he (mul eA (rec eB))) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp he (mul eA (rec eB) (rec w))) w))
      (send (cat (exp he (mul eA (rec eB) (rec w))) w))))
  (label 17)
  (parent 4)
  (unrealized (0 0) (0 1) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C name) (eB e l expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) eB)) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e eB) ((exp (gen) eB) (gen)))
  (non-orig (privk B) (privk C) l)
  (uniq-gen eB e l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 18)
  (parent 5)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (eB l e l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0)))) (he (exp (gen) eB)) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l l (rec l-0))))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e eB) ((exp (gen) eB) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (uniq-gen eB l e l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 19)
  (parent 5)
  (unrealized (0 1))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C name) (eA e l expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk C) l)
  (uniq-gen eA e l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 20)
  (parent 6)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (eA l e l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) e e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) (mul l l (rec l-0)))))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (uniq-gen eA l e l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 21)
  (parent 6)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C self peer name) (hl base) (eA eB l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 2) (0 3)) ((3 2) (1 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk C) l-0)
  (uniq-gen eA eB e l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 22)
  (parent 7)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C self peer name) (hl base)
    (eA eB l l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB e))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 2) (0 3)) ((3 2) (1 3)) ((4 0) (0 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) (mul l l (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-1)
  (uniq-gen eA eB l e l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 23)
  (parent 7)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C name) (eA eB l expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk C) l)
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 24)
  (parent 8)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) eB l))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) (mul l l (rec l-0)))))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 25)
  (parent 8)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 3)) ((3 0) (1 3)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk C) l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 26)
  (parent 9)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self self) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) (mul l l (rec l-0)))))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk C) (privk self) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 27)
  (parent 9)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B C self name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 3)) ((3 0) (1 3)) ((4 0) (0 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 28)
  (parent 9)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (lA eA eB l expn) (w expr))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB w))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l)
  (precur (3 0))
  (operation nonce-test
    (contracted (he (exp (gen) (mul (rec eA) eB w)))) (gen) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 29)
  (parent 10)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (eB l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0)))) (he (exp (gen) (mul eB w)))
    (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l l (rec l-0))))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 30)
  (parent 10)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (lA eA expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) w e e))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 31)
  (parent 10)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C self peer name) (hl base) (lA eA eB l expn)
    (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB w e))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) lA)) (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (precur (3 0))
  (uniq-gen eA eB l e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 32)
  (parent 10)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (lA eA eB expn) (w expr) (l expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB w l))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 33)
  (parent 10)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C self name) (lA eA eB l expn) (w expr)
    (l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 34)
  (parent 10)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eB l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) eB)) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e eB) ((exp (gen) eB) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-0)
  (uniq-gen eB l e l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 35)
  (parent 11)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eB l e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) eB)) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e eB) ((exp (gen) eB) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (uniq-gen eB l e l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 36)
  (parent 11)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eB l l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1)))) (he (exp (gen) eB)) (l l-1)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e eB) ((exp (gen) eB) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (uniq-gen eB l l-0 e l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 37)
  (parent 11)
  (unrealized (0 1))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-0)
  (uniq-gen eA l e l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 38)
  (parent 12)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA l e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (uniq-gen eA l e l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 39)
  (parent 12)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eA l l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) e e))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l-0 (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l-0 l-1)
  (uniq-gen eA l l-0 e l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 40)
  (parent 12)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D self peer name) (hl base)
    (eA eB l l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB e))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3)) ((4 2) (1 3)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-1)
  (uniq-gen eA eB l e l-1)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 41)
  (parent 13)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D self peer name) (hl base)
    (eA eB l l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB e))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3)) ((4 2) (1 3)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-1)
  (uniq-gen eA eB l e l-1)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 42)
  (parent 13)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D self peer name) (hl base)
    (eA eB l l-0 l-1 e l-2 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-2))))
    (he (exp (gen) (mul (rec eA) eB e))) (l l-2) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 1 (self A) (l l-2))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3)) ((4 2) (1 3))
    ((5 0) (0 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-2))
    (eA (exp (gen) (mul l l-0 (rec l-2)))))
  (fn-of ("principal-of" (l-2 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-2)
  (uniq-gen eA eB l l-0 e l-2)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-2))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) A (privk A)))))
  (label 43)
  (parent 13)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 44)
  (parent 14)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 45)
  (parent 14)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l-0 (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l-0 l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 46)
  (parent 14)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 47)
  (parent 15)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 48)
  (parent 15)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l-0 (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 49)
  (parent 15)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D self name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 50)
  (parent 16)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D self name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 51)
  (parent 16)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D self name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self self) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB l-1))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l-0 (rec l-1)))))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk C) (privk D) (privk self) l l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 5 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 52)
  (parent 16)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B C D self name) (eA eB l l-0 l-1 l-2 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-2))))
    (he (exp (gen) (mul (rec eA) eB l-1))) (l l-2) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-2))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3))
    ((5 0) (0 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-2))
    (eA (exp (gen) (mul l l-0 (rec l-2)))))
  (fn-of ("principal-of" (l-2 A) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-2)
  (uniq-gen eA eB l l-0 l-1 l-2)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-2))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) A (privk A)))))
  (label 53)
  (parent 16)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA eA eB l l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB w))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (contracted (he (exp (gen) (mul (rec eA) eB w)))) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 54)
  (parent 17)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eB l l-0 expn) (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1)))) (he (exp (gen) (mul eB w)))
    (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 55)
  (parent 17)
  (unrealized (0 0) (0 1))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA eA l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) w e e))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 56)
  (parent 17)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D self peer name) (hl base)
    (lA eA eB l l-0 expn) (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB w e))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (precur (4 0))
  (uniq-gen eA eB l l-0 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 57)
  (parent 17)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 58)
  (parent 17)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 59)
  (parent 17)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D self name) (lA eA eB l l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB w l-1))) (l lA) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((exp (gen) (mul w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-1)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 60)
  (parent 17)
  (unrealized (0 0) (0 1) (0 3))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB e l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) eB)) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e eB) ((exp (gen) eB) (gen)))
  (non-orig (privk B) l)
  (uniq-gen eB e l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 61)
  (parent 18)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A B) (B B) (C B) (D B) (hl (exp (gen) l))
        (he (exp (gen) eB)) (lA l) (lB l) (eA e) (eB eB))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B name) (eA e l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (uniq-gen eA e l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 62)
  (parent 20)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (eA eB l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 2) (0 3)) ((3 2) (1 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) l-0)
  (uniq-gen eA eB e l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 63)
  (parent 22)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 64)
  (parent 24)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 3)) ((3 0) (1 3)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 65)
  (parent 26)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C name) (eA eB expn) (w expr) (l expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk B) (privk C) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 66)
  (parent 29)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) eB w))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) (mul l l (rec l-0)))))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 67)
  (parent 29)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C name) (eB expn) (w expr) (e l expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB w))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk B) (privk C) l)
  (precur (3 0))
  (uniq-gen eB e l)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 68)
  (parent 30)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (eB l expn) (w expr) (e l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0)))) (he (exp (gen) (mul eB w)))
    (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) (mul l l (rec l-0))))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (precur (3 0))
  (uniq-gen eB l e l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 69)
  (parent 30)
  (unrealized (0 1))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C name) (eA expn) (w expr) (e l expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) w e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk B) (privk C) l)
  (precur (3 0))
  (uniq-gen eA e l)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 70)
  (parent 31)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (eA expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) w e e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) (mul l l (rec l-0)))))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (precur (3 0))
  (uniq-gen eA l e l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 71)
  (parent 31)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C self peer name) (hl base) (eA eB expn) (w expr)
    (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk B) (privk C) l-0)
  (precur (3 0))
  (uniq-gen eA eB e l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 72)
  (parent 32)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C self peer name) (hl base) (eA eB l expn)
    (w expr) (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB w e))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0))
    ((5 0) (0 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) (mul l l (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-1)
  (precur (3 0))
  (uniq-gen eA eB l e l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 73)
  (parent 32)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C name) (eA eB expn) (w expr) (l expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) (privk C) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 74)
  (parent 33)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) eB w l))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) (mul l l (rec l-0)))))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 75)
  (parent 33)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C self name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) (privk C) l-0)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 76)
  (parent 34)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C self name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self self) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) (mul l l (rec l-0)))))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk B) (privk C) (privk self) l l-0)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 77)
  (parent 34)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B C self name) (eA eB l expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0))
    ((5 0) (0 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk A) (privk B) (privk C) l l-1)
  (precur (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 78)
  (parent 34)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB e l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) eB)) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e eB) ((exp (gen) eB) (gen)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eB e l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 79)
  (parent 35)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A B) (B B) (C D) (D D) (hl (exp (gen) l-0))
        (he (exp (gen) eB)) (lA l) (lB l) (eA e) (eB eB))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB e l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) eB)) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e eB) ((exp (gen) eB) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eB e l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 80)
  (parent 36)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A D) (B B) (C B) (D D) (hl (exp (gen) l-0))
        (he (exp (gen) eB)) (lA l) (lB l-0) (eA e) (eB eB))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA e l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA e l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 81)
  (parent 38)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA e l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA e l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 82)
  (parent 39)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (eA eB l e l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3)) ((4 2) (1 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (uniq-gen eA eB e l-0 l-1)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 83)
  (parent 41)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (eA eB l e l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3)) ((4 2) (1 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB e l-0 l-1)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 84)
  (parent 42)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 85)
  (parent 44)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 86)
  (parent 45)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 87)
  (parent 47)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 88)
  (parent 48)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 89)
  (parent 50)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 90)
  (parent 51)
  (unrealized (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 91)
  (parent 54)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 92)
  (parent 54)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eA eB l l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB w))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l-0 (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 93)
  (parent 54)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eB l expn) (w expr) (e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB w))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eB l e l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 94)
  (parent 55)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eB l expn) (w expr) (e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB w))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l e l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 95)
  (parent 55)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eB l l-0 expn) (w expr) (e l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1)))) (he (exp (gen) (mul eB w)))
    (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) (mul l l-0 (rec l-1))))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 96)
  (parent 55)
  (unrealized (0 1))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA l expn) (w expr) (e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) w e e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA l e l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 97)
  (parent 56)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) w e e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l e l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 98)
  (parent 56)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eA l expn) (w expr) (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) w e e))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l-0 (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 99)
  (parent 56)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D self peer name) (hl base) (eA eB l expn)
    (w expr) (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w e))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-1)
  (precur (4 0))
  (uniq-gen eA eB l e l-1)
  (operation encryption-test (displaced 6 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 100)
  (parent 57)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D self peer name) (hl base) (eA eB l expn)
    (w expr) (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w e))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen eA eB l e l-1)
  (operation encryption-test (displaced 6 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 101)
  (parent 57)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D self peer name) (hl base) (eA eB l l-0 expn)
    (w expr) (l-1 e l-2 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-2))))
    (he (exp (gen) (mul (rec eA) eB w e))) (l l-2) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 1 (self A) (l l-2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)) ((6 0) (0 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-2))
    (eA (exp (gen) (mul l l-0 (rec l-2)))))
  (fn-of ("principal-of" (l-2 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-2)
  (precur (4 0))
  (uniq-gen eA eB l l-0 e l-2)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-2))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) A (privk A)))))
  (label 102)
  (parent 57)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 103)
  (parent 58)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 104)
  (parent 58)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eA eB l expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l-0 (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 105)
  (parent 58)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 106)
  (parent 59)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 107)
  (parent 59)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eA eB l expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l-0 (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 108)
  (parent 59)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D self name) (eA eB l expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 6 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 109)
  (parent 60)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D self name) (eA eB l expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer C)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk B) (privk C) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 6 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 110)
  (parent 60)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D self name) (eA eB l l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self self) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB w l-1))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l-0 (rec l-1)))))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((exp (gen) (mul w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-1)) (gen)))
  (non-orig (privk B) (privk C) (privk D) (privk self) l l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 6 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 111)
  (parent 60)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B C D self name) (eA eB l l-0 expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-2))))
    (he (exp (gen) (mul (rec eA) eB w l-1))) (l l-2) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)) ((6 0) (0 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-2))
    (eA (exp (gen) (mul l l-0 (rec l-2)))))
  (fn-of ("principal-of" (l-2 A) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((exp (gen) (mul w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-1)) (gen)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-2)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1 l-2)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-2))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) A (privk A)))))
  (label 112)
  (parent 60)
  (unrealized (0 1) (0 3))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA e l expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) e e (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (uniq-gen eA e l)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) e e (rec w))) w))
    (exp (gen) (mul (rec eA) e e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul (rec eA) e e (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) e e (rec w))) w))))
  (label 113)
  (parent 62)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (eA eB l e l-0 expn)
    (w expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) (mul (rec eA) eB e (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (1 3)) ((3 2) (4 0)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) l-0)
  (uniq-gen eA eB e l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB e (rec w))) w))
    (exp (gen) (mul (rec eA) eB e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eA) eB e (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB e (rec w))) w))))
  (label 114)
  (parent 63)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
    (exp (gen) (mul (rec eA) eB l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))))
  (label 115)
  (parent 64)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (1 3)) ((3 0) (4 0)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) l-0)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
    (exp (gen) (mul (rec eA) eB l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))))
  (label 116)
  (parent 65)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB expn) (w expr) (l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 117)
  (parent 66)
  (unrealized (0 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (e l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB w))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB e l)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 118)
  (parent 68)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (e l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) w e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA e l)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 119)
  (parent 70)
  (unrealized (0 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (eA eB expn) (w expr)
    (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB e l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 120)
  (parent 72)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB expn) (w expr) (l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 121)
  (parent 74)
  (unrealized (0 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 122)
  (parent 76)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA e l l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec eA) e e (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA e l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) e e (rec w))) w))
    (exp (gen) (mul (rec eA) e e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) e e (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) e e (rec w))) w))))
  (label 123)
  (parent 81)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA e l l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) e e (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA e l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) e e (rec w))) w))
    (exp (gen) (mul (rec eA) e e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) e e (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) e e (rec w))) w))))
  (label 124)
  (parent 82)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (eA eB l e l-0 l-1 expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) (mul (rec eA) eB e (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (uniq-gen eA eB e l-0 l-1)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB e (rec w))) w))
    (exp (gen) (mul (rec eA) eB e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eA) eB e (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB e (rec w))) w))))
  (label 125)
  (parent 83)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (eA eB l e l-0 l-1 expn) (w expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) (mul (rec eA) eB e (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB e l-0 l-1)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB e (rec w))) w))
    (exp (gen) (mul (rec eA) eB e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eA) eB e (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB e (rec w))) w))))
  (label 126)
  (parent 84)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
    (exp (gen) (mul (rec eA) eB l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))))
  (label 127)
  (parent 85)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB l-0 (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB l-0 (rec w))) w))
    (exp (gen) (mul (rec eA) eB l-0)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) eB l-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB l-0 (rec w))) w))))
  (label 128)
  (parent 86)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec eA) eB l-0 (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB l-0 (rec w))) w))
    (exp (gen) (mul (rec eA) eB l-0)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) eB l-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB l-0 (rec w))) w))))
  (label 129)
  (parent 87)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
    (exp (gen) (mul (rec eA) eB l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))))
  (label 130)
  (parent 88)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (uniq-gen eA eB l l-0 l-1)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
    (exp (gen) (mul (rec eA) eB l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))))
  (label 131)
  (parent 89)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 expn) (w expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB l l-0 l-1)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
    (exp (gen) (mul (rec eA) eB l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB l (rec w))) w))))
  (label 132)
  (parent 90)
  (unrealized (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 133)
  (parent 91)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 134)
  (parent 92)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB expn) (w expr) (e l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB w))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB e l l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 135)
  (parent 94)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB expn) (w expr) (e l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB w))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB e l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 136)
  (parent 95)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA expn) (w expr) (e l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) w e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA e l l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 137)
  (parent 97)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA expn) (w expr) (e l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) w e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA e l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 138)
  (parent 98)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (eA eB expn) (w expr)
    (l e l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB w e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA eB e l-0 l-1)
  (operation encryption-test (displaced 6 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 139)
  (parent 100)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (eA eB expn) (w expr)
    (l e l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB w e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA eB e l-0 l-1)
  (operation encryption-test (displaced 6 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 140)
  (parent 101)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 141)
  (parent 103)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 142)
  (parent 104)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 143)
  (parent 106)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 144)
  (parent 107)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB expn) (w expr)
    (l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB w l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 6 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 145)
  (parent 109)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB expn) (w expr)
    (l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB w l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 6 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 146)
  (parent 110)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (uniq-gen l eA e)
  (precur (3 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e))) (gen) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 147)
  (parent 113)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e l e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e e (rec e-0)))) (l l)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (e-0 e)
    ((exp (gen) (mul e e (rec e-0))) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-1)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 148)
  (parent 113)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) e)))
      (send (cat (exp (gen) e) (mul (rec eA) e)))))
  (label 149)
  (parent 113)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 hl) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 150)
  (parent 113)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA e l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e e (rec l))))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA e l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) e e (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) e e (rec l))))))
  (label 151)
  (parent 113)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 152)
  (parent 113)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA eB e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (1 3)) ((3 2) (4 0)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) l-0)
  (uniq-gen l-0 eA eB e)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul (rec eA) eB e)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e)))))
  (label 153)
  (parent 114)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB e l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB e (rec e-0)))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (1 3)) ((3 2) (4 0)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e-0 eB)
    ((exp (gen) (mul eB e (rec e-0))) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eB e l-0 e-0)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB e (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))))
  (label 154)
  (parent 114)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA e l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (1 3)) ((3 2) (4 0)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl)
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0))) key)))
  (neq ((exp (gen) e) (gen)) (eA e-0)
    ((exp (gen) (mul (rec eA) e e-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA e l-0 e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e)))))
  (label 155)
  (parent 114)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (1 3)) ((3 2) (4 0)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (displaced 5 3 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB)))))
  (label 156)
  (parent 114)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 eA eB e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (1 3)) ((3 2) (4 0)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 157)
  (parent 114)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB e (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (1 3)) ((3 2) (4 0)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB e (rec l-0))))))
  (label 158)
  (parent 114)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (1 3)) ((3 2) (4 0)) ((4 1) (0 3))
    ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 159)
  (parent 114)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (precur (3 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-0 l) (w (mul (rec eA) eB l)))
    (gen) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB l)))
      (send (cat (gen) (mul (rec eA) eB l)))))
  (label 160)
  (parent 115)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l (rec e)))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB l (rec e) (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (e eB)
    ((exp (gen) (mul eB l (rec e))) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l (rec e) (rec e))))))
  (label 161)
  (parent 115)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) l e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l e))) key)))
  (neq ((exp (gen) l) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) l e)) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) l e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l)))
      (send (cat (exp (gen) e) (mul (rec eA) l)))))
  (label 162)
  (parent 115)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-0 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-0 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-0 (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 163)
  (parent 115)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB)))
      (send (cat (exp (gen) l) (mul (rec eA) eB)))))
  (label 164)
  (parent 115)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 165)
  (parent 115)
  (unrealized (0 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (1 3)) ((3 0) (4 0)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 166)
  (parent 116)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB l (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul eB l (rec e) (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (1 3)) ((3 0) (4 0)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (e eB)
    ((exp (gen) (mul eB l (rec e))) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l (rec e))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eB l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l (rec e) (rec e))))))
  (label 167)
  (parent 116)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) l e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (1 3)) ((3 0) (4 0)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l e))) key)))
  (neq ((exp (gen) l) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) l e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) l e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l)))
      (send (cat (exp (gen) e) (mul (rec eA) l)))))
  (label 168)
  (parent 116)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (1 3)) ((3 0) (4 0)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 169)
  (parent 116)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (1 3)) ((3 0) (4 0)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))))
  (label 170)
  (parent 116)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (1 3)) ((3 0) (4 0)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 171)
  (parent 116)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (1 3)) ((3 0) (4 0)) ((4 1) (0 3))
    ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 172)
  (parent 116)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) (e eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 173)
  (parent 117)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) eA))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) eA)) (send (cat (gen) eA))))
  (label 174)
  (parent 117)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA eB e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 175)
  (parent 117)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) l)))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l)) (gen)) (eA eB)
    ((exp (gen) l) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) l)))
      (send (cat (gen) (mul eA (rec eB) l)))))
  (label 176)
  (parent 117)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 177)
  (parent 117)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB expn) (w expr) (l expn) (w-0 expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))))
  (label 178)
  (parent 117)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (e l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB w))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk B) l)
  (uniq-gen eB e l)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 179)
  (parent 118)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A B) (B B) (C B) (D B) (hl (exp (gen) l))
        (he (exp (gen) (mul eB w))) (lA l) (lB l) (eA e) (eB eB))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B name) (e l e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e-0)) (l l) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) e-0 e-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0))) key)))
  (neq ((exp (gen) (mul (rec e) e-0 e-0)) (gen)) (e-0 e)
    ((exp (gen) e-0) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-1)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 180)
  (parent 119)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e))))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec e))))))
  (label 181)
  (parent 119)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) e-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 hl) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) e-0)) (gen)) (eA e)
    ((exp (gen) e-0) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 182)
  (parent 119)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA e l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) l))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l)))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((exp (gen) (mul eA (rec e) l)) (gen)) (eA e)
    ((exp (gen) l) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA e l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l)))))
  (label 183)
  (parent 119)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) l-0)) (gen)) (eA e)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 184)
  (parent 119)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (e l expn) (w-0 expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) w e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (deflistener (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) w e e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))))
  (label 185)
  (parent 119)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB e l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l-0) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) e-0 e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e-0 e-0))) key)))
  (neq ((exp (gen) (mul (rec eB) e-0 e-0)) (gen)) (e-0 eB)
    ((exp (gen) e-0) (gen)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eB e l-0 e-0)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-1)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 186)
  (parent 120)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA e l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) eA)) (l l-0) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl)
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e-0))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e-0) ((exp (gen) e-0) (gen)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA e l-0 e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 187)
  (parent 120)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (displaced 5 4 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 188)
  (parent 120)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 eA eB e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0))
    ((5 2) (0 3)) ((5 2) (3 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e-0)) (gen)) (eA eB)
    ((exp (gen) e-0) (gen)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 189)
  (parent 120)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 190)
  (parent 120)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0))
    ((5 0) (0 3)) ((5 0) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 191)
  (parent 120)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (eA eB expn) (w expr)
    (l e l-0 expn) (w-0 expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))))
  (label 192)
  (parent 120)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) (e eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))))
  (label 193)
  (parent 121)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l))))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l))))
      (send (cat (exp (gen) l) (mul eA (rec l))))))
  (label 194)
  (parent 121)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (0 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) e)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 195)
  (parent 121)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB))))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l)) (gen)) (eA eB)
    ((exp (gen) l) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB))))
      (send (cat (exp (gen) l) (mul eA (rec eB))))))
  (label 196)
  (parent 121)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 197)
  (parent 121)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB expn) (w expr) (l expn) (w-0 expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB w l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) w))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))))
  (label 198)
  (parent 121)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) e e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l-0)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) (e eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 199)
  (parent 122)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec l))))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l))))
      (send (cat (exp (gen) l) (mul eA (rec l)))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 200)
  (parent 122)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0))
    ((5 2) (0 3)) ((5 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 201)
  (parent 122)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 202)
  (parent 122)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 4 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 203)
  (parent 122)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0))
    ((5 0) (0 3)) ((5 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 204)
  (parent 122)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB expn) (w expr) (l l-0 expn)
    (w-0 expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))))
  (label 205)
  (parent 122)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA e)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e))) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 206)
  (parent 123)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e e (rec e-0)))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (e-0 e)
    ((exp (gen) (mul e e (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l e l-0 e-0)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 207)
  (parent 123)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) e)))
      (send (cat (exp (gen) e) (mul (rec eA) e)))))
  (label 208)
  (parent 123)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 209)
  (parent 123)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))))
  (label 210)
  (parent 123)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))))
  (label 211)
  (parent 123)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 212)
  (parent 123)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA e)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e))) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 213)
  (parent 124)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e l-0 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e e (rec e-0)))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (e-0 e)
    ((exp (gen) (mul e e (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l e l-0 e-0)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 214)
  (parent 124)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) e)))
      (send (cat (exp (gen) e) (mul (rec eA) e)))))
  (label 215)
  (parent 124)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 216)
  (parent 124)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))))
  (label 217)
  (parent 124)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))))
  (label 218)
  (parent 124)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 219)
  (parent 124)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA eB e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (uniq-gen l-0 l-1 eA eB e)
  (precur (5 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul (rec eA) eB e)))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e)))))
  (label 220)
  (parent 125)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eB e l-1 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB e (rec e-0)))) (l l-1)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e-0 eB)
    ((exp (gen) (mul eB e (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0))
  (uniq-gen l-0 eB e l-1 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB e (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))))
  (label 221)
  (parent 125)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA e l-1 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e-0))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e)))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0))) key)))
  (neq ((exp (gen) e) (gen)) (eA e-0)
    ((exp (gen) (mul (rec eA) e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0))
  (uniq-gen l-0 eA e l-1 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e)))))
  (label 222)
  (parent 125)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 6 4 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB)))))
  (label 223)
  (parent 125)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 224)
  (parent 125)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))))
  (label 225)
  (parent 125)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))))
  (label 226)
  (parent 125)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 227)
  (parent 125)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA eB e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen l-0 l-1 eA eB e)
  (precur (5 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul (rec eA) eB e)))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e)))))
  (label 228)
  (parent 126)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eB e l-1 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB e (rec e-0)))) (l l-1)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e-0 eB)
    ((exp (gen) (mul eB e (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (5 0))
  (uniq-gen l-0 eB e l-1 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB e (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))))
  (label 229)
  (parent 126)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA e l-1 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) e e-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e)))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0))) key)))
  (neq ((exp (gen) e) (gen)) (eA e-0)
    ((exp (gen) (mul (rec eA) e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (5 0))
  (uniq-gen l-0 eA e l-1 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e)))))
  (label 230)
  (parent 126)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 6 4 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB)))))
  (label 231)
  (parent 126)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (5 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 232)
  (parent 126)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (5 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))))
  (label 233)
  (parent 126)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (5 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))))
  (label 234)
  (parent 126)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (5 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 235)
  (parent 126)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 236)
  (parent 127)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-0 (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))))
  (label 237)
  (parent 127)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) l-0 e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 e))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) l-0 e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0)))))
  (label 238)
  (parent 127)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 239)
  (parent 127)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 240)
  (parent 127)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))))
  (label 241)
  (parent 127)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 242)
  (parent 127)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 243)
  (parent 128)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB l (rec e) (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (e eB)
    ((exp (gen) (mul eB l (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l (rec e) (rec e))))))
  (label 244)
  (parent 128)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) l-0 e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 e))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) l-0 e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0)))))
  (label 245)
  (parent 128)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 246)
  (parent 128)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 247)
  (parent 128)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))))
  (label 248)
  (parent 128)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 249)
  (parent 128)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 250)
  (parent 129)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB l (rec e) (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (e eB)
    ((exp (gen) (mul eB l (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l (rec e) (rec e))))))
  (label 251)
  (parent 129)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) l e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l e))) key)))
  (neq ((exp (gen) l) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) l e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) l e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l)))
      (send (cat (exp (gen) e) (mul (rec eA) l)))))
  (label 252)
  (parent 129)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 253)
  (parent 129)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))))
  (label 254)
  (parent 129)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 255)
  (parent 129)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 256)
  (parent 129)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 257)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-0 (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))))
  (label 258)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) l e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l e))) key)))
  (neq ((exp (gen) l) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) l e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) l e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l)))
      (send (cat (exp (gen) e) (mul (rec eA) l)))))
  (label 259)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 260)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))))
  (label 261)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 262)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 263)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1)))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB l-1)
  (precur (5 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul (rec eA) eB l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1)))))
  (label 264)
  (parent 131)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-0 (rec e)))) (l l-1)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))))
  (label 265)
  (parent 131)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) l-0 e))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 e))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) l-0 e)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0))
  (uniq-gen l eA l-0 l-1 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0)))))
  (label 266)
  (parent 131)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-2)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 267)
  (parent 131)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))))
  (label 268)
  (parent 131)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))))
  (label 269)
  (parent 131)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB)))))
  (label 270)
  (parent 131)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 271)
  (parent 131)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1)))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB l-1)
  (precur (5 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul (rec eA) eB l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1)))))
  (label 272)
  (parent 132)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-0 (rec e)))) (l l-1)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))))
  (label 273)
  (parent 132)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) l-0 e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 e))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) l-0 e)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0))
  (uniq-gen l eA l-0 l-1 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0)))))
  (label 274)
  (parent 132)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 275)
  (parent 132)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))))
  (label 276)
  (parent 132)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))))
  (label 277)
  (parent 132)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB)))))
  (label 278)
  (parent 132)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 279)
  (parent 132)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) e e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) (e eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 280)
  (parent 133)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) eA))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) eA)) (send (cat (gen) eA))))
  (label 281)
  (parent 133)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA eB e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 282)
  (parent 133)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 283)
  (parent 133)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 284)
  (parent 133)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 285)
  (parent 133)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn)
    (w-0 expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))))
  (label 286)
  (parent 133)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) (e eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 287)
  (parent 134)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) eA))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) eA)) (send (cat (gen) eA))))
  (label 288)
  (parent 134)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA eB e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 289)
  (parent 134)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 290)
  (parent 134)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 291)
  (parent 134)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 292)
  (parent 134)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn)
    (w-0 expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq ((exp (gen) w) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))))
  (label 293)
  (parent 134)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB expn) (w expr) (e l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB w))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eB e l l-0)
  (operation generalization deleted (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 294)
  (parent 135)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A B) (B B) (C D) (D D) (hl (exp (gen) l-0))
        (he (exp (gen) (mul eB w))) (lA l) (lB l) (eA e) (eB eB))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB expn) (w expr) (e l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB w))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (e eB)
    ((exp (gen) (mul eB w)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eB e l l-0)
  (operation generalization deleted (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 295)
  (parent 136)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A D) (B B) (C B) (D D) (hl (exp (gen) l-0))
        (he (exp (gen) (mul eB w))) (lA l) (lB l-0) (eA e) (eB eB))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e-0)) (l l-0) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) e-0 e-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0))) key)))
  (neq ((exp (gen) (mul (rec e) e-0 e-0)) (gen)) (e-0 e)
    ((exp (gen) e-0) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l e l-0 e-0)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-1)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 296)
  (parent 137)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec e))))))
  (label 297)
  (parent 137)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) e-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) e-0)) (gen)) (eA e)
    ((exp (gen) e-0) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 298)
  (parent 137)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) l-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) l-0)) (gen)) (eA e)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))))
  (label 299)
  (parent 137)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) l-0)) (gen)) (eA e)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))))
  (label 300)
  (parent 137)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) l-1))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec e) l-1)) (gen)) (eA e)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 301)
  (parent 137)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA expn) (w expr) (e l l-0 expn)
    (w-0 expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) w e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (deflistener (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) w e e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))))
  (label 302)
  (parent 137)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e l-0 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e-0)) (l l-0) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) e-0 e-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0))) key)))
  (neq ((exp (gen) (mul (rec e) e-0 e-0)) (gen)) (e-0 e)
    ((exp (gen) e-0) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l e l-0 e-0)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-1)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 303)
  (parent 138)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec e))))))
  (label 304)
  (parent 138)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) e-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) e-0)) (gen)) (eA e)
    ((exp (gen) e-0) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 305)
  (parent 138)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) l-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) l-0)) (gen)) (eA e)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))))
  (label 306)
  (parent 138)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) l-0)) (gen)) (eA e)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))))
  (label 307)
  (parent 138)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) l-1))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec e) l-1)) (gen)) (eA e)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 308)
  (parent 138)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA expn) (w expr) (e l l-0 expn)
    (w-0 expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) w e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (deflistener (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) w e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) w e e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))))
  (label 309)
  (parent 138)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eB e l-1 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l-1) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) e-0 e-0))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0))) key)))
  (neq ((exp (gen) (mul (rec eB) e-0 e-0)) (gen)) (e-0 eB)
    ((exp (gen) e-0) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l-0 eB e l-1 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 310)
  (parent 139)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA e l-1 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) eA)) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e-0))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e-0) ((exp (gen) e-0) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l-0 eA e l-1 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 311)
  (parent 139)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 6 5 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 312)
  (parent 139)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)) ((6 2) (0 3)) ((6 2) (4 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e-0)) (gen)) (eA eB)
    ((exp (gen) e-0) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 313)
  (parent 139)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 314)
  (parent 139)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 315)
  (parent 139)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-2)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)) ((6 0) (0 3)) ((6 0) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-2)) (gen)) (eA eB)
    ((exp (gen) l-2) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 316)
  (parent 139)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (eA eB expn) (w expr)
    (l e l-0 l-1 expn) (w-0 expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB w e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA eB e l-0 l-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))))
  (label 317)
  (parent 139)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eB e l-1 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l-1) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) e-0 e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0))) key)))
  (neq ((exp (gen) (mul (rec eB) e-0 e-0)) (gen)) (e-0 eB)
    ((exp (gen) e-0) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 eB e l-1 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 318)
  (parent 140)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA e l-1 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) e-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) eA)) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e-0))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e-0) ((exp (gen) e-0) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 eA e l-1 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 319)
  (parent 140)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 6 5 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 320)
  (parent 140)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) e-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) e-0))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)) ((6 2) (0 3)) ((6 2) (4 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e-0)) (gen)) (eA eB)
    ((exp (gen) e-0) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 321)
  (parent 140)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 322)
  (parent 140)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 323)
  (parent 140)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) l-2)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-2))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)) ((6 0) (0 3)) ((6 0) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-2)) (gen)) (eA eB)
    ((exp (gen) l-2) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 324)
  (parent 140)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (eA eB expn) (w expr)
    (l e l-0 l-1 expn) (w-0 expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB w e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((exp (gen) (mul w e)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA eB e l-0 l-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))))
  (label 325)
  (parent 140)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) e e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) (e eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))))
  (label 326)
  (parent 141)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0))))))
  (label 327)
  (parent 141)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 328)
  (parent 141)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 329)
  (parent 141)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))))
  (label 330)
  (parent 141)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 331)
  (parent 141)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn)
    (w-0 expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))))
  (label 332)
  (parent 141)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) (e eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))))
  (label 333)
  (parent 142)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0))))))
  (label 334)
  (parent 142)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 335)
  (parent 142)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 336)
  (parent 142)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))))
  (label 337)
  (parent 142)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 338)
  (parent 142)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn)
    (w-0 expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w l-0)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w l-0 (rec w-0))) w-0))))
  (label 339)
  (parent 142)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) e e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) (e eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))))
  (label 340)
  (parent 143)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l))))
      (send (cat (exp (gen) l) (mul eA (rec l))))))
  (label 341)
  (parent 143)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 342)
  (parent 143)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))))
  (label 343)
  (parent 143)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 344)
  (parent 143)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 345)
  (parent 143)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn)
    (w-0 expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w l-0)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w l-0 (rec w-0))) w-0))))
  (label 346)
  (parent 143)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) (e eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))))
  (label 347)
  (parent 144)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l))))
      (send (cat (exp (gen) l) (mul eA (rec l))))))
  (label 348)
  (parent 144)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 349)
  (parent 144)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))))
  (label 350)
  (parent 144)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 351)
  (parent 144)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 352)
  (parent 144)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn)
    (w-0 expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) w))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))))
  (label 353)
  (parent 144)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) e e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) (e eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 354)
  (parent 145)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 355)
  (parent 145)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)) ((6 2) (0 3)) ((6 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 356)
  (parent 145)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 357)
  (parent 145)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 358)
  (parent 145)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 359)
  (parent 145)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-2)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)) ((6 0) (0 3)) ((6 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-2)) (gen)) (eA eB)
    ((exp (gen) l-2) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 360)
  (parent 145)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB expn) (w expr)
    (l l-0 l-1 expn) (w-0 expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB w l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))))
  (label 361)
  (parent 145)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) (e eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 362)
  (parent 146)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) eA)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) eA) (gen)) (eA e) ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 363)
  (parent 146)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)) ((6 2) (0 3)) ((6 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) (eA eB)
    ((exp (gen) e) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 364)
  (parent 146)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 365)
  (parent 146)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 366)
  (parent 146)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 367)
  (parent 146)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-2)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)) ((6 0) (0 3)) ((6 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-2)) (gen)) (eA eB)
    ((exp (gen) l-2) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 368)
  (parent 146)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB expn) (w expr)
    (l l-0 l-1 expn) (w-0 expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB w l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w l))) key)))
  (neq ((exp (gen) (mul w l)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB w l)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))))
  (label 369)
  (parent 146)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA eB expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) w)) (w-1 w))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 370)
  (parent 178)
  (unrealized (1 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w e e)))
      (send (cat (gen) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 371)
  (parent 178)
  (unrealized (1 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 372)
  (parent 178)
  (unrealized (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) w e)))
      (send (cat (gen) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 373)
  (parent 178)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB expn) (w expr) (l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w l)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) w l)))
      (send (cat (gen) (mul eA (rec eB) w l))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 374)
  (parent 178)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) w l-0)))
      (send (cat (gen) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 375)
  (parent 178)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA e expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w-0 (mul eA (rec e) (rec e) w)) (e-0 e)
      (w-1 w)) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 376)
  (parent 185)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (e expn) (w expr) (l e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e-0))) (l l) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w e-0 e-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0))) key)))
  (neq ((exp (gen) (mul (rec e) w e-0 e-0)) (gen)) (e-0 e)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0 e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 377)
  (parent 185)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w (rec e))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w (rec e))))
      (send (cat (exp (gen) e) (mul eA w (rec e)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 378)
  (parent 185)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w e-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 hl) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 379)
  (parent 185)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA e expn) (w expr) (l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w l))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l))) key)))
  (neq ((exp (gen) (mul eA (rec e) w l)) (gen)) (eA e)
    ((exp (gen) (mul w l)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 380)
  (parent 185)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 381)
  (parent 185)
  (unrealized (1 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA eB e expn)
    (w expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec e) w))
      (e-0 e) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 382)
  (parent 192)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB e expn) (w expr)
    (l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0))) (l l-0) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e-0 e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul (rec eB) w e-0 e-0)) (gen)) (e-0 eB)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB e l-0 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 383)
  (parent 192)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA e expn) (w expr)
    (l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-0) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl)
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e-0)))
        key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e-0)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 384)
  (parent 192)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB expn) (w expr)
    (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (displaced 6 4 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 385)
  (parent 192)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 eA eB e expn) (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 386)
  (parent 192)
  (unrealized (1 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB e expn)
    (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 387)
  (parent 192)
  (unrealized (1 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e expn) (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 388)
  (parent 192)
  (unrealized (1 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec l) w))
      (l-0 l) (w-1 w)) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 389)
  (parent 198)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eB) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 390)
  (parent 198)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA w (rec l))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA w (rec l))))
      (send (cat (exp (gen) l) (mul eA w (rec l)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 391)
  (parent 198)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB l-0 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 392)
  (parent 198)
  (unrealized (1 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB expn) (w expr) (l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 393)
  (parent 198)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 394)
  (parent 198)
  (unrealized (1 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 395)
  (parent 205)
  (unrealized (1 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 396)
  (parent 205)
  (unrealized (1 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 397)
  (parent 205)
  (unrealized (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (hl base)
    (l l-0 eA eB l-1 expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 398)
  (parent 205)
  (unrealized (1 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 399)
  (parent 205)
  (unrealized (1 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 400)
  (parent 205)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 401)
  (parent 205)
  (unrealized (1 3) (3 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) w)) (w-1 w))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 402)
  (parent 286)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w e e)))
      (send (cat (gen) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 403)
  (parent 286)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 404)
  (parent 286)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w e)))
      (send (cat (gen) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 405)
  (parent 286)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w l-0)))
      (send (cat (gen) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 406)
  (parent 286)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w l-0)))
      (send (cat (gen) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 407)
  (parent 286)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w l-1)))
      (send (cat (gen) (mul eA (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 408)
  (parent 286)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB expn) (w expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) w)) (w-1 w))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 409)
  (parent 293)
  (unrealized (1 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w e e)))
      (send (cat (gen) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 410)
  (parent 293)
  (unrealized (1 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 411)
  (parent 293)
  (unrealized (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w e)))
      (send (cat (gen) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 412)
  (parent 293)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w l-0)))
      (send (cat (gen) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 413)
  (parent 293)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w l-0)))
      (send (cat (gen) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 414)
  (parent 293)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w l-1)))
      (send (cat (gen) (mul eA (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 415)
  (parent 293)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA e expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w-0 (mul eA (rec e) (rec e) w)) (e-0 e)
      (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 416)
  (parent 302)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e-0))) (l l-0) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w e-0 e-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0))) key)))
  (neq ((exp (gen) (mul (rec e) w e-0 e-0)) (gen)) (e-0 e)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0 e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 417)
  (parent 302)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w (rec e))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w (rec e))))
      (send (cat (exp (gen) e) (mul eA w (rec e)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 418)
  (parent 302)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA e expn)
    (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w e-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 419)
  (parent 302)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w l-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 420)
  (parent 302)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 421)
  (parent 302)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w l-1))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec e) w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 422)
  (parent 302)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA e expn) (w expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w-0 (mul eA (rec e) (rec e) w)) (e-0 e)
      (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 423)
  (parent 309)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e-0))) (l l-0) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w e-0 e-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0))) key)))
  (neq ((exp (gen) (mul (rec e) w e-0 e-0)) (gen)) (e-0 e)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 424)
  (parent 309)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w (rec e))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w (rec e))))
      (send (cat (exp (gen) e) (mul eA w (rec e)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 425)
  (parent 309)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA e expn)
    (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w e-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 426)
  (parent 309)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w l-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 427)
  (parent 309)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 428)
  (parent 309)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w l-1))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec e) w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 429)
  (parent 309)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA eB e expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec e) w))
      (e-0 e) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 430)
  (parent 317)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0))) (l l-1) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e-0 e-0)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul (rec eB) w e-0 e-0)) (gen)) (e-0 eB)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 e-0)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 431)
  (parent 317)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0)))
        key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e-0)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA e l-1 e-0)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 432)
  (parent 317)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 7 5 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 433)
  (parent 317)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e expn) (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 434)
  (parent 317)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB e expn)
    (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 435)
  (parent 317)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB e expn)
    (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 436)
  (parent 317)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e expn) (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-2)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 437)
  (parent 317)
  (unrealized (1 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA eB e expn) (w expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec e) w))
      (e-0 e) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 438)
  (parent 325)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0))) (l l-1) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e-0 e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul (rec eB) w e-0 e-0)) (gen)) (e-0 eB)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 e-0)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 439)
  (parent 325)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0)))
        key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e-0)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA e l-1 e-0)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 440)
  (parent 325)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 7 5 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 441)
  (parent 325)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e expn) (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e-0)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 442)
  (parent 325)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB e expn)
    (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 443)
  (parent 325)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB e expn)
    (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 444)
  (parent 325)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e expn) (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-2)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 445)
  (parent 325)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 446)
  (parent 332)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 447)
  (parent 332)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w (rec l-0))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA w (rec l-0)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 448)
  (parent 332)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB l-1 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 449)
  (parent 332)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 450)
  (parent 332)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 451)
  (parent 332)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 452)
  (parent 332)
  (unrealized (1 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 453)
  (parent 339)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 454)
  (parent 339)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w (rec l-0))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA w (rec l-0)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 455)
  (parent 339)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB l-1 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 456)
  (parent 339)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 457)
  (parent 339)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 458)
  (parent 339)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 459)
  (parent 339)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 460)
  (parent 346)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 461)
  (parent 346)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec l) w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 462)
  (parent 346)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB l-1 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 463)
  (parent 346)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 464)
  (parent 346)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 465)
  (parent 346)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 466)
  (parent 346)
  (unrealized (1 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 467)
  (parent 353)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 468)
  (parent 353)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec l) w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 469)
  (parent 353)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB l-1 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 470)
  (parent 353)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 471)
  (parent 353)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 472)
  (parent 353)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 473)
  (parent 353)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn) (w expr))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 474)
  (parent 361)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 475)
  (parent 361)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 476)
  (parent 361)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 477)
  (parent 361)
  (unrealized (1 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 478)
  (parent 361)
  (unrealized (1 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 479)
  (parent 361)
  (unrealized (1 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 480)
  (parent 361)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 expn)
    (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 481)
  (parent 361)
  (unrealized (1 3) (4 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn) (w expr))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 482)
  (parent 369)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 483)
  (parent 369)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 484)
  (parent 369)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 485)
  (parent 369)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 486)
  (parent 369)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 487)
  (parent 369)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 488)
  (parent 369)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 expn)
    (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 489)
  (parent 369)
  (unrealized (1 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 490)
  (parent 370)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 491)
  (parent 370)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 492)
  (parent 370)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB (rec e) l-0))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul eB (rec e) l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 493)
  (parent 371)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expr) (l e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (mul w e e)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-absence eB (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 494)
  (parent 371)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (mul (rec eB) w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-absence e (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 495)
  (parent 371)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eA l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 496)
  (parent 372)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expr) (l e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eA w) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-absence eA w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 497)
  (parent 372)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-listener eA) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv eA) (send eA)))
  (label 498)
  (parent 372)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 499)
  (parent 373)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eB expn) (w expr)
    (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eA (mul (rec eB) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w e))
    (mul (rec eB) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w e)))
      (send (cat (gen) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 500)
  (parent 373)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA expn) (w expr)
    (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eB (mul eA w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-absence eB (mul eA w e)) (mul eA w e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w e))) (send (cat (gen) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 501)
  (parent 373)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (mul eA (rec eB) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 502)
  (parent 373)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 503)
  (parent 374)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (l eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eA (mul (rec eB) w l)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((exp (gen) (mul (rec eB) w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l))
    (mul (rec eB) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w l)))
      (send (cat (gen) (mul (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec eA))))
      (send (cat (exp (gen) l) (mul w (rec eA))))))
  (label 504)
  (parent 374)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (l eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w l)))
  (deflistener (cat (exp (gen) l) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (mul eA w l)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l eB))) key)))
  (neq ((exp (gen) (mul eA w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-absence eB (mul eA w l)) (mul eA w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w l))) (send (cat (gen) (mul eA w l))))
    ((recv (cat (exp (gen) l) (mul w eB)))
      (send (cat (exp (gen) l) (mul w eB)))))
  (label 505)
  (parent 374)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB expn) (w expr) (l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (l (mul eA (rec eB) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (added-absence l (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l))))))
  (label 506)
  (parent 374)
  (unrealized (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 507)
  (parent 375)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 508)
  (parent 375)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eA (mul (rec eB) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 509)
  (parent 375)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (mul eA w l-0)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 eB))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 510)
  (parent 375)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 511)
  (parent 375)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 512)
  (parent 376)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (gen) (mul (rec eA) e e l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((exp (gen) (mul e l-0)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (gen) (mul (rec eA) e e l-0)))
      (send (cat (gen) (mul (rec eA) e e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 513)
  (parent 376)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l e expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eA (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 514)
  (parent 376)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (gen) (mul w e e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (mul eA w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e)
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e)))))
  (label 515)
  (parent 376)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e e (rec e-0)))) (l l)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (e-0 e)
    ((exp (gen) (mul e e (rec e-0))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 516)
  (parent 377)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e e-0 l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e e (rec e-0) l-0))) (l l)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((exp (gen) (mul e l-0)) (gen)) (e-0 e)
    ((exp (gen) (mul e e (rec e-0) l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 517)
  (parent 377)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expr) (l e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e-0 e-0))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e e-0))) (l l) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul w e e e-0)) (gen)) (e e-0)
    ((exp (gen) (mul w e e-0 e-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 518)
  (parent 377)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (e expn) (w expr) (l e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e-0)))) (l l) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (e-0 e)
    ((exp (gen) (mul w (rec e-0))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 519)
  (parent 377)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul (rec eA) e)) (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) e)))
      (send (cat (exp (gen) e) (mul (rec eA) e)))))
  (label 520)
  (parent 378)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((exp (gen) (mul e l-0)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 521)
  (parent 378)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expr) (l e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eA (mul w (rec e))) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-absence eA (mul w (rec e)))
    (mul w (rec e)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 522)
  (parent 378)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (mul eA w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))))
  (label 523)
  (parent 378)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 hl) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul (rec eA) e e (rec e-0)))
      (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 524)
  (parent 379)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA e e-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 hl) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 525)
  (parent 379)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 e expn) (w expr)
    (e-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w e-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eA (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 hl) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) (mul (rec e) w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 526)
  (parent 379)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA expn) (w expr)
    (e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e-0 e-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e e-0))) (l l) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul eA w e)) (e (exp (gen) l-0)) (e hl)
    (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul eA w e e-0)) (gen)) (eA e-0)
    ((exp (gen) (mul w e e-0 e-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (operation nonce-test (added-absence e-0 (mul eA w e)) (mul eA w e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eA w e)))
      (send (cat (exp (gen) e-0) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 527)
  (parent 379)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul eA (rec e) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 hl) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (operation nonce-test (added-absence e-0 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 528)
  (parent 379)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA e l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e e (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e (rec l)))
      (l-0 l)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l) (mul (rec eA) e e (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) e e (rec l))))))
  (label 529)
  (parent 380)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA e l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e e (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((exp (gen) (mul e l-0)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec eA) e e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e e (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 530)
  (parent 380)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e expn) (w expr) (l eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w l))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eA (mul (rec e) (rec e) w l)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((exp (gen) (mul (rec e) w l)) (gen)) (eA e)
    ((exp (gen) (mul w l (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w l))
    (mul (rec e) (rec e) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec eA))))
      (send (cat (exp (gen) l) (mul w (rec eA))))))
  (label 531)
  (parent 380)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l)))
  (deflistener (cat (exp (gen) l) (mul w e e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (mul eA w l)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l e e))) key)))
  (neq ((exp (gen) (mul eA w l e)) (gen)) (eA e)
    ((exp (gen) (mul w l e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-absence e (mul eA w l)) (mul eA w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w l)))
      (send (cat (exp (gen) e) (mul eA w l))))
    ((recv (cat (exp (gen) l) (mul w e e)))
      (send (cat (exp (gen) l) (mul w e e)))))
  (label 532)
  (parent 380)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA e expn) (w expr) (l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (l (mul eA (rec e) (rec e) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (operation nonce-test (added-absence l (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l))))))
  (label 533)
  (parent 380)
  (unrealized (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 534)
  (parent 381)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((exp (gen) (mul e l-0)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 535)
  (parent 381)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA e l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 536)
  (parent 381)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eA (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 537)
  (parent 381)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0 e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (mul eA w l-0)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-absence e (mul eA w l-0)) (mul eA w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 538)
  (parent 381)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul eA (rec e) (rec e) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 539)
  (parent 381)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA eB e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul (rec eA) eB e)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e)))))
  (label 540)
  (parent 382)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul (rec eA) eB e l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e l-1)))
        key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-1)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul (rec eA) eB e l-1)))
      (send (cat (gen) (mul (rec eA) eB e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 541)
  (parent 382)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 542)
  (parent 382)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (eB (mul eA (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w))
    (mul eA (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 543)
  (parent 382)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w e)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (mul eA (rec eB) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e)))))
  (label 544)
  (parent 382)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eB e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB e (rec e-0)))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e-0 eB)
    ((exp (gen) (mul eB e (rec e-0))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e) (w (mul eB e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB e (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))))
  (label 545)
  (parent 383)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eB e e-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB e (rec e-0) l-1)))
    (l l-0) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e l-1)))
        key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (e-0 eB)
    ((exp (gen) (mul eB e (rec e-0) l-1)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB e (rec e-0) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 546)
  (parent 383)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l e expn) (w expr)
    (l-0 e-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 eB))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (eB (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0 eB)))
        key)))
  (neq ((exp (gen) (mul w e-0 e-0)) (gen)) (e-0 eB)
    ((exp (gen) (mul w e-0 eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB)))))
  (label 547)
  (parent 383)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB expn) (w expr)
    (l-0 e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) w e e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e-0 (mul (rec eB) w e e)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 hl) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((exp (gen) (mul (rec eB) w e e e-0)) (gen)) (e eB)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l-0 e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eB) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 548)
  (parent 383)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB e expn) (w expr)
    (l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec e-0)))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e-0 (mul (rec eB) (rec e) w)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e-0 eB)
    ((exp (gen) (mul w (rec e-0))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB e l-0 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 549)
  (parent 383)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 e eA e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) e-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 hl)
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0))) key)))
  (neq ((exp (gen) e-0) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) e-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e eA e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e-0) (w (mul (rec eA) e-0))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) e-0)))))
  (label 550)
  (parent 384)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 e eA e-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) e-0 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e-0 l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 hl)
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0 l-1)))
        key)))
  (neq ((exp (gen) (mul e-0 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) e-0 l-1)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e eA e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) e-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 551)
  (parent 384)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l e expn) (w expr)
    (l-0 e-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (eA (mul (rec e) w)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) w) (gen)) (eA e-0)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) w))
    (mul (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA))))))
  (label 552)
  (parent 384)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA expn) (w expr)
    (l-0 e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e-0 (mul eA w)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 hl) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e e-0)))
        key)))
  (neq ((exp (gen) (mul eA w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (operation nonce-test (added-absence e-0 (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eA w)))
      (send (cat (exp (gen) e-0) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 553)
  (parent 384)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 e eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e eB))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul e (rec eA) eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB)))))
  (label 554)
  (parent 385)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 e eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e eB l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul e (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 555)
  (parent 385)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB expn) (w expr)
    (l-0 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) w)))
      (send (cat (exp (gen) e) (mul (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 556)
  (parent 385)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA expn) (w expr)
    (l-0 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (eB (mul eA w)) (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 557)
  (parent 385)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 eA eB e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-1 e)
      (w (mul (rec eA) eB e (rec e-0))) (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 558)
  (parent 386)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 self-1 name)
    (hl hl-0 base) (l l-0 l-1 eA eB e e-0 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-2)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 559)
  (parent 386)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 eB e expn) (w expr) (e-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eA (mul (rec eB) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 hl-0) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w e-0))
    (mul (rec eB) (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 560)
  (parent 386)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 eA e expn) (w expr) (e-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eB (mul eA (rec e) w e-0)) (e-0 (exp (gen) l-1)) (e-0 hl-0)
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0 eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w e-0))
    (mul eA (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 561)
  (parent 386)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 eA eB expn) (w expr) (e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) w e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eA (rec eB) w e)) (e (exp (gen) l-1)) (e hl-0)
    (e-0 (exp (gen) l)) (e-0 hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e e-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) w e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 562)
  (parent 386)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 eA eB e expn) (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eA (rec eB) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 hl-0) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 563)
  (parent 386)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB e (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul (rec eA) eB e (rec l-0))) (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB e (rec l-0))))))
  (label 564)
  (parent 387)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l eA eB e l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e l-1)))
        key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-1)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 565)
  (parent 387)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB e expn) (w expr)
    (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) (rec e) w l-0)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB e l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w l-0))
    (mul (rec eB) (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 566)
  (parent 387)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA e expn) (w expr)
    (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (eB (mul eA (rec e) w l-0)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w l-0))
    (mul eA (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 567)
  (parent 387)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB expn) (w expr)
    (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0 e)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) (mul w e)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (e (mul eA (rec eB) w l-0)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-0 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w e)))
      (send (cat (exp (gen) l-0) (mul w e)))))
  (label 568)
  (parent 387)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB e expn)
    (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 569)
  (parent 387)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul (rec eA) eB e (rec l-1))) (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 570)
  (parent 388)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e l-1)))
        key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-1)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 571)
  (parent 388)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (hl base)
    (l l-0 eA eB e l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 572)
  (parent 388)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eA (mul (rec eB) (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w l-1))
    (mul (rec eB) (rec e) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 573)
  (parent 388)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eB (mul eA (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w l-1))
    (mul eA (rec e) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 574)
  (parent 388)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB expn) (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1 e)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (mul eA (rec eB) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-1 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w l-1))
    (mul eA (rec eB) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 575)
  (parent 388)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e expn) (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 576)
  (parent 388)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul (rec eA) eB l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-0 l) (w (mul (rec eA) eB l)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul (rec eA) eB l)))
      (send (cat (gen) (mul (rec eA) eB l)))))
  (label 577)
  (parent 389)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l l-0))) key)))
  (neq ((exp (gen) (mul l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (gen) (mul (rec eA) eB l l-0)))
      (send (cat (gen) (mul (rec eA) eB l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 578)
  (parent 389)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l) w))
    (mul (rec eB) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 579)
  (parent 389)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w)))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (mul eA (rec l) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-absence eB (mul eA (rec l) w))
    (mul eA (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec l) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 580)
  (parent 389)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB expn) (w expr) (l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (l (mul eA (rec eB) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (added-absence l (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l))) (send (cat (gen) (mul w l)))))
  (label 581)
  (parent 389)
  (unrealized (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l (rec e)))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) e) (mul eB l (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (e eB)
    ((exp (gen) (mul eB l (rec e))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul eB l (rec e) (rec e))) (l-0 l)
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) e) (mul eB l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l (rec e) (rec e))))))
  (label 582)
  (parent 390)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l (rec e) l-0))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul eB l (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l l-0))) key)))
  (neq ((exp (gen) (mul l l-0)) (gen)) (e eB)
    ((exp (gen) (mul eB l (rec e) l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l (rec e) l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul eB l (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB l (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 583)
  (parent 390)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expr) (l e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (mul w (rec l) e e)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-absence eB (mul w (rec l) e e))
    (mul w (rec l) e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 584)
  (parent 390)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (e l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e l))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (l (mul (rec eB) w e e)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l)) (gen)) (e eB)
    ((exp (gen) (mul w e l)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l)
  (operation nonce-test (added-absence l (mul (rec eB) w e e))
    (mul (rec eB) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l)))
      (send (cat (exp (gen) e) (mul w l)))))
  (label 585)
  (parent 390)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (mul (rec eB) w (rec l))) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-absence e (mul (rec eB) w (rec l)))
    (mul (rec eB) w (rec l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eB) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 586)
  (parent 390)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (e eA l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l))) key)))
  (neq ((exp (gen) l) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e eA l)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul (rec eA) l)) (l-0 l)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l)))
      (send (cat (exp (gen) e) (mul (rec eA) l)))))
  (label 587)
  (parent 391)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (e eA l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l l-0))) key)))
  (neq ((exp (gen) (mul l l-0)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e eA l l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 588)
  (parent 391)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expr) (l e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eA (mul w (rec l))) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-absence eA (mul w (rec l)))
    (mul w (rec l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 589)
  (parent 391)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (e l expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (l (mul eA w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e l))) key)))
  (neq ((exp (gen) (mul eA w l)) (gen)) (eA e)
    ((exp (gen) (mul w e l)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (operation nonce-test (added-absence l (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA w)))
      (send (cat (exp (gen) l) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l)))
      (send (cat (exp (gen) e) (mul w l)))))
  (label 590)
  (parent 391)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-0 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0)
      (w (mul (rec eA) eB l-0 (rec e))) (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-0 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-0 (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 591)
  (parent 392)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l eA eB l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-0 (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-0 (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-0 (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 592)
  (parent 392)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB l-0 expn) (w expr)
    (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eA (mul (rec eB) (rec l-0) w e)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eB l-0 e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w e))
    (mul (rec eB) (rec l-0) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 593)
  (parent 392)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA l-0 expn) (w expr)
    (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eB (mul eA (rec l-0) w e)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e eB)))
        key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA l-0 e eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w e))
    (mul eA (rec l-0) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 594)
  (parent 392)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB expn) (w expr)
    (e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e l-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (l-0 (mul eA (rec eB) w e)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e l-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 595)
  (parent 392)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB l-0 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (mul eA (rec eB) (rec l-0) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 596)
  (parent 392)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l eB))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul l (rec eA) eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB)))
      (send (cat (exp (gen) l) (mul (rec eA) eB)))))
  (label 597)
  (parent 393)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l eB l-0))) key)))
  (neq ((exp (gen) (mul l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul l (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 598)
  (parent 393)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (l eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((exp (gen) (mul (rec eB) w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) w)))
      (send (cat (exp (gen) l) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l) (mul w (rec eA))))
      (send (cat (exp (gen) l) (mul w (rec eA))))))
  (label 599)
  (parent 393)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (l eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA w)))
  (deflistener (cat (exp (gen) l) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l eB))) key)))
  (neq ((exp (gen) (mul eA w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA w)))
      (send (cat (exp (gen) l) (mul eA w))))
    ((recv (cat (exp (gen) l) (mul w eB)))
      (send (cat (exp (gen) l) (mul w eB)))))
  (label 600)
  (parent 393)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul (rec eA) eB l (rec l-0))) (l-2 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 601)
  (parent 394)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l l-0))) key)))
  (neq ((exp (gen) (mul l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 602)
  (parent 394)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l l-1))) key)))
  (neq ((exp (gen) (mul l l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 603)
  (parent 394)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eA (mul (rec eB) (rec l) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l) w l-0))
    (mul (rec eB) (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 604)
  (parent 394)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (mul eA (rec l) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 eB))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l) w l-0))
    (mul eA (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 605)
  (parent 394)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l l-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul eA (rec eB) w l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l l-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w l))
    (mul eA (rec eB) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 606)
  (parent 394)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul eA (rec eB) (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) (rec l) w))
    (mul eA (rec eB) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 607)
  (parent 394)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 608)
  (parent 395)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0 l-0))) key)))
  (neq ((exp (gen) (mul l-0 l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (gen) (mul (rec eA) eB l-0 l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 609)
  (parent 395)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0 l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul (rec eA) eB l-0 l-1)))
      (send (cat (gen) (mul (rec eA) eB l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 610)
  (parent 395)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB l-0 expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 611)
  (parent 395)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA l-0 expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (mul eA (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 612)
  (parent 395)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 613)
  (parent 395)
  (unrealized (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-0 (rec e)))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test
    (contracted (eB-0 eB) (l-1 l-0) (w (mul eB l-0 (rec e) (rec e)))
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))))
  (label 614)
  (parent 396)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB (rec e) l-0 l-0))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0 l-0))) key)))
  (neq ((exp (gen) (mul l-0 l-0)) (gen)) (e eB)
    ((exp (gen) (mul eB (rec e) l-0 l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0 l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 615)
  (parent 396)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-0 (rec e) l-1))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0 l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e) l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0 (rec e) l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 616)
  (parent 396)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (mul (rec l) w e e)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eB)))
        key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul (rec l) w e e))
    (mul (rec l) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 617)
  (parent 396)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (l-0 (mul (rec eB) w e e)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 618)
  (parent 396)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (e (mul (rec eB) (rec l) w)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eB) (rec l) w))
    (mul (rec eB) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 619)
  (parent 396)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eA l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (l-1 l-0) (w (mul (rec eA) l-0))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0)))))
  (label 620)
  (parent 397)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eA l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l-0 l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-0))) key)))
  (neq ((exp (gen) (mul l-0 l-0)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0 l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0 l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0 l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 621)
  (parent 397)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e eA l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l-0 l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0 l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 622)
  (parent 397)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eA (mul (rec l) w)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul (rec l) w))
    (mul (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 623)
  (parent 397)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (l-0 (mul eA w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e l-0))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 624)
  (parent 397)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul (rec eA) eB l-1 (rec e))) (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 625)
  (parent 398)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1 l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1 l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1 l-1))) key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1 l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 626)
  (parent 398)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer self-1 name) (hl base)
    (l l-0 eA eB l-1 e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1 l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1 l-2))) key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 627)
  (parent 398)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (hl base)
    (l l-0 eB l-1 expn) (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eA (mul (rec eB) (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w e))
    (mul (rec eB) (rec l-1) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 628)
  (parent 398)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (hl base)
    (l l-0 eA l-1 expn) (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eB (mul eA (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w e))
    (mul eA (rec l-1) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 629)
  (parent 398)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (hl base)
    (l l-0 eA eB expn) (w expr) (e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (l-1 (mul eA (rec eB) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 630)
  (parent 398)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (hl base)
    (l l-0 eA eB l-1 expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e (mul eA (rec eB) (rec l-1) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) (rec l-1) w))
    (mul eA (rec eB) (rec l-1) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 631)
  (parent 398)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul (rec eA) eB l (rec l-0))) (l-2 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))))
  (label 632)
  (parent 399)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0 l-0))) key)))
  (neq ((exp (gen) (mul l-0 l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0 l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 633)
  (parent 399)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l l-1)))
        key)))
  (neq ((exp (gen) (mul l l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l l-1)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 634)
  (parent 399)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l) w l-0))
    (mul (rec eB) (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 635)
  (parent 399)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (mul eA (rec l) w l-0)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l) w l-0))
    (mul eA (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 636)
  (parent 399)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w l))
    (mul eA (rec eB) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 637)
  (parent 399)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) (rec l) w)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) (rec l) w))
    (mul eA (rec eB) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 638)
  (parent 399)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 639)
  (parent 400)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0 l-0))) key)))
  (neq ((exp (gen) (mul l-0 l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 640)
  (parent 400)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eB l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 641)
  (parent 400)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 642)
  (parent 400)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 eB))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 643)
  (parent 400)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 644)
  (parent 401)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1 l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1 l-1))) key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 645)
  (parent 401)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0 l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 646)
  (parent 401)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l eA eB l-0 l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0 l-2))) key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 647)
  (parent 401)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 648)
  (parent 401)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-1 eB))) key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 649)
  (parent 401)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 650)
  (parent 401)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 651)
  (parent 401)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 652)
  (parent 402)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul (rec eA) eB l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 653)
  (parent 402)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 654)
  (parent 402)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 655)
  (parent 402)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB (rec e) l-0))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul eB (rec e) l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 656)
  (parent 403)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB (rec e) l-1))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (e eB)
    ((exp (gen) (mul eB (rec e) l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 657)
  (parent 403)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul w e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 658)
  (parent 403)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul (rec eB) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 659)
  (parent 403)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eA l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 660)
  (parent 404)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eA l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 661)
  (parent 404)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA w) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 662)
  (parent 404)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-listener eA) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv eA) (send eA)))
  (label 663)
  (parent 404)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 664)
  (parent 405)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-2)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 665)
  (parent 405)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eB expn)
    (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul (rec eB) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w e))
    (mul (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w e)))
      (send (cat (gen) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 666)
  (parent 405)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA expn)
    (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul eA w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul eA w e)) (mul eA w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w e))) (send (cat (gen) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 667)
  (parent 405)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul eA (rec eB) w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 668)
  (parent 405)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 669)
  (parent 406)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 670)
  (parent 406)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 671)
  (parent 406)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA w l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 672)
  (parent 406)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 673)
  (parent 406)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 674)
  (parent 407)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 675)
  (parent 407)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 676)
  (parent 407)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA w l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 677)
  (parent 407)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 678)
  (parent 407)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 679)
  (parent 408)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 680)
  (parent 408)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-2)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 681)
  (parent 408)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul (rec eB) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-1))
    (mul (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-1)))
      (send (cat (gen) (mul (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 682)
  (parent 408)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul eA w l-1)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w l-1)) (mul eA w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w l-1)))
      (send (cat (gen) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 683)
  (parent 408)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 684)
  (parent 408)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul (rec eA) eB l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 685)
  (parent 409)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 686)
  (parent 409)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 687)
  (parent 409)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB (rec e) l-1))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (e eB)
    ((exp (gen) (mul eB (rec e) l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 688)
  (parent 410)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul w e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 689)
  (parent 410)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul (rec eB) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 690)
  (parent 410)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eA l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 691)
  (parent 411)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA w) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 692)
  (parent 411)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA e)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-listener eA) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv eA) (send eA)))
  (label 693)
  (parent 411)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 694)
  (parent 412)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eB expn)
    (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul (rec eB) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w e))
    (mul (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w e)))
      (send (cat (gen) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 695)
  (parent 412)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA expn)
    (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul eA w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul eA w e)) (mul eA w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w e))) (send (cat (gen) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 696)
  (parent 412)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul eA (rec eB) w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 697)
  (parent 412)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 698)
  (parent 413)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 699)
  (parent 413)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA w l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 700)
  (parent 413)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 701)
  (parent 413)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 702)
  (parent 414)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 703)
  (parent 414)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA w l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 704)
  (parent 414)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 705)
  (parent 414)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 706)
  (parent 415)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 707)
  (parent 415)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul (rec eB) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-1))
    (mul (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-1)))
      (send (cat (gen) (mul (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 708)
  (parent 415)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul eA w l-1)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w l-1)) (mul eA w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w l-1)))
      (send (cat (gen) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 709)
  (parent 415)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 710)
  (parent 415)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 711)
  (parent 416)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (gen) (mul (rec eA) e e l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((exp (gen) (mul e l-0)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (gen) (mul (rec eA) e e l-0)))
      (send (cat (gen) (mul (rec eA) e e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 712)
  (parent 416)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (gen) (mul (rec eA) e e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (gen) (mul (rec eA) e e l-1)))
      (send (cat (gen) (mul (rec eA) e e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 713)
  (parent 416)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 714)
  (parent 416)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (gen) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul eA w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e)))))
  (label 715)
  (parent 416)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e e (rec e-0)))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (e-0 e)
    ((exp (gen) (mul e e (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 716)
  (parent 417)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e e-0 l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e e (rec e-0) l-0))) (l l)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((exp (gen) (mul e l-0)) (gen)) (e-0 e)
    ((exp (gen) (mul e e (rec e-0) l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 717)
  (parent 417)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e e-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e e (rec e-0) l-1))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (e-0 e)
    ((exp (gen) (mul e e (rec e-0) l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 718)
  (parent 417)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e-0 e-0))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e e-0))) (l l-0) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul w e e e-0)) (gen)) (e e-0)
    ((exp (gen) (mul w e e-0 e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 719)
  (parent 417)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e-0)))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (e-0 e)
    ((exp (gen) (mul w (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 720)
  (parent 417)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul (rec eA) e)) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) e)))
      (send (cat (exp (gen) e) (mul (rec eA) e)))))
  (label 721)
  (parent 418)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((exp (gen) (mul e l-0)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 722)
  (parent 418)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 723)
  (parent 418)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul w (rec e))) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w (rec e)))
    (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 724)
  (parent 418)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))))
  (label 725)
  (parent 418)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul (rec eA) e e (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 726)
  (parent 419)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA e e-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 hl) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 727)
  (parent 419)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e e-0 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 728)
  (parent 419)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 e expn)
    (w expr) (e-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w e-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 hl) (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) (mul (rec e) w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 729)
  (parent 419)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA expn)
    (w expr) (e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0 e-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e e-0))) (l l)
    (e e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eA w e)) (e (exp (gen) l-1)) (e hl)
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul eA w e e-0)) (gen)) (eA e-0)
    ((exp (gen) (mul w e e-0 e-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-absence e-0 (mul eA w e)) (mul eA w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA w e)))
      (send (cat (exp (gen) e-0) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 730)
  (parent 419)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA e expn)
    (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eA (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 hl) (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-absence e-0 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 731)
  (parent 419)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))))
  (label 732)
  (parent 420)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA e l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) e e (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((exp (gen) (mul e l-0)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec eA) e e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) e e (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 733)
  (parent 420)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA e l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 734)
  (parent 420)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w l-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 735)
  (parent 420)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 e e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0 e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul eA w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 e e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-absence e (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 736)
  (parent 420)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 737)
  (parent 420)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))))
  (label 738)
  (parent 421)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((exp (gen) (mul e l-0)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 739)
  (parent 421)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA e l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 740)
  (parent 421)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 741)
  (parent 421)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0 e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul eA w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-absence e (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-0 e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 742)
  (parent 421)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 743)
  (parent 421)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 744)
  (parent 422)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA e l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 745)
  (parent 422)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 746)
  (parent 422)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA e l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 747)
  (parent 422)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w l-1))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec e) w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 748)
  (parent 422)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1 e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul eA w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-1 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-1 e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test (added-absence e (mul eA w l-1)) (mul eA w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-1 e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-1)))
      (send (cat (exp (gen) e) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 749)
  (parent 422)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 750)
  (parent 422)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 751)
  (parent 423)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (gen) (mul (rec eA) e e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (gen) (mul (rec eA) e e l-1)))
      (send (cat (gen) (mul (rec eA) e e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 752)
  (parent 423)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 753)
  (parent 423)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (gen) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e)))))
  (label 754)
  (parent 423)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e e (rec e-0)))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (e-0 e)
    ((exp (gen) (mul e e (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 755)
  (parent 424)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e e-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e e (rec e-0) l-1))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (e-0 e)
    ((exp (gen) (mul e e (rec e-0) l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 756)
  (parent 424)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e-0 e-0))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e e-0))) (l l) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul w e e e-0)) (gen)) (e e-0)
    ((exp (gen) (mul w e e-0 e-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 757)
  (parent 424)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e-0)))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (e-0 e)
    ((exp (gen) (mul w (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 758)
  (parent 424)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul (rec eA) e)) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) e)))
      (send (cat (exp (gen) e) (mul (rec eA) e)))))
  (label 759)
  (parent 425)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 760)
  (parent 425)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul w (rec e))) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w (rec e)))
    (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 761)
  (parent 425)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))))
  (label 762)
  (parent 425)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul (rec eA) e e (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 763)
  (parent 426)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e e-0 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-2))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 764)
  (parent 426)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 e expn)
    (w expr) (e-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w e-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 hl) (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) (mul (rec e) w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 765)
  (parent 426)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA expn)
    (w expr) (e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0 e-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e e-0))) (l l-0)
    (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eA w e)) (e (exp (gen) l-1)) (e hl)
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul eA w e e-0)) (gen)) (eA e-0)
    ((exp (gen) (mul w e e-0 e-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-absence e-0 (mul eA w e)) (mul eA w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA w e)))
      (send (cat (exp (gen) e-0) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 766)
  (parent 426)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA e expn)
    (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eA (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 hl) (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-absence e-0 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 767)
  (parent 426)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))))
  (label 768)
  (parent 427)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA e l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 769)
  (parent 427)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w l-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 770)
  (parent 427)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0 e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul eA w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-absence e (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 771)
  (parent 427)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 772)
  (parent 427)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))))
  (label 773)
  (parent 428)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA e l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 774)
  (parent 428)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 775)
  (parent 428)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 e e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0 e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul eA w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-absence e (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-0 e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 776)
  (parent 428)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 777)
  (parent 428)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 778)
  (parent 429)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 779)
  (parent 429)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA e l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-2))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 780)
  (parent 429)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w l-1))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul (rec e) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec e) w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 781)
  (parent 429)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1 e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul eA w l-1)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-1 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-1 e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test (added-absence e (mul eA w l-1)) (mul eA w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-1)))
      (send (cat (exp (gen) e) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 782)
  (parent 429)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 783)
  (parent 429)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA eB e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul (rec eA) eB e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e)))))
  (label 784)
  (parent 430)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul (rec eA) eB e l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-1)))
        key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul (rec eA) eB e l-1)))
      (send (cat (gen) (mul (rec eA) eB e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 785)
  (parent 430)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul (rec eA) eB e l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul (rec eA) eB e l-2)))
      (send (cat (gen) (mul (rec eA) eB e l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 786)
  (parent 430)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eB e expn)
    (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 787)
  (parent 430)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA e expn)
    (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w))
    (mul eA (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 788)
  (parent 430)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (mul eA (rec eB) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e)))))
  (label 789)
  (parent 430)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eB e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB e (rec e-0)))) (l l-1)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e-0 eB)
    ((exp (gen) (mul eB e (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e) (w (mul eB e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB e (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))))
  (label 790)
  (parent 431)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eB e e-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eB e (rec e-0) l-1)))
    (l l-0) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-1)))
        key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (e-0 eB)
    ((exp (gen) (mul eB e (rec e-0) l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e e-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB e (rec e-0) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 791)
  (parent 431)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e e-0 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB e (rec e-0) l-2)))
    (l l-1) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (e-0 eB)
    ((exp (gen) (mul eB e (rec e-0) l-2)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB e (rec e-0) l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 792)
  (parent 431)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 e expn)
    (w expr) (l-1 e-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 eB))) (l l-1)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 e-0))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eB)))
        key)))
  (neq ((exp (gen) (mul w e-0 e-0)) (gen)) (e-0 eB)
    ((exp (gen) (mul w e-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 e l-1 e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB)))))
  (label 793)
  (parent 431)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB expn)
    (w expr) (l-1 e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e e-0)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) w e e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e-0 (mul (rec eB) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((exp (gen) (mul (rec eB) w e e e-0)) (gen)) (e eB)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB l-1 e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eB) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 794)
  (parent 431)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec e-0)))) (l l-1)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e-0 (mul (rec eB) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e-0 eB)
    ((exp (gen) (mul w (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 795)
  (parent 431)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 e eA e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) e-0))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 hl) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0))) key)))
  (neq ((exp (gen) e-0) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) e-0)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eA e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e-0) (w (mul (rec eA) e-0))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) e-0)))))
  (label 796)
  (parent 432)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 e eA e-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e (rec eA) e-0 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e-0 l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 hl) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-1)))
        key)))
  (neq ((exp (gen) (mul e-0 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) e-0 l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e eA e-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) e-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 797)
  (parent 432)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 e eA e-0 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) e-0 l-2)))
    (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e-0 l-2))) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 hl) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-2)))
        key)))
  (neq ((exp (gen) (mul e-0 l-2)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) e-0 l-2)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eA e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) e-0 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e-0 l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 798)
  (parent 432)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 e expn)
    (w expr) (l-1 e-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) w) (gen)) (eA e-0)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 e l-1 e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) w))
    (mul (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA))))))
  (label 799)
  (parent 432)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA expn)
    (w expr) (l-1 e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e-0))) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e-0 (mul eA w)) (e (exp (gen) l-1)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 hl) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e e-0)))
        key)))
  (neq ((exp (gen) (mul eA w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA l-1 e e-0)
  (operation nonce-test (added-absence e-0 (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA w)))
      (send (cat (exp (gen) e-0) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 800)
  (parent 432)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 e eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eB))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul e (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB)))))
  (label 801)
  (parent 433)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 e eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eB l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul e (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA eB l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 802)
  (parent 433)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 e eA eB l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eB l-2))) key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul e (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 803)
  (parent 433)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB expn)
    (w expr) (l-1 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) w)))
      (send (cat (exp (gen) e) (mul (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 804)
  (parent 433)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA expn)
    (w expr) (l-1 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA w)) (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 805)
  (parent 433)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-1 e)
      (w (mul (rec eA) eB e (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 806)
  (parent 434)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 self-1 name)
    (hl hl-0 base) (l l-0 l-1 eA eB e e-0 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-2)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 0) (0 1)) ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-2)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) (privk self-1) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e e-0 l-2)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 807)
  (parent 434)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 self-1 name)
    (hl hl-0 base) (l l-0 l-1 l-2 eA eB e e-0 l-3 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-3)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-3)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-3 self-1) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-3)))
        key)))
  (neq ((exp (gen) (mul e l-3)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-3)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-3))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 808)
  (parent 434)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eB e expn) (w expr) (e-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 hl-0) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w e-0))
    (mul (rec eB) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 809)
  (parent 434)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA e expn) (w expr) (e-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w e-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eB (mul eA (rec e) w e-0)) (e-0 (exp (gen) l-2)) (e-0 hl-0)
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w e-0))
    (mul eA (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 810)
  (parent 434)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB expn) (w expr) (e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) w e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul eA (rec eB) w e)) (e (exp (gen) l-2)) (e hl-0)
    (e-0 (exp (gen) l)) (e-0 hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e e-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) w e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 811)
  (parent 434)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e expn) (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul eA (rec eB) (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 hl-0) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 812)
  (parent 434)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul (rec eA) eB e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))))
  (label 813)
  (parent 435)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l eA eB e l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-1)))
        key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 814)
  (parent 435)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 815)
  (parent 435)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w l-1))
    (mul (rec eB) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 816)
  (parent 435)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w l-1))
    (mul eA (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 817)
  (parent 435)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 e))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1 e)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (mul eA (rec eB) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 e)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w l-1))
    (mul eA (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))))
  (label 818)
  (parent 435)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB e expn)
    (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 819)
  (parent 435)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul (rec eA) eB e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))))
  (label 820)
  (parent 436)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-1)))
        key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 821)
  (parent 436)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 822)
  (parent 436)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w l-1))
    (mul (rec eB) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 823)
  (parent 436)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w l-1))
    (mul eA (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 824)
  (parent 436)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1 e)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (mul eA (rec eB) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w l-1))
    (mul eA (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))))
  (label 825)
  (parent 436)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB e expn)
    (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 826)
  (parent 436)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul (rec eA) eB e (rec l-2))) (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 827)
  (parent 437)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (hl base)
    (l l-0 eA eB e l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 0) (0 1)) ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-2)) (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) (privk self-1) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e l-1 l-2)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 828)
  (parent 437)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 829)
  (parent 437)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 eA eB e l-2 l-3 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-3)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-3)))
        key)))
  (neq ((exp (gen) (mul e l-3)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-3)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-3))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 830)
  (parent 437)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e expn) (w expr) (l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w l-2)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w l-2))
    (mul (rec eB) (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 831)
  (parent 437)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e expn) (w expr) (l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (mul eA (rec e) w l-2)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w l-2))
    (mul eA (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eB)))
      (send (cat (exp (gen) l-2) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 832)
  (parent 437)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (l-2 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-2 e)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (mul eA (rec eB) w l-2)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w l-2))
    (mul eA (rec eB) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w e)))
      (send (cat (exp (gen) l-2) (mul w e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 833)
  (parent 437)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e expn) (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 834)
  (parent 437)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eA eB e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul (rec eA) eB e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e)))))
  (label 835)
  (parent 438)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul (rec eA) eB e l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul (rec eA) eB e l-2)))
      (send (cat (gen) (mul (rec eA) eB e l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 836)
  (parent 438)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eB e expn)
    (w expr) (eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 837)
  (parent 438)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA e expn)
    (w expr) (eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w))
    (mul eA (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 838)
  (parent 438)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (mul eA (rec eB) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e)))))
  (label 839)
  (parent 438)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 eB e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB e (rec e-0)))) (l l-1)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (e-0 eB)
    ((exp (gen) (mul eB e (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e) (w (mul eB e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB e (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))))
  (label 840)
  (parent 439)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e e-0 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB e (rec e-0) l-2)))
    (l l-1) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (e-0 eB)
    ((exp (gen) (mul eB e (rec e-0) l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB e (rec e-0) l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 841)
  (parent 439)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 e expn)
    (w expr) (l-1 e-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 eB))) (l l-1)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eB)))
        key)))
  (neq ((exp (gen) (mul w e-0 e-0)) (gen)) (e-0 eB)
    ((exp (gen) (mul w e-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 e l-1 e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB)))))
  (label 842)
  (parent 439)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB expn)
    (w expr) (l-1 e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e e e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) w e e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e-0 (mul (rec eB) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((exp (gen) (mul (rec eB) w e e e-0)) (gen)) (e eB)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB l-1 e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eB) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 843)
  (parent 439)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec e-0)))) (l l-1)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e-0 (mul (rec eB) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e-0 eB)
    ((exp (gen) (mul w (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 844)
  (parent 439)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 e eA e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e (rec eA) e-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 hl) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0))) key)))
  (neq ((exp (gen) e-0) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eA e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e-0) (w (mul (rec eA) e-0))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) e-0)))))
  (label 845)
  (parent 440)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 e eA e-0 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e (rec eA) e-0 l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e-0 l-2))) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 hl) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-2)))
        key)))
  (neq ((exp (gen) (mul e-0 l-2)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) e-0 l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eA e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) e-0 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e-0 l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 846)
  (parent 440)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 e expn)
    (w expr) (l-1 e-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) w) (gen)) (eA e-0)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 e l-1 e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) w))
    (mul (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA))))))
  (label 847)
  (parent 440)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA expn)
    (w expr) (l-1 e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e-0))) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e-0 (mul eA w)) (e (exp (gen) l-1)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 hl) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e e-0)))
        key)))
  (neq ((exp (gen) (mul eA w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA l-1 e e-0)
  (operation nonce-test (added-absence e-0 (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA w)))
      (send (cat (exp (gen) e-0) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 848)
  (parent 440)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 l-1 e eA eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eB))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul e (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB)))))
  (label 849)
  (parent 441)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 e eA eB l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eB l-2))) key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul e (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 850)
  (parent 441)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB expn)
    (w expr) (l-1 e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) w)))
      (send (cat (exp (gen) e) (mul (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 851)
  (parent 441)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA expn)
    (w expr) (l-1 e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 852)
  (parent 441)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-1 e)
      (w (mul (rec eA) eB e (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 853)
  (parent 442)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 self-1 name)
    (hl hl-0 base) (l l-0 l-1 l-2 eA eB e e-0 l-3 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-3)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-3))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-3)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 hl-0) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-3 self-1) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-3)))
        key)))
  (neq ((exp (gen) (mul e l-3)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-3)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-3))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 854)
  (parent 442)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eB e expn) (w expr) (e-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e-0))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 hl-0) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w e-0))
    (mul (rec eB) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 855)
  (parent 442)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA e expn) (w expr) (e-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e-0))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eB (mul eA (rec e) w e-0)) (e-0 (exp (gen) l-2)) (e-0 hl-0)
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w e-0))
    (mul eA (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 856)
  (parent 442)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB expn) (w expr) (e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e e-0)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) w e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul eA (rec eB) w e)) (e (exp (gen) l-2)) (e hl-0)
    (e-0 (exp (gen) l)) (e-0 hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e e-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) w e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 857)
  (parent 442)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e expn) (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul eA (rec eB) (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 hl-0) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 858)
  (parent 442)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul (rec eA) eB e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))))
  (label 859)
  (parent 443)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 860)
  (parent 443)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w l-1))
    (mul (rec eB) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 861)
  (parent 443)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w l-1))
    (mul eA (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 862)
  (parent 443)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1 e)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (mul eA (rec eB) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w l-1))
    (mul eA (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))))
  (label 863)
  (parent 443)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB e expn)
    (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 864)
  (parent 443)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul (rec eA) eB e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))))
  (label 865)
  (parent 444)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 866)
  (parent 444)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w l-1))
    (mul (rec eB) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 867)
  (parent 444)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w l-1))
    (mul eA (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 868)
  (parent 444)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 e))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1 e)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (e (mul eA (rec eB) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w l-1))
    (mul eA (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))))
  (label 869)
  (parent 444)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB e expn)
    (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 870)
  (parent 444)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul (rec eA) eB e (rec l-2))) (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 871)
  (parent 445)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 872)
  (parent 445)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 eA eB e l-2 l-3 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-3)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-3))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-3)))
        key)))
  (neq ((exp (gen) (mul e l-3)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-3)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-3))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 873)
  (parent 445)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e expn) (w expr) (l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-2))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w l-2)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w l-2))
    (mul (rec eB) (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 874)
  (parent 445)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e expn) (w expr) (l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (mul eA (rec e) w l-2)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w l-2))
    (mul eA (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eB)))
      (send (cat (exp (gen) l-2) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 875)
  (parent 445)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (l-2 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-2 e)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (mul eA (rec eB) w l-2)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w l-2))
    (mul eA (rec eB) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w e)))
      (send (cat (exp (gen) l-2) (mul w e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 876)
  (parent 445)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e expn) (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 877)
  (parent 445)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 878)
  (parent 446)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l l-0))) key)))
  (neq ((exp (gen) (mul l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (gen) (mul (rec eA) eB l l-0)))
      (send (cat (gen) (mul (rec eA) eB l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 879)
  (parent 446)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul (rec eA) eB l-0 l-1)))
      (send (cat (gen) (mul (rec eA) eB l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 880)
  (parent 446)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 881)
  (parent 446)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 882)
  (parent 446)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 883)
  (parent 446)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-0 (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul eB l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))))
  (label 884)
  (parent 447)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB l (rec e) l-0))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul eB l (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l l-0))) key)))
  (neq ((exp (gen) (mul l l-0)) (gen)) (e eB)
    ((exp (gen) (mul eB l (rec e) l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l (rec e) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul eB l (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB l (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 885)
  (parent 447)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-0 (rec e) l-1))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e) l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0 (rec e) l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 886)
  (parent 447)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul w (rec l-0) e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 887)
  (parent 447)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul (rec eB) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 888)
  (parent 447)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul (rec eB) w (rec l-0))) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul (rec eB) w (rec l-0)))
    (mul (rec eB) w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 889)
  (parent 447)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e eA l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eA l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul (rec eA) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0)))))
  (label 890)
  (parent 448)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (e eA l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l l-0))) key)))
  (neq ((exp (gen) (mul l l-0)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e eA l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 891)
  (parent 448)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l e eA l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l-0 l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eA l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 892)
  (parent 448)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul w (rec l-0))) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w (rec l-0)))
    (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 893)
  (parent 448)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-0)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-absence l-0 (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 894)
  (parent 448)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul (rec eA) eB l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 895)
  (parent 449)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l eA eB l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-0 (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-0 (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-0 (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 896)
  (parent 449)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 eA eB l-1 e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 897)
  (parent 449)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB l-1 expn)
    (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul (rec eB) (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w e))
    (mul (rec eB) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 898)
  (parent 449)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA l-1 expn)
    (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul eA (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w e))
    (mul eA (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 899)
  (parent 449)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e l-1)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul eA (rec eB) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 900)
  (parent 449)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB l-1 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul eA (rec eB) (rec l-1) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) (rec l-1) w))
    (mul eA (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 901)
  (parent 449)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 902)
  (parent 450)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eB l-0))) key)))
  (neq ((exp (gen) (mul l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul l (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 903)
  (parent 450)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 904)
  (parent 450)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 905)
  (parent 450)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 906)
  (parent 450)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul (rec eA) eB l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))))
  (label 907)
  (parent 451)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l l-0))) key)))
  (neq ((exp (gen) (mul l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 908)
  (parent 451)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l l-1))) key)))
  (neq ((exp (gen) (mul l l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 909)
  (parent 451)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l) w l-0))
    (mul (rec eB) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 910)
  (parent 451)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l) w l-0))
    (mul eA (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 911)
  (parent 451)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l l-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l l-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w l))
    (mul eA (rec eB) w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 912)
  (parent 451)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) (rec l) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) (rec l) w))
    (mul eA (rec eB) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 913)
  (parent 451)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 914)
  (parent 452)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l l-1))) key)))
  (neq ((exp (gen) (mul l l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 915)
  (parent 452)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 916)
  (parent 452)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 917)
  (parent 452)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 918)
  (parent 452)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 919)
  (parent 452)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 920)
  (parent 452)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 921)
  (parent 452)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 922)
  (parent 453)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul (rec eA) eB l-0 l-1)))
      (send (cat (gen) (mul (rec eA) eB l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 923)
  (parent 453)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 924)
  (parent 453)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 925)
  (parent 453)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 926)
  (parent 453)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB l-0 (rec e)))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test
    (contracted (eB-0 eB) (l-1 l-0) (w (mul eB l-0 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-0 (rec e))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))))
  (label 927)
  (parent 454)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB l-0 (rec e) l-1))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e) l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-0 (rec e) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 928)
  (parent 454)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul (rec l) w e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul (rec l) w e e))
    (mul (rec l) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 929)
  (parent 454)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul (rec eB) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 930)
  (parent 454)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul (rec eB) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eB) (rec l) w))
    (mul (rec eB) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 931)
  (parent 454)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e eA l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eA l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul (rec eA) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0)))))
  (label 932)
  (parent 455)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l e eA l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-0 l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eA l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 933)
  (parent 455)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul w (rec l-0))) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w (rec l-0)))
    (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 934)
  (parent 455)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-0)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-absence l-0 (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 935)
  (parent 455)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul (rec eA) eB l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 936)
  (parent 456)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 eA eB l-1 e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 937)
  (parent 456)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB l-1 expn)
    (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul (rec eB) (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w e))
    (mul (rec eB) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 938)
  (parent 456)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA l-1 expn)
    (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul eA (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w e))
    (mul eA (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 939)
  (parent 456)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e l-1)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul eA (rec eB) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 940)
  (parent 456)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB l-1 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul eA (rec eB) (rec l-1) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) (rec l-1) w))
    (mul eA (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 941)
  (parent 456)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 942)
  (parent 457)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 943)
  (parent 457)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 944)
  (parent 457)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 945)
  (parent 457)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul (rec eA) eB l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))))
  (label 946)
  (parent 458)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l l-1))) key)))
  (neq ((exp (gen) (mul l l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 947)
  (parent 458)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l) w l-0))
    (mul (rec eB) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 948)
  (parent 458)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l) w l-0))
    (mul eA (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 949)
  (parent 458)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l l-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l l-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w l))
    (mul eA (rec eB) w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 950)
  (parent 458)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) (rec l) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) (rec l) w))
    (mul eA (rec eB) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 951)
  (parent 458)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 952)
  (parent 459)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 953)
  (parent 459)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 954)
  (parent 459)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 955)
  (parent 459)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 956)
  (parent 459)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 957)
  (parent 459)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 958)
  (parent 459)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 959)
  (parent 460)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-0)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-0)))
        key)))
  (neq ((exp (gen) (mul l-0 l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (gen) (mul (rec eA) eB l-0 l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 960)
  (parent 460)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul (rec eA) eB l-0 l-1)))
      (send (cat (gen) (mul (rec eA) eB l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 961)
  (parent 460)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 962)
  (parent 460)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 963)
  (parent 460)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 964)
  (parent 460)
  (unrealized (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB l-0 (rec e)))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test
    (contracted (eB-0 eB) (l-1 l-0) (w (mul eB l-0 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-0 (rec e))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))))
  (label 965)
  (parent 461)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB (rec e) l-0 l-0))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-0)))
        key)))
  (neq ((exp (gen) (mul l-0 l-0)) (gen)) (e eB)
    ((exp (gen) (mul eB (rec e) l-0 l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) l-0 l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0 l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 966)
  (parent 461)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB l-0 (rec e) l-1))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e) l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-0 (rec e) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 967)
  (parent 461)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul (rec l) w e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul (rec l) w e e))
    (mul (rec l) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 968)
  (parent 461)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul (rec eB) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 969)
  (parent 461)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul (rec eB) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eB) (rec l) w))
    (mul (rec eB) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 970)
  (parent 461)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e eA l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l e eA l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (l-1 l-0) (w (mul (rec eA) l-0))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0)))))
  (label 971)
  (parent 462)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eA l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-0 l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-0))) key)))
  (neq ((exp (gen) (mul l-0 l-0)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0 l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0 l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0 l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 972)
  (parent 462)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l e eA l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-0 l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l e eA l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 973)
  (parent 462)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul (rec l) w))
    (mul (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 974)
  (parent 462)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-0)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 975)
  (parent 462)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul (rec eA) eB l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 976)
  (parent 463)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1 l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1 l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 977)
  (parent 463)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 eA eB l-1 e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 978)
  (parent 463)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB l-1 expn)
    (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul (rec eB) (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w e))
    (mul (rec eB) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 979)
  (parent 463)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA l-1 expn)
    (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul eA (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w e))
    (mul eA (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 980)
  (parent 463)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul eA (rec eB) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 981)
  (parent 463)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB l-1 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul eA (rec eB) (rec l-1) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) (rec l-1) w))
    (mul eA (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 982)
  (parent 463)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul (rec eA) eB l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))))
  (label 983)
  (parent 464)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-0)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-0)))
        key)))
  (neq ((exp (gen) (mul l-0 l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0 l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 984)
  (parent 464)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l l-1))) key)))
  (neq ((exp (gen) (mul l l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 985)
  (parent 464)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l) w l-0))
    (mul (rec eB) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 986)
  (parent 464)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l) w l-0))
    (mul eA (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 987)
  (parent 464)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l l-0)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l l-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w l))
    (mul eA (rec eB) w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 988)
  (parent 464)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) (rec l) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) (rec l) w))
    (mul eA (rec eB) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 989)
  (parent 464)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 990)
  (parent 465)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-0)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-0)))
        key)))
  (neq ((exp (gen) (mul l-0 l-0)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 991)
  (parent 465)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 992)
  (parent 465)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 993)
  (parent 465)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 994)
  (parent 465)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 995)
  (parent 466)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 996)
  (parent 466)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 997)
  (parent 466)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 998)
  (parent 466)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 999)
  (parent 466)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1000)
  (parent 466)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 1001)
  (parent 466)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1002)
  (parent 466)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 1003)
  (parent 467)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul (rec eA) eB l-0 l-1)))
      (send (cat (gen) (mul (rec eA) eB l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1004)
  (parent 467)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 1005)
  (parent 467)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 1006)
  (parent 467)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 1007)
  (parent 467)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-0 (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul eB l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))))
  (label 1008)
  (parent 468)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-0 (rec e) l-1))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e) l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0 (rec e) l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1009)
  (parent 468)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul w (rec l-0) e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 1010)
  (parent 468)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul (rec eB) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 1011)
  (parent 468)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (mul (rec eB) w (rec l-0))) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul (rec eB) w (rec l-0)))
    (mul (rec eB) w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 1012)
  (parent 468)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e eA l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eA l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (l-1 l-0) (w (mul (rec eA) l-0))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0)))))
  (label 1013)
  (parent 469)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l e eA l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l-0 l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eA l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1014)
  (parent 469)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul (rec l) w))
    (mul (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 1015)
  (parent 469)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-0)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 1016)
  (parent 469)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul (rec eA) eB l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1017)
  (parent 470)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 eA eB l-1 e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1018)
  (parent 470)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB l-1 expn)
    (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul (rec eB) (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w e))
    (mul (rec eB) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1019)
  (parent 470)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA l-1 expn)
    (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul eA (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w e))
    (mul eA (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1020)
  (parent 470)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul eA (rec eB) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1021)
  (parent 470)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB l-1 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul eA (rec eB) (rec l-1) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) (rec l-1) w))
    (mul eA (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1022)
  (parent 470)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul (rec eA) eB l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))))
  (label 1023)
  (parent 471)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l l-1))) key)))
  (neq ((exp (gen) (mul l l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1024)
  (parent 471)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l) w l-0))
    (mul (rec eB) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 1025)
  (parent 471)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l) w l-0))
    (mul eA (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 1026)
  (parent 471)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l l-0)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l l-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w l))
    (mul eA (rec eB) w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 1027)
  (parent 471)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (l-0 (mul eA (rec eB) (rec l) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) (rec l) w))
    (mul eA (rec eB) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 1028)
  (parent 471)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 1029)
  (parent 472)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1030)
  (parent 472)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 1031)
  (parent 472)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 1032)
  (parent 472)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1033)
  (parent 473)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1034)
  (parent 473)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1035)
  (parent 473)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1036)
  (parent 473)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1037)
  (parent 473)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 1038)
  (parent 473)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 1039)
  (parent 473)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul (rec eA) eB l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1)))))
  (label 1040)
  (parent 474)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul (rec eA) eB l-0 l-1)))
      (send (cat (gen) (mul (rec eA) eB l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1041)
  (parent 474)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul (rec eA) eB l-1 l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1042)
  (parent 474)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul (rec eA) eB l-1 l-2)))
      (send (cat (gen) (mul (rec eA) eB l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1043)
  (parent 474)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 expn) (w expr)
    (eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w))
    (mul (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 1044)
  (parent 474)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 expn) (w expr)
    (eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w))
    (mul eA (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 1045)
  (parent 474)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 1046)
  (parent 474)
  (unrealized (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-1 (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (e eB)
    ((exp (gen) (mul eB l-1 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test
    (contracted (eB-0 eB) (l-2 l-1) (w (mul eB l-1 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-1 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e))))))
  (label 1047)
  (parent 475)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eB l-0 (rec e) l-1))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (e eB)
    ((exp (gen) (mul eB l-0 (rec e) l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-0 (rec e) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1048)
  (parent 475)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB (rec e) l-1 l-1))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (e eB)
    ((exp (gen) (mul eB (rec e) l-1 l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1049)
  (parent 475)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-1 (rec e) l-2))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l D) (l-0 B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (e eB)
    ((exp (gen) (mul eB l-1 (rec e) l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-1 (rec e) l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1050)
  (parent 475)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 expn) (w expr)
    (l-1 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul (rec l-0) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eB (exp (gen) l-1)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec l-0) w e e))
    (mul (rec l-0) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 1051)
  (parent 475)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul (rec eB) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-1)) (gen)) (e eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 1052)
  (parent 475)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (e (mul (rec eB) (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eB (exp (gen) l-1)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 1053)
  (parent 475)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eA l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (l-2 l-1) (w (mul (rec eA) l-1))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-1)))))
  (label 1054)
  (parent 476)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e eA l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e (rec eA) l-0 l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-0 l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1055)
  (parent 476)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eA l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l-1 l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1))) key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-1 l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1056)
  (parent 476)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 e eA l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e (rec eA) l-1 l-2))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l D) (l-0 B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1057)
  (parent 476)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 expn) (w expr)
    (l-1 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec l-0) w)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (operation nonce-test (added-absence eA (mul (rec l-0) w))
    (mul (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 1058)
  (parent 476)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-absence l-1 (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 1059)
  (parent 476)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-2)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2)
      (w (mul (rec eA) eB l-2 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1060)
  (parent 477)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer self-1 name) (hl base)
    (l l-0 eA eB l-1 e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 0) (0 1)) ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-2)) (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1061)
  (parent 477)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-2 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2 l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-2)))
        key)))
  (neq ((exp (gen) (mul l-2 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2 l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1062)
  (parent 477)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer self-1 name) (hl base)
    (l l-0 l-1 eA eB l-2 e l-3 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-3)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-2 l-3))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e) l-3)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-0 D) (l B) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-3)))
        key)))
  (neq ((exp (gen) (mul l-2 l-3)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2 l-3)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e) l-3)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1063)
  (parent 477)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eB l-2 expn) (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul (rec eB) (rec l-2) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-2) w e))
    (mul (rec eB) (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1064)
  (parent 477)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA l-2 expn) (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eB (mul eA (rec l-2) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-2) w e))
    (mul eA (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1065)
  (parent 477)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e l-2)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (l-2 (mul eA (rec eB) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1066)
  (parent 477)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (mul eA (rec eB) (rec l-2) w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) (rec l-2) w))
    (mul eA (rec eB) (rec l-2) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1067)
  (parent 477)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))))
  (label 1068)
  (parent 478)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (eA eB l l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB l l-1)))
        key)))
  (neq ((exp (gen) (mul l l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1069)
  (parent 478)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1070)
  (parent 478)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-2)))
    (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0 l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1071)
  (parent 478)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 1072)
  (parent 478)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 eB))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 1073)
  (parent 478)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 1074)
  (parent 478)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 1075)
  (parent 478)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))))
  (label 1076)
  (parent 479)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1077)
  (parent 479)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1078)
  (parent 479)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-0 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0 l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1079)
  (parent 479)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 1080)
  (parent 479)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 1081)
  (parent 479)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 1082)
  (parent 479)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 1083)
  (parent 479)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB)))))
  (label 1084)
  (parent 480)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 (rec eA) eB l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1085)
  (parent 480)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1086)
  (parent 480)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1087)
  (parent 480)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 1088)
  (parent 480)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 1089)
  (parent 480)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-1)
      (w (mul (rec eA) eB l-1 (rec l-2))) (l-4 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1090)
  (parent 481)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l eA eB l-0 l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul (rec eA) eB l-0 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 0) (0 1)) ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-2)) (eA (exp (gen) l))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eB l-0 l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1091)
  (parent 481)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-2 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-2)))
        key)))
  (neq ((exp (gen) (mul l-2 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1092)
  (parent 481)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1093)
  (parent 481)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 l-3 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-3)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-3))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-3)))
        key)))
  (neq ((exp (gen) (mul l-1 l-3)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-3)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1094)
  (parent 481)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 expn) (w expr)
    (l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul (rec eB) (rec l-1) w l-2)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-1) w l-2))
    (mul (rec eB) (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1095)
  (parent 481)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 expn) (w expr)
    (l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (mul eA (rec l-1) w l-2)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w l-2))
    (mul eA (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eB)))
      (send (cat (exp (gen) l-2) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1096)
  (parent 481)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1 l-2)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul eA (rec eB) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 D) (l B) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) w l-1))
    (mul eA (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 1097)
  (parent 481)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 expn)
    (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul eA (rec eB) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (added-absence l-2 (mul eA (rec eB) (rec l-1) w))
    (mul eA (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1098)
  (parent 481)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul (rec eA) eB l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1)))))
  (label 1099)
  (parent 482)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul (rec eA) eB l-1 l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1100)
  (parent 482)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul (rec eA) eB l-1 l-2)))
      (send (cat (gen) (mul (rec eA) eB l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1101)
  (parent 482)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 expn) (w expr)
    (eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w))
    (mul (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 1102)
  (parent 482)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 expn) (w expr)
    (eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w))
    (mul eA (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 1103)
  (parent 482)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 1104)
  (parent 482)
  (unrealized (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-1 (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (e eB)
    ((exp (gen) (mul eB l-1 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test
    (contracted (eB-0 eB) (l-2 l-1) (w (mul eB l-1 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-1 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e))))))
  (label 1105)
  (parent 483)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB (rec e) l-1 l-1))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (e eB)
    ((exp (gen) (mul eB (rec e) l-1 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1106)
  (parent 483)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eB l-1 (rec e) l-2))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l B) (l-0 D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (e eB)
    ((exp (gen) (mul eB l-1 (rec e) l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-1 (rec e) l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1107)
  (parent 483)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 expn) (w expr)
    (l-1 e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul (rec l-0) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec l-0) w e e))
    (mul (rec l-0) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 1108)
  (parent 483)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul (rec eB) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-1)) (gen)) (e eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 1109)
  (parent 483)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (e (mul (rec eB) (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 1110)
  (parent 483)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eA l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (l-2 l-1) (w (mul (rec eA) l-1))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-1)))))
  (label 1111)
  (parent 484)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eA l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-1 l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1))) key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-1 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1112)
  (parent 484)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 e eA l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-1 l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA e)
    ((exp (gen) (mul e (rec eA) l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1113)
  (parent 484)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 expn) (w expr)
    (l-1 e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec l-0) w)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (operation nonce-test (added-absence eA (mul (rec l-0) w))
    (mul (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 1114)
  (parent 484)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-absence l-1 (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 1115)
  (parent 484)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2)
      (w (mul (rec eA) eB l-2 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1116)
  (parent 485)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-2 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2 l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-2)))
        key)))
  (neq ((exp (gen) (mul l-2 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2 l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1117)
  (parent 485)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer self-1 name) (hl base)
    (l l-0 l-1 eA eB l-2 e l-3 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-3)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-2 l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e) l-3)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-3)))
        key)))
  (neq ((exp (gen) (mul l-2 l-3)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2 l-3)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e) l-3)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1118)
  (parent 485)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eB l-2 expn) (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul (rec eB) (rec l-2) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 e eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-2) w e))
    (mul (rec eB) (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1119)
  (parent 485)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA l-2 expn) (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eB (mul eA (rec l-2) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-2) w e))
    (mul eA (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1120)
  (parent 485)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e l-2)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (l-2 (mul eA (rec eB) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1121)
  (parent 485)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (mul eA (rec eB) (rec l-2) w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) (rec l-2) w))
    (mul eA (rec eB) (rec l-2) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 1122)
  (parent 485)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))))
  (label 1123)
  (parent 486)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1124)
  (parent 486)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-0 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0 l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1125)
  (parent 486)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 1126)
  (parent 486)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 1127)
  (parent 486)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 1128)
  (parent 486)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 1129)
  (parent 486)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))))
  (label 1130)
  (parent 487)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1131)
  (parent 487)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-2)))
    (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0 l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1132)
  (parent 487)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 1133)
  (parent 487)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 eB))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 1134)
  (parent 487)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 1135)
  (parent 487)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 1136)
  (parent 487)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB)))))
  (label 1137)
  (parent 488)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1138)
  (parent 488)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1139)
  (parent 488)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 1140)
  (parent 488)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (absent (eB (mul eA w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 1141)
  (parent 488)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-1)
      (w (mul (rec eA) eB l-1 (rec l-2))) (l-4 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1142)
  (parent 489)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-2 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-2)))
        key)))
  (neq ((exp (gen) (mul l-2 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1143)
  (parent 489)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1144)
  (parent 489)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 l-3 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-3)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-3)))
        key)))
  (neq ((exp (gen) (mul l-1 l-3)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-3)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1145)
  (parent 489)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 expn) (w expr)
    (l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul (rec eB) (rec l-1) w l-2)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-1) w l-2))
    (mul (rec eB) (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1146)
  (parent 489)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 expn) (w expr)
    (l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (mul eA (rec l-1) w l-2)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w l-2))
    (mul eA (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eB)))
      (send (cat (exp (gen) l-2) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1147)
  (parent 489)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1 l-2)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul eA (rec eB) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) w l-1))
    (mul eA (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 1148)
  (parent 489)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 expn)
    (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul eA (rec eB) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (added-absence l-2 (mul eA (rec eB) (rec l-1) w))
    (mul eA (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1149)
  (parent 489)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1150)
  (parent 491)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB)
  (operation nonce-test (added-listener eB) (mul w eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1151)
  (parent 492)
  (unrealized (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expr) (l e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eB (mul w e e)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1152)
  (parent 494)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (e (mul (rec eB) w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1153)
  (parent 495)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expr) (l e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eA w) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1154)
  (parent 497)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eB expn) (w expr)
    (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul (rec eB) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w e)))
      (send (cat (gen) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 1155)
  (parent 500)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA expn) (w expr)
    (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul eA w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w e))) (send (cat (gen) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 1156)
  (parent 501)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (6 0)) ((6 1) (4 0)))
  (absent (e (mul eA (rec eB) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e) (send e)))
  (label 1157)
  (parent 502)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (l eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eA (mul (rec eB) w l)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((exp (gen) (mul (rec eB) w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w l)))
      (send (cat (gen) (mul (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec eA))))
      (send (cat (exp (gen) l) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1158)
  (parent 504)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (l eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w l)))
  (deflistener (cat (exp (gen) l) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eB (mul eA w l)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l eB))) key)))
  (neq ((exp (gen) (mul eA w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-listener eB) (mul w eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w l))) (send (cat (gen) (mul eA w l))))
    ((recv (cat (exp (gen) l) (mul w eB)))
      (send (cat (exp (gen) l) (mul w eB)))) ((recv eB) (send eB)))
  (label 1159)
  (parent 505)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul (rec eB) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eA) (send eA)))
  (label 1160)
  (parent 509)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul eA w l-0)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 eB))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eB) (send eB)))
  (label 1161)
  (parent 510)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (6 0)) ((6 1) (4 0)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-0) (send l-0)))
  (label 1162)
  (parent 511)
  (unrealized (1 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l e expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eA (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1163)
  (parent 514)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (e (mul eA w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e)
  (operation nonce-test (added-listener e) (mul w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv e) (send e)))
  (label 1164)
  (parent 515)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expr) (l e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e-0 e-0))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e e-0))) (l l) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul w e e e-0)) (gen)) (e e-0)
    ((exp (gen) (mul w e e-0 e-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1165)
  (parent 518)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e expn) (w expr) (l e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e-0)))) (l l) (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (e-0 e)
    ((exp (gen) (mul w (rec e-0))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1166)
  (parent 519)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expr) (l e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eA (mul w (rec e))) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1167)
  (parent 522)
  (unrealized (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (e (mul eA w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-listener e) (mul w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))) ((recv e) (send e)))
  (label 1168)
  (parent 523)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 e expn) (w expr)
    (e-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w e-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 hl) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) (mul (rec e) w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv eA) (send eA)))
  (label 1169)
  (parent 526)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA expn) (w expr)
    (e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e-0 e-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e e-0))) (l l) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (e-0 (mul eA w e)) (e (exp (gen) l-0)) (e hl)
    (e-0 (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul eA w e e-0)) (gen)) (eA e-0)
    ((exp (gen) (mul w e e-0 e-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eA w e)))
      (send (cat (exp (gen) e-0) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e-0) (send e-0)))
  (label 1170)
  (parent 527)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (6 0)) ((6 1) (4 0)))
  (absent (e-0 (mul eA (rec e) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 hl) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv e-0) (send e-0)))
  (label 1171)
  (parent 528)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e expn) (w expr) (l eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w l))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eA (mul (rec e) (rec e) w l)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((exp (gen) (mul (rec e) w l)) (gen)) (eA e)
    ((exp (gen) (mul w l (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec eA))))
      (send (cat (exp (gen) l) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1172)
  (parent 531)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l)))
  (deflistener (cat (exp (gen) l) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (e (mul eA w l)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l e e))) key)))
  (neq ((exp (gen) (mul eA w l e)) (gen)) (eA e)
    ((exp (gen) (mul w l e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-listener e) (mul w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w l)))
      (send (cat (exp (gen) e) (mul eA w l))))
    ((recv (cat (exp (gen) l) (mul w e e)))
      (send (cat (exp (gen) l) (mul w e e)))) ((recv e) (send e)))
  (label 1173)
  (parent 532)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eA) (send eA)))
  (label 1174)
  (parent 537)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0 e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (e (mul eA w l-0)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv e) (send e)))
  (label 1175)
  (parent 538)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (6 0)) ((6 1) (4 0)))
  (absent (l-0 (mul eA (rec e) (rec e) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-0) (send l-0)))
  (label 1176)
  (parent 539)
  (unrealized (1 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1177)
  (parent 542)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1178)
  (parent 543)
  (unrealized (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul eA (rec eB) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e))))
    ((recv e) (send e)))
  (label 1179)
  (parent 544)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l e expn) (w expr)
    (l-0 e-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 eB))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0 eB)))
        key)))
  (neq ((exp (gen) (mul w e-0 e-0)) (gen)) (e-0 eB)
    ((exp (gen) (mul w e-0 eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1180)
  (parent 547)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB expn) (w expr)
    (l-0 e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) w e e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec eB) w e e)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 hl) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((exp (gen) (mul (rec eB) w e e e-0)) (gen)) (e eB)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l-0 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eB) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1181)
  (parent 548)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB e expn) (w expr)
    (l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec e-0)))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec eB) (rec e) w)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e-0 eB)
    ((exp (gen) (mul w (rec e-0))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB e l-0 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1182)
  (parent 549)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l e expn) (w expr)
    (l-0 e-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec e) w)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e hl) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) w) (gen)) (eA e-0)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1183)
  (parent 552)
  (unrealized (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA expn) (w expr)
    (l-0 e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e-0 (mul eA w)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 hl) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e e-0)))
        key)))
  (neq ((exp (gen) (mul eA w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eA w)))
      (send (cat (exp (gen) e-0) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1184)
  (parent 553)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB expn) (w expr)
    (l-0 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) w)))
      (send (cat (exp (gen) e) (mul (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1185)
  (parent 556)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA expn) (w expr)
    (l-0 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA w)) (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1186)
  (parent 557)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 eB e expn) (w expr) (e-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 hl-0) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e e-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1187)
  (parent 560)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 eA e expn) (w expr) (e-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA (rec e) w e-0)) (e-0 (exp (gen) l-1)) (e-0 hl-0)
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0 eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1188)
  (parent 561)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 eA eB expn) (w expr) (e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) w e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (7 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eA (rec eB) w e)) (e (exp (gen) l-1)) (e hl-0)
    (e-0 (exp (gen) l)) (e-0 hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e e-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) w e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1189)
  (parent 562)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 eA eB e expn) (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-1)
    (e e-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eA (rec eB) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 hl-0) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1190)
  (parent 563)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB e expn) (w expr)
    (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec e) w l-0)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1191)
  (parent 566)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA e expn) (w expr)
    (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec e) w l-0)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1192)
  (parent 567)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB expn) (w expr)
    (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0 e)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul eA (rec eB) w l-0)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-0 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w e)))
      (send (cat (exp (gen) l-0) (mul w e)))) ((recv e) (send e)))
  (label 1193)
  (parent 568)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1194)
  (parent 573)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1195)
  (parent 574)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB expn) (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1 e)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (7 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (e (mul eA (rec eB) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-1 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 1196)
  (parent 575)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e expn) (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-1) (send l-1)))
  (label 1197)
  (parent 576)
  (unrealized (1 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eA (mul (rec eB) (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1198)
  (parent 579)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w)))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eB (mul eA (rec l) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-listener eB) (mul w eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec l) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1199)
  (parent 580)
  (unrealized (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expr) (l e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eB (mul w (rec l) e e)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1200)
  (parent 584)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (l e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (e (mul (rec eB) w (rec l))) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eB) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1201)
  (parent 586)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expr) (l e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eA (mul w (rec l))) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1202)
  (parent 589)
  (unrealized (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eB l-0 expn) (w expr)
    (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul (rec eB) (rec l-0) w e)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eB l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 1203)
  (parent 593)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA l-0 expn) (w expr)
    (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul eA (rec l-0) w e)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e eB)))
        key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 1204)
  (parent 594)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (hl base) (l eA eB l-0 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (6 0)) ((6 1) (4 0)))
  (absent (e (mul eA (rec eB) (rec l-0) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e) (send e)))
  (label 1205)
  (parent 596)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB expn) (w expr) (l eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((exp (gen) (mul (rec eB) w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) w)))
      (send (cat (exp (gen) l) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l) (mul w (rec eA))))
      (send (cat (exp (gen) l) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1206)
  (parent 599)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA expn) (w expr) (l eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA w)))
  (deflistener (cat (exp (gen) l) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l eB))) key)))
  (neq ((exp (gen) (mul eA w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-listener eB) (mul w eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA w)))
      (send (cat (exp (gen) l) (mul eA w))))
    ((recv (cat (exp (gen) l) (mul w eB)))
      (send (cat (exp (gen) l) (mul w eB)))) ((recv eB) (send eB)))
  (label 1207)
  (parent 600)
  (unrealized (1 3) (3 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul (rec eB) (rec l) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eA) (send eA)))
  (label 1208)
  (parent 604)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul eA (rec l) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 eB))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eB) (send eB)))
  (label 1209)
  (parent 605)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (6 0)) ((6 1) (4 0)))
  (absent (l-0 (mul eA (rec eB) (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-0) (send l-0)))
  (label 1210)
  (parent 607)
  (unrealized (1 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB l-0 expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1211)
  (parent 611)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA l-0 expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1212)
  (parent 612)
  (unrealized (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul eA (rec eB))) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB)))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1213)
  (parent 613)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (6 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0)))
  (label 1214)
  (parent 613)
  (unrealized (1 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec l) w e e)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eB)))
        key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1215)
  (parent 617)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-0))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) e e l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eB) e e)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1216)
  (parent 618)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (6 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eB) w e e)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1217)
  (parent 618)
  (unrealized (1 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eB) (rec l) w)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1218)
  (parent 619)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec l) w)) (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1219)
  (parent 623)
  (unrealized (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) eA))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 eA) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e l-0))) key)))
  (neq ((exp (gen) (mul eA l-0)) (gen)) (eA e)
    ((exp (gen) (mul e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) eA)) (send (cat (exp (gen) l-0) eA)))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1220)
  (parent 624)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (6 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eA w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e l-0))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1221)
  (parent 624)
  (unrealized (1 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (hl base)
    (l l-0 eB l-1 expn) (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 1222)
  (parent 628)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (hl base)
    (l l-0 eA l-1 expn) (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 1223)
  (parent 629)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (3 0)) ((6 1) (4 0)))
  (absent (l-1 (mul eA (rec eB) e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul e l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1224)
  (parent 630)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (hl base)
    (l l-0 eA eB expn) (w expr) (e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (7 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1)))
  (label 1225)
  (parent 630)
  (unrealized (1 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (hl base)
    (l l-0 eA eB l-1 expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 2) (3 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul eA (rec eB) (rec l-1) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e) (send e)))
  (label 1226)
  (parent 631)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1227)
  (parent 635)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec l) w l-0)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1228)
  (parent 636)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul eA (rec eB) l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul l l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1229)
  (parent 637)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (6 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1230)
  (parent 637)
  (unrealized (1 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1231)
  (parent 642)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 eB))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1232)
  (parent 643)
  (unrealized (1 3) (3 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1233)
  (parent 648)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-1 eB))) key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1234)
  (parent 649)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (3 0)) ((6 1) (4 0)))
  (absent (l-1 (mul eA (rec eB) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1235)
  (parent 650)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (7 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1)))
  (label 1236)
  (parent 650)
  (unrealized (1 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 0) (3 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-1) (send l-1)))
  (label 1237)
  (parent 651)
  (unrealized (1 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1238)
  (parent 654)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1239)
  (parent 655)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul w e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1240)
  (parent 658)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eB) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1241)
  (parent 659)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA w) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1242)
  (parent 662)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eB expn)
    (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w e)))
      (send (cat (gen) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 1243)
  (parent 666)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA expn)
    (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w e))) (send (cat (gen) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 1244)
  (parent 667)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul eA (rec eB) w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e) (send e)))
  (label 1245)
  (parent 668)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1246)
  (parent 671)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA w l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1247)
  (parent 672)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1248)
  (parent 676)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA w l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1249)
  (parent 677)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 1250)
  (parent 678)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-1)))
      (send (cat (gen) (mul (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1251)
  (parent 682)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA w l-1)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w l-1)))
      (send (cat (gen) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1252)
  (parent 683)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1253)
  (parent 684)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1254)
  (parent 686)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1255)
  (parent 687)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul w e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1256)
  (parent 689)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eB) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1257)
  (parent 690)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA w) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1258)
  (parent 692)
  (unrealized (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eB expn)
    (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w e)))
      (send (cat (gen) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 1259)
  (parent 695)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA expn)
    (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w e))) (send (cat (gen) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 1260)
  (parent 696)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul eA (rec eB) w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e) (send e)))
  (label 1261)
  (parent 697)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1262)
  (parent 699)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA w l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1263)
  (parent 700)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1264)
  (parent 703)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA w l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1265)
  (parent 704)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-1)))
      (send (cat (gen) (mul (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1266)
  (parent 708)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA w l-1)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w l-1)))
      (send (cat (gen) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1267)
  (parent 709)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1268)
  (parent 710)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1269)
  (parent 714)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul eA w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv e) (send e)))
  (label 1270)
  (parent 715)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e-0 e-0))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e e-0))) (l l-0) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul w e e e-0)) (gen)) (e e-0)
    ((exp (gen) (mul w e e-0 e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1271)
  (parent 719)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e-0)))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (e-0 e)
    ((exp (gen) (mul w (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1272)
  (parent 720)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul w (rec e))) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1273)
  (parent 724)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))) ((recv e) (send e)))
  (label 1274)
  (parent 725)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 e expn)
    (w expr) (e-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w e-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 hl) (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) (mul (rec e) w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv eA) (send eA)))
  (label 1275)
  (parent 729)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA expn)
    (w expr) (e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0 e-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e e-0))) (l l)
    (e e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eA w e)) (e (exp (gen) l-1)) (e hl)
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul eA w e e-0)) (gen)) (eA e-0)
    ((exp (gen) (mul w e e-0 e-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA w e)))
      (send (cat (exp (gen) e-0) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e-0) (send e-0)))
  (label 1276)
  (parent 730)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA e expn)
    (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eA (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 hl) (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv e-0) (send e-0)))
  (label 1277)
  (parent 731)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w l-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1278)
  (parent 735)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 e e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0 e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul eA w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 e e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 1279)
  (parent 736)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1280)
  (parent 741)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0 e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul eA w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-0 e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 1281)
  (parent 742)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 1282)
  (parent 743)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w l-1))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec e) w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1283)
  (parent 748)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1 e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (e (mul eA w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-1 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-1 e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-1 e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-1)))
      (send (cat (exp (gen) e) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 1284)
  (parent 749)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1285)
  (parent 750)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1286)
  (parent 753)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv e) (send e)))
  (label 1287)
  (parent 754)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e-0 e-0))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e e-0))) (l l) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul w e e e-0)) (gen)) (e e-0)
    ((exp (gen) (mul w e e-0 e-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1288)
  (parent 757)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e-0)))) (l l-0)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (e-0 e)
    ((exp (gen) (mul w (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1289)
  (parent 758)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul w (rec e))) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1290)
  (parent 761)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))) ((recv e) (send e)))
  (label 1291)
  (parent 762)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 e expn)
    (w expr) (e-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w e-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 hl) (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) (mul (rec e) w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv eA) (send eA)))
  (label 1292)
  (parent 765)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA expn)
    (w expr) (e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0 e-0))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e e-0))) (l l-0)
    (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eA w e)) (e (exp (gen) l-1)) (e hl)
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e-0 e-0)))
        key)))
  (neq ((exp (gen) (mul eA w e e-0)) (gen)) (eA e-0)
    ((exp (gen) (mul w e e-0 e-0)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA w e)))
      (send (cat (exp (gen) e-0) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e-0) (send e-0)))
  (label 1293)
  (parent 766)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA e expn)
    (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eA (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 hl) (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv e-0) (send e-0)))
  (label 1294)
  (parent 767)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w l-0))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1295)
  (parent 770)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0 e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul eA w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 1296)
  (parent 771)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1297)
  (parent 775)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 e e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0 e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul eA w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-0 e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-0 e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 1298)
  (parent 776)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w l-1))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec e) w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1299)
  (parent 781)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1 e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (e (mul eA w l-1)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 e e)))
        key)))
  (neq ((exp (gen) (mul eA w l-1 e)) (gen)) (eA e)
    ((exp (gen) (mul w l-1 e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-1)))
      (send (cat (exp (gen) e) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 1300)
  (parent 782)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1301)
  (parent 783)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eB e expn)
    (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1302)
  (parent 787)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA e expn)
    (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1303)
  (parent 788)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e (mul eA (rec eB) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e))))
    ((recv e) (send e)))
  (label 1304)
  (parent 789)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 e expn)
    (w expr) (l-1 e-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 eB))) (l l-1)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 e-0))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eB)))
        key)))
  (neq ((exp (gen) (mul w e-0 e-0)) (gen)) (e-0 eB)
    ((exp (gen) (mul w e-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 e l-1 e-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1305)
  (parent 793)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB expn)
    (w expr) (l-1 e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e e-0)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) w e e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul (rec eB) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((exp (gen) (mul (rec eB) w e e e-0)) (gen)) (e eB)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB l-1 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eB) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1306)
  (parent 794)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec e-0)))) (l l-1)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul (rec eB) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e-0 eB)
    ((exp (gen) (mul w (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1307)
  (parent 795)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 e expn)
    (w expr) (l-1 e-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) w) (gen)) (eA e-0)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 e l-1 e-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1308)
  (parent 799)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA expn)
    (w expr) (l-1 e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e-0))) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul eA w)) (e (exp (gen) l-1)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 hl) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e e-0)))
        key)))
  (neq ((exp (gen) (mul eA w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA l-1 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA w)))
      (send (cat (exp (gen) e-0) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1309)
  (parent 800)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB expn)
    (w expr) (l-1 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) w)))
      (send (cat (exp (gen) e) (mul (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1310)
  (parent 804)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA expn)
    (w expr) (l-1 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA w)) (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1311)
  (parent 805)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eB e expn) (w expr) (e-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 hl-0) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e e-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1312)
  (parent 809)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA e expn) (w expr) (e-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w e-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul eA (rec e) w e-0)) (e-0 (exp (gen) l-2)) (e-0 hl-0)
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1313)
  (parent 810)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB expn) (w expr) (e e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) w e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (8 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (e-0 (mul eA (rec eB) w e)) (e (exp (gen) l-2)) (e hl-0)
    (e-0 (exp (gen) l)) (e-0 hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e e-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) w e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1314)
  (parent 811)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e expn) (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e-0 (mul eA (rec eB) (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 hl-0) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1315)
  (parent 812)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1316)
  (parent 816)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 1317)
  (parent 817)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 e))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1 e)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e (mul eA (rec eB) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 e)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))) ((recv e) (send e)))
  (label 1318)
  (parent 818)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1319)
  (parent 823)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 1320)
  (parent 824)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1 e)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e (mul eA (rec eB) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))) ((recv e) (send e)))
  (label 1321)
  (parent 825)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB e expn)
    (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (7 0)) ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 1) (0 3)) ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1)))
  (label 1322)
  (parent 826)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e expn) (w expr) (l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w l-2)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1323)
  (parent 831)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e expn) (w expr) (l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul eA (rec e) w l-2)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eB)))
      (send (cat (exp (gen) l-2) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1324)
  (parent 832)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (l-2 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-2 e)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (8 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (e (mul eA (rec eB) w l-2)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w e)))
      (send (cat (exp (gen) l-2) (mul w e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 1325)
  (parent 833)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e expn) (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1326)
  (parent 834)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eB e expn)
    (w expr) (eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1327)
  (parent 837)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA e expn)
    (w expr) (eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1328)
  (parent 838)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 l-1 eA eB expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e (mul eA (rec eB) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e))))
    ((recv e) (send e)))
  (label 1329)
  (parent 839)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 e expn)
    (w expr) (l-1 e-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 eB))) (l l-1)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eB)))
        key)))
  (neq ((exp (gen) (mul w e-0 e-0)) (gen)) (e-0 eB)
    ((exp (gen) (mul w e-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 e l-1 e-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1330)
  (parent 842)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB expn)
    (w expr) (l-1 e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e e e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) w e e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul (rec eB) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((exp (gen) (mul (rec eB) w e e e-0)) (gen)) (e eB)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB l-1 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eB) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1331)
  (parent 843)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec e-0)))) (l l-1)
    (e e-0))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul (rec eB) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e-0 eB)
    ((exp (gen) (mul w (rec e-0))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1332)
  (parent 844)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 e expn)
    (w expr) (l-1 e-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e hl) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) w) (gen)) (eA e-0)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 e l-1 e-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1333)
  (parent 847)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA expn)
    (w expr) (l-1 e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e-0))) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul eA w)) (e (exp (gen) l-1)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 hl) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e e-0)))
        key)))
  (neq ((exp (gen) (mul eA w e-0)) (gen)) (eA e)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA l-1 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA w)))
      (send (cat (exp (gen) e-0) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1334)
  (parent 848)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB expn)
    (w expr) (l-1 e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) w)))
      (send (cat (exp (gen) e) (mul (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1335)
  (parent 851)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA expn)
    (w expr) (l-1 e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1336)
  (parent 852)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eB e expn) (w expr) (e-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e-0))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 hl-0) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e e-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1337)
  (parent 855)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA e expn) (w expr) (e-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e-0))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul eA (rec e) w e-0)) (e-0 (exp (gen) l-2)) (e-0 hl-0)
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1338)
  (parent 856)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB expn) (w expr) (e e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e e-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e e-0)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) w e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (8 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (e-0 (mul eA (rec eB) w e)) (e (exp (gen) l-2)) (e hl-0)
    (e-0 (exp (gen) l)) (e-0 hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e e-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e e-0)) (gen)) (eA eB)
    ((exp (gen) (mul w e e-0)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) w e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1339)
  (parent 857)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e expn) (w expr) (e-0 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e-0 (mul eA (rec eB) (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 hl-0) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1340)
  (parent 858)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1341)
  (parent 861)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 1342)
  (parent 862)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1 e)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e (mul eA (rec eB) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))) ((recv e) (send e)))
  (label 1343)
  (parent 863)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1344)
  (parent 867)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec e) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 1345)
  (parent 868)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 e))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1 e)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e (mul eA (rec eB) w l-1)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))) ((recv e) (send e)))
  (label 1346)
  (parent 869)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e expn) (w expr) (l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-2))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w l-2)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1347)
  (parent 874)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e expn) (w expr) (l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul eA (rec e) w l-2)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eB)))
      (send (cat (exp (gen) l-2) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1348)
  (parent 875)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (l-2 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-2 e)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (8 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (e (mul eA (rec eB) w l-2)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2 e)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2 e)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2 e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w e)))
      (send (cat (exp (gen) l-2) (mul w e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 1349)
  (parent 876)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e expn) (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1350)
  (parent 877)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1351)
  (parent 881)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1352)
  (parent 882)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec l-0) e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1353)
  (parent 887)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eB) w (rec l-0))) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1354)
  (parent 889)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul w (rec l-0))) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1355)
  (parent 893)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB l-1 expn)
    (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 1356)
  (parent 898)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA l-1 expn)
    (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 1357)
  (parent 899)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB l-1 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul eA (rec eB) (rec l-1) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e) (send e)))
  (label 1358)
  (parent 901)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1359)
  (parent 905)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1360)
  (parent 906)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1361)
  (parent 910)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1362)
  (parent 911)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) (rec l) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 1363)
  (parent 913)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1364)
  (parent 918)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1365)
  (parent 919)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1366)
  (parent 921)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1367)
  (parent 924)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1368)
  (parent 925)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec l) w e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1369)
  (parent 929)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eB) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1370)
  (parent 931)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul w (rec l-0))) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1371)
  (parent 934)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB l-1 expn)
    (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 1372)
  (parent 938)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA l-1 expn)
    (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 1373)
  (parent 939)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB l-1 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul eA (rec eB) (rec l-1) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e) (send e)))
  (label 1374)
  (parent 941)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1375)
  (parent 944)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1376)
  (parent 945)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1377)
  (parent 948)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1378)
  (parent 949)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1379)
  (parent 955)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1380)
  (parent 956)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1381)
  (parent 958)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1382)
  (parent 962)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1383)
  (parent 963)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (4 0)))
  (absent (l-0 (mul eA (rec eB))) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) (eA eB)
    ((exp (gen) l-0) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB)))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1384)
  (parent 964)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0)))
  (label 1385)
  (parent 964)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec l) w e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1386)
  (parent 968)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-0))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) e e l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eB) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul e l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1387)
  (parent 969)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eB) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1388)
  (parent 969)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eB) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1389)
  (parent 970)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1390)
  (parent 974)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) eA))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (4 0)))
  (absent (l-0 eA) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-0))) key)))
  (neq ((exp (gen) (mul eA l-0)) (gen)) (eA e)
    ((exp (gen) (mul e l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) eA)) (send (cat (exp (gen) l-0) eA)))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1391)
  (parent 975)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eA w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-0)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1392)
  (parent 975)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB l-1 expn)
    (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 1393)
  (parent 979)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA l-1 expn)
    (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 1394)
  (parent 980)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (4 0)))
  (absent (l-1 (mul eA (rec eB) e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul e l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1395)
  (parent 981)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB expn)
    (w expr) (e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (7 0)) ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1)))
  (label 1396)
  (parent 981)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB l-1 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul eA (rec eB) (rec l-1) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e) (send e)))
  (label 1397)
  (parent 982)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1398)
  (parent 986)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1399)
  (parent 987)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (4 0)))
  (absent (l-0 (mul eA (rec eB) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul l l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1400)
  (parent 988)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l l-0)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l l-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1401)
  (parent 988)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1402)
  (parent 993)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1403)
  (parent 994)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1404)
  (parent 999)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1405)
  (parent 1000)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (4 0)))
  (absent (l-1 (mul eA (rec eB) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1406)
  (parent 1001)
  (unrealized (1 3) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (7 0)) ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0))
    ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1407)
  (parent 1001)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1408)
  (parent 1002)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 expn) (w expr) (eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1409)
  (parent 1005)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 expn) (w expr) (eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1410)
  (parent 1006)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec l-0) e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1411)
  (parent 1010)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eB) w (rec l-0))) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1412)
  (parent 1012)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l expn) (w expr) (l-0 e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1413)
  (parent 1015)
  (unrealized (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eB l-1 expn)
    (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 1414)
  (parent 1019)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA l-1 expn)
    (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA (rec l-1) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 1415)
  (parent 1020)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (hl base) (l l-0 eA eB l-1 expn)
    (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul eA (rec eB) (rec l-1) w)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e) (send e)))
  (label 1416)
  (parent 1022)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1417)
  (parent 1025)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1418)
  (parent 1026)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1419)
  (parent 1031)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 1420)
  (parent 1032)
  (unrealized (1 3) (4 0) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1421)
  (parent 1036)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1422)
  (parent 1037)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1423)
  (parent 1039)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 expn) (w expr)
    (eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1424)
  (parent 1044)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 expn) (w expr)
    (eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1425)
  (parent 1045)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul eA (rec eB))) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB)))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1426)
  (parent 1046)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1)))
  (label 1427)
  (parent 1046)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 expn) (w expr)
    (l-1 e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec l-0) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eB (exp (gen) l-1)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1428)
  (parent 1051)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) e e l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eB) e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) e e l-1)) (gen)) (e eB)
    ((exp (gen) (mul e l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1429)
  (parent 1052)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eB) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-1)) (gen)) (e eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1430)
  (parent 1052)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eB) (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eB (exp (gen) l-1)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1431)
  (parent 1053)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 expn) (w expr)
    (l-1 e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec l-0) w)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-1 B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1432)
  (parent 1058)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) eA))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 eA) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-1))) key)))
  (neq ((exp (gen) (mul eA l-1)) (gen)) (eA e)
    ((exp (gen) (mul e l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) eA)) (send (cat (exp (gen) l-1) eA)))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1433)
  (parent 1059)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1434)
  (parent 1059)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eB l-2 expn) (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-2) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 1435)
  (parent 1064)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA l-2 expn) (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul eA (rec l-2) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 1436)
  (parent 1065)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) e l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul eA (rec eB) e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul e l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1437)
  (parent 1066)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e l-2)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (l-2 (mul eA (rec eB) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2)))
  (label 1438)
  (parent 1066)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e (mul eA (rec eB) (rec l-2) w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e) (send e)))
  (label 1439)
  (parent 1067)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1440)
  (parent 1072)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 eB))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 1441)
  (parent 1073)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0 l-1)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1442)
  (parent 1074)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1443)
  (parent 1074)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1444)
  (parent 1080)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 1445)
  (parent 1081)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1446)
  (parent 1082)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1447)
  (parent 1082)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (7 0)) ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0))
    ((6 1) (0 3)) ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1)))
  (label 1448)
  (parent 1083)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1449)
  (parent 1088)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 1450)
  (parent 1089)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 expn) (w expr)
    (l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-1) w l-2)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1451)
  (parent 1095)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 expn) (w expr)
    (l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul eA (rec l-1) w l-2)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eB)))
      (send (cat (exp (gen) l-2) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1452)
  (parent 1096)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-1 l-2)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul eA (rec eB) l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1453)
  (parent 1097)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1 l-2)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (l-2 (mul eA (rec eB) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 D) (l B) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1454)
  (parent 1097)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 expn)
    (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul eA (rec eB) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1455)
  (parent 1098)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 expn) (w expr)
    (eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 1456)
  (parent 1102)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 expn) (w expr)
    (eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 1457)
  (parent 1103)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul eA (rec eB))) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) (eA eB)
    ((exp (gen) l-1) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB)))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1458)
  (parent 1104)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1)))
  (label 1459)
  (parent 1104)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 expn) (w expr)
    (l-1 e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e eB))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec l-0) w e e)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e eB))) key)))
  (neq ((exp (gen) (mul w e e)) (gen)) (e eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 1460)
  (parent 1108)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) e e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eB) e e)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) e e l-1)) (gen)) (e eB)
    ((exp (gen) (mul e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1461)
  (parent 1109)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eB) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-1)) (gen)) (e eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1462)
  (parent 1109)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec e)))) (l l-1) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eB) (rec l-0) w)) (e (exp (gen) l-1))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (e eB)
    ((exp (gen) (mul w (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1463)
  (parent 1110)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 expn) (w expr)
    (l-1 e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec l-0) w)) (e (exp (gen) l-1)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) w) (gen)) (eA e)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1464)
  (parent 1114)
  (unrealized (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) eA))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 eA) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-1))) key)))
  (neq ((exp (gen) (mul eA l-1)) (gen)) (eA e)
    ((exp (gen) (mul e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) eA)) (send (cat (exp (gen) l-1) eA)))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1465)
  (parent 1115)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1466)
  (parent 1115)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eB l-2 expn) (w expr) (e eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-2) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 e eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 1467)
  (parent 1119)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA l-2 expn) (w expr) (e eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul eA (rec l-2) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 1468)
  (parent 1120)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) e l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul eA (rec eB) e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul e l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1469)
  (parent 1121)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e l-2)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (l-2 (mul eA (rec eB) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2)))
  (label 1470)
  (parent 1121)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 expn) (w expr) (e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (4 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e (mul eA (rec eB) (rec l-2) w)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e) (send e)))
  (label 1471)
  (parent 1122)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1472)
  (parent 1126)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 1473)
  (parent 1127)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-0 l-1)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1474)
  (parent 1128)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1475)
  (parent 1128)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1476)
  (parent 1133)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 eB))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 1477)
  (parent 1134)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1478)
  (parent 1135)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1479)
  (parent 1135)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 1480)
  (parent 1140)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (absent (eB (mul eA w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 1481)
  (parent 1141)
  (unrealized (1 3) (4 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 expn) (w expr)
    (l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-1) w l-2)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1482)
  (parent 1146)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 expn) (w expr)
    (l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul eA (rec l-1) w l-2)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eB)))
      (send (cat (exp (gen) l-2) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1483)
  (parent 1147)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) l-1 l-2)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul eA (rec eB) l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1484)
  (parent 1148)
  (unrealized (1 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1 l-2)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (l-2 (mul eA (rec eB) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1485)
  (parent 1148)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 expn)
    (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul eA (rec eB) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1486)
  (parent 1149)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1487)
  (parent 1162)
  (unrealized (1 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec e) (rec e) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1488)
  (parent 1176)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e expn) (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1489)
  (parent 1197)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1490)
  (parent 1210)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1491)
  (parent 1214)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eB) w e e)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1492)
  (parent 1217)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul eA w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e l-0))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1493)
  (parent 1221)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB expn) (w expr) (e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1494)
  (parent 1225)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1495)
  (parent 1230)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1496)
  (parent 1236)
  (unrealized (1 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1497)
  (parent 1237)
  (unrealized (1 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1498)
  (parent 1250)
  (unrealized (1 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1499)
  (parent 1253)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1500)
  (parent 1268)
  (unrealized (1 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1501)
  (parent 1282)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1502)
  (parent 1285)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec e) w))) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) (eA e)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1503)
  (parent 1301)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB e expn) (w expr) (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB e l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1504)
  (parent 1322)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e expn) (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1505)
  (parent 1326)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e expn) (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) (rec e) w)) (e (exp (gen) l)) (e hl)
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1506)
  (parent 1350)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) (rec l) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1507)
  (parent 1363)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1508)
  (parent 1366)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1509)
  (parent 1381)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB expn) (w expr) (l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1510)
  (parent 1385)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eB) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1511)
  (parent 1388)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul eA w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-0)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1512)
  (parent 1392)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB expn) (w expr) (e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w e)) (e (exp (gen) l-0)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1513)
  (parent 1396)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l l-0)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l l-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1514)
  (parent 1401)
  (unrealized (1 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1515)
  (parent 1407)
  (unrealized (1 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1516)
  (parent 1408)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1517)
  (parent 1423)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1518)
  (parent 1427)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e e l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eB) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-1)) (gen)) (e eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1519)
  (parent 1430)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1520)
  (parent 1434)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e l-2)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1521)
  (parent 1438)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1522)
  (parent 1443)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1523)
  (parent 1447)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1524)
  (parent 1448)
  (unrealized (1 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1 l-2)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1525)
  (parent 1454)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 expn)
    (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1526)
  (parent 1455)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB expn) (w expr)
    (l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1527)
  (parent 1459)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eB) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w e e l-1)) (gen)) (e eB)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1528)
  (parent 1462)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1529)
  (parent 1466)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w e l-2)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) w e)) (e (exp (gen) l-1)) (e hl)
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1530)
  (parent 1470)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1531)
  (parent 1475)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1532)
  (parent 1479)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1 l-2)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1533)
  (parent 1485)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 expn)
    (w expr) (l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) w) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1534)
  (parent 1486)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1535)
  (parent 1487)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-0 (mul (rec eB) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1536)
  (parent 1487)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul eA w)) (l-0 (mul eA w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1537)
  (parent 1487)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1538)
  (parent 1488)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1539)
  (parent 1488)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1540)
  (parent 1488)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (e (mul eA w)) (l-0 (mul eA w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1541)
  (parent 1488)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul (rec eA) eB e)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1542)
  (parent 1489)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1543)
  (parent 1489)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (l-1 (mul (rec eB) (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1544)
  (parent 1489)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec e) w)) (l-1 (mul eA (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w))
    (mul eA (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1545)
  (parent 1489)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB expn) (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (e (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1546)
  (parent 1489)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l (one)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0)))
      (send (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1547)
  (parent 1490)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1548)
  (parent 1490)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l) w)) (l-0 (mul (rec eB) (rec l) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l) w))
    (mul (rec eB) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1549)
  (parent 1490)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec l) w)) (l-0 (mul eA (rec l) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l) w))
    (mul eA (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1550)
  (parent 1490)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w)) (l (mul eA (rec eB) w))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l-0)))
        key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l) (mul w (rec l) l-0)))
      (send (cat (exp (gen) l) (mul w (rec l) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1551)
  (parent 1490)
  (unrealized (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul l-0 (rec eA) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul l-0 (rec eA) eB)))
      (send (cat (gen) (mul l-0 (rec eA) eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1552)
  (parent 1491)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eB l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 (rec eA) eB l-1)))
      (send (cat (gen) (mul l-0 (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1553)
  (parent 1491)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-0 (mul (rec eB) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w l-0 (rec eA))))
      (send (cat (gen) (mul w l-0 (rec eA))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1554)
  (parent 1491)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (gen) (mul w l-0 eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul eA w)) (l-0 (mul eA w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 eB))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (gen) (mul w l-0 eB)))
      (send (cat (gen) (mul w l-0 eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1555)
  (parent 1491)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eB e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 eB (rec e)))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul l-0 eB (rec e))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul eB (rec e) (rec e))) (e-0 e)) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 eB (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1556)
  (parent 1492)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 eB (rec e) l-1))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eB l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (e eB)
    ((exp (gen) (mul l-0 eB (rec e) l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 eB (rec e) l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1557)
  (parent 1492)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expr) (l e l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0 eB))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eB)))
        key)))
  (neq ((exp (gen) (mul w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0 eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eB)))
      (send (cat (exp (gen) e) (mul w l-0 eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1558)
  (parent 1492)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB expn) (w expr) (l l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec e)))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (e (mul (rec eB) w)) (l-0 (mul (rec eB) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (e eB)
    ((exp (gen) (mul w l-0 (rec e))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1559)
  (parent 1492)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul e l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (contracted (eA-0 eA) (w (rec eA))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eA))))
      (send (cat (exp (gen) e) (mul l-0 (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1560)
  (parent 1493)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e l-0 eA l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-0 (rec eA) l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eA) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e l-0 (rec eA) l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-0 (rec eA) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eA) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec eA) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1561)
  (parent 1493)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expr) (l e l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA w) (l-0 w) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-0 (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1562)
  (parent 1493)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (absent (l-0 (mul eA w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e l-0))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-listener eA) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1563)
  (parent 1493)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eB))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec e)))
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e))))
      (send (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1564)
  (parent 1494)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eB l-2))) key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1565)
  (parent 1494)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eB expn)
    (w expr) (e l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w e)) (l-1 (mul (rec eB) w e))
    (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w e))
    (mul (rec eB) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1566)
  (parent 1494)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eA expn)
    (w expr) (e l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w e)) (l-1 (mul eA w e)) (e (exp (gen) l-0))
    (e hl) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w e)) (mul eA w e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA w e)))
      (send (cat (exp (gen) l-1) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eB)))
      (send (cat (exp (gen) e) (mul w l-1 eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1567)
  (parent 1494)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB expn) (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (e (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1568)
  (parent 1494)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l (rec eA) eB))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l (one)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eB))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul l (rec eA) eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1569)
  (parent 1495)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l (rec eA) eB l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eB l-1)))
        key)))
  (neq ((exp (gen) (mul l l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1570)
  (parent 1495)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB expn) (w expr) (l l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w l)) (l-0 (mul (rec eB) w l))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l))
    (mul (rec eB) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec eA))))
      (send (cat (exp (gen) l) (mul w l-0 (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1571)
  (parent 1495)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA expn) (w expr) (l l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul eA w l)) (l-0 (mul eA w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w l)) (mul eA w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w l)))
      (send (cat (exp (gen) l-0) (mul eA w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eB)))
      (send (cat (exp (gen) l) (mul w l-0 eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1572)
  (parent 1495)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w l (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w)) (l (mul eA (rec eB) w))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w l))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l)) (gen)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w l (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1573)
  (parent 1495)
  (unrealized (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-1)))
      (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1574)
  (parent 1496)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec eA) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eB l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1575)
  (parent 1496)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eB l-2))) key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1576)
  (parent 1496)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1577)
  (parent 1496)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1578)
  (parent 1496)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1579)
  (parent 1496)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1580)
  (parent 1497)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1 l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1 l-1))) key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1581)
  (parent 1497)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1 l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1 l-2))) key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1582)
  (parent 1497)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1583)
  (parent 1497)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1584)
  (parent 1497)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1585)
  (parent 1497)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1586)
  (parent 1498)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-0 (mul (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1587)
  (parent 1498)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul eA w)) (l-0 (mul eA w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1588)
  (parent 1498)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1589)
  (parent 1499)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-2)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1590)
  (parent 1499)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1591)
  (parent 1499)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1592)
  (parent 1499)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1593)
  (parent 1500)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1594)
  (parent 1500)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1595)
  (parent 1500)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA e expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1596)
  (parent 1501)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1597)
  (parent 1501)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1598)
  (parent 1501)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (e (mul eA w)) (l-0 (mul eA w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1599)
  (parent 1501)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1600)
  (parent 1502)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1))) key)))
  (neq ((exp (gen) (mul e l-1)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) e e l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1601)
  (parent 1502)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1602)
  (parent 1502)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1603)
  (parent 1502)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul eA w)) (l-1 (mul eA w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1604)
  (parent 1502)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1605)
  (parent 1503)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-2))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA e)
    ((exp (gen) (mul (rec eA) e e l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1606)
  (parent 1503)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1607)
  (parent 1503)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul eA w)) (l-1 (mul eA w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1608)
  (parent 1503)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul (rec eA) eB e)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1609)
  (parent 1504)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1610)
  (parent 1504)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (l-1 (mul (rec eB) (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1611)
  (parent 1504)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eB (mul eA (rec e) w)) (l-1 (mul eA (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w))
    (mul eA (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1612)
  (parent 1504)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB expn) (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (e (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1613)
  (parent 1504)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 l-2 eA eB e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 l-2 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul (rec eA) eB e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1614)
  (parent 1505)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-2)) (eA (exp (gen) l-0)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-2)) (exp (gen) (mul eB e l-2)))
        key)))
  (neq ((exp (gen) (mul e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)))
  (non-orig (privk B) (privk self-1) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 l-1 eA eB e l-2)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1615)
  (parent 1505)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 l-2 eA eB e l-3 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-3)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-2 l-3) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-3)))
        key)))
  (neq ((exp (gen) (mul e l-3)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-3)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 l-2 eA eB e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-3))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1616)
  (parent 1505)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e expn) (w expr) (l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (l-2 (mul (rec eB) (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1617)
  (parent 1505)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e expn) (w expr) (l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul eA (rec e) w)) (l-2 (mul eA (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w))
    (mul eA (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eB))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1618)
  (parent 1505)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (l-2 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (e (mul eA (rec eB) w)) (l-2 (mul eA (rec eB) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1619)
  (parent 1505)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 l-2 eA eB e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq ((exp (gen) e) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 l-2 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul (rec eA) eB e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1620)
  (parent 1506)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 l-2 eA eB e l-3 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-3)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-3))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-2 l-3) (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-3)))
        key)))
  (neq ((exp (gen) (mul e l-3)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB e l-3)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 l-2 eA eB e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-3))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1621)
  (parent 1506)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e expn) (w expr) (l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (l-2 (mul (rec eB) (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1622)
  (parent 1506)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e expn) (w expr) (l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul eA (rec e) w)) (l-2 (mul eA (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w))
    (mul eA (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eB))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1623)
  (parent 1506)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (l-2 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (e (mul eA (rec eB) w)) (l-2 (mul eA (rec eB) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1624)
  (parent 1506)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0)) (gen)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0)))
      (send (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1625)
  (parent 1507)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1626)
  (parent 1507)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l) w)) (l-0 (mul (rec eB) (rec l) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l) w))
    (mul (rec eB) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1627)
  (parent 1507)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul eA (rec l) w)) (l-0 (mul eA (rec l) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l) w))
    (mul eA (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1628)
  (parent 1507)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w)) (l (mul eA (rec eB) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l) (mul w (rec l) l-0)))
      (send (cat (exp (gen) l) (mul w (rec l) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1629)
  (parent 1507)
  (unrealized (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l D) (l-1 B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1630)
  (parent 1508)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) (rec eA) eB l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1631)
  (parent 1508)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1632)
  (parent 1508)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1633)
  (parent 1508)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1634)
  (parent 1508)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l D) (l-1 B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1635)
  (parent 1508)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1636)
  (parent 1509)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1637)
  (parent 1509)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1638)
  (parent 1509)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1639)
  (parent 1509)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1640)
  (parent 1509)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul l-0 (rec eA) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul l-0 (rec eA) eB)))
      (send (cat (gen) (mul l-0 (rec eA) eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1641)
  (parent 1510)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 (rec eA) eB l-1)))
      (send (cat (gen) (mul l-0 (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1642)
  (parent 1510)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-0 (mul (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w l-0 (rec eA))))
      (send (cat (gen) (mul w l-0 (rec eA))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1643)
  (parent 1510)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (gen) (mul w l-0 eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul eA w)) (l-0 (mul eA w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (gen) (mul w l-0 eB)))
      (send (cat (gen) (mul w l-0 eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1644)
  (parent 1510)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eB e expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 eB (rec e)))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (e eB)
    ((exp (gen) (mul l-0 eB (rec e))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul eB (rec e) (rec e))) (e-0 e)) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eB (rec e))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1645)
  (parent 1511)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 eB (rec e) l-1))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (e eB)
    ((exp (gen) (mul l-0 eB (rec e) l-1)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eB (rec e) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1646)
  (parent 1511)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expr) (l e l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0 eB))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0 eB)))
        key)))
  (neq ((exp (gen) (mul w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0 eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eB)))
      (send (cat (exp (gen) e) (mul w l-0 eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1647)
  (parent 1511)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB expn) (w expr) (l l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec e)))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (e (mul (rec eB) w)) (l-0 (mul (rec eB) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (e eB)
    ((exp (gen) (mul w l-0 (rec e))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1648)
  (parent 1511)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul e l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (contracted (eA-0 eA) (w (rec eA))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eA))))
      (send (cat (exp (gen) e) (mul l-0 (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1649)
  (parent 1512)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e l-0 eA l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-0 (rec eA) l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eA) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e l-0 (rec eA) l-1)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-0 (rec eA) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eA) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec eA) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1650)
  (parent 1512)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expr) (l e l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA w) (l-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-0 (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1651)
  (parent 1512)
  (unrealized (0 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA expn) (w expr) (l e l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (3 0)))
  (absent (l-0 (mul eA w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-0)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-listener eA) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1652)
  (parent 1512)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eB))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec e)))
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e))))
      (send (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1653)
  (parent 1513)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-1 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1654)
  (parent 1513)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eB expn)
    (w expr) (e l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w e)) (l-1 (mul (rec eB) w e))
    (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w e))
    (mul (rec eB) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1655)
  (parent 1513)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eA expn)
    (w expr) (e l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w e l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eB (mul eA w e)) (l-1 (mul eA w e)) (e (exp (gen) l-0))
    (e hl) (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1 eB)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w e)) (mul eA w e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA w e)))
      (send (cat (exp (gen) l-1) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eB)))
      (send (cat (exp (gen) e) (mul w l-1 eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1656)
  (parent 1513)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB expn) (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (e (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1657)
  (parent 1513)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l)) (he (exp (gen) (mul l (rec eA) eB))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l)) (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eB))) key)))
  (neq ((exp (gen) l) (gen)) (eA eB)
    ((exp (gen) (mul l (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1658)
  (parent 1514)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l)) (he (exp (gen) (mul l (rec eA) eB l-1))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l)) (he (exp (gen) (mul l l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eB l-1))) key)))
  (neq ((exp (gen) (mul l l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1659)
  (parent 1514)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB expn) (w expr) (l l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul (rec eB) w l)) (l-0 (mul (rec eB) w l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l))
    (mul (rec eB) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec eA))))
      (send (cat (exp (gen) l) (mul w l-0 (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1660)
  (parent 1514)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA expn) (w expr) (l l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul eA w l)) (l-0 (mul eA w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0 eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul eA w l)) (mul eA w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w l)))
      (send (cat (exp (gen) l-0) (mul eA w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eB)))
      (send (cat (exp (gen) l) (mul w l-0 eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1661)
  (parent 1514)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB expn) (w expr) (l l-0 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w l (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul eA (rec eB) w)) (l (mul eA (rec eB) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l)) (gen)) (eA eB)
    ((exp (gen) (mul w l)) (gen)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w l (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1662)
  (parent 1514)
  (unrealized (1 3) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-1)))
      (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1663)
  (parent 1515)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec eA) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1664)
  (parent 1515)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1665)
  (parent 1515)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1666)
  (parent 1515)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0 l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1667)
  (parent 1515)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1668)
  (parent 1515)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1669)
  (parent 1516)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1670)
  (parent 1516)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1671)
  (parent 1516)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1672)
  (parent 1516)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1673)
  (parent 1516)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1674)
  (parent 1516)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1675)
  (parent 1517)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1676)
  (parent 1517)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1677)
  (parent 1517)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1678)
  (parent 1517)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1679)
  (parent 1517)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul l-1 (rec eA) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul l-1 (rec eA) eB)))
      (send (cat (gen) (mul l-1 (rec eA) eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1680)
  (parent 1518)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 (rec eA) eB l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 (rec eA) eB l-1)))
      (send (cat (gen) (mul l-0 (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1681)
  (parent 1518)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 (rec eA) eB l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 (rec eA) eB l-2)))
      (send (cat (gen) (mul l-1 (rec eA) eB l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1682)
  (parent 1518)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w l-1 (rec eA))))
      (send (cat (gen) (mul w l-1 (rec eA))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1683)
  (parent 1518)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (gen) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (gen) (mul w l-1 eB)))
      (send (cat (gen) (mul w l-1 eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1684)
  (parent 1518)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eB e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 eB (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) key)))
  (neq ((exp (gen) l-1) (gen)) (e eB)
    ((exp (gen) (mul l-1 eB (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul eB (rec e) (rec e))) (e-0 e)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eB (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1685)
  (parent 1519)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eB e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 eB (rec e) l-1))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (e eB)
    ((exp (gen) (mul l-0 eB (rec e) l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eB (rec e) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eB (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1686)
  (parent 1519)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 eB (rec e) l-2))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (e eB)
    ((exp (gen) (mul l-1 eB (rec e) l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eB (rec e) l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1687)
  (parent 1519)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l expn) (w expr)
    (l-0 e l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1 eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eB)))
        key)))
  (neq ((exp (gen) (mul w e e l-1)) (gen)) (e eB)
    ((exp (gen) (mul w e l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1 eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eB)))
      (send (cat (exp (gen) e) (mul w l-1 eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1688)
  (parent 1519)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (e eB)
    ((exp (gen) (mul w l-1 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1689)
  (parent 1519)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA e)
    ((exp (gen) (mul e l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (contracted (eA-0 eA) (w (rec eA))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1690)
  (parent 1520)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e l-0 eA l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-0 (rec eA) l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eA) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA e)
    ((exp (gen) (mul e l-0 (rec eA) l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-0 (rec eA) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eA) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec eA) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1691)
  (parent 1520)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 e l-1 eA l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1 (rec eA) l-2))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA e)
    ((exp (gen) (mul e l-1 (rec eA) l-2)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1 (rec eA) l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec eA) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1692)
  (parent 1520)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l expn) (w expr)
    (l-0 e l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA w) (l-1 w) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) key)))
  (neq ((exp (gen) (mul w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w e l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-absence eA w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1693)
  (parent 1520)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-listener eA) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1694)
  (parent 1520)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 l-2 eA eB e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-2 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-2)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eB))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul l-2 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e))))
      (send (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1695)
  (parent 1521)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 eA eB e l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul l-1 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-2)) (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec eA) eB (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1696)
  (parent 1521)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 l-2 eA eB e l-3 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-2 (rec eA) eB l-3)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-2 l-3))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e) l-3)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-2 l-3) (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eB l-3)))
        key)))
  (neq ((exp (gen) (mul l-2 l-3)) (gen)) (eA eB)
    ((exp (gen) (mul l-2 (rec eA) eB l-3)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 (rec eA) eB l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e) l-3)))
      (send (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1697)
  (parent 1521)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB expn) (w expr) (e l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) w e)) (l-2 (mul (rec eB) w e))
    (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w e))
    (mul (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1698)
  (parent 1521)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA expn) (w expr) (e l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul eA w e)) (l-2 (mul eA w e)) (e (exp (gen) l-1))
    (e hl) (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul eA w e)) (mul eA w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA w e)))
      (send (cat (exp (gen) l-2) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eB)))
      (send (cat (exp (gen) e) (mul w l-2 eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1699)
  (parent 1521)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (l-2 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (e (mul eA (rec eB) w)) (l-2 (mul eA (rec eB) w))
    (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1700)
  (parent 1521)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1701)
  (parent 1522)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l (rec eA) eB l-1)))
    (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l eB l-1)))
        key)))
  (neq ((exp (gen) (mul l l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l (rec eA) eB (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1702)
  (parent 1522)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB l-2)))
    (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1703)
  (parent 1522)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1704)
  (parent 1522)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0 l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1705)
  (parent 1522)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1706)
  (parent 1522)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1707)
  (parent 1523)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 (rec eA) eB l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec eA) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB l-1)))
        key)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1708)
  (parent 1523)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1709)
  (parent 1523)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1710)
  (parent 1523)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1711)
  (parent 1523)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1712)
  (parent 1523)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1713)
  (parent 1524)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1)))
        key)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1714)
  (parent 1524)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1715)
  (parent 1524)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1716)
  (parent 1524)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1717)
  (parent 1524)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1718)
  (parent 1524)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-2)))
      (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1719)
  (parent 1525)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul l-0 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-2))
    (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1720)
  (parent 1525)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec eA) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec eA) eB)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec eA) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1721)
  (parent 1525)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-3)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-3))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-3)))
        key)))
  (neq ((exp (gen) (mul l-1 l-3)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-3)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1722)
  (parent 1525)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB expn) (w expr)
    (l-1 l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1 l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) w l-1)) (l-2 (mul (rec eB) w l-1))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-1))
    (mul (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eA))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1723)
  (parent 1525)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA expn) (w expr)
    (l-1 l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2 eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1 l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul eA w l-1)) (l-2 (mul eA w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul eA w l-1)) (mul eA w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA w l-1)))
      (send (cat (exp (gen) l-2) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eB)))
      (send (cat (exp (gen) l-1) (mul w l-2 eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1724)
  (parent 1525)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1725)
  (parent 1525)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-2)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 D) (l B) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2) (w (mul (rec eA) eB l-2)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1726)
  (parent 1526)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-2))
    (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1727)
  (parent 1526)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-2 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-2)))
        key)))
  (neq ((exp (gen) (mul l-2 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2 l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1728)
  (parent 1526)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-3)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-2 l-3))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-0 D) (l B) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-3)))
        key)))
  (neq ((exp (gen) (mul l-2 l-3)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2 l-3)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1729)
  (parent 1526)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 expn) (w expr)
    (l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) (rec l-1) w))
    (l-2 (mul (rec eB) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w))
    (mul (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1730)
  (parent 1526)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 expn) (w expr)
    (l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul eA (rec l-1) w)) (l-2 (mul eA (rec l-1) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w))
    (mul eA (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eB))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1731)
  (parent 1526)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 D) (l B) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1732)
  (parent 1526)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul l-1 (rec eA) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul l-1 (rec eA) eB)))
      (send (cat (gen) (mul l-1 (rec eA) eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1733)
  (parent 1527)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 (rec eA) eB l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 (rec eA) eB l-2)))
      (send (cat (gen) (mul l-1 (rec eA) eB l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1734)
  (parent 1527)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w l-1 (rec eA))))
      (send (cat (gen) (mul w l-1 (rec eA))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1735)
  (parent 1527)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (gen) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (gen) (mul w l-1 eB)))
      (send (cat (gen) (mul w l-1 eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1736)
  (parent 1527)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eB e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 eB (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) key)))
  (neq ((exp (gen) l-1) (gen)) (e eB)
    ((exp (gen) (mul l-1 eB (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul eB (rec e) (rec e))) (e-0 e)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eB (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1737)
  (parent 1528)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eB e l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 eB (rec e) l-2))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (e eB)
    ((exp (gen) (mul l-1 eB (rec e) l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eB (rec e) l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1738)
  (parent 1528)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l expn) (w expr)
    (l-0 e l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1 eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eB)))
        key)))
  (neq ((exp (gen) (mul w e e l-1)) (gen)) (e eB)
    ((exp (gen) (mul w e l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1 eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eB)))
      (send (cat (exp (gen) e) (mul w l-1 eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1739)
  (parent 1528)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (e eB)
    ((exp (gen) (mul w l-1 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1740)
  (parent 1528)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA e)
    ((exp (gen) (mul e l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (contracted (eA-0 eA) (w (rec eA))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1741)
  (parent 1529)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 e l-1 eA l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1 (rec eA) l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA e)
    ((exp (gen) (mul e l-1 (rec eA) l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1 (rec eA) l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec eA) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1742)
  (parent 1529)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l expn) (w expr)
    (l-0 e l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA w) (l-1 w) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) key)))
  (neq ((exp (gen) (mul w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w e l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-absence eA w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1743)
  (parent 1529)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 e l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul eA w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-1)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w e l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-listener eA) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1744)
  (parent 1529)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 l-2 eA eB e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-2 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eB))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul l-2 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e))))
      (send (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1745)
  (parent 1530)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 l-2 eA eB e l-3 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-2 (rec eA) eB l-3)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-2 l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e) l-3)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-2 l-3) (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eB l-3)))
        key)))
  (neq ((exp (gen) (mul l-2 l-3)) (gen)) (eA eB)
    ((exp (gen) (mul l-2 (rec eA) eB l-3)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 (rec eA) eB l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e) l-3)))
      (send (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1746)
  (parent 1530)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB expn) (w expr) (e l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) w e)) (l-2 (mul (rec eB) w e))
    (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w e))
    (mul (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1747)
  (parent 1530)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA expn) (w expr) (e l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul eA w e)) (l-2 (mul eA w e)) (e (exp (gen) l-1))
    (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul eA w e)) (mul eA w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA w e)))
      (send (cat (exp (gen) l-2) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eB)))
      (send (cat (exp (gen) e) (mul w l-2 eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1748)
  (parent 1530)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (l-2 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (e (mul eA (rec eB) w)) (l-2 (mul eA (rec eB) w))
    (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1749)
  (parent 1530)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1750)
  (parent 1531)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1751)
  (parent 1531)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1752)
  (parent 1531)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0 l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1753)
  (parent 1531)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1754)
  (parent 1531)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l-1)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1755)
  (parent 1532)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB l-2)))
    (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1756)
  (parent 1532)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1757)
  (parent 1532)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1758)
  (parent 1532)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l-1) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1759)
  (parent 1532)
  (unrealized (1 3) (4 0) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-2)))
      (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1760)
  (parent 1533)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec eA) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec eA) eB)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec eA) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1761)
  (parent 1533)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-3)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-3)))
        key)))
  (neq ((exp (gen) (mul l-1 l-3)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-3)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1762)
  (parent 1533)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB expn) (w expr)
    (l-1 l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1 l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) w l-1)) (l-2 (mul (rec eB) w l-1))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-1))
    (mul (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eA))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1763)
  (parent 1533)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA expn) (w expr)
    (l-1 l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2 eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1 l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul eA w l-1)) (l-2 (mul eA w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul eA w l-1)) (mul eA w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA w l-1)))
      (send (cat (exp (gen) l-2) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eB)))
      (send (cat (exp (gen) l-1) (mul w l-2 eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1764)
  (parent 1533)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1765)
  (parent 1533)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq ((exp (gen) l-2) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2) (w (mul (rec eA) eB l-2)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1766)
  (parent 1534)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-2 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-2)))
        key)))
  (neq ((exp (gen) (mul l-2 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2 l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1767)
  (parent 1534)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-3)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-2 l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-3)))
        key)))
  (neq ((exp (gen) (mul l-2 l-3)) (gen)) (eA eB)
    ((exp (gen) (mul (rec eA) eB l-2 l-3)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1768)
  (parent 1534)
  (unrealized (0 3) (6 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 expn) (w expr)
    (l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) (rec l-1) w))
    (l-2 (mul (rec eB) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w))
    (mul (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1769)
  (parent 1534)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 expn) (w expr)
    (l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul eA (rec l-1) w)) (l-2 (mul eA (rec l-1) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w))
    (mul eA (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eB))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1770)
  (parent 1534)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1771)
  (parent 1534)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul (rec eB) w)) (l-0 (mul (rec eB) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) (rec eA))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1772)
  (parent 1536)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul eA w)) (l-0 (mul eA w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1773)
  (parent 1537)
  (unrealized (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) (rec eA))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1774)
  (parent 1540)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul eA w)) (l-0 (mul eA w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 1775)
  (parent 1541)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (l-1 (mul (rec eB) (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1776)
  (parent 1544)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA (rec e) w)) (l-1 (mul eA (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1777)
  (parent 1545)
  (unrealized (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB expn) (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (8 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 1778)
  (parent 1546)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul (rec eB) (rec l) w)) (l-0 (mul (rec eB) (rec l) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) (rec eA))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1779)
  (parent 1549)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul eA (rec l) w)) (l-0 (mul eA (rec l) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1780)
  (parent 1550)
  (unrealized (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul (rec eB) w)) (l-0 (mul (rec eB) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w l-0 (rec eA))))
      (send (cat (gen) (mul w l-0 (rec eA))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1781)
  (parent 1554)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (gen) (mul w l-0 eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul eA w)) (l-0 (mul eA w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 eB))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (gen) (mul w l-0 eB)))
      (send (cat (gen) (mul w l-0 eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1782)
  (parent 1555)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expr) (l e l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0 eB))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eB)))
        key)))
  (neq ((exp (gen) (mul w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0 eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eB)))
      (send (cat (exp (gen) e) (mul w l-0 eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1783)
  (parent 1558)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB expn) (w expr) (l l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec e)))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eB) w)) (l-0 (mul (rec eB) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (e eB)
    ((exp (gen) (mul w l-0 (rec e))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 1784)
  (parent 1559)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul e l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul l-0 (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eA))))
      (send (cat (exp (gen) e) (mul l-0 (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1785)
  (parent 1560)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expr) (l e l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA w) (l-0 w) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-0 (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1786)
  (parent 1562)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eB expn)
    (w expr) (e l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul (rec eB) w e)) (l-1 (mul (rec eB) w e))
    (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1787)
  (parent 1566)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eA expn)
    (w expr) (e l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eB (mul eA w e)) (l-1 (mul eA w e)) (e (exp (gen) l-0))
    (e hl) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA w e)))
      (send (cat (exp (gen) l-1) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eB)))
      (send (cat (exp (gen) e) (mul w l-1 eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1788)
  (parent 1567)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB expn) (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (8 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (e (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 1789)
  (parent 1568)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB expn) (w expr) (l l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul (rec eB) w l)) (l-0 (mul (rec eB) w l))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec eA))))
      (send (cat (exp (gen) l) (mul w l-0 (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1790)
  (parent 1571)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA expn) (w expr) (l l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul eA w l)) (l-0 (mul eA w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w l)))
      (send (cat (exp (gen) l-0) (mul eA w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eB)))
      (send (cat (exp (gen) l) (mul w l-0 eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1791)
  (parent 1572)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1792)
  (parent 1577)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1793)
  (parent 1578)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (8 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1794)
  (parent 1579)
  (unrealized (1 3) (3 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1795)
  (parent 1583)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1796)
  (parent 1584)
  (unrealized (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (8 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1797)
  (parent 1585)
  (seen 1886)
  (unrealized (1 3) (3 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul (rec eB) w)) (l-0 (mul (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) (rec eA))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1798)
  (parent 1587)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul eA w)) (l-0 (mul eA w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1799)
  (parent 1588)
  (unrealized (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1800)
  (parent 1591)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1801)
  (parent 1592)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1802)
  (parent 1594)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1803)
  (parent 1595)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) (rec eA))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1804)
  (parent 1598)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (e (mul eA w)) (l-0 (mul eA w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 1805)
  (parent 1599)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1806)
  (parent 1603)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul eA w)) (l-1 (mul eA w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1807)
  (parent 1604)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec e) w)) (gen)) (eA e)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1808)
  (parent 1607)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul eA w)) (l-1 (mul eA w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) key)))
  (neq ((exp (gen) (mul eA w e)) (gen)) (eA e)
    ((exp (gen) (mul w e e)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1809)
  (parent 1608)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eB e expn)
    (w expr) (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (l-1 (mul (rec eB) (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1810)
  (parent 1611)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eA e expn)
    (w expr) (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA (rec e) w)) (l-1 (mul eA (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1811)
  (parent 1612)
  (unrealized (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB expn) (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0))
    ((4 2) (8 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 1812)
  (parent 1613)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e expn) (w expr) (l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (l-2 (mul (rec eB) (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) (rec eA))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1813)
  (parent 1617)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e expn) (w expr) (l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul eA (rec e) w)) (l-2 (mul eA (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eB))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1814)
  (parent 1618)
  (unrealized (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (l-2 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (9 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e (mul eA (rec eB) w)) (l-2 (mul eA (rec eB) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w (rec l-2) e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1815)
  (parent 1619)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e expn) (w expr) (l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (l-2 (mul (rec eB) (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) (rec eA))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1816)
  (parent 1622)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e expn) (w expr) (l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul eA (rec e) w)) (l-2 (mul eA (rec e) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eB))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1817)
  (parent 1623)
  (unrealized (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (l-2 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (9 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e (mul eA (rec eB) w)) (l-2 (mul eA (rec eB) w))
    (e (exp (gen) l)) (e hl) (eB (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen)) (eA eB)
    ((exp (gen) (mul w e)) (gen)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w (rec l-2) e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1818)
  (parent 1624)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul (rec eB) (rec l) w)) (l-0 (mul (rec eB) (rec l) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) (rec eA))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1819)
  (parent 1627)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul eA (rec l) w)) (l-0 (mul eA (rec l) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eB)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1820)
  (parent 1628)
  (unrealized (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1821)
  (parent 1633)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1822)
  (parent 1634)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1823)
  (parent 1638)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1824)
  (parent 1639)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB expn) (w expr) (l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul (rec eB) w)) (l-0 (mul (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w l-0 (rec eA))))
      (send (cat (gen) (mul w l-0 (rec eA))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1825)
  (parent 1643)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA expn) (w expr) (l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (gen) (mul w l-0 eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul eA w)) (l-0 (mul eA w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (gen) (mul w l-0 eB)))
      (send (cat (gen) (mul w l-0 eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1826)
  (parent 1644)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expr) (l e l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0 eB))) (l l) (e e))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0 eB)))
        key)))
  (neq ((exp (gen) (mul w e e l-0)) (gen)) (e eB)
    ((exp (gen) (mul w e l-0 eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eB)))
      (send (cat (exp (gen) e) (mul w l-0 eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1827)
  (parent 1647)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB expn) (w expr) (l l-0 e expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec e)))) (l l)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (e (mul (rec eB) w)) (l-0 (mul (rec eB) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (e eB)
    ((exp (gen) (mul w l-0 (rec e))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 1828)
  (parent 1648)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA e)
    ((exp (gen) (mul e l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul l-0 (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eA))))
      (send (cat (exp (gen) e) (mul l-0 (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1829)
  (parent 1649)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expr) (l e l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA w) (l-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0))) key)))
  (neq ((exp (gen) (mul w l-0)) (gen)) (eA e)
    ((exp (gen) (mul w e l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-0 (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1830)
  (parent 1651)
  (unrealized (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eB expn)
    (w expr) (e l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul (rec eB) w e)) (l-1 (mul (rec eB) w e))
    (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1831)
  (parent 1655)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base) (l l-0 eA expn)
    (w expr) (e l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w e l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eB (mul eA w e)) (l-1 (mul eA w e)) (e (exp (gen) l-0))
    (e hl) (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-1 eB)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA w e)))
      (send (cat (exp (gen) l-1) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eB)))
      (send (cat (exp (gen) e) (mul w l-1 eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1832)
  (parent 1656)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (hl base)
    (l l-0 eA eB expn) (w expr) (l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 2) (3 0)) ((5 2) (8 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (e (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (e (exp (gen) l-0)) (e hl) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 1833)
  (parent 1657)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB expn) (w expr) (l l-0 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul (rec eB) w l)) (l-0 (mul (rec eB) w l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec eA))))
      (send (cat (exp (gen) l) (mul w l-0 (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1834)
  (parent 1660)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA expn) (w expr) (l l-0 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul eA w l)) (l-0 (mul eA w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l l-0 eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w l)))
      (send (cat (exp (gen) l-0) (mul eA w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eB)))
      (send (cat (exp (gen) l) (mul w l-0 eB)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1835)
  (parent 1661)
  (unrealized (1 3) (3 0) (7 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1836)
  (parent 1666)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0 l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1837)
  (parent 1667)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 0) (3 0)) ((5 0) (8 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1838)
  (parent 1668)
  (unrealized (1 3) (3 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1839)
  (parent 1672)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1840)
  (parent 1673)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (8 0)) ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1)))
  (label 1841)
  (parent 1674)
  (seen 1887)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1842)
  (parent 1677)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1843)
  (parent 1678)
  (unrealized (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w l-1 (rec eA))))
      (send (cat (gen) (mul w l-1 (rec eA))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1844)
  (parent 1683)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (gen) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (gen) (mul w l-1 eB)))
      (send (cat (gen) (mul w l-1 eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1845)
  (parent 1684)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l expn) (w expr)
    (l-0 e l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1 eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e e l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eB)))
        key)))
  (neq ((exp (gen) (mul w e e l-1)) (gen)) (e eB)
    ((exp (gen) (mul w e l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1 eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eB)))
      (send (cat (exp (gen) e) (mul w l-1 eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1846)
  (parent 1688)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (e eB)
    ((exp (gen) (mul w l-1 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1847)
  (parent 1689)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul e l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA e)
    ((exp (gen) (mul e l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1848)
  (parent 1690)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l expn) (w expr)
    (l-0 e l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA w) (l-1 w) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) key)))
  (neq ((exp (gen) (mul w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w e l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1849)
  (parent 1693)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB expn) (w expr) (e l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul (rec eB) w e)) (l-2 (mul (rec eB) w e))
    (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1850)
  (parent 1698)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA expn) (w expr) (e l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul eA w e)) (l-2 (mul eA w e)) (e (exp (gen) l-1))
    (e hl) (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA w e)))
      (send (cat (exp (gen) l-2) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eB)))
      (send (cat (exp (gen) e) (mul w l-2 eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1851)
  (parent 1699)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (l-2 e expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e (mul eA (rec eB) w)) (l-2 (mul eA (rec eB) w))
    (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1852)
  (parent 1700)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1853)
  (parent 1704)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0 l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1854)
  (parent 1705)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1855)
  (parent 1710)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1856)
  (parent 1711)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (8 0)) ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1)))
  (label 1857)
  (parent 1712)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1858)
  (parent 1716)
  (unrealized (1 3) (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1859)
  (parent 1717)
  (unrealized (3 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (3 0))
    ((4 0) (8 0)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1860)
  (parent 1718)
  (seen 1888)
  (unrealized (1 3) (3 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB expn) (w expr)
    (l-1 l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1 l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul (rec eB) w l-1)) (l-2 (mul (rec eB) w l-1))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eA))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1861)
  (parent 1723)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA expn) (w expr)
    (l-1 l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2 eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1 l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul eA w l-1)) (l-2 (mul eA w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA w l-1)))
      (send (cat (exp (gen) l-2) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eB)))
      (send (cat (exp (gen) l-1) (mul w l-2 eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1862)
  (parent 1724)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-1 (rec l-2))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1863)
  (parent 1725)
  (unrealized (1 3) (4 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 expn) (w expr)
    (l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-1) w))
    (l-2 (mul (rec eB) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) (rec eA))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1864)
  (parent 1730)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 expn) (w expr)
    (l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul eA (rec l-1) w)) (l-2 (mul eA (rec l-1) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eB))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1865)
  (parent 1731)
  (unrealized (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-2))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (9 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 D) (l B) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-1) l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1866)
  (parent 1732)
  (seen 1889)
  (unrealized (1 3) (4 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB expn) (w expr)
    (l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w l-1 (rec eA))))
      (send (cat (gen) (mul w l-1 (rec eA))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1867)
  (parent 1735)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA expn) (w expr)
    (l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (gen) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (gen) (mul w l-1 eB)))
      (send (cat (gen) (mul w l-1 eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1868)
  (parent 1736)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l expn) (w expr)
    (l-0 e l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w e l-1 eB))) (l l-0) (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eB)))
        key)))
  (neq ((exp (gen) (mul w e e l-1)) (gen)) (e eB)
    ((exp (gen) (mul w e l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1 eB))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eB)))
      (send (cat (exp (gen) e) (mul w l-1 eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1869)
  (parent 1739)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 l-1 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1 (rec e)))) (l l-0)
    (e e))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (e eB)
    ((exp (gen) (mul w l-1 (rec e))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e)))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1870)
  (parent 1740)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA e)
    ((exp (gen) (mul e l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1871)
  (parent 1741)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l expn) (w expr)
    (l-0 e l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA w) (l-1 w) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) key)))
  (neq ((exp (gen) (mul w l-1)) (gen)) (eA e)
    ((exp (gen) (mul w e l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1872)
  (parent 1743)
  (unrealized (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB expn) (w expr) (e l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w e l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul (rec eB) w e)) (l-2 (mul (rec eB) w e))
    (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1873)
  (parent 1747)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA expn) (w expr) (e l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul eA w e)) (l-2 (mul eA w e)) (e (exp (gen) l-1))
    (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w e l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w e l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA w e)))
      (send (cat (exp (gen) l-2) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eB)))
      (send (cat (exp (gen) e) (mul w l-2 eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1874)
  (parent 1748)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB expn) (w expr) (l-2 e expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (4 0)) ((6 2) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e (mul eA (rec eB) w)) (l-2 (mul eA (rec eB) w))
    (e (exp (gen) l-1)) (e hl) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1875)
  (parent 1749)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1876)
  (parent 1752)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0 l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1877)
  (parent 1753)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1878)
  (parent 1757)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l-0)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1879)
  (parent 1758)
  (unrealized (1 3) (4 0) (8 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB expn) (w expr)
    (l-1 l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1 l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul (rec eB) w l-1)) (l-2 (mul (rec eB) w l-1))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eA))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1880)
  (parent 1763)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA expn) (w expr)
    (l-1 l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2 eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1 l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul eA w l-1)) (l-2 (mul eA w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 l-2 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA w l-1)))
      (send (cat (exp (gen) l-2) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eB)))
      (send (cat (exp (gen) l-1) (mul w l-2 eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1881)
  (parent 1764)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-1 (rec l-2))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1882)
  (parent 1765)
  (unrealized (1 3) (4 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 expn) (w expr)
    (l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-1) w))
    (l-2 (mul (rec eB) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen)) (eA eB)
    ((exp (gen) (mul w (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) (rec eA))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1883)
  (parent 1769)
  (unrealized (1 3) (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 expn) (w expr)
    (l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul eA (rec l-1) w)) (l-2 (mul eA (rec l-1) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) key)))
  (neq ((exp (gen) (mul eA w)) (gen)) (eA eB)
    ((exp (gen) (mul w eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eB))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1884)
  (parent 1770)
  (unrealized (4 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-2))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (9 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen)) (eA eB)
    ((exp (gen) (mul w l-2)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-1) l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1885)
  (parent 1771)
  (seen 1890)
  (unrealized (1 3) (4 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1886)
  (parent 1794)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1887)
  (parent 1838)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB expn) (w expr)
    (l-0 l-1 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1888)
  (parent 1857)
  (unrealized (1 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1889)
  (parent 1863)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB expn) (w expr)
    (l-1 l-2 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1890)
  (parent 1882)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 l-1 eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1891)
  (parent 1886)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)) ((9 0) (4 0)) ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eB l-2))) key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1892)
  (parent 1886)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (l-0 (mul (rec eB) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1893)
  (parent 1886)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (l-0 (mul eA w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 eB))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1894)
  (parent 1886)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 l-1 eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1895)
  (parent 1887)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)) ((9 0) (4 0))
    ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1896)
  (parent 1887)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (l-0 (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1897)
  (parent 1887)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (l-0 (mul eA w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1898)
  (parent 1887)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 l-1 eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB))) key)))
  (neq ((exp (gen) l-0) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1899)
  (parent 1888)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)) ((9 0) (4 0))
    ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1900)
  (parent 1888)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (l-0 (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1901)
  (parent 1888)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (l-0 (mul eA w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1902)
  (parent 1888)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 l-2 eA eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (l-1 (one)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1903)
  (parent 1889)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 expn))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul l-0 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-1)
    (hl (exp (gen) l-2)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)) ((9 0) (0 1)) ((9 0) (1 1))
    ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-2))
    (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eB l-2)))
        key)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen)) (eA eB)
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 3 10 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) (rec eA) eB l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1904)
  (parent 1889)
  (unrealized (0 3) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 l-2 eA eB l-3 expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-3)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-3))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 0) (5 0)) ((10 1) (4 0)))
  (absent (l-2 l-3) (l-1 l-3) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-3)))
        key)))
  (neq ((exp (gen) (mul l-1 l-3)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-3)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1905)
  (parent 1889)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB expn) (w expr)
    (l-1 l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eA (mul (rec eB) w)) (l-2 (mul (rec eB) w))
    (l-1 (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1906)
  (parent 1889)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA expn) (w expr)
    (l-1 l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eB (mul eA w)) (l-2 (mul eA w)) (l-1 (mul eA w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1907)
  (parent 1889)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 l-2 eA eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (l-1 (one)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) key)))
  (neq ((exp (gen) l-1) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1908)
  (parent 1890)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 l-2 eA eB l-3 expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-3)))
    (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul l-1 l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 0) (5 0)) ((10 1) (4 0)))
  (absent (l-2 l-3) (l-1 l-3) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-3)))
        key)))
  (neq ((exp (gen) (mul l-1 l-3)) (gen)) (eA eB)
    ((exp (gen) (mul l-1 (rec eA) eB l-3)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-3)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1909)
  (parent 1890)
  (unrealized (0 3) (5 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB expn) (w expr)
    (l-1 l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eA (mul (rec eB) w)) (l-2 (mul (rec eB) w))
    (l-1 (mul (rec eB) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1910)
  (parent 1890)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA expn) (w expr)
    (l-1 l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eB (mul eA w)) (l-2 (mul eA w)) (l-1 (mul eA w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1911)
  (parent 1890)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (l-0 (mul (rec eB) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA)
    (mul w l-0 (rec l-1) (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1912)
  (parent 1893)
  (unrealized (1 3) (3 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer B)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (l-0 (mul eA w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0 eB))) key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec l-1) eB)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1913)
  (parent 1894)
  (unrealized (1 3) (3 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (l-0 (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA)
    (mul w l-0 (rec l-1) (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1914)
  (parent 1897)
  (unrealized (1 3) (3 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (l-0 (mul eA w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec l-1) eB)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1915)
  (parent 1898)
  (unrealized (1 3) (3 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB expn) (w expr)
    (l-0 l-1 eA expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w))
    (l-0 (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 (rec eA))) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA)
    (mul w l-0 (rec l-1) (rec eA)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eA)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1916)
  (parent 1901)
  (unrealized (1 3) (3 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA expn) (w expr)
    (l-0 l-1 eB expn))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer self-0)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (1 3)) ((4 1) (0 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)) (l-0 (mul eA w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-0)) (gen)) (eA eB)
    ((exp (gen) (mul w l-0 eB)) (gen)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec l-1) eB)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eB))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1917)
  (parent 1902)
  (unrealized (1 3) (3 0) (9 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB expn) (w expr)
    (l-1 l-2 eA expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-2 (mul (rec eB) w))
    (l-1 (mul (rec eB) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA)
    (mul w l-1 (rec l-2) (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1918)
  (parent 1906)
  (unrealized (1 3) (4 0) (10 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA expn) (w expr)
    (l-1 l-2 eB expn))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (10 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eB (mul eA w)) (l-2 (mul eA w)) (l-1 (mul eA w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec l-2) eB)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1919)
  (parent 1907)
  (unrealized (1 3) (4 0) (10 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB expn) (w expr)
    (l-1 l-2 eA expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-2 (mul (rec eB) w))
    (l-1 (mul (rec eB) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 (rec eA))) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA)
    (mul w l-1 (rec l-2) (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1920)
  (parent 1910)
  (unrealized (1 3) (4 0) (10 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA expn) (w expr)
    (l-1 l-2 eB expn))
  (defstrand participant 5 (key key) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (key key) (self B) (peer D)
    (hl (exp (gen) l)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (10 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eB (mul eA w)) (l-2 (mul eA w)) (l-1 (mul eA w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        key)))
  (neq ((exp (gen) (mul eA w l-1)) (gen)) (eA eB)
    ((exp (gen) (mul w l-1 eB)) (gen)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec l-2) eB)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1921)
  (parent 1911)
  (unrealized (1 3) (4 0) (10 0))
  (comment "empty cohort"))

(comment "Nothing left to do")

(defprotocol iadh-um diffie-hellman
  (defrole participant
    (vars (l e expn) (hl he base) (self peer name) (key data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))
      (recv he) (send key))
    (uniq-gen e)
    (fn-of (foo ((hash (exp hl l) (exp he e)) key)))
    (neq (he (gen)))
    (absent (e (exp (gen) l)) (e hl)))
  (defrole ltx-gen
    (vars (self name) (l expn))
    (trace (send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    (uniq-gen l)
    (fn-of ("principal-of" (l self)))))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (hl hl-0 base)
    (e1 e2 l l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e1))
  (defstrand participant 3 (self self) (peer peer) (hl hl-0) (l l-0)
    (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e1 e2))))
  (absent (e2 (exp (gen) l-0)) (e2 hl-0) (e1 (exp (gen) l)) (e1 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e1 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e1))
      (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl-0 peer (privk peer))) (send (exp (gen) e2)))
    ((recv (hash (exp hl l) (exp (gen) (mul e1 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e1 e2))))))
  (label 1922)
  (unrealized (0 0) (0 1) (0 3) (2 0))
  (preskeleton)
  (comment "Not a skeleton"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (hl hl-0 base)
    (e1 e2 l l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e1))
  (defstrand participant 3 (self self) (peer peer) (hl hl-0) (l l-0)
    (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e1 e2))))
  (precedes ((0 2) (2 0)) ((1 2) (0 3)) ((1 2) (2 0)))
  (absent (e2 (exp (gen) l-0)) (e2 hl-0) (e1 (exp (gen) l)) (e1 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e1 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e1))
      (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl-0 peer (privk peer))) (send (exp (gen) e2)))
    ((recv (hash (exp hl l) (exp (gen) (mul e1 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e1 e2))))))
  (label 1923)
  (parent 1922)
  (unrealized (0 0) (0 1) (2 0))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B name) (hl base) (l e2 expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e2 e2))))
  (precedes ((0 2) (1 0)))
  (absent (e2 (exp (gen) l)) (e2 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e2 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (operation collapsed 1 0)
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e2))
      (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp hl l) (exp (gen) (mul e2 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e2 e2))))))
  (label 1924)
  (parent 1923)
  (unrealized (0 0) (0 1) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (hl hl-0 base)
    (e1 e2 l l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e1))
  (defstrand participant 3 (self self) (peer peer) (hl hl-0) (l l-0)
    (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e1 e2))))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((3 1) (2 0)))
  (absent (e2 (exp (gen) l-0)) (e2 hl-0) (e1 (exp (gen) l)) (e1 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e1 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (operation encryption-test
    (added-listener (cat (exp hl l) (exp (gen) (mul e1 e2))))
    (hash (exp hl l) (exp (gen) (mul e1 e2))) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e1))
      (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl-0 peer (privk peer))) (send (exp (gen) e2)))
    ((recv (hash (exp hl l) (exp (gen) (mul e1 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e1 e2))))
      (send (cat (exp hl l) (exp (gen) (mul e1 e2))))))
  (label 1925)
  (parent 1923)
  (unrealized (0 0) (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B name) (hl base) (l e2 expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e2 e2))))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)))
  (absent (e2 (exp (gen) l)) (e2 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e2 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (operation encryption-test
    (added-listener (cat (exp hl l) (exp (gen) (mul e2 e2))))
    (hash (exp hl l) (exp (gen) (mul e2 e2))) (1 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e2))
      (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp hl l) (exp (gen) (mul e2 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e2 e2))))
      (send (cat (exp hl l) (exp (gen) (mul e2 e2))))))
  (label 1926)
  (parent 1924)
  (unrealized (0 0) (0 1) (2 0))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (hl hl-0 base)
    (e1 e2 l l-0 expn) (w expr))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e1))
  (defstrand participant 3 (self self) (peer peer) (hl hl-0) (l l-0)
    (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) (mul e1 e2 (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (e2 (exp (gen) l-0)) (e2 hl-0) (e1 (exp (gen) l)) (e1 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e1 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e1 e2 (rec w))) w))
    (exp (gen) (mul e1 e2)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e1))
      (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl-0 peer (privk peer))) (send (exp (gen) e2)))
    ((recv (hash (exp hl l) (exp (gen) (mul e1 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e1 e2))))
      (send (cat (exp hl l) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) (mul e1 e2 (rec w))) w))
      (send (cat (exp (gen) (mul e1 e2 (rec w))) w))))
  (label 1927)
  (parent 1925)
  (unrealized (0 0) (0 1) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B name) (hl base) (l e2 expn) (w expr))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul e2 e2 (rec w))) w))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)))
  (absent (e2 (exp (gen) l)) (e2 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e2 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e2 e2 (rec w))) w))
    (exp (gen) (mul e2 e2)) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e2))
      (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp hl l) (exp (gen) (mul e2 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e2 e2))))
      (send (cat (exp hl l) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul e2 e2 (rec w))) w))
      (send (cat (exp (gen) (mul e2 e2 (rec w))) w))))
  (label 1928)
  (parent 1926)
  (unrealized (0 0) (0 1) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (hl hl-0 base)
    (l l-0 e1 e2 expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e1))
  (defstrand participant 3 (self self) (peer peer) (hl hl-0) (l l-0)
    (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e1 e2))))
  (deflistener (cat (gen) (mul e1 e2)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (e2 (exp (gen) l-0)) (e2 hl-0) (e1 (exp (gen) l)) (e1 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e1 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (precur (4 0))
  (operation nonce-test (contracted (e1-0 e1) (e2-0 e2) (w (mul e1 e2)))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e1))
      (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl-0 peer (privk peer))) (send (exp (gen) e2)))
    ((recv (hash (exp hl l) (exp (gen) (mul e1 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e1 e2))))
      (send (cat (exp hl l) (exp (gen) (mul e1 e2)))))
    ((recv (cat (gen) (mul e1 e2))) (send (cat (gen) (mul e1 e2)))))
  (label 1929)
  (parent 1927)
  (unrealized (0 0) (0 1) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (hl hl-0 base)
    (l e2 l-0 e expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l-0) (e e))
  (defstrand participant 3 (self self) (peer peer) (hl hl-0) (l l)
    (e e2))
  (deflistener (hash (exp hl l-0) (exp (gen) (mul e2 e))))
  (deflistener (cat (exp hl l-0) (exp (gen) (mul e2 e))))
  (deflistener (cat (exp (gen) e) e2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl) (e2 (exp (gen) l)) (e2 hl-0))
  (fn-of (foo ((hash (exp hl l-0) (exp (gen) (mul e2 e))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (precur (4 0))
  (uniq-gen e2 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl-0 peer (privk peer))) (send (exp (gen) e2)))
    ((recv (hash (exp hl l-0) (exp (gen) (mul e2 e))))
      (send (hash (exp hl l-0) (exp (gen) (mul e2 e)))))
    ((recv (cat (exp hl l-0) (exp (gen) (mul e2 e))))
      (send (cat (exp hl l-0) (exp (gen) (mul e2 e)))))
    ((recv (cat (exp (gen) e) e2)) (send (cat (exp (gen) e) e2))))
  (label 1930)
  (parent 1927)
  (unrealized (0 0) (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (hl hl-0 base)
    (l e1 l-0 e expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e)) (l l) (e e1))
  (defstrand participant 3 (self self) (peer peer) (hl hl-0) (l l-0)
    (e e))
  (deflistener (hash (exp hl l) (exp (gen) (mul e1 e))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e1 e))))
  (deflistener (cat (exp (gen) e) e1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl-0) (e1 (exp (gen) l)) (e1 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e1 e))) key)))
  (neq ((exp (gen) e) (gen)))
  (non-orig (privk A) (privk B))
  (precur (4 0))
  (uniq-gen e1 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e1))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl-0 peer (privk peer))) (send (exp (gen) e)))
    ((recv (hash (exp hl l) (exp (gen) (mul e1 e))))
      (send (hash (exp hl l) (exp (gen) (mul e1 e)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e1 e))))
      (send (cat (exp hl l) (exp (gen) (mul e1 e)))))
    ((recv (cat (exp (gen) e) e1)) (send (cat (exp (gen) e) e1))))
  (label 1931)
  (parent 1927)
  (unrealized (0 0) (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self peer self-0 peer-0 name)
    (hl hl-0 hl-1 base) (l l-0 l-1 e1 e2 e expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e1))
  (defstrand participant 3 (self self) (peer peer) (hl hl-0) (l l-0)
    (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) e) (mul e1 e2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-1) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e hl-1) (e2 (exp (gen) l-0)) (e2 hl-0)
    (e1 (exp (gen) l)) (e1 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e1 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (precur (4 0))
  (uniq-gen e1 e2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e1))
      (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl-0 peer (privk peer))) (send (exp (gen) e2)))
    ((recv (hash (exp hl l) (exp (gen) (mul e1 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e1 e2))))
      (send (cat (exp hl l) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) e) (mul e1 e2 (rec e))))
      (send (cat (exp (gen) e) (mul e1 e2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl-1 peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 1932)
  (parent 1927)
  (unrealized (0 0) (0 1) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self peer self-0 name) (hl hl-0 base)
    (l l-0 e1 e2 l-1 expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e1))
  (defstrand participant 3 (self self) (peer peer) (hl hl-0) (l l-0)
    (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) l-1) (mul e1 e2 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)) ((5 0) (4 0)))
  (absent (e2 (exp (gen) l-0)) (e2 hl-0) (e1 (exp (gen) l)) (e1 hl))
  (fn-of ("principal-of" (l-1 self-0))
    (foo ((hash (exp hl l) (exp (gen) (mul e1 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (precur (4 0))
  (uniq-gen e1 e2 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e1))
      (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl-0 peer (privk peer))) (send (exp (gen) e2)))
    ((recv (hash (exp hl l) (exp (gen) (mul e1 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e1 e2))))
      (send (cat (exp hl l) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) l-1) (mul e1 e2 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul e1 e2 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 1933)
  (parent 1927)
  (unrealized (0 0) (0 1) (3 0) (4 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B name) (hl base) (l e2 expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e2 e2))))
  (deflistener (cat (gen) (mul e2 e2)))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)))
  (absent (e2 (exp (gen) l)) (e2 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e2 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (precur (3 0))
  (operation nonce-test (contracted (e2-0 e2) (w (mul e2 e2))) (gen)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e2))
      (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp hl l) (exp (gen) (mul e2 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e2 e2))))
      (send (cat (exp hl l) (exp (gen) (mul e2 e2)))))
    ((recv (cat (gen) (mul e2 e2))) (send (cat (gen) (mul e2 e2)))))
  (label 1934)
  (parent 1928)
  (unrealized (0 0) (0 1) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B name) (hl base) (l e expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e)) (l l) (e e))
  (deflistener (hash (exp hl l) (exp (gen) (mul e e))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e e))))
  (deflistener (cat (exp (gen) e) e))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)))
  (absent (e (exp (gen) l)) (e hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e e))) key)))
  (neq ((exp (gen) e) (gen)))
  (non-orig (privk A) (privk B))
  (precur (3 0))
  (uniq-gen e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (hash (exp hl l) (exp (gen) (mul e e))))
      (send (hash (exp hl l) (exp (gen) (mul e e)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e e))))
      (send (cat (exp hl l) (exp (gen) (mul e e)))))
    ((recv (cat (exp (gen) e) e)) (send (cat (exp (gen) e) e))))
  (label 1935)
  (parent 1928)
  (unrealized (0 0) (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (hl hl-0 base)
    (l l-0 e2 e expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) e) (mul e2 e2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl-0) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e hl-0) (e2 (exp (gen) l)) (e2 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e2 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (precur (3 0))
  (uniq-gen e2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e2))
      (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp hl l) (exp (gen) (mul e2 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e2 e2))))
      (send (cat (exp hl l) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) e) (mul e2 e2 (rec e))))
      (send (cat (exp (gen) e) (mul e2 e2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl-0 peer (privk peer))) (send (exp (gen) e))))
  (label 1936)
  (parent 1928)
  (unrealized (0 0) (0 1) (2 0) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self name) (hl base) (l e2 l-0 expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) l-0) (mul e2 e2 (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (3 0)))
  (absent (e2 (exp (gen) l)) (e2 hl))
  (fn-of ("principal-of" (l-0 self))
    (foo ((hash (exp hl l) (exp (gen) (mul e2 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (precur (3 0))
  (uniq-gen e2 l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e2))
      (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp hl l) (exp (gen) (mul e2 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e2 e2))))
      (send (cat (exp hl l) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) l-0) (mul e2 e2 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e2 e2 (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 1937)
  (parent 1928)
  (unrealized (0 0) (0 1) (2 0) (3 0))
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B name) (hl base) (l e2 expn))
  (defstrand participant 5 (key key) (self A) (peer B) (hl hl)
    (he (exp (gen) e2)) (l l) (e e2))
  (deflistener (hash (exp hl l) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp hl l) (exp (gen) (mul e2 e2))))
  (deflistener (cat (gen) (mul e2 e2)))
  (deflistener e2)
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0)))
  (absent (e2 (exp (gen) l)) (e2 hl))
  (fn-of (foo ((hash (exp hl l) (exp (gen) (mul e2 e2))) key)))
  (neq ((exp (gen) e2) (gen)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (precur (3 0))
  (operation nonce-test (added-listener e2) (mul e2 e2) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" hl B (privk B))) (send (exp (gen) e2))
      (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp hl l) (exp (gen) (mul e2 e2))))
      (send (hash (exp hl l) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp hl l) (exp (gen) (mul e2 e2))))
      (send (cat (exp hl l) (exp (gen) (mul e2 e2)))))
    ((recv (cat (gen) (mul e2 e2))) (send (cat (gen) (mul e2 e2))))
    ((recv e2) (send e2)))
  (label 1938)
  (parent 1934)
  (unrealized (0 0) (0 1) (4 0))
  (comment "empty cohort"))

(comment "Nothing left to do")
