(herald "Plain diffie-hellman protocol with challenge-response"
  (algebra diffie-hellman))

(comment "CPSA 3.4.0")
(comment "All input read from plaindh.scm")

(defprotocol plaindh diffie-hellman
  (defrole init
    (vars (x expn) (h base) (n text))
    (trace (send (exp (gen) x)) (recv h) (send (enc n (exp h x)))
      (recv n))
    (uniq-orig n)
    (uniq-gen x))
  (defrole resp
    (vars (y expn) (h base) (n text))
    (trace (recv h) (send (exp (gen) y)) (recv (enc n (exp h y)))
      (send n))
    (uniq-gen y)
    (absent (y h)))
  (comment "Diffie-hellman key exchange followed by an encryption"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn))
  (defstrand init 4 (n n) (h h) (x x))
  (uniq-gen x)
  (uniq-orig n)
  (traces
    ((send (exp (gen) x)) (recv h) (send (enc n (exp h x))) (recv n)))
  (label 0)
  (unrealized (0 3))
  (origs (n (0 2)))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn))
  (defstrand init 4 (n n) (h (exp h y)) (x x))
  (defstrand resp 4 (n n) (h (exp h x)) (y y))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3)))
  (absent (y (exp h x)))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 4) n (0 3)
    (enc n (exp h (mul x y))))
  (traces
    ((send (exp (gen) x)) (recv (exp h y))
      (send (enc n (exp h (mul x y)))) (recv n))
    ((recv (exp h x)) (send (exp (gen) y))
      (recv (enc n (exp h (mul x y)))) (send n)))
  (label 1)
  (parent 0)
  (unrealized (0 1) (1 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn))
  (defstrand init 4 (n n) (h h) (x x))
  (deflistener (exp h x))
  (precedes ((0 0) (1 0)) ((1 1) (0 3)))
  (uniq-gen x)
  (uniq-orig n)
  (operation nonce-test (added-listener (exp h x)) n (0 3)
    (enc n (exp h x)))
  (traces
    ((send (exp (gen) x)) (recv h) (send (enc n (exp h x))) (recv n))
    ((recv (exp h x)) (send (exp h x))))
  (label 2)
  (parent 0)
  (unrealized (1 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 4 (n n) (h (gen)) (y y))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3)))
  (absent (y (gen)))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (h (exp (gen) (rec x)))) (gen)
    (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))
      (send n)))
  (label 3)
  (parent 1)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) x)) (y y))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3)))
  (absent (y (exp (gen) x)))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 2 0 init 1) (exp (gen) x-0) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul y x)))) (recv n))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x)))) (send n)))
  (label 4)
  (parent 1)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3))
    ((2 0) (1 0)))
  (absent (y (exp (gen) x-0)))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0)))) (send n))
    ((send (exp (gen) x-0))))
  (label 5)
  (parent 1)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3))
    ((2 1) (1 0)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))) (recv n))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))) (send n))
    ((recv h) (send (exp (gen) y-0))))
  (label 6)
  (parent 1)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr))
  (defstrand init 4 (n n) (h (exp h y)) (x x))
  (defstrand resp 4 (n n) (h (exp h x)) (y y))
  (deflistener (cat (exp h (mul x (rec w))) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3))
    ((2 1) (1 0)))
  (absent (y (exp h x)))
  (uniq-gen x y)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test (added-listener (cat (exp h (mul x (rec w))) w))
    (exp h x) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp h y))
      (send (enc n (exp h (mul x y)))) (recv n))
    ((recv (exp h x)) (send (exp (gen) y))
      (recv (enc n (exp h (mul x y)))) (send n))
    ((recv (cat (exp h (mul x (rec w))) w))
      (send (cat (exp h (mul x (rec w))) w))))
  (label 7)
  (parent 1)
  (unrealized (0 1) (2 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (precedes ((0 0) (1 0)) ((1 1) (0 3)))
  (uniq-gen x)
  (uniq-orig n)
  (operation nonce-test (contracted (h (exp (gen) (rec x)))) (gen)
    (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen))))
  (label 8)
  (parent 2)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (gen)) (x x))
  (deflistener (exp (gen) x))
  (precedes ((0 0) (1 0)) ((1 1) (0 3)))
  (uniq-gen x)
  (uniq-orig n)
  (operation nonce-test (displaced 2 0 init 1) (exp (gen) x-0) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x)))
      (recv n)) ((recv (exp (gen) x)) (send (exp (gen) x))))
  (label 9)
  (parent 2)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((1 1) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0))))
  (label 10)
  (parent 2)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (deflistener (exp (gen) y))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (1 0)) ((1 1) (0 3)) ((2 1) (0 1)) ((2 1) (1 0)))
  (absent (y h))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv h) (send (exp (gen) y))))
  (label 11)
  (parent 2)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr))
  (defstrand init 4 (n n) (h h) (x x))
  (deflistener (exp h x))
  (deflistener (cat (exp h (mul x (rec w))) w))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test (added-listener (cat (exp h (mul x (rec w))) w))
    (exp h x) (1 0))
  (traces
    ((send (exp (gen) x)) (recv h) (send (enc n (exp h x))) (recv n))
    ((recv (exp h x)) (send (exp h x)))
    ((recv (cat (exp h (mul x (rec w))) w))
      (send (cat (exp h (mul x (rec w))) w))))
  (label 12)
  (parent 2)
  (unrealized (2 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x y expn) (w expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 4 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) (mul (rec x) y (rec w))) w))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 3) (0 3))
    ((2 1) (0 1)))
  (absent (y (gen)))
  (uniq-gen x y)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y (rec w))) w))
    (exp (gen) (mul (rec x) y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))
      (send n))
    ((recv (cat (exp (gen) (mul (rec x) y (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y (rec w))) w))))
  (label 13)
  (parent 3)
  (unrealized (2 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton plaindh
  (vars (n text) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) y)) (x x))
  (absent (y (exp (gen) x)))
  (uniq-gen x)
  (uniq-orig n)
  (operation generalization deleted (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul y x)))) (recv n)))
  (label 14)
  (parent 4)
  (unrealized)
  (origs (n (0 2)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x y x-0 expn) (w expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y x-0 (rec w))) w))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 0) (1 0)) ((3 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y x-0 (rec w))) w))
    (exp (gen) (mul (rec x) y x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0)))) (send n))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) y x-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y x-0 (rec w))) w))))
  (label 15)
  (parent 5)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y y-0 expn) (w expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y y-0 (rec w))) w))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y y-0 (rec w))) w))
    (exp (gen) (mul (rec x) y y-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))) (recv n))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))) (send n))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) (mul (rec x) y y-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y y-0 (rec w))) w))))
  (label 16)
  (parent 6)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x y expn) (w expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y w))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (gen) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3))
    ((2 1) (1 0)))
  (absent (y (exp (gen) w)))
  (uniq-gen x y)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test (contracted (h (exp (gen) (mul (rec x) w))))
    (gen) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w)))
      (send (enc n (exp (gen) (mul y w)))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w)))) (send n))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 17)
  (parent 7)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton plaindh
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y w))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul w x))) (y y))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3))
    ((2 1) (1 0)))
  (absent (y (exp (gen) (mul w x))))
  (precur (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 3 0 init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y w)))
      (send (enc n (exp (gen) (mul y w x)))) (recv n))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w x)))) (send n))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 18)
  (parent 7)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x y expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y w x-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)))
  (absent (y (exp (gen) (mul w x-0))))
  (precur (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w x-0)))
      (send (enc n (exp (gen) (mul y w x-0)))) (recv n))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w x-0)))) (send n))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0))))
  (label 19)
  (parent 7)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y w y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul w y-0))) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)))
  (absent (y-0 h) (y (exp (gen) (mul w y-0))))
  (precur (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w y-0)))
      (send (enc n (exp (gen) (mul y w y-0)))) (recv n))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w y-0)))) (send n))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0))))
  (label 20)
  (parent 7)
  (unrealized (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (exp (gen) (mul (rec x) (rec w))) w))
  (precedes ((0 0) (1 0)) ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (0 1)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) (rec w))) w))
    (exp (gen) (rec x)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) (mul (rec x) (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) (rec w))) w))))
  (label 21)
  (parent 8)
  (unrealized (2 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (gen)) (x x))
  (uniq-gen x)
  (uniq-orig n)
  (operation generalization deleted (1 0))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x)))
      (recv n)))
  (label 22)
  (parent 9)
  (unrealized)
  (shape)
  (maps ((0) ((x x) (h (gen)) (n n))))
  (origs (n (0 2))))

(defskeleton plaindh
  (vars (n text) (x x-0 expn) (w expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) x-0 (rec w))) w))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 0) (1 0))
    ((2 0) (3 0)) ((3 1) (0 1)))
  (uniq-gen x x-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) x-0 (rec w))) w))
    (exp (gen) (mul (rec x) x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) x-0 (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) x-0 (rec w))) w))))
  (label 23)
  (parent 10)
  (unrealized (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (deflistener (exp (gen) y))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) (mul (rec x) y (rec w))) w))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((2 1) (3 0)) ((3 1) (0 1)))
  (absent (y h))
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y (rec w))) w))
    (exp (gen) (mul (rec x) y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) (mul (rec x) y (rec w))) w))
      (send (cat (exp (gen) (mul (rec x) y (rec w))) w))))
  (label 24)
  (parent 11)
  (unrealized (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) w))) (x x))
  (deflistener (exp (gen) w))
  (deflistener (cat (gen) w))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test (contracted (h (exp (gen) (mul (rec x) w))))
    (gen) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w)))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 25)
  (parent 12)
  (unrealized (0 1))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton plaindh
  (vars (n text) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)))
  (precur (2 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation nonce-test (displaced 3 0 init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul w x)))) (recv n))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 26)
  (parent 12)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) w x-0))) (x x))
  (deflistener (exp (gen) (mul w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 0) (0 1))
    ((3 0) (2 0)))
  (precur (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w x-0)))
      (send (enc n (exp (gen) (mul w x-0)))) (recv n))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) (mul w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0))))
  (label 27)
  (parent 12)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) w y))) (x x))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 1) (0 1))
    ((3 1) (2 0)))
  (absent (y h))
  (precur (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w y)))
      (send (enc n (exp (gen) (mul w y)))) (recv n))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv h) (send (exp (gen) y))))
  (label 28)
  (parent 12)
  (unrealized (0 1))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 4 (n n) (h (gen)) (y y))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 3) (0 3))
    ((2 1) (0 1)))
  (absent (y (gen)))
  (uniq-gen x y)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test (contracted (x-0 x) (y-0 y) (w (mul (rec x) y)))
    (gen) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))
      (send n))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 29)
  (parent 13)
  (unrealized (0 1) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 4 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 3) (0 3))
    ((2 1) (0 1)))
  (absent (y (gen)))
  (precur (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 3 0 init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))
      (send n))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 30)
  (parent 13)
  (unrealized (0 1) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 4 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 3) (0 3))
    ((2 1) (0 1)) ((3 0) (2 0)))
  (absent (y (gen)))
  (precur (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))
      (send n))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 31)
  (parent 13)
  (unrealized (0 1) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 4 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 3) (0 3))
    ((2 1) (0 1)))
  (absent (y (gen)))
  (precur (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 3 1 resp 2) (exp (gen) y-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))
      (send n))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 32)
  (parent 13)
  (unrealized (0 1) (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 4 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (2 0)) ((1 3) (0 3))
    ((2 1) (0 1)) ((3 1) (2 0)))
  (absent (y-0 h) (y (gen)))
  (precur (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))
      (send n))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 33)
  (parent 13)
  (unrealized (0 1) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) y)) (x x))
  (uniq-gen x)
  (uniq-orig n)
  (operation generalization forgot ((absent (y (exp (gen) x)))))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul y x)))) (recv n)))
  (label 34)
  (parent 14)
  (unrealized)
  (shape)
  (maps ((0) ((x x) (h (exp (gen) y)) (n n))))
  (origs (n (0 2))))

(defskeleton plaindh
  (vars (n text) (x y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul (rec x) y x-0)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 0) (1 0)) ((3 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (contracted (x-1 x) (y-0 y) (x-2 x-0) (w (mul (rec x) y x-0))) (gen)
    (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0)))) (send n))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul (rec x) y x-0)))
      (send (cat (gen) (mul (rec x) y x-0)))))
  (label 35)
  (parent 15)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (y x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y x (rec x-0)))) (x x-0))
  (defstrand resp 4 (n n) (h (exp (gen) x)) (y y))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul y x (rec x-0) (rec x-0))))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 0) (1 0)) ((3 1) (0 1)))
  (absent (y (exp (gen) x)))
  (precur (3 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-1) (3 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul y x (rec x-0))))
      (send (enc n (exp (gen) (mul y x)))) (recv n))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x)))) (send n))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul y x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul y x (rec x-0) (rec x-0))))))
  (label 36)
  (parent 15)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 0) (1 0)) ((3 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (precur (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 init 1) (exp (gen) x-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0)))) (send n))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y)))
      (send (cat (exp (gen) x-0) (mul (rec x) y)))))
  (label 37)
  (parent 15)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x y x-0 x-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul (rec x) y x-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 0) (1 0)) ((3 1) (0 1)) ((4 0) (3 0)))
  (absent (y (exp (gen) x-0)))
  (precur (3 0))
  (uniq-gen x y x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0)))) (send n))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul (rec x) y x-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul (rec x) y x-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 38)
  (parent 15)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) x-0 y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 0) (1 0)) ((3 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (precur (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0 y)))
      (send (enc n (exp (gen) (mul x-0 y)))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x-0 y)))) (send n))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 39)
  (parent 15)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y x-0 y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y x-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) x-0)) (y y))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y x-0 (rec y-0))))
  (defstrand resp 2 (h h) (y y-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 0) (1 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y-0 h) (y (exp (gen) x-0)))
  (precur (3 0))
  (uniq-gen x y x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y x-0)))
      (send (enc n (exp (gen) (mul y x-0)))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0)))) (send n))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y x-0 (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y x-0 (rec y-0)))))
    ((recv h) (send (exp (gen) y-0))))
  (label 40)
  (parent 15)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (gen) (mul (rec x) y y-0)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (contracted (x-0 x) (y-1 y) (y-2 y-0) (w (mul (rec x) y y-0))) (gen)
    (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))) (recv n))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))) (send n))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (gen) (mul (rec x) y y-0)))
      (send (cat (gen) (mul (rec x) y y-0)))))
  (label 41)
  (parent 16)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y y-0 x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y y-0 (rec x)))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x) (mul y y-0 (rec x) (rec x))))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (precur (3 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y y-0 (rec x))))
      (send (enc n (exp (gen) (mul y y-0)))) (recv n))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))) (send n))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x) (mul y y-0 (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y y-0 (rec x) (rec x))))))
  (label 42)
  (parent 16)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y y-0 x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y y-0 (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)) ((4 0) (3 0)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (precur (3 0))
  (uniq-gen x y y-0 x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))) (recv n))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))) (send n))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y y-0 (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y y-0 (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 43)
  (parent 16)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) y)) (y y-0))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)))
  (absent (y-0 (exp (gen) y)) (y h))
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul y y-0)))) (send n))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y)))
      (send (cat (exp (gen) y-0) (mul (rec x) y)))))
  (label 44)
  (parent 16)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y)))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) y-0)))
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 resp 2) (exp (gen) y-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))) (recv n))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))) (send n))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y)))
      (send (cat (exp (gen) y-0) (mul (rec x) y)))))
  (label 45)
  (parent 16)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (x y y-0 y-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) y-0)) (y y))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y-1) (mul (rec x) y y-0 (rec y-1))))
  (defstrand resp 2 (h h-0) (y y-1))
  (precedes ((0 0) (1 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y-1 h-0) (y-0 h) (y (exp (gen) y-0)))
  (precur (3 0))
  (uniq-gen x y y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y y-0)))
      (send (enc n (exp (gen) (mul y y-0)))) (recv n))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))) (send n))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y-1) (mul (rec x) y y-0 (rec y-1))))
      (send (cat (exp (gen) y-1) (mul (rec x) y y-0 (rec y-1)))))
    ((recv h-0) (send (exp (gen) y-1))))
  (label 46)
  (parent 16)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) x)) (y y))
  (deflistener (cat (gen) x))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3))
    ((2 1) (1 0)))
  (absent (y (exp (gen) x)))
  (precur (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 3 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))) (recv n))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x y)))) (send n))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 47)
  (parent 17)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x y expn) (w w-0 expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y w))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)))
  (absent (y (exp (gen) w)))
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w)))
      (send (enc n (exp (gen) (mul y w)))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w)))) (send n))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w (rec w-0))) w-0))))
  (label 48)
  (parent 17)
  (unrealized (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton plaindh
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y w))) (x x))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (1 0)) ((1 1) (0 1)))
  (absent (y (exp (gen) (mul w x))))
  (precur (1 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation generalization deleted (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y w)))
      (send (enc n (exp (gen) (mul y w x)))) (recv n))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 49)
  (parent 18)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) y)) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) x)) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)))
  (absent (y (exp (gen) x)))
  (precur (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))) (recv n))
    ((recv (exp (gen) x)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x y)))) (send n))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 50)
  (parent 19)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x y expn) (w expr) (x-0 expn) (w-0 expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y w x-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul w x-0))))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w x-0)))
      (send (enc n (exp (gen) (mul y w x-0)))) (recv n))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w x-0)))) (send n))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w x-0 (rec w-0))) w-0))))
  (label 51)
  (parent 19)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) y-0)) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) x)) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (0 1)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)))
  (absent (y-0 (exp (gen) x)) (y h))
  (precur (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))) (recv n))
    ((recv (exp (gen) x)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x y-0)))) (send n))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((recv h) (send (exp (gen) y))))
  (label 52)
  (parent 20)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn) (w-0 expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y w y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul w y-0))) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y-0 h) (y (exp (gen) (mul w y-0))))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) y w y-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y w y-0)))
      (send (enc n (exp (gen) (mul y w y-0)))) (recv n))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y w y-0)))) (send n))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) y w y-0 (rec w-0))) w-0))))
  (label 53)
  (parent 20)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 0) (1 0)) ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (0 1)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (gen) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 54)
  (parent 21)
  (unrealized (0 1) (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 0) (1 0)) ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (0 1)))
  (precur (2 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation nonce-test (displaced 3 0 init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 55)
  (parent 21)
  (unrealized (0 1) (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (0 1))
    ((3 0) (2 0)))
  (precur (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 56)
  (parent 21)
  (unrealized (0 1) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (1 0)) ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (0 1))
    ((3 1) (2 0)))
  (absent (y h))
  (precur (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((recv h) (send (exp (gen) y))))
  (label 57)
  (parent 21)
  (unrealized (0 1) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0)))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 0) (1 0))
    ((2 0) (3 0)) ((3 1) (0 1)))
  (uniq-gen x x-0)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (gen) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul (rec x) x-0)))
      (send (cat (gen) (mul (rec x) x-0)))))
  (label 58)
  (parent 23)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (deflistener (exp (gen) x))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 0) (1 0))
    ((2 0) (3 0)) ((3 1) (0 1)))
  (precur (3 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-1) (3 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))) (recv n))
    ((recv (exp (gen) x)) (send (exp (gen) x))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))))
  (label 59)
  (parent 23)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (rec x)))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 0) (1 0))
    ((2 0) (3 0)) ((3 1) (0 1)))
  (precur (3 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 init 1) (exp (gen) x-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (rec x)))
      (send (cat (exp (gen) x-0) (rec x)))))
  (label 60)
  (parent 23)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 x-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 0) (1 0))
    ((2 0) (3 0)) ((3 1) (0 1)) ((4 0) (3 0)))
  (precur (3 0))
  (uniq-gen x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 61)
  (parent 23)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 0) (1 0))
    ((2 0) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y h))
  (precur (3 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) x-0 (rec y)))))
    ((recv h) (send (exp (gen) y))))
  (label 62)
  (parent 23)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (deflistener (exp (gen) y))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((2 1) (3 0)) ((3 1) (0 1)))
  (absent (y h))
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0))
  (operation nonce-test (contracted (x-0 x) (y-0 y) (w (mul (rec x) y)))
    (gen) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 63)
  (parent 24)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (deflistener (exp (gen) y))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((2 1) (3 0)) ((3 1) (0 1)))
  (absent (y h))
  (precur (3 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 64)
  (parent 24)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (deflistener (exp (gen) y))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 0) (3 0)))
  (absent (y h))
  (precur (3 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 65)
  (parent 24)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (deflistener (exp (gen) y))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((2 1) (3 0)) ((3 1) (0 1)))
  (absent (y h))
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 2 resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 66)
  (parent 24)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (x y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (deflistener (exp (gen) y))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 2 (h h-0) (y y-0))
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y-0 h-0) (y h))
  (precur (3 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((recv h-0) (send (exp (gen) y-0))))
  (label 67)
  (parent 24)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (gen)) (x x))
  (deflistener (exp (gen) x))
  (deflistener (cat (gen) x))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test (contracted (x-0 x) (w x)) (gen) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x)))
      (recv n)) ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (gen) x)) (send (cat (gen) x))))
  (label 68)
  (parent 25)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) x)) (x x))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (gen) (mul x x)))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)))
  (precur (2 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation nonce-test (displaced 3 0 init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))) (recv n))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (gen) (mul x x))) (send (cat (gen) (mul x x)))))
  (label 69)
  (parent 25)
  (unrealized (1 0) (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) x-0)) (x x))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (gen) (mul x x-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 0) (0 1))
    ((3 0) (2 0)))
  (precur (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))) (recv n))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (gen) (mul x x-0))) (send (cat (gen) (mul x x-0))))
    ((send (exp (gen) x-0))))
  (label 70)
  (parent 25)
  (unrealized (1 0) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) y)) (x x))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (gen) (mul x y)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 1) (0 1))
    ((3 1) (2 0)))
  (absent (y h))
  (precur (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))) (recv n))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (gen) (mul x y))) (send (cat (gen) (mul x y))))
    ((recv h) (send (exp (gen) y))))
  (label 71)
  (parent 25)
  (unrealized (1 0) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w w-0 expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) w))) (x x))
  (deflistener (exp (gen) w))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w)))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w (rec w-0))) w-0))))
  (label 72)
  (parent 25)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton plaindh
  (vars (n text) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (1 0)) ((1 1) (0 3)))
  (precur (1 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation generalization deleted (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul w x)))) (recv n))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 73)
  (parent 26)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (gen)) (x x))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 0) (0 1))
    ((3 0) (2 0)))
  (precur (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w (mul x (rec x-0))) (x-2 x-0)) (gen) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x)))
      (recv n)) ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 74)
  (parent 27)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) x-0)) (x x-0))
  (deflistener (exp (gen) (mul x-0 x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
  (defstrand init 1 (x x))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 0) (0 1))
    ((3 0) (2 0)))
  (precur (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-1) (0 1))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x-0 x-0)))) (recv n))
    ((recv (exp (gen) (mul x-0 x-0))) (send (exp (gen) (mul x-0 x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) x-0 x-0))))
    ((send (exp (gen) x))))
  (label 75)
  (parent 27)
  (unrealized (1 0) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) x-0)) (x x))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (exp (gen) x-0) x))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 0) (0 1))
    ((3 0) (2 0)))
  (precur (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 4 3 init 1) (exp (gen) x-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))) (recv n))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (exp (gen) x-0) x)) (send (cat (exp (gen) x-0) x)))
    ((send (exp (gen) x-0))))
  (label 76)
  (parent 27)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 x-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) x-1)) (x x))
  (deflistener (exp (gen) (mul x x-1)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
  (defstrand init 1 (x x-0))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 0) (0 1))
    ((3 0) (2 0)) ((4 0) (0 1)) ((4 0) (2 0)))
  (precur (2 0))
  (uniq-gen x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-1))
      (send (enc n (exp (gen) (mul x x-1)))) (recv n))
    ((recv (exp (gen) (mul x x-1))) (send (exp (gen) (mul x x-1))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) x-1))))
    ((send (exp (gen) x-0))) ((send (exp (gen) x-1))))
  (label 77)
  (parent 27)
  (unrealized (1 0) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) y)) (x x))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) y)))
  (defstrand init 1 (x x-0))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 0) (0 1))
    ((3 0) (2 0)) ((4 1) (0 1)) ((4 1) (2 0)))
  (absent (y h))
  (precur (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))) (recv n))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) y))))
    ((send (exp (gen) x-0))) ((recv h) (send (exp (gen) y))))
  (label 78)
  (parent 27)
  (unrealized (1 0) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn) (w-0 expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) w x-0))) (x x))
  (deflistener (exp (gen) (mul w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w x-0)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w x-0)))
      (send (enc n (exp (gen) (mul w x-0)))) (recv n))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) (mul w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w x-0 (rec w-0))) w-0))))
  (label 79)
  (parent 27)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn))
  (defstrand init 4 (n n) (h (gen)) (x x))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) y) (mul x (rec y))))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 1) (0 1))
    ((3 1) (2 0)))
  (absent (y h))
  (precur (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w (mul x (rec y))) (y-0 y))
    (gen) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (gen)) (send (enc n (exp (gen) x)))
      (recv n)) ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul x (rec y))))
      (send (cat (exp (gen) y) (mul x (rec y)))))
    ((recv h) (send (exp (gen) y))))
  (label 80)
  (parent 28)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) x)) (x x))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) x x)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 1) (0 1))
    ((3 1) (2 0)))
  (absent (y h))
  (precur (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))) (recv n))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (exp (gen) y) (mul (rec y) x x)))
      (send (cat (exp (gen) y) (mul (rec y) x x))))
    ((recv h) (send (exp (gen) y))))
  (label 81)
  (parent 28)
  (unrealized (1 0) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) x-0)) (x x))
  (deflistener (exp (gen) (mul x x-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) x-0)))
  (defstrand resp 2 (h h) (y y))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 1) (0 1))
    ((3 1) (2 0)) ((4 0) (0 1)) ((4 0) (2 0)))
  (absent (y h))
  (precur (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x-0))
      (send (enc n (exp (gen) (mul x x-0)))) (recv n))
    ((recv (exp (gen) (mul x x-0))) (send (exp (gen) (mul x x-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) x-0))))
    ((recv h) (send (exp (gen) y))) ((send (exp (gen) x-0))))
  (label 82)
  (parent 28)
  (unrealized (1 0) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) y)) (x x))
  (deflistener (exp (gen) (mul x y)))
  (deflistener (cat (exp (gen) y) x))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 1) (0 1))
    ((3 1) (2 0)))
  (absent (y h))
  (precur (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 3 resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y))
      (send (enc n (exp (gen) (mul x y)))) (recv n))
    ((recv (exp (gen) (mul x y))) (send (exp (gen) (mul x y))))
    ((recv (cat (exp (gen) y) x)) (send (cat (exp (gen) y) x)))
    ((recv h) (send (exp (gen) y))))
  (label 83)
  (parent 28)
  (unrealized (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (x y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) y-0)) (x x))
  (deflistener (exp (gen) (mul x y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) y-0)))
  (defstrand resp 2 (h h) (y y))
  (defstrand resp 2 (h h-0) (y y-0))
  (precedes ((0 0) (2 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 1) (0 1))
    ((3 1) (2 0)) ((4 1) (0 1)) ((4 1) (2 0)))
  (absent (y-0 h-0) (y h))
  (precur (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) y-0))
      (send (enc n (exp (gen) (mul x y-0)))) (recv n))
    ((recv (exp (gen) (mul x y-0))) (send (exp (gen) (mul x y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) y-0))))
    ((recv h) (send (exp (gen) y))) ((recv h-0) (send (exp (gen) y-0))))
  (label 84)
  (parent 28)
  (unrealized (1 0) (2 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y expn) (w-0 expr))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) w y))) (x x))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
    (exp (gen) (mul (rec x) w y)) (0 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) w y)))
      (send (enc n (exp (gen) (mul w y)))) (recv n))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec x) w y (rec w-0))) w-0))))
  (label 85)
  (parent 28)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (defstrand resp 4 (n n) (h (gen)) (y y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((0 2) (1 2)) ((1 1) (2 0))
    ((1 3) (0 3)) ((2 1) (0 1)) ((3 1) (2 0)))
  (absent (y (gen)))
  (precur (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))
      (send n))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 86)
  (parent 32)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test
    (contracted (x-0 x) (y-0 y) (w-0 (mul x w)) (w-1 (mul w y)) (y-1 y)
      (w (mul w y))) (gen) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))) (send n))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y)))))
  (label 87)
  (parent 48)
  (unrealized (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (w expr) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul w x x))) (y y))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)))
  (absent (y (exp (gen) (mul w x x))))
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))) (recv n))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x x y)))) (send n))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))))
  (label 88)
  (parent 48)
  (unrealized (0 1) (1 0) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)) ((4 0) (2 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (precur (3 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))) (send n))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0))))
  (label 89)
  (parent 48)
  (unrealized (0 1) (1 0) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 4 1 resp 2) (exp (gen) y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))) (send n))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 90)
  (parent 48)
  (unrealized (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)) ((4 1) (2 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (precur (3 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))) (recv n))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))) (send n))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 91)
  (parent 48)
  (unrealized (0 1) (1 0) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y w))) (x x))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (1 0)))
  (absent (y (exp (gen) (mul w x))))
  (precur (1 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation generalization deleted (1 1))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y w)))
      (send (enc n (exp (gen) (mul y w x)))) (recv n))
    ((recv (cat (exp (gen) x) w))))
  (label 92)
  (parent 49)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (y-0 y) (w-0 (mul x (rec x-0) w)) (x-2 x-0)
      (w-1 (mul w y)) (y-1 y) (w (mul w y))) (gen) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))) (send n))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y)))))
  (label 93)
  (parent 51)
  (unrealized (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 y))) (x x-0))
  (defstrand resp 4 (n n) (h (exp (gen) (mul w x-0 x-0))) (y y))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul w x-0 x-0))))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-1) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul w x-0 x-0 y)))) (recv n))
    ((recv (exp (gen) (mul w x-0 x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x-0 x-0 y)))) (send n))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))))
  (label 94)
  (parent 51)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w x-0))))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 init 1) (exp (gen) x-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))) (send n))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))))
  (label 95)
  (parent 51)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn) (w expr) (x-1 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-1 y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w x-1))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w y)))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)) ((5 0) (2 0)))
  (absent (y (exp (gen) (mul x w x-1))))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1 y)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-1 y)))
      (send (enc n (exp (gen) (mul x w x-1 y)))) (recv n))
    ((recv (exp (gen) (mul x w x-1))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-1 y)))) (send n))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w y)))
      (send (cat (exp (gen) x-1) (mul w y)))) ((send (exp (gen) x-1))))
  (label 96)
  (parent 51)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) (mul x w))))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))) (send n))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 97)
  (parent 51)
  (unrealized (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x x-0 expn) (w expr) (y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (2 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))) (recv n))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))) (send n))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))))
  (label 98)
  (parent 51)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) (mul w y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y h) (y-0 (exp (gen) (mul x w))))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w-0 (mul x (rec y) w)) (y-2 y)
      (w-1 (mul w y-0)) (y-3 y-0) (w (mul w y-0))) (gen) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y-0)))) (send n))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) (mul w y-0))) (send (cat (gen) (mul w y-0)))))
  (label 99)
  (parent 53)
  (unrealized (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (y expn) (w expr) (x y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul w x x))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x x)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul w y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y h) (y-0 (exp (gen) (mul w x x))))
  (precur (4 0) (2 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y-0)))
      (send (enc n (exp (gen) (mul w x x y-0)))) (recv n))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w x x y-0)))) (send n))
    ((recv (cat (exp (gen) y) (mul (rec y) w x x)))
      (send (cat (exp (gen) y) (mul (rec y) w x x))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul w y-0)))
      (send (cat (exp (gen) x) (mul w y-0)))))
  (label 100)
  (parent 53)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr) (x-0 y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w x-0))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) (mul w y-0)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 0) (2 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w x-0))))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y-0)))
      (send (enc n (exp (gen) (mul x w x-0 y-0)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w x-0 y-0)))) (send n))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul w y-0)))
      (send (cat (exp (gen) x-0) (mul w y-0))))
    ((send (exp (gen) x-0))))
  (label 101)
  (parent 53)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y-0 (exp (gen) (mul x w))) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 1 resp 2) (exp (gen) y-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y-0)))) (send n))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w))))
  (label 102)
  (parent 53)
  (unrealized (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 2) (exp (gen) y-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))) (recv n))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))) (send n))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))))
  (label 103)
  (parent 53)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (x y expn) (w expr) (y-0 y-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y-0 y-1))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w y-0))) (y y-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w y-1)))
  (defstrand resp 2 (h h-0) (y y-0))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (2 0)))
  (absent (y-0 h-0) (y h) (y-1 (exp (gen) (mul x w y-0))))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 y-1)))
      (send (enc n (exp (gen) (mul x w y-0 y-1)))) (recv n))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) y-1))
      (recv (enc n (exp (gen) (mul x w y-0 y-1)))) (send n))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w y-1)))
      (send (cat (exp (gen) y-0) (mul w y-1))))
    ((recv h-0) (send (exp (gen) y-0))))
  (label 104)
  (parent 53)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (0 1))
    ((3 1) (2 0)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (2 0))
  (operation nonce-test (added-listener x) (rec x) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 105)
  (parent 54)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (1 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (0 1))
    ((3 1) (2 0)))
  (precur (2 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 106)
  (parent 55)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (deflistener (exp (gen) x-0))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (rec x)))
  (deflistener x)
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 0) (1 0))
    ((2 0) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (precur (3 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (rec x)))
      (send (cat (exp (gen) x-0) (rec x)))) ((recv x) (send x)))
  (label 107)
  (parent 60)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (deflistener (exp (gen) y))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (1 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((2 1) (3 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y h))
  (precur (3 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 108)
  (parent 66)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) x)) (x x))
  (deflistener (exp (gen) (mul x x)))
  (deflistener (cat (gen) (mul x x)))
  (deflistener x)
  (precedes ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0)) ((3 1) (2 0)))
  (precur (2 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x x) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) x))
      (send (enc n (exp (gen) (mul x x)))) (recv n))
    ((recv (exp (gen) (mul x x))) (send (exp (gen) (mul x x))))
    ((recv (cat (gen) (mul x x))) (send (cat (gen) (mul x x))))
    ((recv x) (send x)))
  (label 109)
  (parent 69)
  (unrealized (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) w))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test (contracted (x-0 x) (w-0 (mul x w)) (w-1 w))
    (gen) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 110)
  (parent 72)
  (unrealized (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x))) (x x))
  (deflistener (exp (gen) (mul w x x)))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (precur (3 0) (2 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation nonce-test (displaced 4 0 init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x)))
      (send (enc n (exp (gen) (mul w x x)))) (recv n))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) (mul w x x))))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 111)
  (parent 72)
  (unrealized (1 0) (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 0) (2 0)) ((4 0) (3 0)))
  (precur (3 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0))))
  (label 112)
  (parent 72)
  (unrealized (1 0) (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)) ((4 1) (3 0)))
  (absent (y h))
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv h) (send (exp (gen) y))))
  (label 113)
  (parent 72)
  (unrealized (1 0) (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (uniq-gen x)
  (uniq-orig n)
  (operation generalization deleted (1 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul w x)))) (recv n)))
  (label 114)
  (parent 73)
  (unrealized)
  (shape)
  (maps ((0) ((x x) (h (exp (gen) w)) (n n))))
  (origs (n (0 2))))

(defskeleton plaindh
  (vars (n text) (x x-0 expn) (w expr))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) w))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w-0 (mul x (rec x-0) w)) (x-2 x-0) (w-1 w))
    (gen) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 115)
  (parent 79)
  (unrealized (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0))) (x x-0))
  (deflistener (exp (gen) (mul w x-0 x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-1) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul w x-0 x-0)))) (recv n))
    ((recv (exp (gen) (mul w x-0 x-0)))
      (send (exp (gen) (mul w x-0 x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 116)
  (parent 79)
  (unrealized (1 0) (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) w))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 init 1) (exp (gen) x-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w))))
  (label 117)
  (parent 79)
  (unrealized (1 0) (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn) (w expr) (x-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-1))) (x x))
  (deflistener (exp (gen) (mul x w x-1)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) w))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-1)))
      (send (enc n (exp (gen) (mul x w x-1)))) (recv n))
    ((recv (exp (gen) (mul x w x-1))) (send (exp (gen) (mul x w x-1))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) w)) (send (cat (exp (gen) x-1) w)))
    ((send (exp (gen) x-1))))
  (label 118)
  (parent 79)
  (unrealized (1 0) (2 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x x-0 expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv h) (send (exp (gen) y))))
  (label 119)
  (parent 79)
  (unrealized (1 0) (2 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) w))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w-0 (mul x (rec y) w)) (y-0 y) (w-1 w)) (gen)
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 120)
  (parent 85)
  (unrealized (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (y expn) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x))) (x x))
  (deflistener (exp (gen) (mul w x x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x x)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (displaced 5 0 init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x)))
      (send (enc n (exp (gen) (mul w x x)))) (recv n))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) (mul w x x))))
    ((recv (cat (exp (gen) y) (mul (rec y) w x x)))
      (send (cat (exp (gen) y) (mul (rec y) w x x))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w))))
  (label 121)
  (parent 85)
  (unrealized (1 0) (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) w))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-strand init 1) (exp (gen) x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((send (exp (gen) x-0))))
  (label 122)
  (parent 85)
  (unrealized (1 0) (2 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) w))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (displaced 5 3 resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w))))
  (label 123)
  (parent 85)
  (unrealized (1 0) (2 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (deflistener (exp (gen) (mul x w y-0)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) w))
  (defstrand resp 2 (h h-0) (y y-0))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)))
  (absent (y-0 h-0) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-strand resp 2) (exp (gen) y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))) (recv n))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) (mul x w y-0))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) w)) (send (cat (exp (gen) y-0) w)))
    ((recv h-0) (send (exp (gen) y-0))))
  (label 124)
  (parent 85)
  (unrealized (1 0) (2 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y (exp (gen) (mul x w))))
  (uniq-gen x y)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))) (send n))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv y) (send y)))
  (label 125)
  (parent 87)
  (unrealized (2 0) (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (w expr) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul w x x))) (y y))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y (exp (gen) (mul w x x))))
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))) (recv n))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x x y)))) (send n))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))) ((recv y) (send y)))
  (label 126)
  (parent 88)
  (unrealized (1 0) (2 0) (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)) ((4 0) (2 0)) ((5 1) (3 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (precur (3 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))) (send n))
    ((recv (cat (gen) (mul x w x-0))) (send (cat (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0)))
    ((recv y) (send y)))
  (label 127)
  (parent 89)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 4 (n n) (h (gen)) (y y))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)))
  (absent (y (gen)))
  (precur (3 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))
      (send n)) ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 128)
  (parent 90)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (w expr) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)))
  (absent (x w) (y (exp (gen) w)))
  (precur (3 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w) w (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))) (send n))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))))
  (label 129)
  (parent 90)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)) ((4 1) (2 0)))
  (absent (y (exp (gen) (mul x w))))
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))) (send n))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 130)
  (parent 90)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (0 1)) ((4 1) (2 0)) ((5 1) (3 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (precur (3 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))) (recv n))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))) (send n))
    ((recv (cat (gen) (mul x w y))) (send (cat (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 131)
  (parent 91)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (y expn) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y w))) (x x))
  (deflistener (cat (exp (gen) x) w))
  (precedes ((0 0) (1 0)))
  (precur (1 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation generalization forgot ((absent (y (exp (gen) (mul w x))))))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y w)))
      (send (enc n (exp (gen) (mul y w x)))) (recv n))
    ((recv (cat (exp (gen) x) w))))
  (label 132)
  (parent 92)
  (seen 114)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul x w))))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y)))) (send n))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv y) (send y)))
  (label 133)
  (parent 93)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 y))) (x x-0))
  (defstrand resp 4 (n n) (h (exp (gen) (mul w x-0 x-0))) (y y))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul w x-0 x-0))))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul w x-0 x-0 y)))) (recv n))
    ((recv (exp (gen) (mul w x-0 x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w x-0 x-0 y)))) (send n))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0 x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0 x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((recv y) (send y)))
  (label 134)
  (parent 94)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (exp (gen) (mul x w x-0))))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-0 y)))) (send n))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((recv y) (send y)))
  (label 135)
  (parent 95)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn) (w expr) (x-1 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-1 y))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w x-1))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w y)))
  (defstrand init 1 (x x-1))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (6 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((6 1) (4 0)))
  (absent (y (exp (gen) (mul x w x-1))))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-1 y)))
      (send (enc n (exp (gen) (mul x w x-1 y)))) (recv n))
    ((recv (exp (gen) (mul x w x-1))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w x-1 y)))) (send n))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w x-1)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w x-1))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w y)))
      (send (cat (exp (gen) x-1) (mul w y)))) ((send (exp (gen) x-1)))
    ((recv y) (send y)))
  (label 136)
  (parent 96)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (y x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y (rec x) x-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) x-0)) (y y))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) x-0)))
  (precur (4 0) (2 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x) x-0)))
      (send (enc n (exp (gen) (mul y x-0)))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y x-0)))) (send n))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0)))
      (send (cat (exp (gen) y) (mul (rec x) x-0)))))
  (label 137)
  (parent 97)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y (rec x-0)))) (x x-0))
  (defstrand resp 4 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w (rec x-0))))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (x-0 (mul (rec x) w)) (y (exp (gen) w)))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w))
    (mul (rec x) w) (2 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w y (rec x-0))))
      (send (enc n (exp (gen) (mul w y)))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))) (send n))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w (rec x-0))))
      (send (cat (exp (gen) y) (mul w (rec x-0))))))
  (label 138)
  (parent 97)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y x-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w x-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)))
  (absent (x-0 (mul x w)) (y (exp (gen) (mul x w x-0))))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w)) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y x-0)))
      (send (enc n (exp (gen) (mul x w y x-0)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y x-0)))) (send n))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w x-0)))
      (send (cat (exp (gen) y) (mul w x-0)))))
  (label 139)
  (parent 97)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x x-0 expn) (w expr) (y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (6 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((6 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))) (recv n))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))) (send n))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w y)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w y))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 140)
  (parent 98)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w))))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0)))
      (send (enc n (exp (gen) (mul x w y-0)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y-0)))) (send n))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) (mul w y-0))) (send (cat (gen) (mul w y-0))))
    ((recv y-0) (send y-0)))
  (label 141)
  (parent 99)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y expn) (w expr) (x y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul w x x))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x x)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul w x x))))
  (precur (4 0) (2 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y-0)))
      (send (enc n (exp (gen) (mul w x x y-0)))) (recv n))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w x x y-0)))) (send n))
    ((recv (cat (exp (gen) y) (mul (rec y) w x x)))
      (send (cat (exp (gen) y) (mul (rec y) w x x))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul w y-0)))
      (send (cat (exp (gen) x) (mul w y-0)))) ((recv y-0) (send y-0)))
  (label 142)
  (parent 100)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr) (x-0 y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w x-0))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w x-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) (mul w y-0)))
  (defstrand init 1 (x x-0))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (6 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((6 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w x-0))))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0 y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y-0)))
      (send (enc n (exp (gen) (mul x w x-0 y-0)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w x-0 y-0)))) (send n))
    ((recv (cat (exp (gen) y) (mul x (rec y) w x-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w x-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul w y-0)))
      (send (cat (exp (gen) x-0) (mul w y-0)))) ((send (exp (gen) x-0)))
    ((recv y-0) (send y-0)))
  (label 143)
  (parent 101)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y x y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y (rec x) y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) y-0)) (y y))
  (deflistener (cat (exp (gen) y-0) (one)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y (exp (gen) y-0)) (y-0 h))
  (precur (4 0) (2 0))
  (uniq-gen y x y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y-0) (w (mul (rec x) y-0))) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x) y-0)))
      (send (enc n (exp (gen) (mul y y-0)))) (recv n))
    ((recv (exp (gen) y-0)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul y y-0)))) (send n))
    ((recv (cat (exp (gen) y-0) (one)))
      (send (cat (exp (gen) y-0) (one))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) y-0)))
      (send (cat (exp (gen) y) (mul (rec x) y-0)))))
  (label 144)
  (parent 102)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (x (mul (rec y) w)) (y-0 (exp (gen) w)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w))
    (mul (rec y) w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))) (send n))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x))))))
  (label 145)
  (parent 102)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w y-0))) (y y))
  (deflistener (cat (exp (gen) y-0) (mul x w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)))
  (absent (y-0 (mul x w)) (y (exp (gen) (mul x w y-0))) (y-0 h))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w)) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))) (recv n))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y y-0)))) (send n))
    ((recv (cat (exp (gen) y-0) (mul x w)))
      (send (cat (exp (gen) y-0) (mul x w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))))
  (label 146)
  (parent 102)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w y))) (y y-0))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (5 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y h) (y-0 (exp (gen) (mul x w y))))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))) (recv n))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul x w y y-0)))) (send n))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))) ((recv y-0) (send y-0)))
  (label 147)
  (parent 103)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (x y expn) (w expr) (y-0 y-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y-0 y-1))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w y-0))) (y y-1))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w y-1)))
  (defstrand resp 2 (h h-0) (y y-0))
  (deflistener y-1)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (6 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((6 1) (4 0)))
  (absent (y-0 h-0) (y h) (y-1 (exp (gen) (mul x w y-0))))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0 y-1)
  (uniq-orig n)
  (operation nonce-test (added-listener y-1) (mul w y-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 y-1)))
      (send (enc n (exp (gen) (mul x w y-0 y-1)))) (recv n))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) y-1))
      (recv (enc n (exp (gen) (mul x w y-0 y-1)))) (send n))
    ((recv (cat (exp (gen) y) (mul x (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul x (rec y) w y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w y-1)))
      (send (cat (exp (gen) y-0) (mul w y-1))))
    ((recv h-0) (send (exp (gen) y-0))) ((recv y-1) (send y-1)))
  (label 148)
  (parent 104)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (gen) (rec x)))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x)))))
  (label 149)
  (parent 110)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (deflistener (exp (gen) w))
  (deflistener (cat (gen) w))
  (deflistener (cat (gen) (mul w (rec x))))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (absent (x w))
  (operation nonce-test (added-absence x w) w (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (gen) (mul w (rec x))))
      (send (cat (gen) (mul w (rec x))))))
  (label 150)
  (parent 110)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (gen) w))
  (deflistener x)
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test (added-listener x) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))) ((recv x) (send x)))
  (label 151)
  (parent 110)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (precur (3 0) (2 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation nonce-test (contracted (w (mul (rec x) (rec x))) (x-0 x))
    (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x))))))
  (label 152)
  (parent 111)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (deflistener (exp (gen) w))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) x) (mul w (rec x) (rec x))))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)))
  (precur (3 0) (2 0))
  (uniq-gen x)
  (uniq-orig n)
  (absent (x w))
  (operation nonce-test (added-absence x w) w (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) x) (mul w (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w (rec x) (rec x))))))
  (label 153)
  (parent 111)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x))) (x x))
  (deflistener (exp (gen) (mul w x x)))
  (deflistener (cat (gen) (mul w x x)))
  (deflistener (cat (exp (gen) x) w))
  (deflistener x)
  (precedes ((0 0) (3 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)))
  (precur (3 0) (2 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w x x) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x)))
      (send (enc n (exp (gen) (mul w x x)))) (recv n))
    ((recv (exp (gen) (mul w x x))) (send (exp (gen) (mul w x x))))
    ((recv (cat (gen) (mul w x x))) (send (cat (gen) (mul w x x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w)))
    ((recv x) (send x)))
  (label 154)
  (parent 111)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 0) (2 0)) ((4 0) (3 0)))
  (precur (3 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w (mul (rec x) (rec x-0))) (x-2 x-0)) (one)
    (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x-0) (mul (rec x) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 155)
  (parent 112)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (w expr) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (gen) (mul w x)))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (defstrand init 1 (x x))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 0) (2 0)) ((4 0) (3 0)))
  (precur (3 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 (mul w x)))
  (operation nonce-test (added-absence x-0 (mul w x)) (mul w x) (2 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))) (recv n))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (gen) (mul w x))) (send (cat (gen) (mul w x))))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0)))))
    ((send (exp (gen) x))))
  (label 156)
  (parent 112)
  (unrealized (0 1) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 0) (2 0)) ((4 0) (3 0)))
  (precur (3 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 (mul x w)))
  (operation nonce-test (added-absence x-0 (mul x w)) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 157)
  (parent 112)
  (unrealized (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y) (mul (rec x) (rec y))))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)) ((4 1) (3 0)))
  (absent (y h))
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (w (mul (rec x) (rec y))) (y-0 y)) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y) (mul (rec x) (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) (rec y)))))
    ((recv h) (send (exp (gen) y))))
  (label 158)
  (parent 113)
  (unrealized (0 1) (3 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (w expr) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)) ((4 1) (3 0)))
  (absent (x (mul w y)) (y h))
  (precur (3 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul w y)) (mul w y) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))) (recv n))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x)))))
    ((recv h) (send (exp (gen) y))))
  (label 159)
  (parent 113)
  (unrealized (0 1) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul w (rec y))))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)) ((4 1) (3 0)))
  (absent (y (mul x w)) (y h))
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x w)) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul w (rec y))))
      (send (cat (exp (gen) y) (mul w (rec y)))))
    ((recv h) (send (exp (gen) y))))
  (label 160)
  (parent 113)
  (unrealized (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul (rec x) x-0)))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0))) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul (rec x) x-0)))
      (send (cat (gen) (mul (rec x) x-0)))))
  (label 161)
  (parent 115)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w (rec x-0)))) (x x-0))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (gen) (mul w (rec x-0))))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 (mul (rec x) w)))
  (operation nonce-test (added-absence x-0 (mul (rec x) w))
    (mul (rec x) w) (2 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w (rec x-0))))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (gen) (mul w (rec x-0))))
      (send (cat (gen) (mul w (rec x-0))))))
  (label 162)
  (parent 115)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w x-0)))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 (mul x w)))
  (operation nonce-test (added-absence x-0 (mul x w)) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w x-0))) (send (cat (gen) (mul w x-0)))))
  (label 163)
  (parent 115)
  (unrealized (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (w (mul x (rec x-0) (rec x-0))) (x-2 x-0)) (one)
    (2 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))) (recv n))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) (rec x-0))))))
  (label 164)
  (parent 116)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (w expr) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x x-0))) (x x))
  (deflistener (exp (gen) (mul w x x x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w x x)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x) (mul w x-0)))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 (mul w x x)))
  (operation nonce-test (added-absence x-0 (mul w x x)) (mul w x x)
    (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x x-0)))
      (send (enc n (exp (gen) (mul w x x x-0)))) (recv n))
    ((recv (exp (gen) (mul w x x x-0)))
      (send (exp (gen) (mul w x x x-0))))
    ((recv (cat (exp (gen) x-0) (mul w x x)))
      (send (cat (exp (gen) x-0) (mul w x x)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x) (mul w x-0)))
      (send (cat (exp (gen) x) (mul w x-0)))))
  (label 165)
  (parent 116)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w (rec x-0)))) (x x-0))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0) (rec x-0))))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 (mul (rec x) w)))
  (operation nonce-test (added-absence x-0 (mul (rec x) w))
    (mul (rec x) w) (2 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w (rec x-0))))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0) (rec x-0))))))
  (label 166)
  (parent 116)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (rec x-0)))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (contracted (x-1 x-0) (w (rec x-0))) (one)
    (2 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))) (recv n))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (rec x-0)))
      (send (cat (exp (gen) x) (rec x-0)))))
  (label 167)
  (parent 117)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (w expr) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) x) w))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 w))
  (operation nonce-test (added-absence x-0 w) w (2 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))) (recv n))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w)))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0))))))
  (label 168)
  (parent 117)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-0) w))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 1) (2 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) w)) (send (cat (exp (gen) x-0) w)))
    ((recv x) (send x)))
  (label 169)
  (parent 117)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 x-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-2 x) (x-3 x-0) (w (mul (rec x) x-0 (rec x-1)))
      (x-4 x-1)) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1))))
      (send (cat (exp (gen) x-1) (mul (rec x) x-0 (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 170)
  (parent 118)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 x-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 (rec x-1)))) (x x-1))
  (deflistener (exp (gen) (mul w x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-1))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1)
  (uniq-orig n)
  (absent (x-1 (mul (rec x) w x-0)))
  (operation nonce-test (added-absence x-1 (mul (rec x) w x-0))
    (mul (rec x) w x-0) (2 0))
  (traces
    ((send (exp (gen) x-1)) (recv (exp (gen) (mul w x-0 (rec x-1))))
      (send (enc n (exp (gen) (mul w x-0)))) (recv n))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) (mul w x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-1))))
      (send (cat (exp (gen) x-0) (mul w (rec x-1)))))
    ((send (exp (gen) x-0))))
  (label 171)
  (parent 118)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 x-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 x-1))) (x x))
  (deflistener (exp (gen) (mul x w x-0 x-1)))
  (deflistener (cat (exp (gen) x-1) (mul x w x-0)))
  (defstrand init 1 (x x-1))
  (deflistener (cat (exp (gen) x-0) (mul w x-1)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1)
  (uniq-orig n)
  (absent (x-1 (mul x w x-0)))
  (operation nonce-test (added-absence x-1 (mul x w x-0)) (mul x w x-0)
    (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 x-1)))
      (send (enc n (exp (gen) (mul x w x-0 x-1)))) (recv n))
    ((recv (exp (gen) (mul x w x-0 x-1)))
      (send (exp (gen) (mul x w x-0 x-1))))
    ((recv (cat (exp (gen) x-1) (mul x w x-0)))
      (send (cat (exp (gen) x-1) (mul x w x-0))))
    ((send (exp (gen) x-1)))
    ((recv (cat (exp (gen) x-0) (mul w x-1)))
      (send (cat (exp (gen) x-0) (mul w x-1))))
    ((send (exp (gen) x-0))))
  (label 172)
  (parent 118)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn) (w expr) (x-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w (rec x-1))))
  (defstrand init 1 (x x-1))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1)
  (uniq-orig n)
  (absent (x-1 (mul x (rec x-0) w)))
  (operation nonce-test (added-absence x-1 (mul x (rec x-0) w))
    (mul x (rec x-0) w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w (rec x-1))))
      (send (cat (exp (gen) x-1) (mul w (rec x-1)))))
    ((send (exp (gen) x-1))))
  (label 173)
  (parent 118)
  (unrealized (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) x-0))) (x x))
  (deflistener (exp (gen) x-0))
  (deflistener (cat (exp (gen) x-0) (one)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (x-2 x-0) (w (mul (rec x) x-0 (rec y))) (y-0 y))
    (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) x-0)))
      (send (enc n (exp (gen) x-0))) (recv n))
    ((recv (exp (gen) x-0)) (send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-0) (one)))
      (send (cat (exp (gen) x-0) (one)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul (rec x) x-0 (rec y))))
      (send (cat (exp (gen) y) (mul (rec x) x-0 (rec y)))))
    ((recv h) (send (exp (gen) y))))
  (label 174)
  (parent 119)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y (rec x-0)))) (x x-0))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w y)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w (rec x-0))))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)))
  (absent (x-0 (mul (rec x) w y)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec x) w y))
    (mul (rec x) w y) (2 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w y (rec x-0))))
      (send (enc n (exp (gen) (mul w y)))) (recv n))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) x) (mul (rec x) w y)))
      (send (cat (exp (gen) x) (mul (rec x) w y))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w (rec x-0))))
      (send (cat (exp (gen) y) (mul w (rec x-0)))))
    ((recv h) (send (exp (gen) y))))
  (label 175)
  (parent 119)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y x-0))) (x x))
  (deflistener (exp (gen) (mul x w y x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w x-0)))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)))
  (absent (x-0 (mul x w y)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x w y)) (mul x w y)
    (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y x-0)))
      (send (enc n (exp (gen) (mul x w y x-0)))) (recv n))
    ((recv (exp (gen) (mul x w y x-0)))
      (send (exp (gen) (mul x w y x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w y)))
      (send (cat (exp (gen) x-0) (mul x w y)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w x-0)))
      (send (cat (exp (gen) y) (mul w x-0))))
    ((recv h) (send (exp (gen) y))))
  (label 176)
  (parent 119)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x x-0 expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w (rec y))))
  (defstrand resp 2 (h h) (y y))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)))
  (absent (y (mul x (rec x-0) w)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x (rec x-0) w))
    (mul x (rec x-0) w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w (rec y))))
      (send (cat (exp (gen) y) (mul w (rec y)))))
    ((recv h) (send (exp (gen) y))))
  (label 177)
  (parent 119)
  (unrealized (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) (mul (rec x) y)))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (y-0 y) (w (mul (rec x) y)))
    (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) (mul (rec x) y)))
      (send (cat (gen) (mul (rec x) y)))))
  (label 178)
  (parent 120)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y expn) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) (mul w (rec x))))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (absent (x (mul (rec y) w)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w))
    (mul (rec y) w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) (mul w (rec x))))
      (send (cat (gen) (mul w (rec x))))))
  (label 179)
  (parent 120)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (absent (y (mul x w)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x w)) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y)))))
  (label 180)
  (parent 120)
  (unrealized (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul y (rec x) (rec x))))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test
    (contracted (y-0 y) (w (mul y (rec x) (rec x))) (x-0 x)) (one)
    (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul y (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul y (rec x) (rec x))))))
  (label 181)
  (parent 121)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (w expr) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x y))) (x x))
  (deflistener (exp (gen) (mul w x x y)))
  (deflistener (cat (exp (gen) y) (mul w x x)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (absent (y (mul w x x)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul w x x)) (mul w x x) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))) (recv n))
    ((recv (exp (gen) (mul w x x y))) (send (exp (gen) (mul w x x y))))
    ((recv (cat (exp (gen) y) (mul w x x)))
      (send (cat (exp (gen) y) (mul w x x))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))))
  (label 182)
  (parent 121)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (y expn) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul w (rec x) (rec x))))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (absent (x (mul (rec y) w)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w))
    (mul (rec y) w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul w (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w (rec x) (rec x))))))
  (label 183)
  (parent 121)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-1 x) (y-0 y) (w (mul (rec x) y (rec x-0))) (x-2 x-0))
    (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul (rec x) y (rec x-0))))
      (send (cat (exp (gen) x-0) (mul (rec x) y (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 184)
  (parent 122)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y expn) (w expr) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (defstrand init 1 (x x))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)))
  (absent (x-0 (mul (rec y) w x)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul (rec y) w x))
    (mul (rec y) w x) (2 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))) (recv n))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) y) (mul (rec y) w x)))
      (send (cat (exp (gen) y) (mul (rec y) w x))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0)))))
    ((send (exp (gen) x))))
  (label 185)
  (parent 122)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (deflistener (exp (gen) (mul x w x-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w x-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)))
  (absent (y (mul x w x-0)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-absence y (mul x w x-0)) (mul x w x-0)
    (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))) (recv n))
    ((recv (exp (gen) (mul x w x-0 y)))
      (send (exp (gen) (mul x w x-0 y))))
    ((recv (cat (exp (gen) y) (mul x w x-0)))
      (send (cat (exp (gen) y) (mul x w x-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0))))
  (label 186)
  (parent 122)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0))))
  (defstrand init 1 (x x-0))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)))
  (absent (x-0 (mul x (rec y) w)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-absence x-0 (mul x (rec y) w))
    (mul x (rec y) w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0)))))
    ((send (exp (gen) x-0))))
  (label 187)
  (parent 122)
  (unrealized (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (contracted (x-0 x) (w (rec x))) (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))))
  (label 188)
  (parent 123)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (w expr) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)))
  (absent (x w) (y h))
  (precur (4 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-absence x w) w (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))) (recv n))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))))
  (label 189)
  (parent 123)
  (unrealized (0 1) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) w))
  (deflistener x)
  (precedes ((0 0) (4 0)) ((0 0) (5 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (2 0)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul x w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv x) (send x)))
  (label 190)
  (parent 123)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (x y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul (rec x) y))) (x x))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
  (defstrand resp 2 (h h-0) (y y-0))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)))
  (absent (y-0 h-0) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test
    (contracted (x-0 x) (y-1 y) (w (mul (rec x) y (rec y-0))) (y-2 y-0))
    (one) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul (rec x) y)))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul (rec x) y (rec y-0))))
      (send (cat (exp (gen) y-0) (mul (rec x) y (rec y-0)))))
    ((recv h-0) (send (exp (gen) y-0))))
  (label 191)
  (parent 124)
  (unrealized (0 1) (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (defstrand resp 2 (h h-0) (y y-0))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)))
  (absent (x (mul (rec y) w y-0)) (y-0 h-0) (y h))
  (precur (4 0) (2 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-absence x (mul (rec y) w y-0))
    (mul (rec y) w y-0) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))) (recv n))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x)))))
    ((recv h-0) (send (exp (gen) y-0))))
  (label 192)
  (parent 124)
  (unrealized (0 1) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (deflistener (exp (gen) (mul x w y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w y)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h-0) (y y))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)))
  (absent (y-0 (mul x w y)) (y h-0) (y-0 h))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x w y)) (mul x w y)
    (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))) (recv n))
    ((recv (exp (gen) (mul x w y y-0)))
      (send (exp (gen) (mul x w y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w y)))
      (send (cat (exp (gen) y-0) (mul x w y))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h-0) (send (exp (gen) y))))
  (label 193)
  (parent 124)
  (unrealized (0 1) (1 0) (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec y-0))))
  (defstrand resp 2 (h h-0) (y y-0))
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)))
  (absent (y-0 (mul x (rec y) w)) (y-0 h-0) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-absence y-0 (mul x (rec y) w))
    (mul x (rec y) w) (2 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w (rec y-0)))))
    ((recv h-0) (send (exp (gen) y-0))))
  (label 194)
  (parent 124)
  (unrealized (2 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton plaindh
  (vars (n text) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (defstrand resp 4 (n n) (h (gen)) (y y))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 3) (0 3)) ((2 1) (1 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (y (gen)))
  (precur (3 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (gen)) (send (exp (gen) y)) (recv (enc n (exp (gen) y)))
      (send n)) ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 195)
  (parent 128)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (w expr) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((0 2) (1 2)) ((1 1) (3 0))
    ((1 3) (0 3)) ((2 1) (1 0)) ((3 1) (0 1)) ((4 1) (3 0)))
  (absent (x w) (y (exp (gen) w)))
  (precur (3 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))) (send n))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))) ((recv x) (send x)))
  (label 196)
  (parent 129)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y (rec x-0)))) (x x-0))
  (defstrand resp 4 (n n) (h (exp (gen) w)) (y y))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (4 0))
    ((1 3) (0 3)) ((2 1) (1 0)) ((3 0) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (absent (x-0 (mul (rec x) w)) (y (exp (gen) w)))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w y (rec x-0))))
      (send (enc n (exp (gen) (mul w y)))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul w y)))) (send n))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w (rec x-0))))
      (send (cat (exp (gen) y) (mul w (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 197)
  (parent 138)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y x-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w x-0))) (y y))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w x-0)))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 0) (2 0)) ((3 0) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (absent (x-0 (mul x w)) (y (exp (gen) (mul x w x-0))))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y x-0)))
      (send (enc n (exp (gen) (mul x w y x-0)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y x-0)))) (send n))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w x-0)))
      (send (cat (exp (gen) y) (mul w x-0)))) ((recv x-0) (send x-0)))
  (label 198)
  (parent 139)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) w)) (y y-0))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((0 2) (1 2)) ((1 1) (4 0))
    ((1 3) (0 3)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (absent (x (mul (rec y) w)) (y-0 (exp (gen) w)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) y-0))
      (recv (enc n (exp (gen) (mul w y-0)))) (send n))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x))))) ((recv x) (send x)))
  (label 199)
  (parent 145)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (defstrand resp 4 (n n) (h (exp (gen) (mul x w y-0))) (y y))
  (deflistener (cat (exp (gen) y-0) (mul x w)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 2) (1 2)) ((1 1) (4 0)) ((1 3) (0 3))
    ((2 1) (1 0)) ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1))
    ((5 1) (4 0)))
  (absent (y-0 (mul x w)) (y (exp (gen) (mul x w y-0))) (y-0 h))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))) (recv n))
    ((recv (exp (gen) (mul x w y-0))) (send (exp (gen) y))
      (recv (enc n (exp (gen) (mul x w y y-0)))) (send n))
    ((recv (cat (exp (gen) y-0) (mul x w)))
      (send (cat (exp (gen) y-0) (mul x w))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0)))) ((recv y-0) (send y-0)))
  (label 200)
  (parent 146)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (gen) (rec x)))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (operation nonce-test (added-listener x) (rec x) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (gen) (rec x))) (send (cat (gen) (rec x))))
    ((recv x) (send x)))
  (label 201)
  (parent 149)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (deflistener (exp (gen) w))
  (deflistener (cat (gen) w))
  (deflistener (cat (gen) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (uniq-gen x)
  (uniq-orig n)
  (precur (3 0) (2 0))
  (absent (x w))
  (operation nonce-test (added-listener x) (mul w (rec x)) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (gen) (mul w (rec x))))
      (send (cat (gen) (mul w (rec x))))) ((recv x) (send x)))
  (label 202)
  (parent 150)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (rec x))) (x x))
  (deflistener (gen))
  (deflistener (cat (gen) (one)))
  (deflistener (cat (exp (gen) x) (mul (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (precur (3 0) (2 0))
  (uniq-gen x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul (rec x) (rec x)) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (rec x)))
      (send (enc n (gen))) (recv n)) ((recv (gen)) (send (gen)))
    ((recv (cat (gen) (one))) (send (cat (gen) (one))))
    ((recv (cat (exp (gen) x) (mul (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 203)
  (parent 152)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (deflistener (exp (gen) w))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) x) (mul w (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (3 0)))
  (precur (3 0) (2 0))
  (uniq-gen x)
  (uniq-orig n)
  (absent (x w))
  (operation nonce-test (added-listener x) (mul w (rec x) (rec x))
    (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) x) (mul w (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 204)
  (parent 153)
  (unrealized (4 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (w expr) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (gen) (mul w x)))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (defstrand init 1 (x x))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 0) (2 0)) ((4 0) (3 0)) ((5 1) (3 0)))
  (precur (3 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 (mul w x)))
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (3 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))) (recv n))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (gen) (mul w x))) (send (cat (gen) (mul w x))))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0)))))
    ((send (exp (gen) x))) ((recv x-0) (send x-0)))
  (label 205)
  (parent 156)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0))))
  (defstrand init 1 (x x-0))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 0) (2 0)) ((4 0) (5 0)) ((5 1) (3 0)))
  (precur (3 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 (mul x w)))
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0)))))
    ((send (exp (gen) x-0))) ((recv x-0) (send x-0)))
  (label 206)
  (parent 157)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (w expr) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (defstrand resp 2 (h h) (y y))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)) ((4 1) (3 0)) ((5 1) (3 0)))
  (absent (x (mul w y)) (y h))
  (precur (3 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))) (recv n))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x)))))
    ((recv h) (send (exp (gen) y))) ((recv x) (send x)))
  (label 207)
  (parent 159)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul w (rec y))))
  (defstrand resp 2 (h h) (y y))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 0) (3 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (0 1)) ((4 1) (2 0)) ((4 1) (5 0)) ((5 1) (3 0)))
  (absent (y (mul x w)) (y h))
  (precur (3 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w (rec y)) (3 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (gen) (mul x w))) (send (cat (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul w (rec y))))
      (send (cat (exp (gen) y) (mul w (rec y)))))
    ((recv h) (send (exp (gen) y))) ((recv y) (send y)))
  (label 208)
  (parent 160)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w (rec x-0)))) (x x-0))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (gen) (mul w (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 (mul (rec x) w)))
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w (rec x-0))))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (gen) (mul w (rec x-0))))
      (send (cat (gen) (mul w (rec x-0))))) ((recv x-0) (send x-0)))
  (label 209)
  (parent 162)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0))) (x x))
  (deflistener (exp (gen) (mul x w x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (gen) (mul w x-0)))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 (mul x w)))
  (operation nonce-test (added-listener x-0) (mul w x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0)))
      (send (enc n (exp (gen) (mul x w x-0)))) (recv n))
    ((recv (exp (gen) (mul x w x-0))) (send (exp (gen) (mul x w x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w)))
      (send (cat (exp (gen) x-0) (mul x w)))) ((send (exp (gen) x-0)))
    ((recv (cat (gen) (mul w x-0))) (send (cat (gen) (mul w x-0))))
    ((recv x-0) (send x-0)))
  (label 210)
  (parent 163)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (w expr) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x x-0))) (x x))
  (deflistener (exp (gen) (mul w x x x-0)))
  (deflistener (cat (exp (gen) x-0) (mul w x x)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x) (mul w x-0)))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 (mul w x x)))
  (operation nonce-test (added-listener x-0) (mul w x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x x-0)))
      (send (enc n (exp (gen) (mul w x x x-0)))) (recv n))
    ((recv (exp (gen) (mul w x x x-0)))
      (send (exp (gen) (mul w x x x-0))))
    ((recv (cat (exp (gen) x-0) (mul w x x)))
      (send (cat (exp (gen) x-0) (mul w x x)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x) (mul w x-0)))
      (send (cat (exp (gen) x) (mul w x-0)))) ((recv x-0) (send x-0)))
  (label 211)
  (parent 165)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w (rec x-0)))) (x x-0))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) x) (mul (rec x) w)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0) (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 (mul (rec x) w)))
  (operation nonce-test (added-listener x-0) (mul w (rec x-0) (rec x-0))
    (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w (rec x-0))))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) x) (mul (rec x) w)))
      (send (cat (exp (gen) x) (mul (rec x) w)))) ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0) (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0) (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 212)
  (parent 166)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul x (rec x-0)))) (x x-0))
  (deflistener (exp (gen) x))
  (deflistener (cat (exp (gen) x) (one)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (rec x-0)))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (rec x-0) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul x (rec x-0))))
      (send (enc n (exp (gen) x))) (recv n))
    ((recv (exp (gen) x)) (send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (one))) (send (cat (exp (gen) x) (one))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (rec x-0)))
      (send (cat (exp (gen) x) (rec x-0)))) ((recv x-0) (send x-0)))
  (label 213)
  (parent 167)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (w expr) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) x) w))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0)
  (uniq-orig n)
  (absent (x-0 w))
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))) (recv n))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) x) w)) (send (cat (exp (gen) x) w)))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0)))))
    ((recv x-0) (send x-0)))
  (label 214)
  (parent 168)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 x-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 (rec x-1)))) (x x-1))
  (deflistener (exp (gen) (mul w x-0)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w x-0)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-1))))
  (defstrand init 1 (x x-0))
  (deflistener x-1)
  (precedes ((0 0) (2 0)) ((0 0) (6 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)) ((6 1) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1)
  (uniq-orig n)
  (absent (x-1 (mul (rec x) w x-0)))
  (operation nonce-test (added-listener x-1) (mul w (rec x-1)) (4 0))
  (traces
    ((send (exp (gen) x-1)) (recv (exp (gen) (mul w x-0 (rec x-1))))
      (send (enc n (exp (gen) (mul w x-0)))) (recv n))
    ((recv (exp (gen) (mul w x-0))) (send (exp (gen) (mul w x-0))))
    ((recv (cat (exp (gen) x) (mul (rec x) w x-0)))
      (send (cat (exp (gen) x) (mul (rec x) w x-0))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-1))))
      (send (cat (exp (gen) x-0) (mul w (rec x-1)))))
    ((send (exp (gen) x-0))) ((recv x-1) (send x-1)))
  (label 215)
  (parent 171)
  (unrealized (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x expn) (w expr) (x-0 x-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 x-1))) (x x))
  (deflistener (exp (gen) (mul x w x-0 x-1)))
  (deflistener (cat (exp (gen) x-1) (mul x w x-0)))
  (defstrand init 1 (x x-1))
  (deflistener (cat (exp (gen) x-0) (mul w x-1)))
  (defstrand init 1 (x x-0))
  (deflistener x-1)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (6 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)) ((6 1) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1)
  (uniq-orig n)
  (absent (x-1 (mul x w x-0)))
  (operation nonce-test (added-listener x-1) (mul w x-1) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 x-1)))
      (send (enc n (exp (gen) (mul x w x-0 x-1)))) (recv n))
    ((recv (exp (gen) (mul x w x-0 x-1)))
      (send (exp (gen) (mul x w x-0 x-1))))
    ((recv (cat (exp (gen) x-1) (mul x w x-0)))
      (send (cat (exp (gen) x-1) (mul x w x-0))))
    ((send (exp (gen) x-1)))
    ((recv (cat (exp (gen) x-0) (mul w x-1)))
      (send (cat (exp (gen) x-0) (mul w x-1)))) ((send (exp (gen) x-0)))
    ((recv x-1) (send x-1)))
  (label 216)
  (parent 172)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (x x-0 expn) (w expr) (x-1 expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) x-1) (mul w (rec x-1))))
  (defstrand init 1 (x x-1))
  (deflistener x-1)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (6 0)) ((6 1) (4 0)))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 x-1)
  (uniq-orig n)
  (absent (x-1 (mul x (rec x-0) w)))
  (operation nonce-test (added-listener x-1) (mul w (rec x-1)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) x-1) (mul w (rec x-1))))
      (send (cat (exp (gen) x-1) (mul w (rec x-1)))))
    ((send (exp (gen) x-1))) ((recv x-1) (send x-1)))
  (label 217)
  (parent 173)
  (unrealized (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y (rec x-0)))) (x x-0))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) x) (mul (rec x) w y)))
  (defstrand init 1 (x x))
  (deflistener (cat (exp (gen) y) (mul w (rec x-0))))
  (defstrand resp 2 (h h) (y y))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (6 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (absent (x-0 (mul (rec x) w y)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w y (rec x-0))))
      (send (enc n (exp (gen) (mul w y)))) (recv n))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) x) (mul (rec x) w y)))
      (send (cat (exp (gen) x) (mul (rec x) w y))))
    ((send (exp (gen) x)))
    ((recv (cat (exp (gen) y) (mul w (rec x-0))))
      (send (cat (exp (gen) y) (mul w (rec x-0)))))
    ((recv h) (send (exp (gen) y))) ((recv x-0) (send x-0)))
  (label 218)
  (parent 175)
  (unrealized (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y x-0))) (x x))
  (deflistener (exp (gen) (mul x w y x-0)))
  (deflistener (cat (exp (gen) x-0) (mul x w y)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w x-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (6 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (absent (x-0 (mul x w y)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w x-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y x-0)))
      (send (enc n (exp (gen) (mul x w y x-0)))) (recv n))
    ((recv (exp (gen) (mul x w y x-0)))
      (send (exp (gen) (mul x w y x-0))))
    ((recv (cat (exp (gen) x-0) (mul x w y)))
      (send (cat (exp (gen) x-0) (mul x w y)))) ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w x-0)))
      (send (cat (exp (gen) y) (mul w x-0))))
    ((recv h) (send (exp (gen) y))) ((recv x-0) (send x-0)))
  (label 219)
  (parent 176)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x x-0 expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) x-0) (mul x (rec x-0) w)))
  (defstrand init 1 (x x-0))
  (deflistener (cat (exp (gen) y) (mul w (rec y))))
  (defstrand resp 2 (h h) (y y))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 0) (2 0)) ((3 0) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (absent (y (mul x (rec x-0) w)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w (rec y)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) x-0) (mul x (rec x-0) w)))
      (send (cat (exp (gen) x-0) (mul x (rec x-0) w))))
    ((send (exp (gen) x-0)))
    ((recv (cat (exp (gen) y) (mul w (rec y))))
      (send (cat (exp (gen) y) (mul w (rec y)))))
    ((recv h) (send (exp (gen) y))) ((recv y) (send y)))
  (label 220)
  (parent 177)
  (unrealized (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y expn) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (x (mul (rec y) w)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) (mul w (rec x))))
      (send (cat (gen) (mul w (rec x))))) ((recv x) (send x)))
  (label 221)
  (parent 179)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y))) (x x))
  (deflistener (exp (gen) (mul x w y)))
  (deflistener (cat (exp (gen) y) (mul x w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (gen) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (mul x w)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y)))
      (send (enc n (exp (gen) (mul x w y)))) (recv n))
    ((recv (exp (gen) (mul x w y))) (send (exp (gen) (mul x w y))))
    ((recv (cat (exp (gen) y) (mul x w)))
      (send (cat (exp (gen) y) (mul x w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (gen) (mul w y))) (send (cat (gen) (mul w y))))
    ((recv y) (send y)))
  (label 222)
  (parent 180)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (w expr) (x y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x y))) (x x))
  (deflistener (exp (gen) (mul w x x y)))
  (deflistener (cat (exp (gen) y) (mul w x x)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul w y)))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (5 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y (mul w x x)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x y)))
      (send (enc n (exp (gen) (mul w x x y)))) (recv n))
    ((recv (exp (gen) (mul w x x y))) (send (exp (gen) (mul w x x y))))
    ((recv (cat (exp (gen) y) (mul w x x)))
      (send (cat (exp (gen) y) (mul w x x))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul w y)))
      (send (cat (exp (gen) x) (mul w y)))) ((recv y) (send y)))
  (label 223)
  (parent 182)
  (unrealized (1 0) (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y expn) (w expr) (x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w (rec x)))) (x x))
  (deflistener (exp (gen) w))
  (deflistener (cat (exp (gen) y) (mul (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul w (rec x) (rec x))))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (x (mul (rec y) w)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x) (rec x))
    (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w (rec x))))
      (send (enc n (exp (gen) w))) (recv n))
    ((recv (exp (gen) w)) (send (exp (gen) w)))
    ((recv (cat (exp (gen) y) (mul (rec y) w)))
      (send (cat (exp (gen) y) (mul (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul w (rec x) (rec x))))
      (send (cat (exp (gen) x) (mul w (rec x) (rec x)))))
    ((recv x) (send x)))
  (label 224)
  (parent 183)
  (unrealized (2 0) (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y expn) (w expr) (x x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x (rec x-0)))) (x x-0))
  (deflistener (exp (gen) (mul w x)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w x)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x) (mul w (rec x-0))))
  (defstrand init 1 (x x))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (6 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (x-0 (mul (rec y) w x)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen y x x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (4 0))
  (traces
    ((send (exp (gen) x-0)) (recv (exp (gen) (mul w x (rec x-0))))
      (send (enc n (exp (gen) (mul w x)))) (recv n))
    ((recv (exp (gen) (mul w x))) (send (exp (gen) (mul w x))))
    ((recv (cat (exp (gen) y) (mul (rec y) w x)))
      (send (cat (exp (gen) y) (mul (rec y) w x))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x) (mul w (rec x-0))))
      (send (cat (exp (gen) x) (mul w (rec x-0)))))
    ((send (exp (gen) x))) ((recv x-0) (send x-0)))
  (label 225)
  (parent 185)
  (unrealized (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x expn) (w expr) (x-0 y expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w x-0 y))) (x x))
  (deflistener (exp (gen) (mul x w x-0 y)))
  (deflistener (cat (exp (gen) y) (mul x w x-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) (mul w y)))
  (defstrand init 1 (x x-0))
  (deflistener y)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (y (mul x w x-0)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x x-0 y)
  (uniq-orig n)
  (operation nonce-test (added-listener y) (mul w y) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w x-0 y)))
      (send (enc n (exp (gen) (mul x w x-0 y)))) (recv n))
    ((recv (exp (gen) (mul x w x-0 y)))
      (send (exp (gen) (mul x w x-0 y))))
    ((recv (cat (exp (gen) y) (mul x w x-0)))
      (send (cat (exp (gen) y) (mul x w x-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul w y)))
      (send (cat (exp (gen) x-0) (mul w y)))) ((send (exp (gen) x-0)))
    ((recv y) (send y)))
  (label 226)
  (parent 186)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (x y expn) (w expr) (x-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) x-0) (mul w (rec x-0))))
  (defstrand init 1 (x x-0))
  (deflistener x-0)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 0) (2 0))
    ((5 0) (6 0)) ((6 1) (4 0)))
  (absent (x-0 (mul x (rec y) w)) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y x-0)
  (uniq-orig n)
  (operation nonce-test (added-listener x-0) (mul w (rec x-0)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) x-0) (mul w (rec x-0))))
      (send (cat (exp (gen) x-0) (mul w (rec x-0)))))
    ((send (exp (gen) x-0))) ((recv x-0) (send x-0)))
  (label 227)
  (parent 187)
  (unrealized (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul y (rec x)))) (x x))
  (deflistener (exp (gen) y))
  (deflistener (cat (exp (gen) y) (one)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) (rec x)))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (y h))
  (precur (4 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (rec x) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul y (rec x))))
      (send (enc n (exp (gen) y))) (recv n))
    ((recv (exp (gen) y)) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (one))) (send (cat (exp (gen) y) (one))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (rec x)))
      (send (cat (exp (gen) y) (rec x)))) ((recv x) (send x)))
  (label 228)
  (parent 188)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h base) (w expr) (y x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y (rec x)))) (x x))
  (deflistener (exp (gen) (mul w y)))
  (deflistener (cat (exp (gen) y) w))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y) (mul w (rec x))))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (5 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (4 0)))
  (absent (x w) (y h))
  (precur (4 0) (2 0))
  (uniq-gen y x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y (rec x))))
      (send (enc n (exp (gen) (mul w y)))) (recv n))
    ((recv (exp (gen) (mul w y))) (send (exp (gen) (mul w y))))
    ((recv (cat (exp (gen) y) w)) (send (cat (exp (gen) y) w)))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y) (mul w (rec x))))
      (send (cat (exp (gen) y) (mul w (rec x))))) ((recv x) (send x)))
  (label 229)
  (parent 189)
  (unrealized (5 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (y expn) (w expr) (y-0 x expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y-0 (rec x)))) (x x))
  (deflistener (exp (gen) (mul w y-0)))
  (deflistener (cat (exp (gen) y) (mul (rec y) w y-0)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec x))))
  (defstrand resp 2 (h h-0) (y y-0))
  (deflistener x)
  (precedes ((0 0) (2 0)) ((0 0) (6 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (absent (x (mul (rec y) w y-0)) (y-0 h-0) (y h))
  (precur (4 0) (2 0))
  (uniq-gen y y-0 x)
  (uniq-orig n)
  (operation nonce-test (added-listener x) (mul w (rec x)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y-0 (rec x))))
      (send (enc n (exp (gen) (mul w y-0)))) (recv n))
    ((recv (exp (gen) (mul w y-0))) (send (exp (gen) (mul w y-0))))
    ((recv (cat (exp (gen) y) (mul (rec y) w y-0)))
      (send (cat (exp (gen) y) (mul (rec y) w y-0))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec x))))
      (send (cat (exp (gen) y-0) (mul w (rec x)))))
    ((recv h-0) (send (exp (gen) y-0))) ((recv x) (send x)))
  (label 230)
  (parent 192)
  (unrealized (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (x expn) (w expr) (y y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) (mul w y y-0))) (x x))
  (deflistener (exp (gen) (mul x w y y-0)))
  (deflistener (cat (exp (gen) y-0) (mul x w y)))
  (defstrand resp 2 (h h) (y y-0))
  (deflistener (cat (exp (gen) y) (mul w y-0)))
  (defstrand resp 2 (h h-0) (y y))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (6 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (4 0)) ((6 1) (4 0)))
  (absent (y-0 (mul x w y)) (y h-0) (y-0 h))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w y-0) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) (mul w y y-0)))
      (send (enc n (exp (gen) (mul x w y y-0)))) (recv n))
    ((recv (exp (gen) (mul x w y y-0)))
      (send (exp (gen) (mul x w y y-0))))
    ((recv (cat (exp (gen) y-0) (mul x w y)))
      (send (cat (exp (gen) y-0) (mul x w y))))
    ((recv h) (send (exp (gen) y-0)))
    ((recv (cat (exp (gen) y) (mul w y-0)))
      (send (cat (exp (gen) y) (mul w y-0))))
    ((recv h-0) (send (exp (gen) y))) ((recv y-0) (send y-0)))
  (label 231)
  (parent 193)
  (unrealized (1 0) (2 0) (6 0))
  (comment "empty cohort"))

(defskeleton plaindh
  (vars (n text) (h h-0 base) (x y expn) (w expr) (y-0 expn))
  (defstrand init 4 (n n) (h (exp (gen) w)) (x x))
  (deflistener (exp (gen) (mul x w)))
  (deflistener (cat (exp (gen) y) (mul x (rec y) w)))
  (defstrand resp 2 (h h) (y y))
  (deflistener (cat (exp (gen) y-0) (mul w (rec y-0))))
  (defstrand resp 2 (h h-0) (y y-0))
  (deflistener y-0)
  (precedes ((0 0) (2 0)) ((0 0) (4 0)) ((1 1) (0 3)) ((2 1) (1 0))
    ((3 1) (2 0)) ((3 1) (4 0)) ((4 1) (0 1)) ((5 1) (2 0))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (absent (y-0 (mul x (rec y) w)) (y-0 h-0) (y h))
  (precur (4 0) (2 0))
  (uniq-gen x y y-0)
  (uniq-orig n)
  (operation nonce-test (added-listener y-0) (mul w (rec y-0)) (4 0))
  (traces
    ((send (exp (gen) x)) (recv (exp (gen) w))
      (send (enc n (exp (gen) (mul x w)))) (recv n))
    ((recv (exp (gen) (mul x w))) (send (exp (gen) (mul x w))))
    ((recv (cat (exp (gen) y) (mul x (rec y) w)))
      (send (cat (exp (gen) y) (mul x (rec y) w))))
    ((recv h) (send (exp (gen) y)))
    ((recv (cat (exp (gen) y-0) (mul w (rec y-0))))
      (send (cat (exp (gen) y-0) (mul w (rec y-0)))))
    ((recv h-0) (send (exp (gen) y-0))) ((recv y-0) (send y-0)))
  (label 232)
  (parent 194)
  (unrealized (2 0) (6 0))
  (comment "empty cohort"))

(comment "Nothing left to do")
